@@ -51,11 +51,13 @@
 #include <config.h>
 #endif
 
-RCSID("$Id: klog.c,v 1.38 2003/06/04 11:46:31 hin Exp $");
+RCSID("$Id: klog.c,v 1.40 2005/04/01 17:36:02 lha Exp $");
 
 #include "appl_locl.h"
 #include "klog.h"
+#ifdef KERBEROS
 #include "kafs.h"
+#endif
 
 /*
  * State variables
@@ -499,7 +501,7 @@ main(int argc, char **argv)
 
     char pwbuf[PASSWD_MAX];
 
-    set_progname (argv[0]);
+    setprogname (argv[0]);
 
     method = log_open (getprogname(), "/dev/stderr:notime");
     if (method == NULL)
