@@ -33,9 +33,11 @@
 
 
 #include "appl_locl.h"
+#ifdef KERBEROS
 #include <kafs.h>
+#endif
 
-RCSID("$Id: up.c,v 1.6 2000/10/03 00:06:41 lha Exp $");
+RCSID("$Id: up.c,v 1.8 2005/11/17 15:44:55 tol Exp $");
 
 static void do_help (int exitval);
 static int  copyacl (char *from, char *to);
@@ -84,8 +86,8 @@ copyacl (char *from, char *to)
     a_params.out_size	= sizeof(buf);
     a_params.out	= buf;
 
-    if (k_pioctl (from, VIOCGETAL, &a_params, 1) != 0) {
-	fprintf (stderr, "k_pioctl(\"%s\", VIOCGETAL) failed %d\n", 
+    if (k_pioctl (from, ARLA_VIOCGETAL, &a_params, 1) != 0) {
+	fprintf (stderr, "k_pioctl(\"%s\", ARLA_VIOCGETAL) failed %d\n", 
 		 from, errno);
 	return errno;
     }
@@ -95,8 +97,8 @@ copyacl (char *from, char *to)
     a_params.out_size	= 0;
     a_params.out	= NULL;
 
-    if (k_pioctl (to, VIOCSETAL, &a_params, 1) != 0) {
-	fprintf (stderr, "k_pioctl(\"%s\", VIOCSETAL) failed %d\n",
+    if (k_pioctl (to, ARLA_VIOCSETAL, &a_params, 1) != 0) {
+	fprintf (stderr, "k_pioctl(\"%s\", ARLA_VIOCSETAL) failed %d\n",
 		 to, errno);
 	return errno;
     }
