@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995-2003, 2005 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -35,7 +35,7 @@
 #include <sl.h>
 #include "bos_local.h"
 
-RCSID("$Id: bos.c,v 1.11 2003/01/17 03:24:48 lha Exp $");
+RCSID("$Id: bos.c,v 1.14 2005/11/25 13:07:18 tol Exp $");
 
 int bos_interactive = 0;
 
@@ -105,7 +105,7 @@ static int
 quit_cmd(int argc, char **argv)
 {
     printf("exiting\n");
-    return 1;
+    return -2;
 }
 
 /*
@@ -159,9 +159,11 @@ main(int argc, char **argv)
     cell_init(0, method);
     ports_init();
     
-    if (argc > 1)
+    if (argc > 1) {
 	ret = sl_command(cmds, argc - 1, argv + 1);
-    else {
+	if (ret == -1)
+	    printf ("%s: Unknown command\n", argv[1]); 
+    } else {
 	bos_interactive = 1;
 	printf("bos - an arla tool for administrating AFS-servers.\n");
 	printf("Type \"help\" to get a list of commands.\n");
