@@ -35,7 +35,7 @@
 #include <config.h>
 #endif
 
-RCSID("$Id: kalog.c,v 1.8.2.1 2004/06/27 13:03:03 lha Exp $");
+RCSID("$Id: kalog.c,v 1.13 2005/12/11 15:10:14 lha Exp $");
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -53,6 +53,7 @@ RCSID("$Id: kalog.c,v 1.8.2.1 2004/06/27 13:03:03 lha Exp $");
 #include <ka-procs.h>
 
 #include <roken.h>
+#include <getarg.h>
 #include <err.h>
 
 #include <vers.h>
@@ -78,15 +79,35 @@ parse_user (char *argv1, const char **user, const char **cell)
     
 }
 
+static int num_hours = 8;
+static int help_flag;
+static int version_flag;
+
+static struct getargs args[] = {
+    {"hours",	  0,	arg_integer,	&num_hours,
+     "hours to get tickets for", NULL},
+    {"version",	0,	arg_flag,	&version_flag,
+     NULL, NULL},
+    {"help",	0,	arg_flag,	&help_flag,
+     NULL, NULL}
+};
+
+static void
+usage (int ret)
+{
+    arg_printusage (args, sizeof(args)/sizeof(*args), NULL, "[principal]");
+    exit (ret);
+}
+
 int
 main (int argc, char **argv)
 {
-    int ret;
+    int ret, optidx = 0;
     Log_method *method;
     const char *cellname;
     const char *user;
 
-    set_progname (argv[0]);
+    setprogname (argv[0]);
     tzset();
 
     method = log_open (getprogname(), "/dev/stderr:notime");
@@ -97,21 +118,29 @@ main (int argc, char **argv)
     
     cellname = cell_getthiscell();
 
-    if (argc == 1)
-	user = get_default_username();
-    else if (strcmp("-version", argv[1]) == 0) {
-	print_version(NULL);
-	return 0;
-    } else if (argc == 2)
-	parse_user (argv[1], &user, &cellname);
-    else {
-	fprintf (stderr, "usage: %s [-version] [username[@cell]]\n",
-		 getprogname());
-	exit (1);
+    if (getarg (args, sizeof(args)/sizeof(*args), argc, argv, &optidx))
+	usage (1);
+
+    argc -= optidx;
+    argv += optidx;
+
+    if (help_flag)
+	usage (0);
+
+    if (version_flag) {
+	print_version (NULL);
+	exit (0);
     }
+    
+    if (argc == 0)
+	user = get_default_username();
+    else if (argc == 1)
+	parse_user (argv[0], &user, &cellname);
+    else
+	usage(1);
 
     printf ("Getting ticket for %s@%s\n", user, cellname);
-    ret = ka_authenticate(user, "", cellname, NULL, 8 * 3600);
+    ret = ka_authenticate(user, "", cellname, NULL, num_hours * 3600);
     if (ret == EINVAL)
 	errx(1, "kalog built without kaserver support");
     else if (ret)
