@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: arlalib.h,v 1.50 2003/04/24 11:51:24 lha Exp $ */
+/* $Id: arlalib.h,v 1.51 2005/10/28 14:33:34 tol Exp $ */
 
 #ifndef ARLALIB_H
 #define ARLALIB_H 1
@@ -142,7 +142,10 @@ int
 fs_get_sysname (char *sys, size_t sys_sz);
 
 int
-fs_setcache(int lv, int hv, int lb, int hb);
+fs_setcacheparam(int64_t high_bytes,
+		 int64_t low_bytes,
+		 int64_t high_vnodes,
+		 int64_t low_vnodes);
 
 int
 fs_wscell (char *cell, size_t cell_sz);
