@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998 - 2001 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1998 - 2001, 2005 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -32,12 +32,28 @@
  */
 
 #include "appl_locl.h"
+#include <arla-pioctl.h>
+#ifdef KERBEROS
 #include <kafs.h>
+#endif
 
-RCSID("$Id: fs_lib.c,v 1.41 2003/01/17 03:04:47 lha Exp $");
+RCSID("$Id: fs_lib.c,v 1.46 2005/11/17 09:52:03 tol Exp $");
 
 enum { PIOCTL_MAXSIZE = 2000 };
 
+/*
+ *
+ */
+
+static int
+arla_k_pioctl(char *a_path,
+	      int o_opcode,
+	      struct arlaViceIoctl *a_paramsP,
+	      int a_followSymlinks)
+{
+    return k_pioctl(a_path, o_opcode, (void *)a_paramsP, a_followSymlinks);
+}
+
 /*
  * fserr, try to implement a generic function for fs error messages
  */
@@ -82,7 +98,7 @@ fserr(const char *progname, int error, const char *realpath)
 int
 fs_getfid(char *path, VenusFid *fid)
 {
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
 
     if (path == NULL || fid == NULL)
 	return EINVAL;
@@ -92,7 +108,7 @@ fs_getfid(char *path, VenusFid *fid)
     a_params.in=NULL;
     a_params.out=(void*) fid;
     
-    if(k_pioctl(path,VIOCGETFID,&a_params,1) == -1)
+    if(arla_k_pioctl(path,ARLA_VIOCGETFID,&a_params,1) == -1)
 	return errno;
 
     return 0;
@@ -105,14 +121,14 @@ fs_getfid(char *path, VenusFid *fid)
 int
 fs_nop(void)
 {
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
 
     a_params.in_size=0;
     a_params.out_size=0;
     a_params.in=NULL;
     a_params.out=NULL;
     
-    if (k_pioctl(NULL,VIOCNOP,&a_params,1) == -1) 
+    if (arla_k_pioctl(NULL,ARLA_VIOCNOP,&a_params,1) == -1) 
 	return errno;
 
     return 0;
@@ -125,14 +141,14 @@ fs_nop(void)
 int
 fs_getfilecellname(char *path, char *cell, size_t len)
 {
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
 
     a_params.in_size=0;
     a_params.out_size=len;
     a_params.in=NULL;
     a_params.out=cell;
     
-    if (k_pioctl(path,VIOC_FILE_CELL_NAME,&a_params,1) == -1) 
+    if (arla_k_pioctl(path,ARLA_VIOC_FILE_CELL_NAME,&a_params,1) == -1) 
 	return errno;
 
     return 0;
@@ -142,45 +158,41 @@ fs_getfilecellname(char *path, char *cell, size_t len)
  * set the level of crypt
  */
 
-#ifdef VIOC_SETRXKCRYPT
 int
 fs_setcrypt (uint32_t n)
 {
-    struct ViceIoctl	a_params;
+    struct arlaViceIoctl	a_params;
 
     a_params.in_size  = sizeof(n);
     a_params.out_size = 0;
     a_params.in	      = (char *)&n;
     a_params.out      = NULL;
 
-    if (k_pioctl (NULL, VIOC_SETRXKCRYPT, &a_params, 0) == -1)
+    if (arla_k_pioctl (NULL, ARLA_VIOC_SETRXKCRYPT, &a_params, 0) == -1)
 	return errno;
 
     return 0;
 }
-#endif
 
 /*
  * get currernt level of crypt
  */
 
-#ifdef VIOC_GETRXKCRYPT
 int
 fs_getcrypt (uint32_t *level)
 {
-    struct ViceIoctl	a_params;
+    struct arlaViceIoctl	a_params;
 
     a_params.in_size  = 0;
     a_params.out_size = sizeof(*level);
     a_params.in	      = NULL;
     a_params.out      = (char *) level;
 
-    if (k_pioctl (NULL, VIOC_GETRXKCRYPT, &a_params, 0) == -1) 
+    if (arla_k_pioctl (NULL, ARLA_VIOC_GETRXKCRYPT, &a_params, 0) == -1) 
 	return errno;
     
     return 0;
 }
-#endif
 
 /*
  * get and set the connect-mode
@@ -189,14 +201,14 @@ fs_getcrypt (uint32_t *level)
 int
 fs_connect(int32_t type, int32_t *flags)
 {
-    struct ViceIoctl   a_params;
+    struct arlaViceIoctl   a_params;
 
     a_params.in_size = sizeof(type);
     a_params.out_size = sizeof (int32_t);
     a_params.in = (char *) &type;
     a_params.out = (char *) flags;
 
-    if (k_pioctl (NULL, VIOCCONNECTMODE, &a_params, 0) == -1)
+    if (arla_k_pioctl (NULL, ARLA_AIOC_CONNECTMODE, &a_params, 0) == -1)
 	return errno;
 
     return 0;
@@ -206,14 +218,13 @@ fs_connect(int32_t type, int32_t *flags)
  *
  */
 
-#ifdef VIOC_FPRIOSTATUS
 int
 fs_setfprio(VenusFid fid, int16_t prio)
 {
-    struct ViceIoctl   a_params;
-    struct vioc_fprio  fprio;
+    struct arlaViceIoctl   a_params;
+    struct arla_vioc_fprio  fprio;
 
-    fprio.cmd = FPRIO_SET;
+    fprio.cmd = arla_FPRIO_SET;
     fprio.Cell = fid.Cell;
     fprio.Volume = fid.fid.Volume;
     fprio.Vnode = fid.fid.Vnode;
@@ -225,21 +236,19 @@ fs_setfprio(VenusFid fid, int16_t prio)
     a_params.in = (char *) &fprio;
     a_params.out = NULL;
 
-    if (k_pioctl (NULL, VIOC_FPRIOSTATUS , &a_params, 0) == -1)
+    if (arla_k_pioctl (NULL, ARLA_VIOC_FPRIOSTATUS , &a_params, 0) == -1)
 	return errno;
 
     return 0;
 }
-#endif
 
-#ifdef VIOC_FPRIOSTATUS
 int
 fs_getfprio(VenusFid fid, int16_t *prio)
 {
-    struct ViceIoctl   a_params;
-    struct vioc_fprio  fprio;
+    struct arlaViceIoctl   a_params;
+    struct arla_vioc_fprio  fprio;
 
-    fprio.cmd = FPRIO_GET;
+    fprio.cmd = arla_FPRIO_GET;
     fprio.Cell = fid.Cell;
     fprio.Volume = fid.fid.Volume;
     fprio.Vnode = fid.fid.Vnode;
@@ -250,21 +259,19 @@ fs_getfprio(VenusFid fid, int16_t *prio)
     a_params.in = (char *) &fprio;
     a_params.out = (char *) prio;
 
-    if (k_pioctl (NULL, VIOC_FPRIOSTATUS , &a_params, 0) == -1)
+    if (arla_k_pioctl (NULL, ARLA_VIOC_FPRIOSTATUS , &a_params, 0) == -1)
 	return errno;
 
     return 0;
 }
-#endif
 
-#ifdef VIOC_FPRIOSTATUS
 int
 fs_setmaxfprio(int16_t maxprio)
 {
-    struct ViceIoctl   a_params;
-    struct vioc_fprio  fprio;
+    struct arlaViceIoctl   a_params;
+    struct arla_vioc_fprio  fprio;
 
-    fprio.cmd = FPRIO_SETMAX;
+    fprio.cmd = arla_FPRIO_SETMAX;
     fprio.prio = maxprio;
 
     a_params.in_size = sizeof(fprio);
@@ -272,33 +279,30 @@ fs_setmaxfprio(int16_t maxprio)
     a_params.in = (char *) &fprio;
     a_params.out = NULL;
 
-    if (k_pioctl (NULL, VIOC_FPRIOSTATUS , &a_params, 0) == -1)
+    if (arla_k_pioctl (NULL, ARLA_VIOC_FPRIOSTATUS , &a_params, 0) == -1)
 	return errno;
 
     return 0;
 }
-#endif
 
-#ifdef VIOC_FPRIOSTATUS
 int
 fs_getmaxfprio(int16_t *maxprio)
 {
-    struct ViceIoctl   a_params;
-    struct vioc_fprio  fprio;
+    struct arlaViceIoctl   a_params;
+    struct arla_vioc_fprio  fprio;
 
-    fprio.cmd = FPRIO_GETMAX;
+    fprio.cmd = arla_FPRIO_GETMAX;
 
     a_params.in_size = sizeof(fprio);
     a_params.out_size = sizeof(*maxprio);
     a_params.in = (char *) &fprio;
     a_params.out = (char *) maxprio;
 
-    if (k_pioctl (NULL, VIOC_FPRIOSTATUS , &a_params, 0) == -1)
+    if (arla_k_pioctl (NULL, ARLA_VIOC_FPRIOSTATUS , &a_params, 0) == -1)
 	return errno;
 
     return 0;
 }
-#endif
 
 /*
  *
@@ -307,14 +311,14 @@ fs_getmaxfprio(int16_t *maxprio)
 static int
 getcacheparam(int32_t opcode, int64_t *val)
 {
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
 
     a_params.in_size  = sizeof(opcode);
     a_params.out_size = sizeof(*val);
     a_params.in       = (char *)&opcode;
     a_params.out      = (char *)val;
 
-    if (k_pioctl (NULL, AIOC_GETCACHEPARAMS , &a_params, 0) == -1)
+    if (arla_k_pioctl (NULL, ARLA_AIOC_GETCACHEPARAMS , &a_params, 0) == -1)
 	return errno;
     return 0;
 }
@@ -328,10 +332,10 @@ fs_getfilecachestats(int64_t *max_bytes,
 		     int64_t *low_vnodes)
 {
     uint32_t oldparms[16];
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
 
 #define query(o,v) \
-	if (v && getcacheparam(GETCACHEPARAMS_OPCODE_##o,v)) goto old
+	if (v && getcacheparam(arla_GETCACHEPARAMS_OPCODE_##o,v)) goto old
 
     query(HIGHBYTES, max_bytes);
     query(USEDBYTES, used_bytes);
@@ -354,7 +358,7 @@ fs_getfilecachestats(int64_t *max_bytes,
 
     memset (oldparms, 0, sizeof(oldparms));
 
-    if (k_pioctl (NULL, VIOCGETCACHEPARAMS , &a_params, 0) == -1)
+    if (arla_k_pioctl (NULL, ARLA_VIOCGETCACHEPARAMS , &a_params, 0) == -1)
 	return errno;
 
     /* param[0] and param[1] send maxbytes and usedbytes in kbytes */
@@ -380,20 +384,19 @@ fs_getfilecachestats(int64_t *max_bytes,
  *
  */
 
-#ifdef VIOC_AVIATOR
 int
 fs_getaviatorstats(uint32_t *max_workers,
 		   uint32_t *used_workers)
 {
     uint32_t parms[16];
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
 
     a_params.in_size = 0;
     a_params.out_size = sizeof(parms);
     a_params.in = NULL;
     a_params.out = (char *) parms;
 
-    if (k_pioctl (NULL, VIOC_AVIATOR , &a_params, 0) == -1)
+    if (arla_k_pioctl (NULL, ARLA_VIOC_AVIATOR , &a_params, 0) == -1)
 	return errno;
 
     if (max_workers)
@@ -403,17 +406,15 @@ fs_getaviatorstats(uint32_t *max_workers,
 
     return 0;
 }
-#endif
 
 /*
  *
  */
 
-#ifdef VIOC_GCPAGS
 int
 fs_gcpags(void)
 {
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
 
     a_params.in_size  = 0;
     a_params.out_size = 0;
@@ -421,31 +422,29 @@ fs_gcpags(void)
     a_params.out      = NULL;
 
 
-    if (k_pioctl(NULL, VIOC_GCPAGS, &a_params, 0) != 0)
+    if (arla_k_pioctl(NULL, ARLA_VIOC_GCPAGS, &a_params, 0) != 0)
 	return errno;
     
     return 0;
 }
-#endif
 
 /*
  *
  */
 
-#ifdef VIOC_CALCULATE_CACHE
 int
 fs_calculate_cache(uint32_t *calculated,
 		   uint32_t *usedbytes)
 {
     uint32_t parms[16];
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
 
     a_params.in_size = 0;
     a_params.out_size = sizeof(parms);
     a_params.in = NULL;
     a_params.out = (char *) parms;
 
-    if (k_pioctl (NULL, VIOC_CALCULATE_CACHE , &a_params, 0) == -1)
+    if (arla_k_pioctl (NULL, ARLA_VIOC_CALCULATE_CACHE , &a_params, 0) == -1)
 	return errno;
 
     if (calculated)
@@ -455,7 +454,6 @@ fs_calculate_cache(uint32_t *calculated,
 
     return 0;
 }
-#endif
 
 /*
  *
@@ -464,14 +462,14 @@ fs_calculate_cache(uint32_t *calculated,
 int
 fs_invalidate (const char *path)
 {
-    struct ViceIoctl   a_params;
+    struct arlaViceIoctl   a_params;
 
     a_params.in_size  = 0;
     a_params.out_size = 0;
     a_params.in       = NULL;
     a_params.out      = NULL;
     
-    if (k_pioctl ((char *)path, VIOC_BREAKCALLBACK, &a_params, 0) < 0)
+    if (arla_k_pioctl ((char *)path, ARLA_VIOC_BREAKCALLBACK, &a_params, 0) < 0)
 	return errno;
     else
 	return 0;
@@ -487,7 +485,7 @@ fs_invalidate (const char *path)
 static int
 debug (int pioctl_cmd, int inflags, int *outflags, char *pathname)
 {
-    struct ViceIoctl   a_params;
+    struct arlaViceIoctl   a_params;
 
     int32_t rinflags = inflags;
     int32_t routflags;
@@ -508,7 +506,7 @@ debug (int pioctl_cmd, int inflags, int *outflags, char *pathname)
 	a_params.out = NULL;
     }
 
-    if (k_pioctl (pathname, pioctl_cmd, &a_params, 0) == -1)
+    if (arla_k_pioctl (pathname, pioctl_cmd, &a_params, 0) == -1)
 	return errno;
     
     if (outflags)
@@ -521,37 +519,31 @@ debug (int pioctl_cmd, int inflags, int *outflags, char *pathname)
  * nnpfs_debug
  */
 
-#ifdef VIOC_NNPFSDEBUG
 int
 nnpfs_debug(int inflags, int *outflags)
 {
-    return debug (VIOC_NNPFSDEBUG, inflags, outflags, NULL);
+    return debug (ARLA_VIOC_NNPFSDEBUG, inflags, outflags, NULL);
 }
-#endif
 
 /*
  * nnpfs_debug_print
  */
 
-#ifdef VIOC_NNPFSDEBUG_PRINT
 int
 nnpfs_debug_print(int inflags, char *pathname)
 {
-    return debug (VIOC_NNPFSDEBUG_PRINT, inflags, NULL, pathname);
+    return debug (ARLA_VIOC_NNPFSDEBUG_PRINT, inflags, NULL, pathname);
 }
-#endif
 
 /*
  * arla_debug
  */
 
-#ifdef VIOC_ARLADEBUG
 int
 arla_debug (int inflags, int *outflags)
 {
-    return debug (VIOC_ARLADEBUG, inflags, outflags, NULL);
+    return debug (ARLA_VIOC_ARLADEBUG, inflags, outflags, NULL);
 }
-#endif
 
 /*
  * checkservers
@@ -563,7 +555,7 @@ arla_debug (int inflags, int *outflags)
 int
 fs_checkservers(char *cell, int32_t flags, uint32_t *hosts, int numhosts)
 {
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
     char *in = NULL;
     int ret;
     size_t insize;
@@ -591,7 +583,7 @@ fs_checkservers(char *cell, int32_t flags, uint32_t *hosts, int numhosts)
 
     ret = 0;
 
-    if (k_pioctl (NULL, VIOCCKSERV, &a_params, 0) == -1)
+    if (arla_k_pioctl (NULL, ARLA_VIOCCKSERV, &a_params, 0) == -1)
 	ret = errno;
     
     if (in)
@@ -607,14 +599,14 @@ fs_checkservers(char *cell, int32_t flags, uint32_t *hosts, int numhosts)
 int
 fs_checkvolumes (void)
 {
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
 
     a_params.in       = NULL;
     a_params.in_size  = 0;
     a_params.out      = NULL;
     a_params.out_size = 0;
 
-    if (k_pioctl (NULL, VIOCCKBACK, &a_params, 0) < 0)
+    if (arla_k_pioctl (NULL, ARLA_VIOCCKBACK, &a_params, 0) < 0)
 	return errno;
     else
 	return 0;
@@ -627,7 +619,7 @@ fs_checkvolumes (void)
 int
 fs_get_sysname (char *sys, size_t sys_sz)
 {
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
     int32_t set = 0;
     char *buf;
 
@@ -640,7 +632,7 @@ fs_get_sysname (char *sys, size_t sys_sz)
     a_params.out      = buf;
     a_params.out_size = sys_sz + 4;
 
-    if(k_pioctl (NULL, VIOC_AFS_SYSNAME, &a_params, 1) < 0)
+    if(arla_k_pioctl (NULL, ARLA_VIOC_AFS_SYSNAME, &a_params, 1) < 0)
 	return errno;
     else {
 	strlcpy (sys, buf + 4, sys_sz);
@@ -655,7 +647,7 @@ fs_get_sysname (char *sys, size_t sys_sz)
 int
 fs_set_sysname (const char *sys)
 {
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
     int32_t set = 1;
 
     a_params.in_size  = sizeof(set) + strlen(sys) + 1;
@@ -667,20 +659,20 @@ fs_set_sysname (const char *sys)
     memcpy (a_params.in, &set, sizeof(set));
     strcpy (a_params.in + sizeof(set), sys);
 
-    if(k_pioctl (NULL, VIOC_AFS_SYSNAME, &a_params, 1) < 0)
+    if(arla_k_pioctl (NULL, ARLA_VIOC_AFS_SYSNAME, &a_params, 1) < 0)
 	return errno;
     else
 	return 0;
 }
 
 /*
- *
+ * fs_setcache -- use the old interface, fallback only
  */
 
-int
+static int
 fs_setcache(int lv, int hv, int lb, int hb)
 {
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
     uint32_t s[4];
 
     s[0] = lv;
@@ -693,12 +685,72 @@ fs_setcache(int lv, int hv, int lb, int hb)
     a_params.in       = (void *)s;
     a_params.out      = NULL;
 
-    if (k_pioctl(NULL, VIOCSETCACHESIZE, &a_params, 0) < 0)
+    if (arla_k_pioctl(NULL, ARLA_VIOCSETCACHESIZE, &a_params, 0) < 0)
 	return errno;
     else
 	return 0;
 }
 
+/*
+ * setcacheparam -- utility function using the new interface
+ */
+
+static int
+setcacheparam(int32_t opcode, int64_t *high, int64_t *low)
+{
+    struct arlaViceIoctl a_params;
+    char buf[sizeof(opcode) + sizeof(*high) + sizeof(*low)];
+    char *p = buf;
+
+    memcpy(p, &opcode, sizeof(opcode));
+    p += sizeof(opcode);
+
+    memcpy(p, high, sizeof(*high));
+    p += sizeof(*high);
+
+    memcpy(p, low, sizeof(*low));
+
+    a_params.in_size  = sizeof(opcode) + sizeof(*high) + sizeof(*low);
+    a_params.out_size = 0;
+    a_params.in       = buf;
+    a_params.out      = NULL;
+
+    if (arla_k_pioctl (NULL, ARLA_AIOC_SETCACHEPARAMS , &a_params, 0) == -1)
+	return errno;
+    return 0;
+}
+
+/*
+ * fs_setcacheparam -- try new interface first, fall back on old if necessary
+ */
+
+int
+fs_setcacheparam(int64_t high_bytes,
+		 int64_t low_bytes,
+		 int64_t high_vnodes,
+		 int64_t low_vnodes)
+{
+    int error = 0;
+
+    if (high_bytes != 0 || low_bytes != 0)
+	error = setcacheparam(arla_SETCACHEPARAMS_OPCODE_BYTES,
+			      &high_bytes, &low_bytes);
+    
+    if (error == 0 && (high_vnodes != 0 || low_vnodes != 0))
+	error = setcacheparam(arla_SETCACHEPARAMS_OPCODE_VNODES,
+			      &high_vnodes, &low_vnodes);
+    
+    if (error == 0)
+	return error;
+    
+    /* fall back on old style, int32 */
+    if (high_bytes > INT_MAX || low_bytes > INT_MAX
+	|| high_vnodes > INT_MAX || low_vnodes > INT_MAX)
+	return EINVAL;
+
+    return fs_setcache(low_vnodes, high_vnodes, low_bytes, high_bytes);
+}
+
 /*
  * return the local cell in `cell' (of size `cell_sz').
  */
@@ -706,14 +758,14 @@ fs_setcache(int lv, int hv, int lb, int hb)
 int
 fs_wscell (char *cell, size_t cell_sz)
 {
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
 
     a_params.in_size  = 0;
     a_params.in       = NULL;
     a_params.out_size = cell_sz;
     a_params.out      = cell;
 
-    if (k_pioctl (NULL, VIOC_GET_WS_CELL, &a_params, 0) < 0)
+    if (arla_k_pioctl (NULL, ARLA_VIOC_GET_WS_CELL, &a_params, 0) < 0)
 	return errno;
     return 0;
 }
@@ -725,14 +777,14 @@ fs_wscell (char *cell, size_t cell_sz)
 int
 fs_flushvolume (const char *path)
 {
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
 
     a_params.in_size  = 0;
     a_params.out_size = 0;
     a_params.in       = NULL;
     a_params.out      = NULL;
 
-    if (k_pioctl ((char *)path, VIOC_FLUSHVOLUME, &a_params, 0) < 0)
+    if (arla_k_pioctl ((char *)path, ARLA_VIOC_FLUSHVOLUME, &a_params, 0) < 0)
 	return errno;
     else
 	return 0;
@@ -745,14 +797,14 @@ fs_flushvolume (const char *path)
 int
 fs_flush (const char *path)
 {
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
 
     a_params.in_size  = 0;
     a_params.out_size = 0;
     a_params.in       = NULL;
     a_params.out      = NULL;
 
-    if (k_pioctl ((char *)path, VIOCFLUSH, &a_params, 0) < 0)
+    if (arla_k_pioctl ((char *)path, ARLA_VIOCFLUSH, &a_params, 0) < 0)
 	return errno;
     else
 	return 0;
@@ -765,7 +817,7 @@ fs_flush (const char *path)
 int
 fs_venuslog (void)
 {
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
     int32_t status = 0;   /* XXX not really right, but anyway */
 
     a_params.in_size  = sizeof(int32_t);
@@ -773,7 +825,7 @@ fs_venuslog (void)
     a_params.in       = (caddr_t) &status;
     a_params.out      = NULL;
 
-    if (k_pioctl (NULL, VIOC_VENUSLOG, &a_params, 0) < 0)
+    if (arla_k_pioctl (NULL, ARLA_VIOC_VENUSLOG, &a_params, 0) < 0)
 	return errno;
     else
 	return 0;
@@ -787,7 +839,7 @@ fs_venuslog (void)
 int
 fs_newcell (const char *cell, int nservers, char **servers)
 {
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
     int len;
     char *buf;
     int i, ret;
@@ -825,7 +877,7 @@ fs_newcell (const char *cell, int nservers, char **servers)
     a_params.in       = (caddr_t)buf;
     a_params.out      = NULL;
 
-    ret = k_pioctl (NULL, VIOCNEWCELL, &a_params, 0);
+    ret = arla_k_pioctl (NULL, ARLA_VIOCNEWCELL, &a_params, 0);
     free (buf);
     if (ret < 0)
 	return errno;
@@ -842,7 +894,7 @@ int
 fs_getcells (int32_t num, uint32_t *server, int numservers,
 	     char *cell, size_t cell_sz)
 {
-    struct ViceIoctl   a_params;
+    struct arlaViceIoctl   a_params;
     int32_t *server_list;
     int i;
     
@@ -862,7 +914,7 @@ fs_getcells (int32_t num, uint32_t *server, int numservers,
 
     server_list = (int32_t *) a_params.out;
 
-    if (k_pioctl (NULL, VIOCGETCELL, &a_params, 0) != 0)
+    if (arla_k_pioctl (NULL, ARLA_VIOCGETCELL, &a_params, 0) != 0)
 	return errno;
     
     if (numservers > GETCELL_MAXSERVER)
@@ -884,14 +936,14 @@ fs_getcells (int32_t num, uint32_t *server, int numservers,
 int
 fs_getcellstatus (char *cellname, uint32_t *flags)
 {
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
 
     a_params.in_size  = strlen (cellname) + 1;
     a_params.out_size = sizeof (uint32_t);
     a_params.in       = cellname;
     a_params.out      = (caddr_t) flags;
 
-    if (k_pioctl (NULL, VIOC_GETCELLSTATUS, &a_params, 0) < 0)
+    if (arla_k_pioctl (NULL, ARLA_VIOC_GETCELLSTATUS, &a_params, 0) < 0)
 	return errno;
     else
 	return 0;
@@ -905,7 +957,7 @@ fs_getcellstatus (char *cellname, uint32_t *flags)
 static int
 internal_mp (const char *path, int pioctl_cmd, char **res)
 {
-    struct ViceIoctl    a_params;
+    struct arlaViceIoctl    a_params;
     char               *last;
     char               *path_bkp;
     int			error;
@@ -929,13 +981,21 @@ internal_mp (const char *path, int pioctl_cmd, char **res)
     if (last != NULL) {
 	*last = '\0';
 	a_params.in = last + 1;
-    } else
+    } else 
 	a_params.in = (char *)path;
 
+    if (strcmp((char *)a_params.in, ".") == 0
+	|| strcmp((char *)a_params.in, "..") == 0) {
+	printf("fs: don't use '%s' as the last component in lsmount\n",
+	       (char *)a_params.in);
+	free (path_bkp);
+	return ENOMEM;
+    }
+    
     a_params.in_size = strlen (a_params.in) + 1;
     a_params.out_size = PIOCTL_MAXSIZE;
 
-    error = k_pioctl (last ? path_bkp : "." ,
+    error = arla_k_pioctl (last ? path_bkp : "." ,
 		      pioctl_cmd, &a_params, 1);
     if (error < 0) {
 	error = errno;
@@ -956,7 +1016,7 @@ int
 fs_lsmount (const char *path)
 {
     char *res;
-    int error = internal_mp (path, VIOC_AFS_STAT_MT_PT, &res);
+    int error = internal_mp (path, ARLA_VIOC_AFS_STAT_MT_PT, &res);
 
     if (error == 0) {
 	printf ("'%s' is a mount point for volume '%s'\n", path, res);
@@ -968,13 +1028,13 @@ fs_lsmount (const char *path)
 int
 fs_rmmount (const char *path)
 {
-    return internal_mp (path, VIOC_AFS_DELETE_MT_PT, NULL);
+    return internal_mp (path, ARLA_VIOC_AFS_DELETE_MT_PT, NULL);
 }
 
 int
 fs_incompat_renumber (int *ret)
 {
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
     unsigned char buf[1024];
 
     a_params.in_size  = 0;
@@ -983,7 +1043,7 @@ fs_incompat_renumber (int *ret)
     a_params.out      = (caddr_t) buf;
 
     /* getcrypt or getinitparams */
-    if (k_pioctl (NULL, _VICEIOCTL(49), &a_params, 0) < 0) {
+    if (arla_k_pioctl (NULL, _VICEIOCTL(49), &a_params, 0) < 0) {
 	if (errno == EINVAL) {
 
 	    /* not openafs or old openafs */
@@ -993,7 +1053,7 @@ fs_incompat_renumber (int *ret)
 	    a_params.in       = 0;
 	    a_params.out      = (caddr_t) buf;
 	    
-	    if (k_pioctl (NULL, _VICEIOCTL(49), &a_params, 0) < 0) {
+	    if (arla_k_pioctl (NULL, _VICEIOCTL(49), &a_params, 0) < 0) {
 		if (errno == EINVAL) {
 		    
 		    a_params.in_size  = 0;
@@ -1003,7 +1063,7 @@ fs_incompat_renumber (int *ret)
 		    
 		    /* might be new interface */
 
-		    if (k_pioctl (NULL, _VICEIOCTL(55), &a_params, 0) < 0)
+		    if (arla_k_pioctl (NULL, _VICEIOCTL(55), &a_params, 0) < 0)
 			return errno; /* dunno */
 		    
 		    *ret = 1;
@@ -1031,10 +1091,10 @@ fs_statistics_list(uint32_t *host, uint32_t *part, int *n)
 {
     uint32_t data[512];
     uint32_t indata;
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
     int i;
 
-    indata = STATISTICS_OPCODE_LIST;
+    indata = arla_STATISTICS_OPCODE_LIST;
 
     a_params.in_size  = sizeof(indata);
     a_params.out_size = sizeof(data);
@@ -1043,7 +1103,7 @@ fs_statistics_list(uint32_t *host, uint32_t *part, int *n)
 
     memset (data, 0, sizeof(data));
 
-    if (k_pioctl (NULL, AIOC_STATISTICS , &a_params, 0) == -1)
+    if (arla_k_pioctl (NULL, ARLA_AIOC_STATISTICS , &a_params, 0) == -1)
 	return errno;
 
     if (data[0] < *n)
@@ -1064,11 +1124,11 @@ fs_statistics_entry(uint32_t host, uint32_t part, uint32_t type,
 {
     uint32_t data[160];
     uint32_t indata[5];
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
     int i;
     int j;
 
-    indata[0] = STATISTICS_OPCODE_GETENTRY;
+    indata[0] = arla_STATISTICS_OPCODE_GETENTRY;
     indata[1] = host;
     indata[2] = part;
     indata[3] = type;
@@ -1081,7 +1141,7 @@ fs_statistics_entry(uint32_t host, uint32_t part, uint32_t type,
 
     memset (data, 0, sizeof(data));
 
-    if (k_pioctl (NULL, AIOC_STATISTICS , &a_params, 0) == -1)
+    if (arla_k_pioctl (NULL, ARLA_AIOC_STATISTICS , &a_params, 0) == -1)
 	return errno;
 
     j = 0;
