@@ -33,7 +33,7 @@
 
 #include "appl_locl.h"
 
-RCSID("$Id: tokens.c,v 1.9 2003/02/01 17:01:11 lha Exp $");
+RCSID("$Id: tokens.c,v 1.10 2005/11/17 15:44:55 tol Exp $");
 
 /*
  * Iterate over all tokens and apply `func' on all of them, passing
@@ -65,7 +65,7 @@ arlalib_token_iter (const char *cell, arlalib_token_iter_func func, void *arg)
 	parms.out = (void *)data;
 	parms.out_size = sizeof(data); 
 	
-	if(k_pioctl(NULL, VIOCGETTOK, &parms, 0) != 0) {
+	if(k_pioctl(NULL, ARLA_VIOCGETTOK, &parms, 0) != 0) {
 	    if(errno == EDOM)
 		break;
 	    return EINVAL;
