@@ -3,6 +3,7 @@
 #import "Controller.h"
 #include <stdio.h>
 #include <sys/ioctl.h>
+#include <arla-pioctl.h>
 #include <kafs.h>
 #include "config.h"
 #include <roken.h>
@@ -10,6 +11,7 @@
 #include <sys/wait.h>
 
 static struct units size_units[] = {
+    { "G", 1024 * 1024 * 1024 },
     { "M", 1024 * 1024 },
     { "k", 1024 },
     { NULL, 0 }
@@ -129,27 +131,6 @@ nnpfs_mount(AuthorizationRef authorization)
     }
 }
 
-static int
-disktool_refresh(AuthorizationRef authorization)
-{
-    char *argv[2];
-    OSStatus status;
-    FILE *output;
-
-    argv[0] = "-r";
-    argv[1] = NULL;
-
-    status = AuthorizationExecuteWithPrivileges(authorization, "/usr/sbin/disktool", 0, argv, &output);
-    if (status == noErr) {
-        drainfile(output);
-        fclose(output);
-	drainproc();
-        return 0;
-    } else {
-        return -1;
-    }
-}
-
 static int
 stoparlad(AuthorizationRef authorization)
 {
@@ -205,6 +186,8 @@ kmodunload(AuthorizationRef authorization)
     OSStatus status;
     FILE *output;
 
+    sleep(2); /* wait 2 secs for arlad to clean up */
+
     argv[0] = "/usr/arla/bin/nnpfs.kext";
     argv[1] = NULL;
 
@@ -263,45 +246,48 @@ mkafsdir(AuthorizationRef authorization)
 }
 
 static int
-getcache(int *max_bytes, int *used_bytes, int *max_vnodes, int *used_vnodes,
-	 int *min_bytes, int *min_vnodes)
+getcacheparam(int32_t opcode, int64_t *val)
 {
-    u_int32_t parms[16];
-    struct ViceIoctl a_params;
+    struct arlaViceIoctl a_params;
 
-    a_params.in_size  = 0;
-    a_params.out_size = sizeof(parms);
-    a_params.in       = NULL;
-    a_params.out      = (char *) parms;
+    a_params.in_size  = sizeof(opcode);
+    a_params.out_size = sizeof(*val);
+    a_params.in       = (char *)&opcode;
+    a_params.out      = (char *)val;
 
-    memset (parms, 0, sizeof(parms));
-
-    if (!k_hasafs())
-        return ENOSYS;
-
-    if (k_pioctl (NULL, VIOCGETCACHEPARAMS , &a_params, 0) == -1)
-        return errno;
-
-    /* param[0] and param[1] send maxbytes and usedbytes in kbytes */
-
-    if (max_vnodes)
-        *max_vnodes = parms[2];
-    if (used_vnodes)
-        *used_vnodes = parms[3];
-    if (max_bytes)
-        *max_bytes = parms[4];
-    if (used_bytes)
-        *used_bytes = parms[5];
-    if (min_bytes)
-	*min_bytes = parms[6];
-    if (min_vnodes)
-	*min_vnodes = parms[7];
+    if (k_pioctl(NULL, ARLA_AIOC_GETCACHEPARAMS, (void *)&a_params, 0) == -1)
+	return errno;
 
     return 0;
 }
 
 static int
-setcache(int max_bytes, int min_bytes, int max_vnodes, int min_vnodes,
+getcache(int64_t *high_bytes,
+	 int64_t *used_bytes,
+	 int64_t *low_bytes,
+	 int64_t *high_vnodes,
+	 int64_t *used_vnodes,
+	 int64_t *low_vnodes)
+{
+    int ret;
+    
+    if (!k_hasafs_recheck())
+	return ENOSYS;
+    
+    if ((ret = getcacheparam(arla_GETCACHEPARAMS_OPCODE_HIGHBYTES, high_bytes) != 0)
+	|| (ret = getcacheparam(arla_GETCACHEPARAMS_OPCODE_USEDBYTES, used_bytes) != 0)
+	|| (ret = getcacheparam(arla_GETCACHEPARAMS_OPCODE_LOWBYTES, low_bytes) != 0)
+	|| (ret = getcacheparam(arla_GETCACHEPARAMS_OPCODE_HIGHVNODES, high_vnodes) != 0)
+	|| (ret = getcacheparam(arla_GETCACHEPARAMS_OPCODE_USEDVNODES, used_vnodes) != 0)
+	|| (ret = getcacheparam(arla_GETCACHEPARAMS_OPCODE_LOWVNODES, low_vnodes) != 0))
+	return ret;
+    
+    return 0;
+}
+
+static int
+setcache(long long high_bytes, long long low_bytes,
+	 long long high_vnodes, long long low_vnodes,
 	 AuthorizationRef authorization)
 {
     char *argv[6];
@@ -315,10 +301,10 @@ setcache(int max_bytes, int min_bytes, int max_vnodes, int min_vnodes,
     argv[4] = malloc(100);
     argv[5] = NULL;
 
-    snprintf(argv[1], 100, "%d", min_vnodes);
-    snprintf(argv[2], 100, "%d", max_vnodes);
-    snprintf(argv[3], 100, "%d", min_bytes);
-    snprintf(argv[4], 100, "%d", max_bytes);
+    snprintf(argv[1], 100, "%lld", high_bytes / 1024);
+    snprintf(argv[2], 100, "%lld", low_bytes / 1024);
+    snprintf(argv[3], 100, "%lld", high_vnodes);
+    snprintf(argv[4], 100, "%lld", low_vnodes);
 
     status = AuthorizationExecuteWithPrivileges(authorization,
 						"/usr/arla/bin/fs",
@@ -496,7 +482,7 @@ row:(int)row {
     char *save = NULL;
     char *n;
     char *v;
-    unsigned val;
+    long long val;
     char *endptr;
     char *line1;
     NSDictionary *dict;
@@ -520,16 +506,27 @@ row:(int)row {
 	return nil;
     }
     
-    val = parse_units(v, size_units, NULL);
-    if(val == (unsigned)-1) {
-	val = strtol(v, &endptr, 0);
-	if (endptr == v) {
-	    free(line1);
-	    return nil;
+    /* parse_units is currently broken, so we roll our own for now */
+    val = strtoll(v, &endptr, 0);
+    
+    if (endptr == v) {
+	free(line1);
+	return nil;
+    }
+
+    if (*endptr != '\0') {
+	struct units *u = size_units;
+	
+	while (u->name != NULL) {
+	    if (!strcmp(endptr, u->name)) {
+		val *= u->mult;
+		break;
+	    }
+	    u++;
 	}
     }
-    
-    dict = [NSDictionary dictionaryWithObject: [NSNumber numberWithInt: val]
+
+    dict = [NSDictionary dictionaryWithObject: [NSNumber numberWithLongLong: val]
 			 forKey: [NSString stringWithCString: n]];
     free(line1);
     return dict;
@@ -570,21 +567,20 @@ row:(int)row {
     NSString *key;
     
     while ((key = [enumerator nextObject])) {
-	[data appendString: [NSString stringWithFormat: @"%s %d\n",
+	[data appendString: [NSString stringWithFormat: @"%s %lld\n",
 				      [key cString],
-				      [[dict objectForKey: key] intValue]]];
+				      [[dict objectForKey: key] longLongValue]]];
     }
 }
 
 - (OSStatus)getCache: (AuthorizationRef) authorization
-	    maxBytes: (int *) maxBytes minBytes: (int *) minBytes
-	    maxFiles: (int *) maxFiles minFiles: (int *) minFiles
-	    curBytes: (int *) curBytes curFiles: (int *) curFiles {
+	    maxBytes: (int64_t *) maxBytes minBytes: (int64_t *) minBytes
+	    maxFiles: (int64_t *) maxFiles minFiles: (int64_t *) minFiles
+	    curBytes: (int64_t *) curBytes curFiles: (int64_t *) curFiles {
     NSMutableString *data;
     NSMutableDictionary *dict;
 
-    if (getcache(maxBytes, curBytes, maxFiles, curFiles,
-		 minBytes, minFiles) == 0)
+    if (getcache(maxBytes, curBytes, minBytes, maxFiles, curFiles, minFiles) == 0)
 	return 0;
 
     data = [[[NSMutableString alloc] init] autorelease];
@@ -594,10 +590,10 @@ row:(int)row {
 
     [self parseConf: data dict: dict];
     
-    *maxBytes = [[dict objectForKey: @"high_bytes"] intValue];
-    *minBytes = [[dict objectForKey: @"low_bytes"] intValue];
-    *maxFiles = [[dict objectForKey: @"high_vnodes"] intValue];
-    *minFiles = [[dict objectForKey: @"low_vnodes"] intValue];
+    *maxBytes = [[dict objectForKey: @"high_bytes"] longLongValue];
+    *minBytes = [[dict objectForKey: @"low_bytes"] longLongValue];
+    *maxFiles = [[dict objectForKey: @"high_vnodes"] longLongValue];
+    *minFiles = [[dict objectForKey: @"low_vnodes"] longLongValue];
     *curBytes = 0;
     *curFiles = 0;
 
@@ -606,8 +602,10 @@ row:(int)row {
     
 
 - (OSStatus)saveConfData: (AuthorizationRef) authorization
-		maxBytes: (int) maxBytes minBytes: (int) minBytes
-		maxFiles: (int) maxFiles minFiles: (int) minFiles
+		maxBytes: (long long) maxBytes
+		minBytes: (long long) minBytes
+		maxFiles: (long long) maxFiles
+		minFiles: (long long) minFiles
 	     startAtBoot: (int) startAtBoot {
     NSMutableString *data;
     NSMutableDictionary *dict;
@@ -621,13 +619,13 @@ row:(int)row {
 
     [data setString: @""];
 
-    [dict setObject: [NSNumber numberWithInt: maxBytes]
+    [dict setObject: [NSNumber numberWithLongLong: maxBytes]
 	  forKey: @"high_bytes"];
-    [dict setObject: [NSNumber numberWithInt: minBytes]
+    [dict setObject: [NSNumber numberWithLongLong: minBytes]
 	  forKey: @"low_bytes"];
-    [dict setObject: [NSNumber numberWithInt: maxFiles]
+    [dict setObject: [NSNumber numberWithLongLong: maxFiles]
 	  forKey: @"high_vnodes"];
-    [dict setObject: [NSNumber numberWithInt: minFiles]
+    [dict setObject: [NSNumber numberWithLongLong: minFiles]
 	  forKey: @"low_vnodes"];
     
     [self writeConf: data dict: dict];
@@ -689,12 +687,10 @@ row:(int)row {
 	select(0, NULL, NULL, NULL, &t);
     }
     nnpfs_mount(authorization);
-    disktool_refresh(authorization);
 }
 
 + (void) stopDaemon: (AuthorizationRef) authorization {
     nnpfs_umount(authorization);
-    disktool_refresh(authorization);
     stoparlad(authorization);
     kmodunload(authorization);
 }
