@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: arlacmm.h,v 1.8 2002/04/23 16:25:45 ahltorp Exp $ */
+/* $Id: arlacmm.h,v 1.9 2005/10/28 14:33:35 tol Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -49,6 +49,7 @@
 
 #include <atypes.h>
 #include <roken.h>
+#include <arla-pioctl.h>
 
 typedef struct ArlaCMMType {
     ContextualMenuInterfaceStruct *_contextualMenuInterface;
@@ -104,6 +105,8 @@ typedef struct ArlaCMMType {
 
 #define kInfoFileNameControl 100
 #define kInfoVolumeName 101
+#define kInfoCellName 104
+#define kInfoFileServers 105
 #define kInfoQuotaUsable 102
 #define kInfoQuotaUsed 103
 
@@ -125,6 +128,12 @@ getfid(UInt8 *path, struct VenusFid *fid);
 int
 isInAFS(UInt8 *path, int *isdir);
 
+int
+getfilecellname(UInt8 *path, CFStringRef *string);
+
+int
+getfileservers(UInt8 *path, CFStringRef *string);
+
 int
 verifyname(CFStringRef path, CFStringRef name);
 
