@@ -33,12 +33,14 @@
 
 #include "arlacmm.h"
 
-RCSID("$Id: infowin.c,v 1.1 2002/04/23 00:16:12 ahltorp Exp $");
+RCSID("$Id: infowin.c,v 1.2 2005/10/28 14:33:35 tol Exp $");
 
 static ControlID textcontrolid = {kControlID, kInfoFileNameControl};
 static ControlID volumenameid = {kControlID, kInfoVolumeName};
 static ControlID quotausableid = {kControlID, kInfoQuotaUsable};
 static ControlID quotausedid = {kControlID, kInfoQuotaUsed};
+static ControlID cellnameid = {kControlID, kInfoCellName};
+static ControlID fileserversid = {kControlID, kInfoFileServers};
 
 static pascal OSStatus
 CloseWindowEventHandler(EventHandlerCallRef handlerRef,
@@ -88,11 +90,15 @@ infoWindow (ArlaCMMType *this, UInt8 *path)
     ControlHandle volumenamecontrol;
     ControlHandle quotausablecontrol;
     ControlHandle quotausedcontrol;
+    ControlHandle cellnamecontrol;
+    ControlHandle fileserverscontrol;
     CFStringRef volname;
     int quota;
     int used;
     CFStringRef quotastring;
     CFStringRef usedstring;
+    CFStringRef cellname;
+    CFStringRef fileservers;
 
     ret = createWindow(this);
     if (ret != noErr)
@@ -102,6 +108,19 @@ infoWindow (ArlaCMMType *this, UInt8 *path)
     if (ret)
 	return ret;
 
+    ret = getfilecellname(path, &cellname);
+    if (ret) {
+        CFRelease(volname);
+	return ret;
+    }
+
+    ret = getfileservers(path, &fileservers);
+    if (ret) {
+        CFRelease(volname);
+	CFRelease(cellname);
+	return ret;
+    }
+
     GetControlByID(this->infowindow, &textcontrolid, &textcontrol);
     this->path = CFStringCreateWithCString(NULL, path, kCFStringEncodingUTF8);
     ret = SetControlData(textcontrol, 0, kControlEditTextCFStringTag,
@@ -125,9 +144,19 @@ infoWindow (ArlaCMMType *this, UInt8 *path)
     ret = SetControlData(quotausedcontrol, 0, kControlEditTextCFStringTag,
 			 sizeof(CFStringRef), &usedstring);
 
+    GetControlByID(this->infowindow, &cellnameid, &cellnamecontrol);
+    ret = SetControlData(cellnamecontrol, 0, kControlEditTextCFStringTag,
+			 sizeof(CFStringRef), &cellname);
+
+    GetControlByID(this->infowindow, &fileserversid, &fileserverscontrol);
+    ret = SetControlData(fileserverscontrol, 0, kControlEditTextCFStringTag,
+			 sizeof(CFStringRef), &fileservers);
+
     CFRelease(volname);
     CFRelease(quotastring);
     CFRelease(usedstring);
+    CFRelease(cellname);
+    CFRelease(fileservers);
 
     ShowWindow(this->infowindow);
     return 0;
