@@ -33,14 +33,14 @@
 
 #include "arlacmm.h"
 
-RCSID("$Id: pioctl.c,v 1.4 2002/04/23 10:10:13 ahltorp Exp $");
+RCSID("$Id: pioctl.c,v 1.6 2005/11/17 15:44:55 tol Exp $");
 
 int
 getfid(UInt8 *path, struct VenusFid *fid)
 {
     struct ViceIoctl a_params;
 
-    if (!k_hasafs())
+    if (!k_hasafs_recheck())
         return errno;
 
     if (path == NULL)
@@ -51,7 +51,7 @@ getfid(UInt8 *path, struct VenusFid *fid)
     a_params.in=NULL;
     a_params.out=(void*) fid;
     
-    if(k_pioctl(path,VIOCGETFID,&a_params,1) == -1)
+    if(k_pioctl(path,ARLA_VIOCGETFID,&a_params,1) == -1)
         return errno;
 
     return 0;
@@ -83,7 +83,7 @@ getacl(UInt8 *path)
     char *s;
     CFStringRef string;
 
-    if (!k_hasafs())
+    if (!k_hasafs_recheck())
         return NULL;
 
     s = malloc(MAXSIZE);
@@ -93,7 +93,7 @@ getacl(UInt8 *path)
     a_params.in = NULL;
     a_params.out = s;
     
-    if(k_pioctl(path,VIOCGETAL,&a_params,1) == -1)
+    if(k_pioctl(path,ARLA_VIOCGETAL,&a_params,1) == -1)
         return NULL;
         
     s[a_params.out_size] = '\0';
@@ -107,7 +107,7 @@ setacl(UInt8 *path, UInt8 *acl)
 {
     struct ViceIoctl a_params;
 
-    if (!k_hasafs())
+    if (!k_hasafs_recheck())
         return NULL;
 
     a_params.in_size = strlen(acl);
@@ -115,7 +115,7 @@ setacl(UInt8 *path, UInt8 *acl)
     a_params.in = acl;
     a_params.out = NULL;
     
-    if(k_pioctl(path,VIOCSETAL,&a_params,1) == -1)
+    if(k_pioctl(path,ARLA_VIOCSETAL,&a_params,1) == -1)
         return paramErr;
         
     return 0;
@@ -152,7 +152,7 @@ getvolstat(UInt8 *path, CFStringRef *string, int *quota, int *used)
 	return memFullErr;
     }
 
-    if(k_pioctl(path,VIOCGETVOLSTAT,&a_params,1)==-1) {
+    if(k_pioctl(path,ARLA_VIOCGETVOLSTAT,&a_params,1)==-1) {
 	free(a_params.out);
 	return paramErr;
     }
@@ -166,3 +166,76 @@ getvolstat(UInt8 *path, CFStringRef *string, int *quota, int *used)
     free(a_params.out);
     return 0;
 }
+
+int
+getfilecellname(UInt8 *path, CFStringRef *string)
+{
+    struct ViceIoctl a_params;
+    char *name;
+
+    a_params.in_size=0;
+    a_params.out_size=MAXSIZE;
+    a_params.in=NULL;
+    a_params.out=malloc(MAXSIZE);
+
+    if (a_params.out == NULL) {
+	return memFullErr;
+    }
+
+    if(k_pioctl(path,ARLA_VIOC_FILE_CELL_NAME,&a_params,1)==-1) {
+	free(a_params.out);
+	return paramErr;
+    }
+
+    name = a_params.out;
+
+    *string = CFStringCreateWithCString(NULL, name, kCFStringEncodingUTF8);
+    free(a_params.out);
+    return 0;
+}
+
+int
+getfileservers(UInt8 *path, CFStringRef *string)
+{
+    struct ViceIoctl a_params;
+    struct in_addr addr;
+    int32_t *curptr;
+    char *name;
+    int i = 0;
+
+    CFMutableStringRef hosts;
+    
+    a_params.in_size = 0;
+    a_params.out_size = 8 * sizeof(int32_t);
+    a_params.in = NULL;
+    a_params.out = malloc(8 * sizeof(int32_t));
+    
+    if (a_params.out == NULL)
+	return memFullErr;
+
+    if (k_pioctl(path, VIOCWHEREIS, &a_params, 1) == -1) {
+	free(a_params.out);
+	return paramErr;
+    }
+    
+    hosts = CFStringCreateMutable(kCFAllocatorDefault, 512);
+    curptr=(int32_t *) a_params.out;
+
+    while (curptr[i] && i < 8) {
+        struct hostent *h;
+	addr.s_addr = curptr[i];
+	h = gethostbyaddr((const char *) &addr, sizeof(addr), AF_INET);
+	if (h == NULL)
+	    name = inet_ntoa(addr);
+	else
+	    name = h->h_name;
+
+	CFStringAppendCString(hosts, name, kCFStringEncodingUTF8);
+	i++;
+    }
+    
+    *string = hosts;
+    free(a_params.out);
+
+    return 0;
+}
