@@ -44,7 +44,8 @@
 #include <strings.h>
 
 #include <krb5.h>
-#include <krbafs.h>
+#include <kafs.h>
+#include <arla-pioctl.h>
 
 #include <fs.h>
 #include <arlalib.h>
@@ -54,7 +55,7 @@
 @implementation mafslog
 
 #ifdef RCSID
-RCSID("$Id: mafslog.m,v 1.3 2002/12/03 15:56:35 lha Exp $");
+RCSID("$Id: mafslog.m,v 1.4 2005/10/28 14:33:36 tol Exp $");
 #endif
 
 static krb5_context mafslog_context = NULL;
@@ -114,7 +115,7 @@ do_afslog(id tt, NSProgressIndicator *p, id sender)
 {
     int ret;
 
-    if (!k_hasafs()) {
+    if (!k_hasafs_recheck()) {
 	if (sender == nil)
 	    return;
         NSBeginAlertSheet(@"Error", @"Ok", nil, nil, [tt window],
@@ -146,7 +147,7 @@ do_afslog(id tt, NSProgressIndicator *p, id sender)
 
     krb5_cc_default(mafslog_context, &mafslog_id);
    
-    if (!k_hasafs())
+    if (!k_hasafs_recheck())
         return;
 
     do_afslog(tokenTable, progress, nil);
@@ -163,7 +164,7 @@ do_afslog(id tt, NSProgressIndicator *p, id sender)
     parms.out = NULL;
     parms.out_size = 0;
 	
-    if(k_pioctl(NULL, VIOCUNLOG, &parms, 0) != 0)
+    if (k_pioctl(NULL, ARLA_VIOCUNLOG, &parms, 0) != 0)
         printf("error unlogging\n");
 
     update_token_array(tokenTable, tokenTableCell, tokenTableStatus, tokenTableExpire);
