@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999 - 2003 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1999 - 2004 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -35,7 +35,7 @@
 #include <config.h>
 #endif
 
-RCSID("$Id: afsfsperf.c,v 1.13 2004/06/07 14:49:04 tol Exp $");
+RCSID("$Id: afsfsperf.c,v 1.18 2004/12/31 01:13:26 haba Exp $");
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -73,6 +73,7 @@ RCSID("$Id: afsfsperf.c,v 1.13 2004/06/07 14:49:04 tol Exp $");
 
 static char *arg_cell = NULL;
 static int   arg_smallrun = 0;
+static int   arg_64bitrun = 0;
 static int   arg_version = 0;
 static char *arg_test_to_run = "all";
 
@@ -298,8 +299,8 @@ do_createfile (struct rx_connection *conn, const AFSFid *parent,
 
 static void
 do_createfilecontent (struct rx_connection *conn, const AFSFid *parent,
-		      const char *name, AFSFid *child, int size,
-		      int times, char *buffer)
+		      const char *name, AFSFid *child, size_t size,
+		      unsigned long times, char *buffer, int use64call)
 {
     static AFSStoreStatus InStatus = { 0,0,0,0,0 };
     AFSFetchStatus OutFidStatus, OutDirStatus;
@@ -308,6 +309,7 @@ do_createfilecontent (struct rx_connection *conn, const AFSFid *parent,
     int ret;
     struct rx_call *call;
     int i;
+    uint64_t filesize = (uint64_t)size * (uint64_t)times;
     
     ret = RXAFS_CreateFile (conn, parent, name, &InStatus,
 			    child, &OutFidStatus, &OutDirStatus,
@@ -320,11 +322,25 @@ do_createfilecontent (struct rx_connection *conn, const AFSFid *parent,
     if (call == NULL)
 	errx (1, "rx_NewCall returned NULL");
 
-    ret = StartRXAFS_StoreData (call, child, &InStatus,
-				0, size*times, size*times);
+    if (use64call)
+	ret = StartRXAFS_StoreData64 (call, child, &InStatus,
+				      0, filesize, filesize);
+    else 
+	ret = RXGEN_OPCODE;
+    if (ret == RXGEN_OPCODE) {
+	rx_EndCall(call, 0);
+	call = rx_NewCall(conn);
+	if (filesize >> 32)
+	    errx(1, "fileserver doesn't support 64 bit ops, "
+		 "but file is larger then 32 bits");
+	use64call = 0;
+	ret = StartRXAFS_StoreData (call, child, &InStatus,
+				    0, filesize, filesize);
+    }
 
     if (ret)
-	errx (1, "StartRXAFS_StoreData returned %d", ret);
+	errx (1, "StartRXAFS_StoreData%s returned %d", 
+	      use64call ? "64" : "", ret);
 
     for (i = 0; i < times; i++) {
 	ret = rx_Write(call, buffer, size);
@@ -333,20 +349,37 @@ do_createfilecontent (struct rx_connection *conn, const AFSFid *parent,
 		  ret, rx_GetCallError(call));
     }
 
-    ret = EndRXAFS_StoreData (call, &OutFidStatus, &a_volSyncP);
+    if (use64call)
+	ret = EndRXAFS_StoreData64 (call, &OutFidStatus, &a_volSyncP);
+    else
+	ret = EndRXAFS_StoreData (call, &OutFidStatus, &a_volSyncP);
 
     if (ret)
-	errx (1, "StartRXAFS_StoreData returned %d", ret);
+	errx (1, "StartRXAFS_StoreData%s returned %d",  
+	      use64call ? "64" : "", ret);
 
     ret = rx_EndCall(call, ret);
     if (ret)
 	errx (1, "rx_EndCall returned %d", ret);
 }
 
+static const void *
+memcchr(const void *mem, int c, size_t sz)
+{
+    const char *str = mem;
+    size_t l = sz;
+    while (l > 0 && c == *str)
+	str++, l--;
+    if (l == 0)
+	return NULL;
+    return &str[sz - l];
+}
+
 static void
 do_readfilecontent (struct rx_connection *conn, const AFSFid *fid,
-		    int size,
-		    int times, char *buffer)
+		    size_t size,
+		    unsigned long times, char *buffer,
+		    int use64call, int verify_content)
 {
     AFSFetchStatus OutFidStatus;
     AFSCallBack CallBack;
@@ -354,26 +387,65 @@ do_readfilecontent (struct rx_connection *conn, const AFSFid *fid,
     int ret;
     struct rx_call *call;
     int i;
+    uint64_t rsize, filesize = (uint64_t)size * (uint64_t)times;
+    uint32_t rsize4;
     
     call = rx_NewCall (conn);
 
     if (call == NULL)
 	errx (1, "rx_NewCall returned NULL");
 
-    ret = StartRXAFS_FetchData (call, fid, 0, size*times);
+    if (use64call)
+	ret = StartRXAFS_FetchData64 (call, fid, 0, filesize);
+    else 
+	ret = RXGEN_OPCODE;
+    if (ret == RXGEN_OPCODE) {
+	if (use64call)
+	    printf("not 64 bit fileserver\n");
+	if (filesize >> 32)
+	    errx(1, "fileserver doesn't support 64 bit ops, "
+		 "but file is larger then 32 bits");
+	use64call = 0;
+	ret = StartRXAFS_FetchData (call, fid, 0, filesize);
+    }
 
     if (ret)
 	errx (1, "StartRXAFS_FetchData returned %d", ret);
 
+    rsize = 0;
+    if (use64call) {
+	    ret = rx_Read (call, &rsize4, sizeof(rsize4));
+	    if (ret != sizeof(rsize4)) {
+		rx_EndCall(call, 0);
+		errx(1, "failed to read size 8");
+	    }
+	    rsize = (uint64_t)htonl(rsize) << 32;
+    } 
+    ret = rx_Read (call, &rsize4, sizeof(rsize4));
+    if (ret != sizeof(rsize4)) {
+	rx_EndCall(call, 0);
+	errx(1, "failed to read size 4");
+    }
+    rsize |= ntohl (rsize4);
+
+    if (rsize != filesize)
+	errx(1, "fileserver returned diffrent size then we asked for");
+
     for (i = 0; i < times; i++) {
 	ret = rx_Read(call, buffer, size);
 	if (ret != size)
 	    errx (1, "rx_Read returned %d rx error: %d", 
 		  ret, rx_GetCallError(call));
+	if (verify_content && memcchr(buffer, 'X', size) != NULL)
+	    errx(1, "not same content on fileserver");
     }
 
-    ret = EndRXAFS_FetchData (call, &OutFidStatus, &CallBack,
-			      &a_volSyncP);
+    if (use64call)
+	ret = EndRXAFS_FetchData64 (call, &OutFidStatus, &CallBack,
+				    &a_volSyncP);
+    else
+	ret = EndRXAFS_FetchData (call, &OutFidStatus, &CallBack,
+				  &a_volSyncP);
     if (ret)
 	errx (1, "StartRXAFS_FetchData returned %d", ret);
 
@@ -558,7 +630,8 @@ create_many_entry (struct rx_connection *conn, AFSFid *parentfid, int num,
 }
 
 static void
-create_and_read_content (struct rx_connection *conn, AFSFid *parentfid, int size)
+create_and_read_content (struct rx_connection *conn, AFSFid *parentfid, 
+			 unsigned long size, int use64call)
 {
     char *dir, *stamp;
     AFSFid fid;
@@ -566,9 +639,9 @@ create_and_read_content (struct rx_connection *conn, AFSFid *parentfid, int size
     char *buffer;
     struct timeval timer_start;
     
-    asprintf (&stamp, "writing   %dM data", size);
+    asprintf (&stamp, "writing   %luM data", (unsigned long)size);
 
-    asprintf (&dir, "write_big_files-%d-time-%d", size, (int)time(NULL));
+    asprintf (&dir, "write_big_files-%lu-time-%d", size, (int)time(NULL));
     do_mkdir (conn, parentfid, dir, &fid);
 
     buffer = malloc(1024*1024);
@@ -576,21 +649,21 @@ create_and_read_content (struct rx_connection *conn, AFSFid *parentfid, int size
     
     start_timer(&timer_start);
     do_createfilecontent (conn, &fid, "foo", &file, 1024*1024,
-			  size, buffer);
+			  size, buffer, use64call);
     end_and_print_timer (stamp, size, 0, &timer_start);
 
     free (stamp);
 
-    asprintf (&stamp, "reading   %dM data", size);
+    asprintf (&stamp, "reading   %luM data", size);
     start_timer(&timer_start);
-    do_readfilecontent (conn, &file, 1024*1024, size, buffer);
+    do_readfilecontent (conn, &file, 1024*1024, size, buffer, use64call, 1);
     end_and_print_timer (stamp, size, 0, &timer_start);
 
     free(stamp);
 
     free(buffer);
 
-    asprintf (&stamp, "removing  %dM data", size);
+    asprintf (&stamp, "removing  %luM data", size);
 
     start_timer(&timer_start);
     do_removefile (conn, &fid, "foo");
@@ -634,15 +707,18 @@ do_gettime(struct rx_connection *conn, AFSFid *root)
 static void
 do_read_content(struct rx_connection *conn, AFSFid *root)
 {
-    create_and_read_content (conn, root, 1);
-    create_and_read_content (conn, root, 2);
-    create_and_read_content (conn, root, 10);
-    create_and_read_content (conn, root, 20);
+    create_and_read_content (conn, root, 1, arg_64bitrun);
+    create_and_read_content (conn, root, 2, arg_64bitrun);
+    create_and_read_content (conn, root, 10, arg_64bitrun);
 
     if (!arg_smallrun) {
-	create_and_read_content (conn, root, 30);
-	create_and_read_content (conn, root, 40);
-	create_and_read_content (conn, root, 50);
+	create_and_read_content (conn, root, 50, arg_64bitrun);
+	create_and_read_content (conn, root, 100, arg_64bitrun);
+	create_and_read_content (conn, root, 500, arg_64bitrun);
+    }
+    if (arg_64bitrun && !arg_smallrun) {
+	create_and_read_content (conn, root, 3000, arg_64bitrun);
+	create_and_read_content (conn, root, 6000, arg_64bitrun);
     }
 }
 
@@ -710,7 +786,7 @@ do_readbench(struct rx_connection *conn, int volume, int vnode, int uniq)
     file.Volume = volume;
     file.Unique = uniq;
     start_timer(&timer_start);
-    do_readfilecontent (conn, &file, 1024*1024, 10, buffer);
+    do_readfilecontent (conn, &file, 1024*1024, 10, buffer, arg_64bitrun, 0);
     end_and_print_timer (stamp, 10, 1, &timer_start);
 
     free(stamp);
@@ -755,8 +831,8 @@ struct {
     int bit;
     void (*func)(struct rx_connection *, AFSFid *);
 } benchmarks[] = {
-    { "gettime",	BENCH_GETTIME,		do_gettime },
-    { "readcontent",	BENCH_READ_CONTENT,	do_read_content },
+    { "get-time",	BENCH_GETTIME,		do_gettime },
+    { "read-content",	BENCH_READ_CONTENT,	do_read_content },
     { "create-files",	BENCH_CREATE_FILES,	do_create_files },
     { "create-dirs",	BENCH_CREATE_DIRS,	do_create_dirs },
     { "bulk-status",	BENCH_BULK_STATUS,	do_bulk_status }
@@ -792,6 +868,9 @@ struct agetargs args[] = {
      "test1,test2", NULL, aarg_optional},
     {"smallrun", 0, aarg_flag, &arg_smallrun,
      "a run a smaller version of the benchmark", NULL, aarg_optional},
+    {"64bitrun", 0, aarg_flag, &arg_64bitrun,
+     "use 64bit rpc and to large tests it its not a small run", 
+     NULL, aarg_optional},
     {"version", 0, aarg_flag, &arg_version,
      "version", NULL, aarg_optional},
    { NULL, 0}
