@@ -36,7 +36,7 @@
 #include "vos_local.h"
 #include <vers.h>
 
-RCSID("$Id: vos.c,v 1.70 2004/06/10 13:07:37 ian Exp $");
+RCSID("$Id: vos.c,v 1.73 2005/11/25 13:06:39 tol Exp $");
 
 int vos_interactive = 0;
 
@@ -74,8 +74,8 @@ static SL_cmd cmds[] = {
     {"partinfo",   vos_partinfo,    "print partition information on a server"},
     {"release",    empty_cmd,       "not yet implemented"},
     {"remove",     empty_cmd,       "not yet implemented"},
-    {"remsite",    empty_cmd,       "not yet implemented"},
-    {"rename",     empty_cmd,       "not yet implemented"},
+    {"remsite",    vos_remsite,     "Remove a replication site from a volume"},
+    {"rename",     vos_rename,      "Rename a volume"},
     {"restore",    empty_cmd,       "not yet implemented"},
     {"status",     vos_status,      "Show volume server transactions"},
     {"syncserv",   empty_cmd,       "not yet implemented"},
@@ -109,7 +109,7 @@ static int
 quit_cmd(int argc, char **argv)
 {
     printf("exiting\n");
-    return 1;
+    return -2;
 }
 
 /*
