@@ -35,7 +35,7 @@
 #include <sl.h>
 #include "vos_local.h"
 
-RCSID("$Id: vos_unlock.c,v 1.5 2001/11/16 16:52:47 mattiasa Exp $");
+RCSID("$Id: vos_unlock.c,v 1.6 2004/10/22 23:33:45 ian Exp $");
 
 /*
  * unlock a volume
@@ -131,7 +131,7 @@ vos_unlock_volume(char *volname)
 static void
 usage(void)
 {
-    aarg_printusage(args, "vos dump", "", AARG_AFSSTYLE);
+    aarg_printusage(args, "vos unlock", "", AARG_AFSSTYLE);
 }
 
 int
