@@ -37,7 +37,7 @@
 
 #include "arla_local.h"
 
-RCSID("$Id: adir.c,v 1.73 2002/07/24 05:56:27 lha Exp $") ;
+RCSID("$Id: adir.c,v 1.75 2005/08/10 09:01:01 lha Exp $") ;
 
 /*
  *
@@ -187,7 +187,7 @@ adir_mkdir (FCacheEntry *dir,
     if (ret)
 	return ret;
 
-    ret = fdir_mkdir (&the_fbuf, dot, dot_dot);
+    ret = fdir_mkdir (&the_fbuf, dot, dot_dot, 0);
     fcache_update_length (dir, fbuf_len(&the_fbuf), fbuf_len(&the_fbuf));
     fbuf_end (&the_fbuf);
     close (fd);
@@ -212,7 +212,7 @@ adir_creat (FCacheEntry *dir,
     if (ret)
 	return ret;
 
-    ret = fdir_creat (&the_fbuf, name, fid);
+    ret = fdir_creat (&the_fbuf, name, NULL, fid);
     fcache_update_length (dir, fbuf_len(&the_fbuf), fbuf_len(&the_fbuf));
     fbuf_end (&the_fbuf);
     close (fd);
