@@ -38,9 +38,9 @@
 #include "arla_local.h"
 #include <getarg.h>
 
-RCSID("$Id: afsdir_check.c,v 1.25 2002/07/24 05:57:41 lha Exp $");
+RCSID("$Id: afsdir_check.c,v 1.29 2005/10/28 14:33:36 tol Exp $");
 
-static int help_flag;
+static int local_help_flag;
 static int verbose = 0;
 
 /*
@@ -75,7 +75,9 @@ getentry (DirPage0 *page0,
 
      page = (DirPage1 *)((char *)page0 +
 			 AFSDIR_PAGESIZE * (num / ENTRIESPERPAGE));
-     if (page->header.pg_tag != htons(AFSDIRMAGIC)) {
+     if (page->header.pg_tag != htons(AFSDIRMAGIC) &&
+	 page->header.pg_tag != htons(AFSDIRMAGIC_UTF8))
+     {
 	 printf ("Bad magic (%d) != %d in directory\n",
 		 ntohs(page->header.pg_tag), AFSDIRMAGIC);
 	 exit (1);
@@ -94,6 +96,7 @@ check_dir (const char *filename)
     unsigned ind;
     unsigned len;
     int ret = 0;
+    int utf8_dir = 0;
     unsigned npages = 0;
     unsigned page_entry_count;
     unsigned hash_entry_count;
@@ -144,6 +147,9 @@ check_dir (const char *filename)
 
     page_entry_count = 0;
 
+    if (page0->header.pg_tag == htons(AFSDIRMAGIC_UTF8))
+	utf8_dir = 1;
+
     for (i = 0; i < npages; ++i) {
 	PageHeader *ph = (PageHeader *)((char *)page0 + i * AFSDIR_PAGESIZE);
 	int start;
@@ -155,7 +161,9 @@ check_dir (const char *filename)
 
 	memset (my_bitmaps[i], 0, sz);
 
-	if (ph->pg_tag != htons(AFSDIRMAGIC)) {
+	if (ph->pg_tag != htons(AFSDIRMAGIC) &&
+	    ph->pg_tag != htons(AFSDIRMAGIC_UTF8))
+	{
 	    printf ("page %d: wrong tag: %u\n", i, htons(ph->pg_tag));
 	    ret = 1;
 	    goto out;
@@ -163,10 +171,18 @@ check_dir (const char *filename)
 	printf ("page %d: count = %u, tag = %u, freecount = %u\n",
 		i, ntohs(ph->pg_pgcount), htons(ph->pg_tag), ph->pg_freecount);
 	if (i == 0) {
-	    if (ph->pg_freecount != 51) {
-		printf ("freecount should be 51!\n");
-		ret = 1;
-		goto out;
+	    if (utf8_dir) {
+		if (ph->pg_freecount != 19) {
+		    printf ("freecount should be 19!\n");
+		    ret = 1;
+		    goto out;
+		}
+	    } else {
+		if (ph->pg_freecount != 51) {
+		    printf ("freecount should be 51!\n");
+		    ret = 1;
+		    goto out;
+		}
 	    }
 	    if (ntohs(ph->pg_pgcount) != npages) {
 		printf ("pgcount should be %u!\n", npages);
@@ -212,12 +228,23 @@ check_dir (const char *filename)
 	    int len;
 	    unsigned off;
 	    unsigned pageno;
+	    const char *name;
 
 	    entry = getentry (page0, ind - 1);
 
+	    name = entry->name;
+	    
+	    if (entry->flag & AFSDIR_UTFENT) {
+		const DirEntry2 *direntry2 = (DirEntry2 *)
+		    (((const char *)entry) + (entry->length << 2));
+		printf("len %d\n", entry->length);
+		printf("raw_offset %x\n", direntry2->raw_offset);
+		name = ((const char *)entry) + direntry2->raw_offset;
+	    }
+	    
 	    if (verbose)
 		printf ("%s - %ld.%ld\n",
-			entry->name, 
+			name, 
 			(long)ntohl(entry->fid.Vnode),
 			(long)ntohl(entry->fid.Unique));
 	    
@@ -255,8 +282,11 @@ check_dir (const char *filename)
 	int j;
 	unsigned unused;
 
-	if (i == 0)
+	if (i == 0) {
 	    unused = 13;
+	    if (utf8_dir)
+		unused += 32;
+	}
 	else
 	    unused = 1;
 
@@ -300,7 +330,7 @@ out:
 static struct getargs args[] = {
     {"verbose",	'v',	arg_flag,	&verbose,
      "run in test mode", NULL},
-    {"help",	0,	arg_flag,	&help_flag,
+    {"help",	0,	arg_flag,	&local_help_flag,
      NULL, NULL},
 };
 
@@ -322,7 +352,7 @@ main(int argc, char **argv)
     argc -= optind;
     argv += optind;
 
-    if (help_flag)
+    if (local_help_flag)
 	usage (0);
 
     if (argc != 1)
