@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2002 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2005 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -36,7 +36,7 @@
 #include <getarg.h>
 #include <vers.h>
 
-RCSID("$Id: arla-cli.c,v 1.37 2003/11/19 16:20:13 mattiasa Exp $");
+RCSID("$Id: arla-cli.c,v 1.45 2005/11/25 13:05:54 tol Exp $");
 
 char *default_log_file = "/dev/stderr";
 char *default_arla_cachedir = ".arlacache";
@@ -60,6 +60,7 @@ static int help(int, char **);
 static int arla_quit(int, char **);
 static int arla_checkserver(int, char **);
 static int arla_conn_status(int, char **);
+static int arla_connect(int, char **);
 static int arla_vol_status(int, char **);
 static int arla_cred_status(int, char **);
 static int arla_fcache_status(int, char **);
@@ -107,6 +108,8 @@ static SL_cmd cmds[] = {
     {"quit", arla_quit, "quit"},
     {"exit"},
     {"sysname", arla_sysname, "sysname"},
+    {"connect", arla_connect,
+     "connect [connected|fetch-only|disconnected|callback-connected]"},
     { NULL }
 };
 
@@ -164,7 +167,7 @@ static int
 arla_quit (int argc, char **argv)
 {
     printf("Thank you for using arla\n");
-    return 1;
+    return -2;
 }
 
 static int
@@ -286,7 +289,7 @@ print_dir_long (VenusFid *fid, const char *name, void *v)
     ti = entry->status.ClientModTime;
     t = localtime (&ti);
     strftime (timestr, sizeof(timestr), "%Y-%m-%d", t);
-    printf ("%c%c%c%c%c%c%c%c%c%c %2d %6d %6d %8d %s ",
+    printf ("%c%c%c%c%c%c%c%c%c%c %2d %6d %6d %8lu %s ",
 	    type,
 	    entry->status.UnixModeBits & 0x100 ? 'r' : '-',
 	    entry->status.UnixModeBits & 0x080 ? 'w' : '-',
@@ -300,7 +303,7 @@ print_dir_long (VenusFid *fid, const char *name, void *v)
 	    entry->status.LinkCount,
 	    entry->status.Owner,
 	    entry->status.Group,
-	    entry->status.Length,
+	    (unsigned long)fcache_get_status_length(&entry->status),
 	    timestr);
 
     printf ("v %d ", entry->status.DataVersion);
@@ -371,6 +374,7 @@ arla_mkdir (int argc, char **argv)
     VenusFid fid;
     int ret;
     FCacheEntry *e;
+    char *argcopy;
     char *dirname;
     char *basename;
     AFSStoreStatus store_attr;
@@ -381,24 +385,26 @@ arla_mkdir (int argc, char **argv)
 	printf ("usage: %s file\n", argv[0]);
 	return 0;
     }
-    dirname = strdup(argv[1]);
-    if (dirname == NULL)
-	err(1, "strdup");
-    basename = strrchr(dirname, '/');
-    if (basename == NULL) {
-	printf ("%s: filename contains no /\n", argv[0]);
-	free(dirname);
-	return 0;
-    }
-    basename[0] = '\0';
-    basename++;
 
-    if(cm_walk (cwd, dirname, &fid) == 0) {
+    argcopy = strdup(argv[1]);
+    if (argcopy == NULL)
+	err(1, "strdup");
+    basename = strrchr(argcopy, '/');
+    if (basename == NULL) {
+	basename = argcopy;
+	dirname = ".";
+    } else {
+	basename[0] = '\0';
+	basename++;
+	dirname = argcopy;
+    }
+
+    if (cm_walk (cwd, dirname, &fid) == 0) {
 
 	ret = fcache_get(&e, fid, ce);
 	if (ret) {
 	    printf ("fcache_get failed: %d\n", ret);
-	    free(dirname);
+	    free(argcopy);
 	    return 0;
 	}
 
@@ -409,18 +415,14 @@ arla_mkdir (int argc, char **argv)
 	store_attr.UnixModeBits = 0;
 	store_attr.SegSize = 0;
 	ret = cm_mkdir(&e, basename, &store_attr, &res, &fetch_attr, &ce);
-	if (ret) {
+	if (ret)
 	    arla_warn (ADEBWARN, ret,
 		       "%s: cannot create directory `%s'",
 		       argv[0], argv[1]);
-	    fcache_release(e);
-	    free(dirname);
-	    return 1;
-	}
 
 	fcache_release(e);
     }
-    free(dirname);
+    free(argcopy);
     return 0;
 }
 
@@ -430,6 +432,7 @@ arla_rmdir (int argc, char **argv)
     VenusFid fid;
     int ret;
     FCacheEntry *e;
+    char *argcopy;
     char *dirname;
     char *basename;
     
@@ -437,19 +440,21 @@ arla_rmdir (int argc, char **argv)
 	printf ("usage: %s file\n", argv[0]);
 	return 0;
     }
-    dirname = strdup(argv[1]);
-    if (dirname == NULL)
-	err(1, "strdup");
-    basename = strrchr(dirname, '/');
-    if (basename == NULL) {
-	printf ("%s: filename contains no /\n", argv[0]);
-	free(dirname);
-	return 0;
-    }
-    basename[0] = '\0';
-    basename++;
 
-    if(cm_walk (cwd, dirname, &fid) == 0) {
+    argcopy = strdup(argv[1]);
+    if (argcopy == NULL)
+	err(1, "strdup");
+    basename = strrchr(argcopy, '/');
+    if (basename == NULL) {
+	basename = argcopy;
+	dirname = ".";
+    } else {
+	basename[0] = '\0';
+	basename++;
+	dirname = argcopy;
+    }
+
+    if (cm_walk (cwd, dirname, &fid) == 0) {
 
 	ret = fcache_get(&e, fid, ce);
 	if (ret) {
@@ -459,18 +464,13 @@ arla_rmdir (int argc, char **argv)
 	}
 
 	ret = cm_rmdir(&e, basename, &ce);
-	if (ret) {
+	if (ret)
 	    arla_warn (ADEBWARN, ret,
 		       "%s: cannot remove directory `%s'",
 		       argv[0], argv[1]);
-	    fcache_release(e);
-	    free(dirname);
-	    return 1;
-	}
-
 	fcache_release(e);
     }
-    free(dirname);
+    free(argcopy);
     return 0;
 }
 
@@ -505,15 +505,10 @@ arla_rm (int argc, char **argv)
 	}
 
 	ret = cm_remove(&e, basename, &ce);
-	if (ret) {
+	if (ret)
 	    arla_warn (ADEBWARN, ret,
 		       "%s: cannot remove file `%s'",
 		       argv[0], argv[1]);
-	    fcache_release(e);
-	    free(dirname);
-	    free(basename);
-	    return 1;
-	}
 
 	fcache_release(e);
     }
@@ -873,6 +868,52 @@ arla_cell_status (int argc, char **argv)
     return 0;
 }
 
+static int
+arla_connect (int argc, char **argv)
+{
+    int error = (argc > 2) ? 1 : 0;
+    int32_t mode;
+    if (argc == 2) {
+	struct nnpfs_cred cred;
+
+	if (strncmp("dis", argv[1], 3) == 0) 
+	    mode = arla_CONNMODE_DISCONN;
+	else if (strncmp("fetch", argv[1], 5) == 0)
+	    mode = arla_CONNMODE_FETCH;
+	else if (strncmp("conn", argv[1], 4) == 0)
+	    mode = arla_CONNMODE_CONN;
+	else if (strncmp(argv[1], "call", 4) == 0)
+	    mode = arla_CONNMODE_CONN_WITHCALLBACKS;
+	else
+	    mode = -1;
+
+	if (mode >= 0)
+	    error = set_connmode(mode, &cred);
+    }
+    if (error) {
+	printf("usage: %s [connected|fetch|disconnected|callback-connected]\n",
+	       argv[0]);
+	return 0;
+    }
+
+    get_connmode(&mode);
+    switch(mode) {
+    case arla_CONNMODE_CONN:
+	printf("Connected mode\n");
+	break;
+    case arla_CONNMODE_FETCH:
+	printf("Fetch only mode\n");
+	break;
+    case arla_CONNMODE_DISCONN:
+	printf("Disconnected mode\n");
+	break;
+    default:
+	printf("Unknown or error\n");
+	break;
+    }
+    return 0;
+}
+
 #ifdef RXDEBUG
 static int
 arla_rx_status(int argc, char **argv)
@@ -1046,7 +1087,7 @@ main (int argc, char **argv)
     int optind = 0;
     int ret;
 
-    set_progname (argv[0]);
+    setprogname (argv[0]);
     tzset();
     srand(time(NULL));
 
