@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2003 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2006 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -39,19 +39,22 @@
 #include <parse_units.h>
 #include <getarg.h>
 
-RCSID("$Id: arla.c,v 1.165 2004/03/26 12:02:43 lha Exp $") ;
+RCSID("$Id: arla.c,v 1.175 2006/04/21 13:31:28 tol Exp $") ;
 
 enum connected_mode connected_mode = CONNECTED;
 
 static void
-initrx (int port)
+initrx (int userport)
 {
+    int port = userport;
     int error;
 
     if (port == 0)
 	port = afscallbackport;
-
+	
     error = rx_Init (htons(port));
+    if (error == RX_ADDRINUSE && userport == 0)
+	error = rx_Init(0);
     if (error)
 	arla_err (1, ADEBERROR, error, "rx_init");
 }
@@ -161,14 +164,26 @@ read_conffile(const char *fname,
 	case CONF_PARAM_INT:
 	case CONF_PARAM_INT64:
 
-	    val = parse_units(v, size_units, NULL);
-	    if(val == -1 || val < 0) {
+	    /* parse_units is currently broken, so we roll our own for now */
+
 #ifdef HAVE_STRTOLL
-		val = strtoll(v, &endptr, 0);
+	    val = strtoll(v, &endptr, 0);
 #else
-		val = strtol(v, &endptr, 0);
+	    val = strtol(v, &endptr, 0);
 #endif
-		if (*endptr != '\0')
+
+	    if (*endptr != '\0') {
+		struct units *u = size_units;
+
+		while (u->name != NULL) {
+		    if (!strcmp(endptr, u->name)) {
+			val *= u->mult;
+			break;
+		    }
+		    u++;
+		}
+
+		if (u->name == NULL)
 		    fprintf (stderr, "%s:%d: bad value `%s'\n",
 			     fname, lineno, v);
 	    }
@@ -217,6 +232,7 @@ static unsigned numvols		= ARLA_NUMVOLS;
 static unsigned dynrootlevel	= DYNROOT_DEFAULT;
 static char *conf_sysname	= NULL;	/* sysname from conf file */
 const char *argv_sysname	= NULL; /* sysname from argv */
+int num_workers = 16;
 #ifdef KERBEROS
 const char *rxkad_level_string = "crypt";
 #endif
@@ -224,6 +240,7 @@ const char *rxkad_level_string = "crypt";
 static struct conf_param conf_params[] = {
     {"dynroot",			CONF_PARAM_BOOL,	&dynrootlevel},
     {"fake_stat",		CONF_PARAM_BOOL,	&fake_stat},
+    {"fake_mp",			CONF_PARAM_BOOL,	&fake_mp},
     {"fetch_block",		CONF_PARAM_INT,		&fetch_block_size},
     {"low_vnodes",		CONF_PARAM_INT,		&low_vnodes},
     {"high_vnodes",		CONF_PARAM_INT,		&high_vnodes},
@@ -233,6 +250,7 @@ static struct conf_param conf_params[] = {
     {"numconns",		CONF_PARAM_INT,		&numconns},
     {"numvols",			CONF_PARAM_INT,		&numvols},
     {"sysname",			CONF_PARAM_STR,		&conf_sysname},
+    {"workers",			CONF_PARAM_INT,		&num_workers},
 #ifdef KERBEROS
     {"rxkad-level",		CONF_PARAM_STR,		&rxkad_level_string},
 #endif
@@ -251,16 +269,16 @@ int recover = 0;
 int dynroot_enable = 0;
 int cm_consistency = 0;
 int fake_stat = 0;
-int default_stacksize = DEFAULT_STACKSIZE;
 
 /*
  * These are exported to other modules
  */
 
-int num_workers = 16;
 char *cache_dir;
 int fake_mp;
 int fork_flag = 1;
+int use_o_largefile = 1;
+char *trace_file = NULL;
 
 /*
  * Global AFS variables, se arla_local.h for comment
@@ -357,10 +375,23 @@ arla_init (void)
 	if (tmp < 0)
 	    errx (1, "bad connected mode: `%s'", connected_mode_string);
 	connected_mode = tmp;
+	if (connected_mode != CONNECTED)
+	    disco_openlog();
     }
 
     read_conffile(conf_file, conf_params);
 
+    if (low_vnodes > high_vnodes)
+	arla_errx (1, ADEBERROR, "low vnode is larger then high vnodes, "
+		   "cowardly refusing to start (%d > %d) "
+		   "(check BUGS section in arla.conf manual page)",
+		   low_vnodes, high_vnodes);
+    if (low_bytes > high_bytes)
+	arla_errx (1, ADEBERROR, "low bytes is larger then high bytes, "
+		   "cowardly refusing to start "
+		   "(check BUGS section in arla.conf manual page)");
+
+
 #ifdef KERBEROS
     conn_rxkad_level = parse_rxkad_level (rxkad_level_string);
     if (conn_rxkad_level < 0)
@@ -428,10 +459,10 @@ arla_init (void)
     cred_init (numcreds);
 
     arla_warnx (ADEBINIT,
-		"fcache low_vnodes = %u, high_vnodes = %u"
-		"low_bytes = %ld, high_bytes = %ld",
+		"fcache low_vnodes = %u, high_vnodes = %u "
+		"low_bytes = %lld, high_bytes = %lld",
 		low_vnodes, high_vnodes,
-		(long)low_bytes, (long)high_bytes);
+		(long long)low_bytes, (long long)high_bytes);
     fcache_init (low_vnodes, high_vnodes,
 		 low_bytes, high_bytes, recover);
 
