@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2003 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2005 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -33,7 +33,7 @@
 
 /*
  *  Include file for whole arlad
- *  $Id: arla_local.h,v 1.91 2004/03/26 12:06:10 lha Exp $
+ *  $Id: arla_local.h,v 1.100 2006/05/24 17:50:58 tol Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -52,8 +52,11 @@
 #if defined(HAVE_DIRENT_H)
 #include <dirent.h>
 #if DIRENT_AND_SYS_DIR_H
+#ifndef __FreeBSD__
+/* <sys/dir.h> is for DIRSIZ macro, freebsd uses _GENERIC_DIRSIZ */
 #include <sys/dir.h>
 #endif
+#endif
 #elif defined(HAVE_SYS_DIR_H)
 #include <sys/dir.h>
 #endif
@@ -80,6 +83,9 @@
 #ifdef HAVE_SYS_PRCTL_H
 #include <sys/prctl.h>
 #endif
+#ifdef HAVE_SYS_IOCTL_H
+#include <sys/ioctl.h>
+#endif
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
@@ -104,7 +110,10 @@
 
 #include <ifaddrs.h>
 
+#include <arla-pioctl.h>
+#ifdef KERBEROS
 #include <kafs.h>
+#endif
 
 #include <uae.h>
 
@@ -152,18 +161,8 @@ extern enum connected_mode connected_mode;
 
 #include "dynroot.h"
 
-#ifndef DEFAULT_STACKSIZE
-#define DEFAULT_STACKSIZE (32*1024)
-#endif
-extern int default_stacksize;
-
-#if 0
 #define assert_flag(e,f)	assert((e)->flags.f)
 #define assert_not_flag(e,f)	assert(!(e)->flags.f)
-#else
-#define assert_flag(e,f)	do { } while(0)
-#define assert_not_flag(e,f)	do { } while(0)
-#endif
 
 #define SYSNAMEMAXLEN 2048
 extern char **sysnamelist;
@@ -193,6 +192,7 @@ extern int fork_flag;		/* if the program should fork */
 extern int num_workers;		/* number of workers program should use */
 extern int client_port;		/* what port the client is using */
 extern int afs_BusyWaitPeriod;	/* number of sec to wait on fs when VBUSY */
+extern int use_o_largefile;     /* turns off largefile cache if set to 0 */
 
 void
 store_state (void);
@@ -207,6 +207,10 @@ get_default_cache_dir (void);
 #define O_BINARY 0
 #endif
 
+#ifndef O_LARGEFILE
+#define O_LARGEFILE 0
+#endif
+
 extern const char *conf_file;
 extern char *log_file;
 extern char *debug_levels;
@@ -223,6 +227,7 @@ extern int recover;
 extern int dynroot_enable;
 extern int cm_consistency;
 extern int fake_stat;
-extern int fetch_block_size;
+extern uint64_t fetch_block_size;
 
 extern char *cache_dir;
+extern char *trace_file;
