@@ -34,7 +34,7 @@
 #include <arla_local.h>
 #include <getarg.h>
 #include <vers.h>
-RCSID("$Id: arlad.c,v 1.23 2004/04/02 15:41:42 lha Exp $");
+RCSID("$Id: arlad.c,v 1.26 2005/08/11 14:47:18 map Exp $");
 
 char *default_log_file = "syslog";
 int client_port = 0;
@@ -194,6 +194,8 @@ static struct getargs args[] = {
      "the NNPFS device to use [/dev/nnpfs0]", "path"},
     {"dynroot", 'D', arg_flag,	&dynroot_enable,
      "if dynroot is enabled", NULL},
+    {"largefile",  0,	arg_negative_flag, &use_o_largefile,
+     "stop using O_LARGEFILE flag when opening cache files", NULL},
     {"log",	'l',	arg_string,	&log_file,
      "where to write log (stderr (default), syslog, or path to file)", NULL},
     {"fake-mp",	  0,	arg_flag,	&fake_mp,
@@ -213,6 +215,8 @@ static struct getargs args[] = {
 #endif
     {"sysname",	 's',	arg_string,	&argv_sysname,
      "set the sysname of this system", NULL},
+    {"tracefile", 0,	arg_string,	&trace_file,
+     "record nnpfs traffic in file", NULL},
     {"workers",	  0,	arg_integer,	&num_workers,
      "number of worker threads", NULL},
     {"version",	0,	arg_flag,	&version_flag,
@@ -234,7 +238,7 @@ main (int argc, char **argv)
     int optind = 0;
     int ret;
 
-    set_progname (argv[0]);
+    setprogname (argv[0]);
     tzset();
     srand(time(NULL));
 
