@@ -38,7 +38,7 @@
 
 #include "arla_local.h"
 #ifdef RCSID
-RCSID("$Id: conn.c,v 1.77 2004/03/26 12:07:03 lha Exp $") ;
+RCSID("$Id: conn.c,v 1.82 2005/10/28 14:33:36 tol Exp $") ;
 #endif
 
 #define CONNCACHESIZE 101
@@ -61,7 +61,7 @@ static unsigned nactive_connections;
 static List *connprobelist;
 
 #ifdef KERBEROS
-int conn_rxkad_level = rxkad_auth;
+int conn_rxkad_level = rxkad_crypt;
 #endif
 
 /*
@@ -264,8 +264,7 @@ conn_init (unsigned nentries)
 	 arla_errx (1, ADEBERROR, "conn_init: listnew failed");
      nconnections = 0;
 
-     if (LWP_CreateProcess (pinger, default_stacksize, 1,
-			    NULL, "pinger", &pinger_pid))
+     if (LWP_CreateProcess (pinger, 0, 1, NULL, "pinger", &pinger_pid))
 	 arla_errx (1, ADEBERROR,
 		    "conn: cannot create pinger thread");
 
@@ -417,7 +416,7 @@ add_connection(int32_t cell,
 							cred->ct.HandShakeKey,
 							cred->ct.AuthHandle,
 							cred->ticket_len,
-							cred->ticket);
+							(void *)cred->ticket);
 	    break;
 	}
 #endif
@@ -725,6 +724,22 @@ conn_status (void)
     hashtabforeach (connhtab, print_conn, NULL);
 }
 
+Bool
+conn_get_fs_support64(const ConnCacheEntry *e)
+{
+    return e->flags.no64support ? FALSE : TRUE;
+}
+
+void
+conn_set_fs_support64(ConnCacheEntry *e, Bool status)
+{
+    e->flags.no64support = status == TRUE ? 0 : 1;
+}
+
+/*
+ *
+ */
+
 struct clear_state {
     clear_state_mask mask;
     int32_t cell;
@@ -793,14 +808,14 @@ host_down (void *ptr, void *arg)
 
     if (s->cell == e->cell) {
 
-	if (!(s->flags & CKSERV_DONTPING)) {
+	if (!(s->flags & arla_CKSERV_DONTPING)) {
 	    conn_probe (e);
 	}
 
 	if (e->flags.alivep)
 	    return FALSE;
 	
-	if (s->flags & CKSERV_FSONLY && e->port != afsport)
+	if (s->flags & arla_CKSERV_FSONLY && e->port != afsport)
 	    return FALSE;
 
 	for (i = 0; i < s->i; ++i)
