@@ -35,7 +35,7 @@
  * Header for connection cache
  */
 
-/* $Id: conn.h,v 1.32 2004/02/04 14:42:21 lha Exp $ */
+/* $Id: conn.h,v 1.34 2004/12/22 12:27:00 lha Exp $ */
 
 #ifndef _CONN_H_
 #define _CONN_H_
@@ -57,6 +57,7 @@ struct conncacheentry {
 	unsigned alivep : 1;
 	unsigned killme : 1;
 	unsigned old : 1;	/* Old server,vldb -> only VL_GetEntryByName */
+	unsigned no64support : 1;
     } flags;
     unsigned refcount;
     Listitem *probe_le;
@@ -110,6 +111,12 @@ conn_serverupp (uint32_t host, uint16_t port, uint16_t service);
 void
 conn_status (void);
 
+Bool
+conn_get_fs_support64(const ConnCacheEntry *e);
+
+void
+conn_set_fs_support64(ConnCacheEntry *e, Bool status);
+
 void
 conn_clearcred (clear_state_mask mask,
 		int32_t cell, nnpfs_pag_t cred, int securityindex);
