@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2002 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2002, 2005-2006 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -36,7 +36,7 @@
  */
 
 #include "arla_local.h"
-RCSID("$Id: inter.c,v 1.139 2003/10/13 02:34:48 lha Exp $") ;
+RCSID("$Id: inter.c,v 1.150 2006/03/20 13:43:32 tol Exp $") ;
 
 #include <nnpfs/nnpfs_message.h>
 
@@ -63,26 +63,41 @@ getrights (FCacheEntry *e, CredCacheEntry *ce)
 
 /*
  * Check to see if the operation(s) mask are allowed to user cred on
- * file e
+ * file e.
+ *
+ * Return 0 on success, else a suitable error code.
  */
 
-static Bool
-checkright (FCacheEntry *e, u_long mask, CredCacheEntry *ce)
+int
+cm_checkright (FCacheEntry *e, u_long mask, CredCacheEntry *ce)
 {
+    uint32_t modify = AWRITE | AINSERT | ADELETE | ALOCK | AADMIN;
+    long voltype = getvoltype(e->fid.fid.Volume, e->volume);
     u_long rights;
 
+    /* We won't be able to modify readonly volumes */
+    if (voltype != RWVOL && (mask & modify) != 0)
+	return EROFS;
+
     if (e->status.FileType == TYPE_LINK &&
 	e->anonaccess & ALIST)
-	return TRUE;
+	return 0;
+
     if ((e->anonaccess & mask) == mask)
-	return TRUE;
+	return 0;
+
     rights = getrights (e, ce);
+    if (e->status.FileType != TYPE_DIR && (rights & AADMIN))
+	rights |= AREAD | AWRITE;
+
     if (e->status.FileType == TYPE_LINK &&
 	rights & ALIST)
-	return TRUE;
+	return 0;
+
     if ((rights & mask) == mask)
-	return TRUE;
-    return FALSE;
+	return 0;
+
+    return EACCES;
 }
 
 static int log_fd;
@@ -203,42 +218,45 @@ cm_open (FCacheEntry *entry, CredCacheEntry *ce, u_int tokens)
 
      switch(tokens) {
      case NNPFS_DATA_R:
-#if 0
+	  mask = AREAD;
+	  break;
      case NNPFS_OPEN_NR:
+#if 0
      case NNPFS_OPEN_SR:
 #endif
 	  mask = AREAD;
+	  tokens |= NNPFS_DATA_R;
 	  break;
      case NNPFS_DATA_W:
 	  mask = AWRITE;
 	  break;
      case NNPFS_OPEN_NW:
 	  mask = AREAD | AWRITE;
-	  tokens |= NNPFS_DATA_R | NNPFS_DATA_W;
+	  tokens |= NNPFS_DATA_R | NNPFS_DATA_W | NNPFS_OPEN_NR;
 	  break;
      default:
 	 arla_warnx (ADEBCM, "cm_open(): unknown token: %d, assuming AREAD",
 		     tokens);
 	 mask = AREAD;
 	 tokens |= NNPFS_DATA_R;
-#if 0
+#if 1
 	 assert(FALSE);
 #endif
      }
 
-     if (checkright (entry, mask, ce)) {
-	  assert(entry->flags.attrusedp);
-	  entry->flags.datausedp = TRUE;
-	  entry->tokens |= tokens;
-	  
-	  log_operation ("open (%ld,%lu,%lu,%lu) %u\n",
-			 entry->fid.Cell,
-			 entry->fid.fid.Volume,
-			 entry->fid.fid.Vnode,
-			 entry->fid.fid.Unique,
-			 mask);
-     } else
-	  error = EACCES;
+     error = cm_checkright(entry, mask, ce);
+     if (!error) {
+	 assert(entry->flags.attrusedp);
+	 entry->flags.datausedp = TRUE;
+	 entry->tokens |= tokens;
+	 
+	 log_operation ("open (%ld,%lu,%lu,%lu) %u\n",
+			entry->fid.Cell,
+			entry->fid.fid.Volume,
+			entry->fid.fid.Vnode,
+			entry->fid.fid.Unique,
+			mask);
+     }
 
      cm_check_consistency();
  
@@ -260,7 +278,7 @@ cm_close (FCacheEntry *entry, int flag,
 	if (flag & NNPFS_FSYNC)
 	    status->Mask |= SS_FSYNC;
 
-	error = write_data (entry, status, ce);
+	error = write_data (entry, NULL, status, ce);
 
 	if (error) {
 	    arla_warn (ADEBCM, error, "writing back file");
@@ -303,9 +321,12 @@ cm_getattr (FCacheEntry *entry,
 
      arla_warnx (ADEBCM, "cm_getattr: done get attr");
 
-     if (checkright (entry,
-		     entry->status.FileType == TYPE_FILE ? AREAD : 0,
-		     ce)) {
+     error = cm_checkright(entry,
+			   entry->status.FileType == TYPE_FILE ? AREAD : 0,
+			   ce);
+     if (error) {
+	 *ae = NULL;
+     } else {
 	 entry->flags.attrusedp = TRUE;
 	 entry->flags.kernelp   = TRUE;
 	 *ae = entry->acccache;
@@ -315,11 +336,8 @@ cm_getattr (FCacheEntry *entry,
 			entry->fid.fid.Volume,
 			entry->fid.fid.Vnode,
 			entry->fid.fid.Unique);
-	 
-     } else {
-	 *ae = NULL;
-	 error = EACCES;
      }
+
      if (!entry->flags.datausedp)
 	 entry->tokens &= ~(NNPFS_DATA_MASK | NNPFS_OPEN_MASK);
      
@@ -343,18 +361,18 @@ cm_setattr (FCacheEntry *entry, AFSStoreStatus *attr, CredCacheEntry* ce)
      if (error)
 	 return error;
 
-     if (checkright (entry, AWRITE, ce)) {
-	  arla_warnx (ADEBCM, "cm_setattr: Writing status");
-	  error = write_attr (entry, attr, ce);
-
-	  log_operation ("setattr (%ld,%lu,%lu,%lu)\n",
-			 entry->fid.Cell,
-			 entry->fid.fid.Volume,
-			 entry->fid.fid.Vnode,
-			 entry->fid.fid.Unique);
-     } else
-	 error = EACCES;
-
+     error = cm_checkright(entry, AWRITE, ce);
+     if (!error) {
+	 arla_warnx (ADEBCM, "cm_setattr: Writing status");
+	 error = write_attr (entry, attr, ce);
+	 
+	 log_operation ("setattr (%ld,%lu,%lu,%lu)\n",
+			entry->fid.Cell,
+			entry->fid.fid.Volume,
+			entry->fid.fid.Vnode,
+			entry->fid.fid.Unique);
+     }
+     
      cm_check_consistency();
      return error;
 }
@@ -379,46 +397,18 @@ cm_ftruncate (FCacheEntry *entry, off_t size,
 	     return error;
      }
 
-     if (checkright (entry, AWRITE, ce)) {
-	  error = truncate_file (entry, size, storestatus, ce);
-
-	  log_operation ("ftruncate (%ld,%lu,%lu,%lu) %lu\n",
-			 entry->fid.Cell,
-			 entry->fid.fid.Volume,
-			 entry->fid.fid.Vnode,
-			 entry->fid.fid.Unique,
-			 (unsigned long)size);
-     } else
-	 error = EACCES;
-
-     cm_check_consistency();
-     return error;
-}
-
-/*
- * access - check if user is allowed to perform operation.
- */
-
-int
-cm_access (FCacheEntry *entry, int mode, CredCacheEntry* ce)
-{
-     int error = 0;
-
-     error = fcache_verify_attr (entry, NULL, NULL, ce);
-     if (error)
-	 return error;
-
-     if (checkright (entry, AWRITE, ce))
-	 error = 0;
-     else
-	 error = EACCES;
-
-     log_operation ("access (%ld,%lu,%lu,%lu)\n",
-		    entry->fid.Cell,
-		    entry->fid.fid.Volume,
-		    entry->fid.fid.Vnode,
-		    entry->fid.fid.Unique);
-
+     error = cm_checkright(entry, AWRITE, ce);
+     if (!error) {
+	 error = truncate_file (entry, size, storestatus, ce);
+	 
+	 log_operation ("ftruncate (%ld,%lu,%lu,%lu) %lu\n",
+			entry->fid.Cell,
+			entry->fid.fid.Volume,
+			entry->fid.fid.Vnode,
+			entry->fid.fid.Unique,
+			(unsigned long)size);
+     }
+     
      cm_check_consistency();
      return error;
 }
@@ -507,28 +497,52 @@ cm_lookup (FCacheEntry **entry,
 	 return error;
 
      /* 
-      * The ".." at the top of a volume just points to the volume root,
-      * so get the real ".." from the volume cache instead.
       *
-      * Or if we are looking up "." we don't want to follow the
-      * mountpoint
+      * Or 
       */
 
      if (strcmp(".", name) == 0) {
 
+	 /*
+	  * If we are looking up "." we don't want to follow the
+	  * mountpoint, do fcache_verify_attr to force resolving of
+	  * fake mountpoints.
+	  */
+
 	 error = fcache_verify_attr (*entry, NULL, NULL, *ce);
 	 if (error)
 	     goto out;
 
 	 *res = (*entry)->fid;
-     } else if (strcmp("..", name) == 0
-		&& VenusFid_cmp(&(*entry)->fid, res) == 0) {
+     } else if (strcmp("..", name) == 0) {
 
-	 error = fcache_verify_attr (*entry, NULL, NULL, *ce);
-	 if (error)
-	     goto out;
+	 /* 
+	  * First make sure we don't following mountpoints for ".."
+	  * First, We are sure, its not a mountpoint. Second since
+	  * following mountpoints lock both parent and child, and
+	  * mountpoints breaks the tree that usully filesystem enforce
+	  * (non-directed graph) we can deadlock one thread looks up
+	  * from "root" -> "directory" and a second from "directory"
+	  * -> "..".
+	  */
+
+	 /*
+	  * The ".." at the top of a volume just points to the volume
+	  * root itself, so try to get the real ".." from the volume
+	  * cache instead.
+	  */
+
+	 if (VenusFid_cmp(&(*entry)->fid, res) == 0) {
+	     long voltype;
+
+	     error = fcache_verify_attr (*entry, NULL, NULL, *ce);
+	     if (error)
+		 goto out;
+	     
+	     voltype = getvoltype((*entry)->fid.fid.Volume, (*entry)->volume);
+	     *res = (*entry)->volume->parent[voltype].fid; /* entry->parent */
+	 }
 
-	 *res = (*entry)->volume->parent_fid; /* entry->parent */
      } else if (follow_mount_point) {
 	 error = followmountpoint (res, &(*entry)->fid, *entry, ce);
 	 if (error)
@@ -561,14 +575,14 @@ cm_create (FCacheEntry **dir, const char *name, AFSStoreStatus *store_attr,
      if (error)
 	 return error;
 
-     if (checkright (*dir, AINSERT, *ce)) {
+     error = cm_checkright(*dir, AINSERT, *ce);
+     if (!error) {
 	 error = create_file (*dir, name, store_attr,
 			      res, fetch_attr, *ce);
 	 if (error == 0)
 	     error = adir_creat (*dir, name, res->fid);
-     } else
-	 error = EACCES;
-
+     }
+     
      log_operation ("create (%ld,%lu,%lu,%lu) %s\n",
 		    (*dir)->fid.Cell,
 		    (*dir)->fid.fid.Volume,
@@ -596,21 +610,20 @@ cm_mkdir (FCacheEntry **dir, const char *name,
      if (error)
 	 return error;
 
-     if (checkright (*dir, AINSERT, *ce)) {
+     error = cm_checkright(*dir, AINSERT, *ce);
+     if (!error) {
 	 error = create_directory (*dir, name, store_attr,
 				   res, fetch_attr, *ce);
 	 if (error == 0)
 	     error = adir_creat (*dir, name, res->fid);
-	 
-     } else
-	 error = EACCES;
+     }
 
      log_operation ("mkdir (%ld,%lu,%lu,%lu) %s\n",
 		    (*dir)->fid.Cell,
 		    (*dir)->fid.fid.Volume,
 		    (*dir)->fid.fid.Vnode,
 		    (*dir)->fid.fid.Unique,
-	      name);
+		    name);
 
      cm_check_consistency();
      return error;
@@ -618,6 +631,10 @@ cm_mkdir (FCacheEntry **dir, const char *name,
 
 /*
  * Create a symlink
+ *
+ * If realfid is non-NULL, we mark the symlink with kernelp flag and
+ * return its fid in realfid.
+ *
  */
 
 int
@@ -635,10 +652,9 @@ cm_symlink (FCacheEntry **dir,
      if (error)
 	 return error;
 
-     if (!checkright (*dir, AINSERT, *ce)) {
-	 error = EACCES;
-	 goto out;
-     }
+     error = cm_checkright(*dir, AINSERT, *ce);
+     if (error)
+	 return error;
 
      /* It seems Transarc insists on mount points having mode bits 0644 */
 
@@ -667,8 +683,13 @@ cm_symlink (FCacheEntry **dir,
       * If the new symlink is a mountpoint and it points
       * to dir_fid we will deadlock if we look it up.
       */
-
-     if (VenusFid_cmp (res, &(*dir)->fid) != 0) {
+     
+     if (realfid == NULL) {
+	 /*
+	  * Caller doesn't care, don't bother.
+	  * ...and don't mark the symlink with kernelp
+	  */
+     } else if (VenusFid_cmp (res, &(*dir)->fid) != 0) {
 
 	 error = fcache_get (&symlink_entry, *res, *ce);
 	 if (error)
@@ -730,15 +751,15 @@ cm_link (FCacheEntry **dir,
      if (error)
 	 goto out;
 
-     if (checkright (*dir, AINSERT, *ce)) {
+     error = cm_checkright(*dir, AINSERT, *ce);
+     if (!error) {
 	 error = create_link (*dir, name, file, *ce);
 	 if (error == 0) {
 	     error = adir_creat (*dir, name, existing_fid.fid);
 	     if (error == 0)
 		 *existing_status = file->status;
 	 }
-     } else 
-	 error = EACCES;
+     }
 
      log_operation ("link (%ld,%lu,%lu,%lu) (%ld,%lu,%lu,%lu) %s\n",
 		    (*dir)->fid.Cell,
@@ -774,12 +795,12 @@ sub_remove (FCacheEntry **dir, const char *name, CredCacheEntry **ce,
      if (error)
 	 return error;
 
-     if (checkright (*dir, ADELETE, *ce)) {
+     error = cm_checkright(*dir, ADELETE, *ce);
+     if (!error) {
 	 error = (*func) (*dir, name, *ce);
 	 if (error == 0)
 	     error = adir_remove (*dir, name);
-     } else 
-	 error = EACCES;
+     }
      
      log_operation ("%s (%ld,%lu,%lu,%lu) %s\n",
 		    operation,
@@ -815,6 +836,485 @@ cm_rmdir(FCacheEntry **dir,
     return sub_remove (dir, name, ce, "rmdir", remove_directory);
 }
 
+
+/*
+ * Apple's Finder doesn't understand EXDEV within a "Volume", so we do
+ * a copy+remove.
+ *
+ * XXX this (or rather, nnpfs_message_rename() in messages.c) probably
+ * does not support rename across cells
+ *
+ * XXX does not properly handle hardlinks
+ */
+
+#define RENAME_MAX_DEPTH 10
+
+/*
+ * Arguments struct for rename_remove_node().
+ */
+
+typedef struct remove_node_args {
+    CredCacheEntry *ce;
+    FCacheEntry *dir;
+    int depth;
+    int error;
+} remove_node_args;
+
+/*
+ * Remove a node or tree to given destination.
+ *
+ * We assume that relevant checks have been performed.
+ * Meant as a fdir_readdir_func.
+ */
+
+static int
+rename_remove_node(VenusFid *fid, const char *name, void *a)
+{
+    remove_node_args *args = (remove_node_args *)a;
+    CredCacheEntry *ce = args->ce;
+    FCacheEntry *entry;
+    int error;
+
+    if (strcmp(".", name) == 0 || strcmp("..", name) == 0)
+	return 0;
+
+    error = fcache_get(&entry, *fid, ce);
+    if (error)
+	return error;
+    
+    if (entry->status.FileType == TYPE_DIR) {
+	remove_node_args args2 = *args;
+	args2.dir = entry;
+	args2.depth++;
+	
+	if (args2.depth >= RENAME_MAX_DEPTH)
+	    error = EXDEV; /* restore original error */
+	else 
+	    error = adir_readdir(&entry,
+				 rename_remove_node,
+				 (void *)&args2,
+				 &ce);
+	if (!error)
+	    error = args2.error;
+
+	fcache_release(entry);
+	
+	if (!error)
+	    error = cm_rmdir(&args->dir, name, &ce);
+	
+    } else {
+	fcache_release(entry);
+	error = cm_remove(&args->dir, name, &ce);
+    }
+    
+    if (error)
+	args->error = error;
+
+    return error;
+}
+
+/*
+ * Remove a node or tree to given destination, usable version.
+ */
+
+static int
+rename_remove_tree(FCacheEntry *dir, VenusFid *fid,
+		   const char *name, CredCacheEntry *ce)
+{
+    remove_node_args args;
+
+    args.ce = ce;
+    args.depth = 0;
+    args.error = 0;
+    args.dir = dir;
+
+    return rename_remove_node(fid, name, (void*)&args);
+}
+
+typedef struct rename_fid_pair {
+    VenusFid old;
+    VenusFid new;
+} rename_fid_pair;
+
+/*
+ * Arguments struct for rename_copy_node().
+ */
+
+typedef struct copy_node_args {
+    CredCacheEntry *ce;
+    FCacheEntry *target;
+    rename_fid_pair *hardlinks;
+    int nlinks;
+    int depth;
+    int error;
+} copy_node_args;
+
+/* Forward */
+static int rename_readdir_copy(VenusFid *fid, const char *name, void *a);
+
+/*
+ * Copy a node or tree to given destination.
+ *
+ * We assume that relevant checks have been performed.
+ */
+
+static int
+rename_copy_node(FCacheEntry *old_entry, const char *name,
+		 copy_node_args *args)
+{
+    AFSFetchStatus fetch_attr;
+    CredCacheEntry *ce = args->ce;
+    AFSStoreStatus status;
+    FCacheEntry *new_entry = NULL;
+    VenusFid new_fid;
+    int error = 0;
+    
+    afsstatus2afsstorestatus(&old_entry->status, &status);
+    
+    if (old_entry->status.FileType == TYPE_DIR) {
+	error = cm_mkdir(&args->target, name, &status, &new_fid,
+			 &fetch_attr, &ce);
+	if (!error)
+	    error = fcache_get(&new_entry, new_fid, ce);
+	
+	if (!error) {
+	    copy_node_args args2 = *args;
+	    args2.target = new_entry;
+	    args2.hardlinks = NULL;
+	    args2.nlinks = 0;
+	    args2.depth++;
+
+	    if (args2.depth >= RENAME_MAX_DEPTH)
+		error = EXDEV; /* restore original error */
+	    else 
+		error = adir_readdir(&old_entry, rename_readdir_copy,
+				     (void *)&args2, &ce);
+	    if (args2.nlinks)
+		free(args2.hardlinks);
+
+	    if (!error)
+		error = args2.error;
+	}
+    } else if (old_entry->status.FileType == TYPE_FILE) {
+	int linkp = 0;
+	int createp = 1;
+
+	if (old_entry->status.LinkCount > 1) {
+	    int i;
+
+	    linkp = 1;
+
+	    for (i = 0; i < args->nlinks; i++)
+		if (VenusFid_cmp(&args->hardlinks[i].old, &old_entry->fid) == 0)
+		    break;
+	    
+	    if (i < args->nlinks) {
+		/* we've already copied this fid */
+		
+		/* XXX hope it's still the same one */
+		error = cm_link(&args->target, name,
+				args->hardlinks[i].new,
+				&fetch_attr, &ce);
+
+		createp = 0;
+	    }
+	}
+	
+	if (!error && createp)
+	    error = cm_create(&args->target, name, &status,
+			      &new_fid, &fetch_attr, &ce);
+	if (!error)
+	    error = fcache_get(&new_entry, new_fid, ce);
+    
+	/* copy file data */
+	if (!error)
+	    error = write_data(new_entry, old_entry, &status, ce);
+
+	if (linkp && createp && !error) {
+	    /* hard linked, add to list and copy the data */
+	    int n = args->nlinks + 1;
+	    if (n < 0) {
+		error = EXDEV;
+	    } else {
+		rename_fid_pair *tmp;
+		tmp = realloc(args->hardlinks, n * sizeof(rename_fid_pair));
+		if (tmp) {
+		    tmp[args->nlinks].old = old_entry->fid;
+		    tmp[args->nlinks].new = new_fid;
+		    args->hardlinks = tmp;
+		    args->nlinks = n;
+		} else {
+		    error = errno;
+		}
+	    }
+	}
+    } else if (old_entry->status.FileType == TYPE_LINK) {
+	char symlink[MAXPATHLEN];
+	int fd, len;
+	
+	fd = fcache_open_file(old_entry, O_RDONLY);
+	/* read the symlink and null-terminate it */
+	if (fd < 0) {
+	    error = errno;
+	    arla_warn(ADEBWARN, errno, "fcache_open_file");
+	}
+	if (!error) {
+	    len = read(fd, symlink, sizeof(symlink) - 1);
+	    if (len <= 0) {
+		error = errno;
+		arla_warnx (ADEBWARN, "cannot read symlink");
+	    } else {
+		symlink[len] = '\0';
+		error = cm_symlink(&args->target, name,
+				   &status, &new_fid,
+				   NULL, &fetch_attr,
+				   symlink, &ce);
+	    }
+	    close(fd);
+	}
+    } else {
+	arla_warnx(ADEBWARN, "rename_copy_node: bad node type");
+    }
+    
+    if (new_entry)
+	fcache_release(new_entry);
+
+    return error;
+}
+
+/*
+ * Copy a node or tree to given destination.
+ *
+ * fdir_readdir_func version.
+ */
+
+static int
+rename_readdir_copy(VenusFid *fid, const char *name, void *a)
+{
+    copy_node_args *args = (copy_node_args *)a;
+    FCacheEntry *old_entry;
+    int error;
+
+    if (strcmp(".", name) == 0 || strcmp("..", name) == 0)
+	return 0;
+
+    error = fcache_get(&old_entry, *fid, args->ce);
+    if (error)
+	return error;
+    
+    error = fcache_get_data(&old_entry, &args->ce, 0);
+    if (!error) {
+	error = rename_copy_node(old_entry, name, args);
+	if (error)
+	    args->error = error;
+    }
+
+    fcache_release(old_entry);
+
+    return error;
+}
+
+/*
+ * Copy a node or tree to given destination, usable version.
+ */
+
+static int
+rename_copy_tree(FCacheEntry *old_entry, FCacheEntry *target,
+		 const char *new_name, CredCacheEntry *ce)
+{
+    copy_node_args args;
+    int ret;
+
+    args.ce = ce;
+    args.depth = 0;
+    args.error = 0;
+    args.target = target;
+    args.hardlinks = NULL;
+    args.nlinks = 0;
+
+    ret = rename_copy_node(old_entry, new_name, &args);
+
+    if (args.nlinks)
+	free(args.hardlinks);
+
+    return ret;
+}
+
+/*
+ * Arguments struct for rename_source_access().
+ */
+
+typedef struct source_access_args {
+    CredCacheEntry *ce;
+    int depth;
+    int error;
+} source_access_args;
+
+/*
+ * Check access rights for source node and recurse.
+ *
+ * Meant as a fdir_readdir_func.
+ */
+
+static int
+rename_source_access(VenusFid *fid, const char *name, void *a)
+{
+    source_access_args *args = (source_access_args *)a;
+    CredCacheEntry *ce = args->ce;
+    FCacheEntry *entry;
+    int error;
+
+    if (strcmp(".", name) == 0 || strcmp("..", name) == 0)
+	return 0;
+    
+    error = fcache_get(&entry, *fid, ce);
+    if (!error)
+	error = fcache_verify_attr (entry, NULL, NULL, ce);
+
+    if (!error) {
+	if (entry->status.FileType == TYPE_DIR) {
+	    error = cm_checkright(entry, ADELETE|AREAD, ce);
+	    if (!error) {
+		/* Let's recurse */
+		args->depth++;
+
+		if (args->depth >= RENAME_MAX_DEPTH)
+		    error = EXDEV; /* restore original error */
+		else 
+		    error = adir_readdir(&entry, rename_source_access,
+					 a, &ce);
+		args->depth--;
+		if (!error)
+		    error = args->error;
+	    }
+	} else if (entry->status.FileType != TYPE_FILE
+		   && entry->status.FileType != TYPE_LINK) {
+	    error = EACCES;
+	}
+
+	fcache_release(entry);
+    }
+    
+    if (error)
+	args->error = error;
+    
+    return error;
+}
+
+/*
+ * Do some checks to be reasonably sure the operation won't fail.
+ *
+ * XXX check size vs quota.
+ */
+
+static int
+rename_check_tree(FCacheEntry *oldnode, CredCacheEntry *ce)
+{
+    source_access_args args;
+    int error;
+    
+    args.error = 0;
+    args.depth = 0;
+    args.ce = ce;
+    
+    error = adir_readdir(&oldnode, 
+			 rename_source_access,
+			 (void *)&args,
+			 &ce);
+    if (!error)
+	error = args.error;
+
+    return error;
+}
+
+static int
+copy_remove_entry(FCacheEntry *old_dir, const char *old_name,
+		  FCacheEntry *new_dir, const char *new_name,
+		  VenusFid *new_fid, CredCacheEntry *ce)
+{
+    VenusFid existing_fid, old_fid;
+    FCacheEntry *old_entry = NULL;
+
+    int exists = 0;
+    int dirp = 0;
+    int error;
+
+    /* until we bother to get a second set of creds */
+    if (new_dir->fid.Cell != old_dir->fid.Cell)
+	return EXDEV;
+
+    error = adir_lookup(new_dir, new_name, &existing_fid);
+    if (!error)
+	exists = 1;
+
+    error = adir_lookup(old_dir, old_name, &old_fid);
+    if (error)
+	return error;
+
+    /* check permissions (and hope they don't change) */
+    error = cm_checkright(old_dir, ADELETE|AREAD, ce);
+    if (!error)
+	error = cm_checkright(new_dir, AINSERT, ce);
+    if (exists && !error)
+	error = cm_checkright(new_dir, ADELETE, ce);
+    if (error)
+	return error;
+    
+    error = fcache_get(&old_entry, old_fid, ce);
+    if (error)
+	return error;
+
+    if (old_entry->status.FileType != TYPE_FILE
+	&& old_entry->status.FileType != TYPE_DIR
+	&& old_entry->status.FileType != TYPE_LINK) {
+	fcache_release(old_entry);
+	return EXDEV;
+    }
+    
+    if (old_entry->status.FileType == TYPE_DIR)
+	dirp = 1;
+    
+    error = fcache_get_data(&old_entry, &ce, 0);
+
+    if (dirp && !error) {
+	if (!adir_emptyp(&old_entry, &ce))
+	    error = rename_check_tree(old_entry, ce);
+    }
+    
+    if (!error && exists) {
+	/* 1. we might be able to do a truncate+write. but not today */
+	/* 2. rmdir fails if not empty. that's correct and well. */
+
+	if (dirp)
+	    error = cm_rmdir(&new_dir, new_name, &ce);
+	else
+	    error = cm_remove(&new_dir, new_name, &ce);
+    }
+
+    if (!error) {
+	error = rename_copy_tree(old_entry, new_dir, new_name, ce);
+
+	if (error) {
+	    /*
+	     * Roll back changes as best we can. Unfortunately, existing
+	     * targets are already removed and permanently lost.
+	     */
+	    
+	    VenusFid created_fid;
+	    int error2 = adir_lookup(new_dir, new_name, &created_fid);
+	    if (!error2)
+		rename_remove_tree(new_dir, &created_fid, new_name, ce);
+	}
+    }
+
+    fcache_release(old_entry);
+
+    if (!error)
+	error = rename_remove_tree(old_dir, &old_fid, old_name, ce);
+
+    return error;
+}
+
 /*
  * Called when the object is being moved to a new directory, to be
  * able to update .. when required.
@@ -891,13 +1391,24 @@ cm_rename(FCacheEntry **old_dir, const char *old_name,
     if (error)
 	return error;
 
-    if (!checkright (*old_dir, ADELETE, *ce)
-	|| !checkright (*new_dir, AINSERT, *ce)) {
-	error = EACCES;
+    error = cm_checkright(*old_dir, ADELETE, *ce);
+    if (!error)
+	error = cm_checkright(*new_dir, AINSERT, *ce);
+    if (error)
 	goto out;
-    }
 	
     error = rename_file (*old_dir, old_name, *new_dir, new_name, *ce);
+    if (error == EXDEV) {
+	/*
+	 * copy_remove_entry() only does normal operations (mkdir,
+	 * remove, etc) and thus gets correct parent/child fids, so we
+	 * can leave update_child unchanged (zero).
+	 */
+	error = copy_remove_entry(*old_dir, old_name, *new_dir,
+				  new_name, child_fid, *ce);
+	goto out;
+    }
+
     if (error)
 	goto out;
 
@@ -954,18 +1465,19 @@ cm_rename(FCacheEntry **old_dir, const char *old_name,
     error = adir_creat (*new_dir, new_name,  new_fid.fid)
 	|| adir_remove (*old_dir, old_name);
     
-    log_operation ("rename (%ld,%lu,%lu,%lu) (%ld,%lu,%lu,%lu) %s %s\n",
-		   (*old_dir)->fid.Cell,
-		   (*old_dir)->fid.fid.Volume,
-		   (*old_dir)->fid.fid.Vnode,
-		   (*old_dir)->fid.fid.Unique,
-		   (*new_dir)->fid.Cell,
-		   (*new_dir)->fid.fid.Volume,
-		   (*new_dir)->fid.fid.Vnode,
-		   (*new_dir)->fid.fid.Unique,
-		   old_name, new_name);
-    
  out:
+    if (!error)
+	log_operation ("rename (%ld,%lu,%lu,%lu) (%ld,%lu,%lu,%lu) %s %s\n",
+		       (*old_dir)->fid.Cell,
+		       (*old_dir)->fid.fid.Volume,
+		       (*old_dir)->fid.fid.Vnode,
+		       (*old_dir)->fid.fid.Unique,
+		       (*new_dir)->fid.Cell,
+		       (*new_dir)->fid.fid.Volume,
+		       (*new_dir)->fid.fid.Vnode,
+		       (*new_dir)->fid.fid.Unique,
+		       old_name, new_name);
+    
     cm_check_consistency();
     return error;
 }
@@ -1039,33 +1551,33 @@ cm_walk (VenusFid fid,
 	    return -1;
 	}
 	error = cm_lookup (&dentry, base, &file, &ce, TRUE);
+	fcache_release(dentry);
 	if (error) {
-	    fcache_release(dentry);
 	    arla_warn (ADEBWARN, error, "lookup(%s)", base);
 	    cred_free(ce);
 	    return -1;
 	}
-	fcache_release(dentry);
+
 	error = fcache_get(&entry, file, ce);
 	if (error) {
 	    arla_warn (ADEBWARN, error, "fcache_get");
 	    cred_free(ce);
 	    return -1;
 	}
-	
-	error = fcache_get_data (&entry, &ce, 0);
-	if (error) {
-	    fcache_release(entry);
-	    arla_warn (ADEBWARN, error, "fcache_get_data");
-	    cred_free(ce);
-	    return -1;
-	}
-	
+		
 	/* handle symlinks here */
 	if (entry->status.FileType == TYPE_LINK) {
 	    int len;
 	    int fd;
 	    
+	    error = fcache_get_data (&entry, &ce, 0);
+	    if (error) {
+		fcache_release(entry);
+		arla_warn (ADEBWARN, error, "fcache_get_data");
+		cred_free(ce);
+		return -1;
+	    }
+
 	    fd = fcache_open_file (entry, O_RDONLY);
 	    /* read the symlink and null-terminate it */
 	    if (fd < 0) {
