@@ -35,13 +35,16 @@
  * The interface to the cache manager.
  */
 
-/* $Id: inter.h,v 1.36 2003/01/09 16:40:10 lha Exp $ */
+/* $Id: inter.h,v 1.38 2006/03/10 12:23:27 tol Exp $ */
 
 #ifndef _INTER_H_
 #define _INTER_H_
 
 #include <cred.h>
 
+int
+cm_checkright (FCacheEntry *e, u_long mask, CredCacheEntry *ce);
+
 void
 cm_init (void);
 
@@ -67,9 +70,6 @@ int
 cm_ftruncate (FCacheEntry *entry, off_t size,
 	      AFSStoreStatus *storeattr, CredCacheEntry *ce);
 
-int
-cm_access (FCacheEntry *entry, int mode, CredCacheEntry *ce);
-
 int
 cm_lookup (FCacheEntry **entry, const char *name, VenusFid *res,
 	   CredCacheEntry **ce, int follow_mount_point);
