@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2003 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2005 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -32,7 +32,7 @@
  */
 
 #include "arla_local.h"
-RCSID("$Id: kernel.c,v 1.37 2004/03/26 12:05:22 lha Exp $");
+RCSID("$Id: kernel.c,v 1.46 2006/02/17 11:47:52 lha Exp $");
 
 /*
  * The fd we use to talk with the kernel on.
@@ -67,12 +67,28 @@ kernel_usedworkers(void)
     return workers_used;
 }
 
+/*
+ * The work threads.
+ */
+
+struct worker {
+    char name[16];
+    char data[NNPFS_MAX_MSG_SIZE];
+    PROCESS pid;
+    int  msg_length;
+    int  busyp;
+    int  number;
+    uint32_t seqno;
+    uint32_t opcode;
+    const char *debuginfo;
+} *workers;
+
 /*
  *
  */
 
 static int
-process_message (int msg_length, char *msg)
+process_message (int msg_length, char *msg, struct worker *self)
 {
     struct nnpfs_message_header *header;
     char *p;
@@ -83,7 +99,11 @@ process_message (int msg_length, char *msg)
 	 msg_length > 0;
 	 p += header->size, msg_length -= header->size) {
 	header = (struct nnpfs_message_header *)p;
+	self->seqno = header->sequence_num;
+	self->opcode = header->opcode;
 	nnpfs_message_receive (kernel_fd, header, header->size);
+	self->seqno = 0;
+	self->opcode = 0;
 	++cnt;
     }
     if (cnt < sizeof(recv_count)/sizeof(recv_count[0]))
@@ -98,32 +118,103 @@ process_message (int msg_length, char *msg)
 
 static int overload = FALSE;
 
+static int trace_fd = -1;
+
+static void
+open_trace(char *filename)
+{
+    char fn[MAXPATHLEN];
+
+    if (filename == NULL)
+	return;
+
+    snprintf (fn, MAXPATHLEN, "%s", filename);
+    trace_fd = open (fn, O_WRONLY | O_CREAT | O_TRUNC, 0700);
+}
+
+static void
+save_message(const char *msg, int msg_length, int direction)
+{
+    struct timeval tv;
+    struct {
+	uint32_t direction;
+	uint32_t tv_sec;
+	uint32_t tv_usec;
+	uint32_t len;
+    } header;
+    struct iovec iov[2];
+
+    if (trace_fd == -1)
+	return;
+    
+    gettimeofday(&tv, NULL);
+    
+    header.direction = direction; /* direction is stored in host byte order */
+    header.tv_sec = htonl(tv.tv_sec);
+    header.tv_usec = htonl(tv.tv_usec);
+    header.len = htonl(msg_length);
+    
+    iov[0].iov_base = &header;
+    iov[0].iov_len = sizeof(header);
+    iov[1].iov_base = (char *)msg;
+    iov[1].iov_len = msg_length;
+
+    writev(trace_fd, iov, 2);
+}
+
+#define ARLAD_LWP_ROCK_CURRENTWORKER_TAG 0x3ce87ac5
+
 /*
- * The work threads.
+ * Sets the debug string for a worker. Does *not* copy the string,
+ * so make sure the pointer lives "forever", e.g. by using a string
+ * constant.
  */
 
-struct worker {
-    char name[16];
-    char data[MAX_XMSG_SIZE];
-    PROCESS pid;
-    int  msg_length;
-    int  busyp;
-    int  number;
-} *workers;
+void
+worker_setdebuginfo(const char *s)
+{
+    struct worker *worker;
+    int ret;
+
+    ret = LWP_GetRock(ARLAD_LWP_ROCK_CURRENTWORKER_TAG, (char **)&worker);
+    if (ret == LWP_SUCCESS) {
+	worker->debuginfo = s;
+    } else {
+        arla_log (ADEBWARN, 
+		  "worker_setdebuginfo: Could not find worker for tag %d",
+		  ARLAD_LWP_ROCK_CURRENTWORKER_TAG);
+    }
+		
+}
+
+void
+worker_printstatus(void)
+{
+    int i;
+
+    arla_log (ADEBVLOG, "worker status:");
+    for (i = 0; i < workers_high; ++i) {
+	arla_log (ADEBVLOG, "    worker %i opcode %u seqno %u %s",
+		  i, workers[i].opcode, workers[i].seqno, workers[i].debuginfo ? workers[i].debuginfo : "");
+    }
+}
 
 static void
 sub_thread (void *v_myself)
 {
     struct worker *self = (struct worker *)v_myself;
 
+    LWP_NewRock(ARLAD_LWP_ROCK_CURRENTWORKER_TAG, (char *)self);
+    worker_setdebuginfo(NULL);
     for (;;) {
 	arla_warnx (ADEBKERNEL, "worker %d waiting", self->number);
+	worker_setdebuginfo("waiting");
 	LWP_WaitProcess (self);
-	self->busyp = 1;
-	++workers_used;
 	arla_warnx (ADEBKERNEL, "worker %d: processing", self->number);
-	process_message (self->msg_length, self->data);
+	worker_setdebuginfo("processing");
+	process_message (self->msg_length, self->data, self);
 	arla_warnx (ADEBKERNEL, "worker %d: done", self->number);
+	worker_setdebuginfo("done");
 	--workers_used;
 	self->busyp = 0;
 	overload = FALSE;
@@ -210,8 +301,8 @@ tcp_write (int fd, const void *data, size_t len)
     int32_t slen = htonl(len);
     char out_len[4];
 
-    memcpy (out_len, &slen, sizeof(len));
-    if (send (fd, out_len, sizeof(len), 0) != sizeof(out_len)) {
+    memcpy (out_len, &slen, sizeof(slen));
+    if (send (fd, out_len, sizeof(out_len), 0) != sizeof(out_len)) {
 	arla_warn (ADEBWARN, errno, "tcp_write: failed to write length");
 	return -1;
     }
@@ -319,14 +410,20 @@ kern_open (const char *filename)
 ssize_t
 kern_read (int fd, void *data, size_t len)
 {
+    ssize_t ret;
+    
     assert (kern_cur != NULL);
-    return (kern_cur->read) (fd, data, len);
+    ret = (kern_cur->read) (fd, data, len);
+    if (ret > 0)
+	save_message(data, ret, 1);
+    return ret;
 }
 
 ssize_t
 kern_write (int fd, const void *data, size_t len)
 {
     assert (kern_cur != NULL);
+    save_message(data, len, 2);
     return (kern_cur->write) (fd, data, len);
 }
 
@@ -357,6 +454,7 @@ kernel_interface (struct kernel_args *args)
 {
     int i;
 
+    open_trace(trace_file);
     assert (kernel_fd >= 0);
 
     workers = malloc (sizeof(*workers) * args->num_workers);
@@ -367,20 +465,20 @@ kernel_interface (struct kernel_args *args)
     workers_high = args->num_workers;
     workers_used = 0;
  
+    message_init();
+
     for (i = 0; i < args->num_workers; ++i) {
 	workers[i].busyp  = 0;
 	workers[i].number = i;
 	snprintf(workers[i].name, sizeof(workers[i].name), "worker %d", i);
 
-	if (LWP_CreateProcess (sub_thread, default_stacksize, 1,
-			       (char *)&workers[i],
+	if (LWP_CreateProcess (sub_thread, 0, 1, (char *)&workers[i],
 			       workers[i].name, &workers[i].pid))
 	    arla_errx (1, ADEBERROR, "CreateProcess of worker failed");
     }
 
-    if (LWP_CreateProcess (version_thread, default_stacksize, 1,
-			   NULL,
-			   "version", &version_pid))
+    if (LWP_CreateProcess (version_thread, 0, 1, NULL, "version", 
+			   &version_pid))
 	arla_errx (1, ADEBERROR, "CreateProcess of version thread failed");
 
     arla_warnx(ADEBKERNEL, "Arla: selecting on fd: %d", kernel_fd);
@@ -407,15 +505,27 @@ kernel_interface (struct kernel_args *args)
 		    if (ret <= 0) {
 			arla_warn (ADEBWARN, errno, "read");
 		    } else {
+			workers[i].busyp = 1;
 			workers[i].msg_length = ret;
+			workers[i].seqno = 0;
+			workers[i].opcode = 0;
+			++workers_used;
 			LWP_SignalProcess (&workers[i]);
 		    }
 		    break;
 		}
 	    }
 	    if (i == args->num_workers) {
+		int j;
 		arla_warnx (ADEBWARN, "kernel: all %u workers busy",
 			    args->num_workers);
+		for (j = 0; j < args->num_workers; ++j) {
+		    arla_warnx (ADEBWARN, 
+				"kernel: worker %i opcode %u seqno %u %s",
+				j, workers[j].opcode, 
+				workers[j].seqno, 
+				workers[j].debuginfo ? workers[j].debuginfo : "");
+		}
 		overload = TRUE;
 		LWP_WaitProcess(&overload);
 	    }
