@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: kernel.h,v 1.9 2002/07/24 06:04:29 lha Exp $ */
+/* $Id: kernel.h,v 1.10 2005/10/28 14:33:37 tol Exp $ */
 
 #ifndef _KERNEL_H_
 #define _KERNEL_H_
@@ -59,4 +59,10 @@ kern_read (int fd, void *data, size_t len);
 ssize_t
 kern_write (int fd, const void *data, size_t len);
 
+void
+worker_setdebuginfo(const char *s);
+
+void
+worker_printstatus(void);
+
 #endif /* _KERNEL_H_ */
