@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2002 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2002, 2005 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -36,7 +36,7 @@
  */
 
 #include "arla_local.h"
-RCSID("$Id: volcache.c,v 1.117 2004/02/19 09:02:00 lha Exp $") ;
+RCSID("$Id: volcache.c,v 1.119 2005/10/28 14:33:37 tol Exp $") ;
 
 static int volcache_timeout = VOLCACHE_TIMEOUT;
 
@@ -224,15 +224,16 @@ recycle_entry (VolCacheEntry *e)
 
     assert (e->refcount == 0 && e->vol_refs == 0);
 
-    for (i = 0; i < MAXTYPES; ++i)
+    for (i = 0; i < MAXTYPES; ++i) {
 	if (e->num_ptr[i].ptr != NULL)
 	    hashtabdel (volidhashtab, &e->num_ptr[i]);
+	if (e->parent[i].volume) {
+	    volcache_volfree (e->parent[i].volume);
+	    e->parent[i].volume = NULL;
+	}
+    }
     if (e->name_ptr.ptr != NULL)
 	hashtabdel (volnamehashtab, &e->name_ptr);
-    if (e->parent) {
-	volcache_volfree (e->parent);
-	e->parent = NULL;
-    }
 
     memset (&e->entry, 0, sizeof(e->entry));
     memset (&e->volsync, 0, sizeof(e->volsync));
@@ -494,7 +495,10 @@ volcache_recover_entry (struct volcache_store *st, void *ptr)
     struct vstore_context *c = (struct vstore_context *)ptr;
 
     e->cell = cell_name2num (st->cell);
-    assert (e->cell != -1);
+    if (e->cell == -1){
+	arla_warnx(ADEBWARN, "can't resolve cell name");
+	return(-1);
+    }
     e->entry = st->entry;
     e->volsync = st->volsync;
     e->refcount = st->refcount;
@@ -527,15 +531,18 @@ volcache_recover_state (void)
 	item = listnext (lrulist, item)) {
 	VolCacheEntry *e = (VolCacheEntry *)listdata(item);
 	VolCacheEntry *parent;
+	int i;
 
 	if (e->cell == -1)
 	    continue;
 
-	parent = getbyid (e->parent_fid.fid.Volume,
-			  e->parent_fid.Cell,
-			  NULL);
-	if (parent != NULL)
-	    volcache_volref (e, parent);
+	for (i = 0; i < MAXTYPES; ++i) {
+	    parent = getbyid (e->parent[i].fid.fid.Volume,
+			      e->parent[i].fid.Cell,
+			      NULL);
+	    if (parent != NULL)
+		volcache_volref (e, parent, i);
+	}
     }
     arla_warnx (ADEBVOLCACHE, "recovered %u entries to volcache", c.n);
 }
@@ -691,7 +698,6 @@ get_info_loop (VolCacheEntry *e, nvldbentry *entry,
 		    "Cannot find any db servers in cell %d(%s) while "
 		    "getting data for volume `%s'",
 		    cell, cell_num2name(cell), name);
-	assert (cell_is_sanep (cell));
 	return ENOENT;
     }
 
@@ -1178,13 +1184,13 @@ volcache_free (VolCacheEntry *e)
  */
 
 void
-volcache_volref (VolCacheEntry *e, VolCacheEntry *parent)
+volcache_volref (VolCacheEntry *e, VolCacheEntry *parent, long voltype)
 {
-    if (e->parent == NULL) {
+    if (e->parent[voltype].volume == NULL) {
 	if (parent->refcount == 0 && parent->vol_refs == 0)
 	    ++nactive_volcacheentries;
 	++parent->vol_refs;
-	e->parent = parent;
+	e->parent[voltype].volume = parent;
     }
 }
 
