@@ -35,7 +35,7 @@
  * Our cache of volume information.
  */
 
-/* $Id: volcache.h,v 1.36 2002/05/15 22:31:16 lha Exp $ */
+/* $Id: volcache.h,v 1.37 2005/10/28 14:33:37 tol Exp $ */
 
 #ifndef _VOLCACHE_
 #define _VOLCACHE_
@@ -73,9 +73,11 @@ struct volcacheentry {
     unsigned refcount;		/* number of files refererring this */
     unsigned vol_refs;		/* number of volumes refing this */
     Listitem *li;
-    VenusFid mp_fid;		/* pointing to this volume */
-    VenusFid parent_fid;	/* .. of this volume */
-    struct volcacheentry *parent; /* parent volume */
+    struct {
+	VenusFid mp_fid;	       /* mp pointing to this volume's rw/ro/bk */
+	VenusFid fid;	               /* .. of this volume */
+	struct volcacheentry *volume;  /* parent volume */
+    } parent[MAXTYPES];
     struct {
 	unsigned validp  : 1;
 	unsigned stablep : 1;
@@ -119,7 +121,7 @@ void volcache_free (VolCacheEntry *e);
 
 void volcache_ref (VolCacheEntry *e);
 
-void volcache_volref (VolCacheEntry *e, VolCacheEntry *parent);
+void volcache_volref (VolCacheEntry *e, VolCacheEntry *parent, long parent_type);
 
 void volcache_volfree (VolCacheEntry *e);
 
