@@ -38,7 +38,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: bits.c,v 1.7 2002/02/07 17:59:15 lha Exp $");
+RCSID("$Id: bits.c,v 1.8 2005/01/07 13:11:13 lha Exp $");
 #endif
 #include <stdio.h>
 #include <string.h>
@@ -140,8 +140,8 @@ int main(int argc, char **argv)
 	f = fopen(argv[1], "w");
     }
     fprintf(f, "/* %s -- this file was generated for %s by\n", fn, HOST);
-    fprintf(f, "   %*s    %s */\n\n", strlen(fn), "", 
-	    "$Id: bits.c,v 1.7 2002/02/07 17:59:15 lha Exp $");
+    fprintf(f, "   %*s    %s */\n\n", (int)strlen(fn), "", 
+	    "$Id: bits.c,v 1.8 2005/01/07 13:11:13 lha Exp $");
     fprintf(f, "#ifndef %s\n", hb);
     fprintf(f, "#define %s\n", hb);
     fprintf(f, "\n");
