@@ -35,7 +35,7 @@
  * The directory structure used by AFS.
  */
 
-/* $Id: afs_dir.h,v 1.8 2002/03/19 12:21:25 tol Exp $ */
+/* $Id: afs_dir.h,v 1.11 2005/08/09 14:51:54 lha Exp $ */
 
 #ifndef _AFS_DIR_
 #define _AFS_DIR_
@@ -44,7 +44,8 @@
 #define ADIRHASHSIZE 128
 #define ENTRIESPERPAGE 64
 
-#define AFSDIR_FIRST 1
+#define AFSDIR_FIRST	1
+#define AFSDIR_UTFENT	4
 
 /* number of pages in map */
 
@@ -72,7 +73,8 @@ typedef struct {
      } u;
 } PageHeader;
 
-#define AFSDIRMAGIC 1234
+#define AFSDIRMAGIC 		1234
+#define AFSDIRMAGIC_UTF8	12345
 
 #define pg_tag		u.s.tag
 #define pg_freecount	u.s.freecount
@@ -93,6 +95,12 @@ typedef struct {
      char fill[4];		/* should use next entry too.  */
 } DirEntry;
 
+typedef struct {
+    uint32_t utf_next;
+    uint16_t raw_offset;
+    uint16_t norm_offset;
+} DirEntry2;
+
 typedef struct {
      PageHeader header;
      DirHeader  dheader;
