@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2003 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2004 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -33,7 +33,7 @@
 
 #include <config.h>
 
-RCSID("$Id: fbuf.c,v 1.22 2003/01/23 10:34:55 tol Exp $") ;
+RCSID("$Id: fbuf.c,v 1.23 2004/12/18 20:55:05 lha Exp $") ;
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -177,7 +177,7 @@ static size_t mmap_max_size = 10 * 1024 * 1024;
  */
 
 static int 
-mmap_copyrx2fd (struct rx_call *call, int fd, off_t off, size_t len)
+mmap_copyrx2fd (struct rx_call *call, int fd, off_t off, off_t len)
 {
     void *buf;
     int r_len;
@@ -227,7 +227,7 @@ mmap_copyrx2fd (struct rx_call *call, int fd, off_t off, size_t len)
  */
 
 static int
-mmap_copyfd2rx (int fd, struct rx_call *call, off_t off, size_t len)
+mmap_copyfd2rx (int fd, struct rx_call *call, off_t off, off_t len)
 {
     void *buf;
     int w_len;
@@ -367,7 +367,7 @@ fail:
  */
 
 static int 
-malloc_copyrx2fd (struct rx_call *call, int fd, off_t off, size_t len)
+malloc_copyrx2fd (struct rx_call *call, int fd, off_t off, off_t len)
 {
     void *buf;
     struct stat statbuf;
@@ -409,7 +409,7 @@ malloc_copyrx2fd (struct rx_call *call, int fd, off_t off, size_t len)
  */
 
 static int
-malloc_copyfd2rx (int fd, struct rx_call *call, off_t off, size_t len)
+malloc_copyfd2rx (int fd, struct rx_call *call, off_t off, off_t len)
 {
     void *buf;
     struct stat statbuf;
@@ -515,7 +515,7 @@ fbuf_truncate (fbuf *f, size_t new_len)
  */
 
 int 
-copyrx2fd (struct rx_call *call, int fd, off_t off, size_t len)
+copyrx2fd (struct rx_call *call, int fd, off_t off, off_t len)
 {
 #ifdef HAVE_MMAP
     return mmap_copyrx2fd (call, fd, off, len);
@@ -531,7 +531,7 @@ copyrx2fd (struct rx_call *call, int fd, off_t off, size_t len)
  */
 
 int
-copyfd2rx (int fd, struct rx_call *call, off_t off, size_t len)
+copyfd2rx (int fd, struct rx_call *call, off_t off, off_t len)
 {
 #ifdef HAVE_MMAP
     return mmap_copyfd2rx (fd, call, off, len);
