@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000, 2002, 2004 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: fbuf.h,v 1.8 2002/09/07 10:44:06 lha Exp $ */
+/* $Id: fbuf.h,v 1.9 2004/12/18 20:56:33 lha Exp $ */
 
 #ifndef _FBUF_H_
 #define _FBUF_H_
@@ -68,7 +68,7 @@ size_t fbuf_len (fbuf *f);
 void *fbuf_buf (fbuf *f);
 
 #ifndef _KERNEL
-int copyrx2fd (struct rx_call *call, int fd, off_t off, size_t len);
-int copyfd2rx (int fd, struct rx_call *call, off_t off, size_t len);
+int copyrx2fd (struct rx_call *call, int fd, off_t off, off_t len);
+int copyfd2rx (int fd, struct rx_call *call, off_t off, off_t len);
 #endif
 #endif /* _FBUF_H_ */
