@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2005 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -38,7 +38,7 @@
 #ifndef _KERNEL
 #include <config.h>
 
-RCSID("$Id: fdir.c,v 1.16 2003/11/04 00:41:41 lha Exp $") ;
+RCSID("$Id: fdir.c,v 1.27 2005/11/23 11:58:08 lha Exp $") ;
 
 #include <sys/types.h>
 #include <sys/errno.h>
@@ -79,13 +79,24 @@ hashentry (const char *sentry)
  */
 
 static unsigned
-additional_entries (const char *filename)
+additional_entries (const char *filename,
+		    const char *raw_utf,
+		    const char *norm_utf)
 {
     static DirEntry dummy;
 
-    return (strlen(filename) - sizeof(dummy.name) + 1
-	    + sizeof(DirEntry) - 1)
-	/ sizeof(DirEntry);
+    size_t len = (strlen(filename) - sizeof(dummy.name) + 1
+		  + sizeof(DirEntry) - 1);
+    if (raw_utf) {
+	/* align to 4 byte boundery */
+	if (len % 4) 
+	    len += 4 - (len % 4);
+	len += sizeof(DirEntry2);
+	len += strlen(raw_utf);
+	if (norm_utf)
+	    len += strlen(norm_utf);
+    }
+    return len / sizeof(DirEntry);
 }
 
 /*
@@ -105,11 +116,21 @@ getpage (DirPage0 *page0, unsigned pageno)
 
 static DirEntry *
 getentry (DirPage0 *page0,
-	  unsigned short num)
+	  unsigned short num,
+	  size_t npages)
 {
-     DirPage1 *page = getpage (page0, num / ENTRIESPERPAGE);
+     DirPage1 *page;
+     unsigned idx = num / ENTRIESPERPAGE;
+
+     if (idx >= npages)
+	 return NULL;
+
+     page = getpage (page0, idx);
+
+     if (page->header.pg_tag != htons(AFSDIRMAGIC) &&
+	 page->header.pg_tag != htons(AFSDIRMAGIC_UTF8))
+	 return NULL;
 
-     assert (page->header.pg_tag == htons(AFSDIRMAGIC));
      return &page->entry[num % ENTRIESPERPAGE];
 }
 
@@ -118,15 +139,18 @@ getentry (DirPage0 *page0,
  */
 
 static DirEntry *
-find_entry(DirPage0 *page0, const char *name)
+find_entry(DirPage0 *page0, const char *name, size_t npages)
 {
     DirEntry *entry;
     unsigned short i;
 
     for (i = ntohs(page0->dheader.hash[hashentry (name)]);
 	 i != 0;
-	 i = ntohs(entry->next)) {
-	entry = getentry (page0, (unsigned short)(i - 1));
+	 i = ntohs(entry->next))
+    {
+	entry = getentry (page0, (unsigned short)(i - 1), npages);
+	if (entry == NULL)
+	    return NULL;
 
 	if (strcmp (entry->name, name) == 0)
 	    return entry;
@@ -144,9 +168,10 @@ static int
 find_by_name (DirPage0 *page0,
 	      const char *name,
 	      VenusFid *fid,
-	      const VenusFid *dir)
+	      const VenusFid *dir,
+	      unsigned npages)
 {
-    const DirEntry *entry = find_entry (page0, name);
+    const DirEntry *entry = find_entry (page0, name, npages);
 
     if (entry == NULL)
 	return -1;
@@ -176,7 +201,7 @@ first_slotp (DirPage1 *page, int off)
 {
     DirEntry *entry = &page->entry[off];
     if (used_slot(page, off + 1) 
-	&& entry->flag == AFSDIR_FIRST)
+	&& (entry->flag & AFSDIR_FIRST))
 	return TRUE;
 
     return FALSE;
@@ -214,10 +239,17 @@ fdir_lookup (fbuf *the_fbuf, const VenusFid *dir,
 {
      DirPage0 *page0;
      unsigned ind;
+     unsigned npages;
+     size_t len = fbuf_len(the_fbuf);
 
      page0 = (DirPage0 *)fbuf_buf(the_fbuf);
      assert (page0);
-     ind = find_by_name (page0, name, file, dir);
+
+     npages = ntohs(page0->header.pg_pgcount);
+     if (npages < len / AFSDIR_PAGESIZE)
+	 npages = len / AFSDIR_PAGESIZE;
+
+     ind = find_by_name (page0, name, file, dir, npages);
 
      if (ind == 0)
 	 return 0;
@@ -234,11 +266,14 @@ fdir_emptyp (fbuf *dir)
 {
      DirPage0 *page0;
      unsigned npages;
+     size_t len = fbuf_len(dir);
 
      page0 = (DirPage0 *)fbuf_buf(dir);
      assert (page0);
 
      npages = ntohs(page0->header.pg_pgcount);
+     if (npages < len / AFSDIR_PAGESIZE)
+	 npages = len / AFSDIR_PAGESIZE;
 
      return (npages == 1) && (page0->dheader.map[0] == 49);
 }
@@ -258,7 +293,7 @@ fdir_readdir (fbuf *the_fbuf,
      DirPage0 *page0;
      unsigned i, j;
      VenusFid fid;
-     unsigned len = fbuf_len(the_fbuf);
+     size_t len = fbuf_len(the_fbuf);
      unsigned npages;
      unsigned first_slot;
      int ret;
@@ -268,7 +303,6 @@ fdir_readdir (fbuf *the_fbuf,
      assert (page0);
 
      npages = ntohs(page0->header.pg_pgcount);
-
      if (npages < len / AFSDIR_PAGESIZE)
 	 npages = len / AFSDIR_PAGESIZE;
 
@@ -276,7 +310,8 @@ fdir_readdir (fbuf *the_fbuf,
 	 i = *offset / ENTRIESPERPAGE;
 	 first_slot = *offset % ENTRIESPERPAGE;
 
-	 assert(i > 0 || first_slot >= 12);
+	 if (i == 0 && first_slot < 12)
+	     return EINVAL;
      } else {
 	 i = 0;
 	 first_slot = 12;
@@ -288,8 +323,11 @@ fdir_readdir (fbuf *the_fbuf,
 	 for (j = first_slot; j < ENTRIESPERPAGE - 1; ++j) {
 	     if (first_slotp (page, j)) {
 		 DirEntry *entry = &page->entry[j];
+		 char *name;
+		 char *raw_utf = NULL;
+		 char *norm_utf = NULL;
 
-		 if (entry->flag != AFSDIR_FIRST)
+		 if ((entry->flag & AFSDIR_FIRST) == 0)
 		     continue;
 
 		 fid.Cell       = dir.Cell;
@@ -297,14 +335,42 @@ fdir_readdir (fbuf *the_fbuf,
 		 fid.fid.Vnode  = ntohl (entry->fid.Vnode);
 		 fid.fid.Unique = ntohl (entry->fid.Unique);
 
-		 ret = (*func)(&fid, entry->name, arg);
+		 if (entry->flag & AFSDIR_UTFENT) {
+		     DirEntry2 *direntry2 = (DirEntry2 *)((char *)entry) + (entry->length << 2);
+
+		     /*
+		      * Check that the end of the entry isn't past end
+		      * of page
+		      */
+		     if ((unsigned char *)direntry2 > 
+			 ((unsigned char *)page) + AFSDIR_PAGESIZE - sizeof(*direntry2))
+			 continue;
+
+		     raw_utf = ((char *)entry) + direntry2->raw_offset;
+		     if (direntry2->raw_offset != direntry2->norm_offset)
+			 norm_utf = ((char *)entry) + direntry2->norm_offset;
+
+		     /* 
+		      * Check that the start of the name isn't over
+		      * the page, we should really check if its inside
+		      * page, but lets skip that for now.
+		      */
+		     if (raw_utf > ((char *)page) + AFSDIR_PAGESIZE)
+			 continue;
+
+		     name = raw_utf;
+		 } else
+		     name = entry->name;
+
+
+		 ret = (*func)(&fid, name, arg);
 		 if (ret) {
 		     if (ret > 0)
 			 j++; /* look at next entry next time */
 		     
 		     goto done;
 		 }
-		 j += additional_entries (entry->name);
+		 j += additional_entries (entry->name, raw_utf, norm_utf);
 	     }
 	 }
 	 first_slot = 0;
@@ -329,7 +395,8 @@ fdir_dirp (fbuf *the_fbuf)
 {
      DirPage0 *page0;
      DirPage1 *page;
-     unsigned num, npages, len = fbuf_len(the_fbuf);
+     unsigned num, npages;
+     size_t len = fbuf_len(the_fbuf);
 
      if (len < AFSDIR_PAGESIZE)
 	 return 0;
@@ -338,14 +405,14 @@ fdir_dirp (fbuf *the_fbuf)
      assert (page0);
 
      npages = ntohs(page0->header.pg_pgcount);
-
      if (npages < len / AFSDIR_PAGESIZE)
 	 npages = len / AFSDIR_PAGESIZE;
 
      for (num = 0; num < npages; num++) {
 	 page = getpage (page0, num);
 
-	 if (page->header.pg_tag != htons(AFSDIRMAGIC))
+	 if (page->header.pg_tag != htons(AFSDIRMAGIC) &&
+	     page->header.pg_tag != htons(AFSDIRMAGIC_UTF8))
 	     return 0;
      }
      return 1;
@@ -359,9 +426,10 @@ fdir_dirp (fbuf *the_fbuf)
 static int
 update_fid_by_name (DirPage0 *page0,
 		    const char *name,
-		    const VenusFid *fid)
+		    const VenusFid *fid,
+		    unsigned npages)
 {
-    DirEntry *entry = find_entry (page0, name);
+    DirEntry *entry = find_entry (page0, name, npages);
 
     if (entry == NULL)
 	return -1;
@@ -399,7 +467,7 @@ set_unused (DirPage1 *page, int off)
 
 static int
 create_new_page (DirPage1 **ret_page,
-		 fbuf *the_fbuf)
+		 fbuf *the_fbuf, int utf8_page)
 {
     int ret;
     DirPage1 *page1;
@@ -411,7 +479,10 @@ create_new_page (DirPage1 **ret_page,
 
     page1 = (DirPage1 *)((char *)fbuf_buf(the_fbuf) + len);
     page1->header.pg_pgcount   = htons(0);
-    page1->header.pg_tag       = htons(AFSDIRMAGIC);
+    if (utf8_page)
+	page1->header.pg_tag       = htons(AFSDIRMAGIC_UTF8);
+    else
+	page1->header.pg_tag       = htons(AFSDIRMAGIC);
     page1->header.pg_freecount = ENTRIESPERPAGE - 1;
     memset (page1->header.pg_bitmap, 0, sizeof(page1->header.pg_bitmap));
     set_used (page1, 0);
@@ -431,13 +502,14 @@ add_to_page (DirPage0 *page0,
 	     DirPage1 *page,
 	     unsigned pageno,
 	     const char *filename,
+	     const char *raw_name,
 	     AFSFid fid,
 	     unsigned next)
 {
     int i, j;
     unsigned n;
 
-    n = 1 + additional_entries (filename);
+    n = 1 + additional_entries (filename, raw_name, NULL);
 
     if (pageno < MAXPAGES && page0->dheader.map[pageno] < n)
 	return -1;
@@ -457,6 +529,33 @@ add_to_page (DirPage0 *page0,
 	    page->entry[i].fid.Vnode  = htonl(fid.Vnode);
 	    page->entry[i].fid.Unique = htonl(fid.Unique);
 	    strcpy (page->entry[i].name, filename);
+	    if (raw_name) {
+		DirEntry2 *entry2;
+		size_t fnl2, fnl = strlen(filename) + 1;
+
+		page->entry[i].flag |= AFSDIR_UTFENT;
+
+		if (fnl % 4)
+		    fnl2 = fnl + (4 - (fnl % 4));
+		else
+		    fnl2 = fnl;
+		memset(page->entry[i].name + fnl, 0, fnl2 - fnl);
+
+		page->entry[i].length = 
+		    (((unsigned char *)&page->entry[i].name) - 
+		     ((unsigned char *)&page->entry[i]) +
+		     fnl2) >> 2;
+
+		entry2 = (DirEntry2 *)(page->entry[i].name + fnl2);
+		entry2->utf_next = 0;
+		entry2->raw_offset = 
+		    ((unsigned char *)&page->entry[i].name) - 
+		    ((unsigned char *)&page->entry[i]) +
+		    fnl2 + sizeof(*entry2);
+		entry2->norm_offset = entry2->raw_offset;
+		strcpy((char *)(((unsigned char *)&page->entry[i]) + entry2->raw_offset),
+		       raw_name);
+	    }
 	    memset(page->entry[i + j - 1].fill, 0, 4);
 	    if (pageno < MAXPAGES)
 		page0->dheader.map[pageno] -= n;
@@ -480,7 +579,7 @@ remove_from_page (DirPage0 *page0,
     DirEntry *entry = &page->entry[off];
     unsigned n, i;
 
-    n = 1 + additional_entries (entry->name);
+    n = 1 + additional_entries (entry->name, NULL, NULL);
 
     if (pageno < MAXPAGES)
 	page0->dheader.map[pageno] += n;
@@ -505,11 +604,18 @@ fdir_changefid (fbuf *the_fbuf,
 		const VenusFid *file)
 {
     DirPage0 *page0;
+    unsigned npages;
+    size_t len = fbuf_len(the_fbuf);
     int ret;
 
     page0 = (DirPage0 *)fbuf_buf(the_fbuf);
     assert (page0);
-    ret = update_fid_by_name (page0, name, file);
+
+    npages = ntohs(page0->header.pg_pgcount);
+    if (npages < len / AFSDIR_PAGESIZE)
+	npages = len / AFSDIR_PAGESIZE;
+    
+    ret = update_fid_by_name (page0, name, file, npages);
 
     if (ret == 0)
 	return 0;
@@ -524,7 +630,8 @@ fdir_changefid (fbuf *the_fbuf,
 int
 fdir_mkdir (fbuf *the_fbuf,
 	    AFSFid dot,
-	    AFSFid dot_dot)
+	    AFSFid dot_dot,
+	    int utf8_dir)
 {
     DirPage0 *page0;
     DirPage1 *page;
@@ -533,7 +640,7 @@ fdir_mkdir (fbuf *the_fbuf,
     int tmp;
     int ret;
 
-    ret = create_new_page (&page, the_fbuf);
+    ret = create_new_page (&page, the_fbuf, utf8_dir);
     if (ret)
 	return ret;
 
@@ -541,16 +648,25 @@ fdir_mkdir (fbuf *the_fbuf,
     memset (&page0->dheader, 0, sizeof(page0->dheader));
     tmp = ENTRIESPERPAGE
 	- (sizeof(PageHeader) + sizeof(DirHeader)) / sizeof(DirEntry);
+    if (utf8_dir)
+	tmp -= 32;
     page0->header.pg_freecount = tmp;
     page0->dheader.map[0]      = tmp;
     page0->header.pg_pgcount   = htons(1);
 
+    /* set hashtable used */
     for (i = 0; i < 13; ++i)
 	set_used (page, i);
 
+    if (utf8_dir) {
+	/* set hashtable_utf used */
+	for (; i < 13 + 32; ++i)
+	    set_used (page, i);
+    }
+
     assert (page0->dheader.hash[hashentry(".")] == 0);
 
-    ind = add_to_page (page0, page, 0, ".", dot, 0);
+    ind = add_to_page (page0, page, 0, ".", NULL, dot, 0);
 
     assert (ind >= 0);
 
@@ -558,7 +674,7 @@ fdir_mkdir (fbuf *the_fbuf,
 
     assert (page0->dheader.hash[hashentry("..")] == 0);
 
-    ind = add_to_page (page0, page, 0, "..", dot_dot, 0);
+    ind = add_to_page (page0, page, 0, "..", NULL, dot_dot, 0);
 
     assert (ind >= 0);
 
@@ -574,46 +690,58 @@ fdir_mkdir (fbuf *the_fbuf,
 int
 fdir_creat (fbuf *dir,
 	    const char *name,
+	    const char *utf8name,
 	    AFSFid fid)
 {
     int ret;
     int i;
-    unsigned npages;
+    size_t npages;
     DirPage0 *page0;
     DirPage1 *page;
     int ind = 0;
     unsigned hash_value, next;
+    int utf8_dir, convert_dir = 0;
 
-    assert (dir->len != 0);
+    if (dir->len == 0)
+	return EINVAL;
 
     page0 = (DirPage0 *)fbuf_buf(dir);
     assert (page0);
-    npages = ntohs(page0->header.pg_pgcount);
 
+    npages = ntohs(page0->header.pg_pgcount);
     if (npages < fbuf_len(dir) / AFSDIR_PAGESIZE)
 	npages = fbuf_len(dir) / AFSDIR_PAGESIZE;
 
-    if (find_entry (page0, name))
+    if (find_entry (page0, name, npages))
 	return EEXIST;
 
+    if (page0->header.pg_tag == htons(AFSDIRMAGIC_UTF8))
+	utf8_dir = 1;
+    else if (utf8name) {
+	convert_dir = 1;
+	utf8_dir = 1;
+	utf8name = NULL;
+    } else
+	utf8_dir = 0;
+
     hash_value = hashentry (name);
     next = page0->dheader.hash[hash_value];
 
     for (i = 0; i < npages; ++i) {
 	page = getpage (page0, i);
-	ind = add_to_page (page0, page, i, name, fid, next);
+	ind = add_to_page (page0, page, i, name, utf8name, fid, next);
 	if (ind >= 0)
 	    break;
     }
     if (i == npages) {
-	ret = create_new_page (&page, dir);
+	ret = create_new_page (&page, dir, utf8_dir);
 	if (ret)
 	    return ret;
 	page0 = (DirPage0 *)fbuf_buf(dir);
 	page0->header.pg_pgcount = htons(npages + 1);
 	if (i < MAXPAGES)
 	    page0->dheader.map[i] = ENTRIESPERPAGE - 1;
-	ind = add_to_page (page0, page, i, name, fid, next);
+	ind = add_to_page (page0, page, i, name, utf8name, fid, next);
 	assert (ind >= 0);
     }
     ind += i * ENTRIESPERPAGE;
@@ -641,10 +769,11 @@ fdir_remove (fbuf *dir,
     DirEntry *prev_entry;
     unsigned pageno;
     int found;
-    unsigned npages;
+    size_t npages;
 
 
     page0 = (DirPage0 *)fbuf_buf(dir);
+
     npages = ntohs(page0->header.pg_pgcount);
     if (npages < len / AFSDIR_PAGESIZE)
 	npages = len / AFSDIR_PAGESIZE;
@@ -654,7 +783,9 @@ fdir_remove (fbuf *dir,
     found = i == 0;
     prev_entry = NULL;
     while (!found) {
-	entry = getentry (page0, i - 1);
+	entry = getentry (page0, i - 1, npages);
+	if (entry == NULL)
+	    return ENOENT;
 
 	if (strcmp (entry->name, name) == 0) {
 	    found = TRUE;
