@@ -35,7 +35,7 @@
  * Interface to directory handling routines
  */
 
-/* $Id: fdir.h,v 1.9 2003/11/04 00:43:37 lha Exp $ */
+/* $Id: fdir.h,v 1.11 2005/08/09 19:13:55 lha Exp $ */
 
 #ifndef _FDIR_H_
 #define _FDIR_H_
@@ -76,9 +76,7 @@ fdir_changefid (fbuf *the_fbuf,
 
 
 int
-fdir_creat (fbuf *dir,
-	    const char *filename,
-	    AFSFid fid);
+fdir_creat (fbuf *, const char *, const char *, AFSFid);
 
 int
 fdir_remove (fbuf *dir,
@@ -88,7 +86,8 @@ fdir_remove (fbuf *dir,
 int
 fdir_mkdir (fbuf *dir,
 	    AFSFid dot,
-	    AFSFid dot_dot);
+	    AFSFid dot_dot,
+	    int utf8_dir);
 
 int
 fdir_dirp (fbuf *);
