@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: ko.h,v 1.36 2002/12/06 05:00:04 lha Exp $ */
+/* $Id: ko.h,v 1.40 2005/03/28 18:32:31 lha Exp $ */
 
 #ifndef __KO_H
 #define __KO_H 1
@@ -48,7 +48,7 @@ typedef int32_t koerr_t;
  * Error messages
  */
 
-const char    *koerr_gettext(koerr_t err);
+const char    *koerr_gettext(koerr_t);
 
 /*
  * sysname
@@ -121,7 +121,7 @@ const char **  cell_thesecells (void);
 void 	       cell_print_cell (const cell_entry *c, FILE *out);
 void	       cell_status (FILE *f);
 time_t	       cell_get_poller_time(const cell_entry *c);
-void	       cell_set_poller_time(cell_entry *c, time_t time);
+void	       cell_set_poller_time(cell_entry *, time_t);
 
 
 /*
