@@ -33,7 +33,7 @@
 
 /*
  *  Private Include file for libko
- *  $Id: ko_locl.h,v 1.14 2002/07/24 07:03:42 lha Exp $
+ *  $Id: ko_locl.h,v 1.16 2006/03/22 16:07:16 tol Exp $
  */
 
 #ifndef _LIBKO_H
@@ -84,11 +84,19 @@
 
 #include "ko.h"
 
-/* resolver independant api for libko */
+/* resolver independent api for libko */
 enum { CELL_INVALID_HOST = 1 };
 
 void	 _ko_resolve_init(void);
 int	 _ko_resolve_cell(const char *, cell_db_entry *, int, int *, int *);
 int	 _ko_resolve_host(const char *, cell_db_entry *);
 
+/* internal, for testing */
+int
+_arla_getsysname_hint(char *buf, 
+		      size_t len,
+		      const char *cpu, 
+		      const char *vendor, 
+		      const char *os);
+
 #endif /* _LIBKO_H */
