@@ -37,7 +37,7 @@
 
 #include "ko_locl.h"
 
-RCSID("$Id: kocell.c,v 1.76 2003/10/03 17:26:00 map Exp $");
+RCSID("$Id: kocell.c,v 1.78 2005/10/28 14:33:38 tol Exp $");
 
 #define TRANSARCSYSCONFDIR "/usr/vice/etc"
 #define CELLFILENAME "CellServDB"
@@ -522,7 +522,13 @@ parse_simple_file (const char *filename, void (*func)(char *))
 	return 1;
     
     while ((fgets (line, sizeof(line), f) != NULL)) {
+        /* remove the end of line character */
 	line[strcspn(line, "\n")] = '\0';
+
+	/* Ignore lines with only whitespace */
+	if(strspn(line, "\r\t ") == strlen(line))
+	  continue;
+
 	(*func) (line);
     }
     fclose (f);
@@ -793,6 +799,9 @@ cell_init (int cellcachesize, Log_method *logm)
     cellaliaslist = listnew();
     if (readaliasdb (SYSCONFDIR "/" ALIASDBFILENAME))
 	readaliasdb (TRANSARCSYSCONFDIR "/" ALIASDBFILENAME);
+
+    if (cell_get_by_name(thiscell) == NULL)
+	errx(1, "Can't find your cell %s (ThisCell)", thiscell);
 }
 
 /*
