@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2003 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2002 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -31,35 +31,13 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: resolve.h,v 1.8 2003/12/06 17:03:46 lha Exp $ */
+/* $Id: resolve.h,v 1.13 2005/03/28 18:40:51 lha Exp $ */
 
 #ifndef __RESOLVE_H__
 #define __RESOLVE_H__
 
 /* We use these, but they are not always present in <arpa/nameser.h> */
 
-#ifndef C_IN
-#define C_IN		1
-#endif
-
-#ifndef T_A
-#define T_A		1
-#endif
-#ifndef T_NS
-#define T_NS		2
-#endif
-#ifndef T_CNAME
-#define T_CNAME		5
-#endif
-#ifndef T_SOA
-#define T_SOA		5
-#endif
-#ifndef T_PTR
-#define T_PTR		12
-#endif
-#ifndef T_MX
-#define T_MX		15
-#endif
 #ifndef T_TXT
 #define T_TXT		16
 #endif
@@ -85,10 +63,6 @@
 #define T_CERT		37
 #endif
 
-#ifndef MAXDNAME
-#define MAXDNAME	1025
-#endif
-
 #define dns_query		rk_dns_query
 #define mx_record		rk_mx_record
 #define srv_record		rk_srv_record
@@ -175,17 +149,8 @@ struct resource_record{
 typedef int HEADER; /* will never be used */
 #endif
 
-struct dns_header {
-    u_int16_t id;
-    u_int16_t misc;
-    u_int16_t qdcount;
-    u_int16_t ancount;
-    u_int16_t nscount;
-    u_int16_t arcount;
-};
-
 struct dns_reply{
-    struct dns_header h;
+    HEADER h;
     struct dns_query q;
     struct resource_record *head;
 };
