@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995-2002 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995-2003 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -33,7 +33,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: snprintf.c,v 1.11 2002/09/12 16:23:16 lha Exp $");
+RCSID("$Id: snprintf.c,v 1.14 2005/03/26 11:12:17 lha Exp $");
 #endif
 #include <stdio.h>
 #include <stdarg.h>
@@ -64,7 +64,7 @@ struct snprintf_state {
   /* XXX - methods */
 };
 
-#if TEST_SNPRINTF
+#if defined(TEST_SNPRINTF)
 #include "snprintf-test.h"
 #endif /* TEST_SNPRINTF */
 
@@ -338,6 +338,8 @@ xyzprintf (struct snprintf_state *state, const char *char_format, va_list ap)
 	  flags |= alternate_flag;
 	else if(c == '0')
 	  flags |= zero_flag;
+	else if(c == '\'')
+	    ; /* just ignore */
 	else
 	  break;
       }
