@@ -33,7 +33,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: strlwr.c,v 1.5 2002/09/12 16:23:25 lha Exp $");
+RCSID("$Id: strlwr.c,v 1.6 2004/08/07 13:33:34 tol Exp $");
 #endif
 #include <string.h>
 #include <ctype.h>
@@ -47,7 +47,7 @@ strlwr(char *str)
   char *s;
 
   for(s = str; *s; s++)
-    *s = tolower(*s);
+    *s = tolower((unsigned char)*s);
   return str;
 }
 #endif
