@@ -33,7 +33,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: sl.c,v 1.9 2003/02/01 19:19:01 lha Exp $");
+RCSID("$Id: sl.c,v 1.10 2005/11/25 13:03:16 tol Exp $");
 #endif
 
 #include "sl_locl.h"
@@ -308,7 +308,7 @@ sl_loop(SL_cmd *cmds, char *prompt)
 {
     void *data = NULL;
     int ret;
-    while((ret = sl_command_loop(cmds, prompt, &data)) == 0)
+    while((ret = sl_command_loop(cmds, prompt, &data)) >= 0)
 	;
     return ret;
 }
