@@ -42,7 +42,7 @@
 #include <lwp.h>
 #include "preempt.h"
 
-RCSID("$Id: plwp.c,v 1.25 2004/03/27 21:47:08 tol Exp $");
+RCSID("$Id: plwp.c,v 1.28 2005/11/25 10:50:20 haba Exp $");
 
 #ifdef	AFS_AIX32_ENV
 #include <ulimit.h>
@@ -321,8 +321,8 @@ LWP_GetRock(int Tag, char **Value)
 #elif WINDOWS_THREADS_LWP
 	    *Value = TlsGetValue((unsigned long) ra[i].val);
 #endif
-	    /**Value =  ra[i].value;*/
-	    return(LWP_SUCCESS);
+	    if (*Value != NULL)
+	      return(LWP_SUCCESS);
 	}
     return(LWP_EBADROCK);
 }
@@ -355,7 +355,7 @@ LWP_NewRock(int Tag, char *Value)
     /* insert new rock in rock list and increment count of rocks */
     if (lwp_cpptr->rused < MAXROCKS) {
         ra[lwp_cpptr->rused].tag = Tag;
-#if defined(LWP_THREADS)
+#ifdef PTHREADS_LWP
 #ifdef HAVE_PTHREAD_KEYCREATE	
         if (pthread_keycreate(&ra[lwp_cpptr->rused].val, NULL))
 #else
@@ -365,7 +365,7 @@ LWP_NewRock(int Tag, char *Value)
 	if (pthread_setspecific(ra[lwp_cpptr->rused].val, Value))
 	    return(LWP_EBADROCK);
 
-#elif defined(WINDOWS_THREADS_LWP)
+#elif WINDOWS_THREADS_LWP
 	ra[lwp_cpptr->rused].val = (LPVOID) TlsAlloc();
 	if (ra[lwp_cpptr->rused].val == (LPVOID) 0xFFFFFFFF)
 	    return(LWP_EBADROCK);
@@ -560,6 +560,10 @@ LWP_CreateProcess(void (*ep)(), int stacksize, int priority,
     
     Debug(0, ("Entered LWP_CreateProcess to create %s at priority %d\n", 
 	      name, priority));
+
+    if (stacksize < AFS_LWP_MINSTACKSIZE)
+	stacksize = AFS_LWP_MINSTACKSIZE;
+
     old_cpptr = lwp_cpptr;
     /* Throw away all dead process control blocks */
     purge_dead_pcbs();
@@ -627,7 +631,7 @@ LWP_CreateProcess(void (*ep)(), int stacksize, int priority,
 #if defined(PTHREADS_LWP)
 #if defined(HAVE_THR_YIELD)
 	thr_yield();
-#elif defined(_POSIX_THREAD_PRIORITY_SCHEDULING)
+#elif defined(HAVE_SCHED_YIELD)
 	sched_yield();
 #else
 	pthread_yield();
