@@ -33,7 +33,7 @@
 
 #include "fsrv_locl.h"
 
-RCSID("$Id: fileserver.c,v 1.30 2002/04/26 16:11:42 lha Exp $");
+RCSID("$Id: fileserver.c,v 1.31 2004/11/17 19:16:06 lha Exp $");
 
 typedef enum { NO_SALVAGE, NORMAL_SALVAGE, SALVAGE_ALL } salvage_options_e;
 
@@ -169,7 +169,7 @@ main(int argc, char **argv)
     Log_method *method;
     int salvage_options = NORMAL_SALVAGE;
     
-    set_progname (argv[0]);
+    setprogname (argv[0]);
 
     if (agetarg (args, argc, argv, &optind, AARG_GNUSTYLE)) {
 	usage (1);
