@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999 - 2003 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1999 - 2003, 2005 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -33,7 +33,7 @@
 
 #include "fsrv_locl.h"
 
-RCSID("$Id: fsprocs.c,v 1.69 2003/11/11 00:13:05 mattiasa Exp $");
+RCSID("$Id: fsprocs.c,v 1.71 2005/03/27 17:32:21 tol Exp $");
 
 #define GETHOST(call) rx_HostOf(rx_PeerOf(rx_ConnectionOf((call))))
 #define GETPORT(call) rx_PortOf(rx_PeerOf(rx_ConnectionOf((call))))
@@ -595,6 +595,52 @@ skipline(char **curptr)
     (*curptr)++;
 }
 
+/*
+ *
+ */
+
+int
+SRXAFS_FetchData64(struct rx_call *call,
+		   const struct AFSFid *fid,
+		   const int64_t pos,
+		   const int64_t length,
+		   struct AFSFetchStatus *outstatus,
+		   struct AFSCallBack *callback,
+		   struct AFSVolSync *sync)
+{
+    mlog_log (MDEBFS, "FetchData64: fid: %u.%u.%u", 
+	      fid->Volume, fid->Vnode, fid->Unique);
+    mlog_log (MDEBFS, "FetchData64: offset=%lld, len=%lld",
+	      pos, length);
+
+    return EPERM;
+}
+
+/*
+ *
+ */
+
+int
+SRXAFS_StoreData64(struct rx_call *call,
+		   const struct AFSFid *fid,
+		   const struct AFSStoreStatus *instatus,
+		   const uint64_t pos,
+		   const uint64_t length,
+		   const uint64_t filelength,
+		   struct AFSFetchStatus *outstatus,
+		   struct AFSVolSync *sync)
+{
+    mlog_log (MDEBFS, "StoreData64: fid: %u.%u.%u", 
+	      fid->Volume, fid->Vnode, fid->Unique);
+    mlog_log (MDEBFS, "StoreData64: "
+	      "offset=0x%llx, len=0x%llx, total=0x%llx",
+	      pos, length, filelength);
+    
+    return EPERM;
+}
+
+
+
 /*
  *
  */
