@@ -38,13 +38,19 @@
 #include <rx/rx_null.h>
 
 #ifdef KERBEROS
+
 #ifdef HAVE_OPENSSL
 #include <openssl/des.h>
 #else
 #include <des.h>
 #endif
+
+#ifdef HAVE_KRB5
 #include <krb5.h>
+#endif
+
 #include <rxkad.h>
+
 #endif
 
 #include <ko.h>
@@ -62,7 +68,7 @@
 #include <unistd.h>
 #endif
 
-RCSID("$Id: netinit.c,v 1.17 2004/03/27 19:27:32 tol Exp $");
+RCSID("$Id: netinit.c,v 1.19 2005/04/06 21:16:59 tol Exp $");
 
 /*
  * Network functions
@@ -84,7 +90,7 @@ network_kerberos_init (char *srvtab)
     }
 }
 
-#ifdef KERBEROS
+#ifdef HAVE_KRB4
 
 static char cell_name[REALM_SZ] = "";
 static char realm_name[REALM_SZ] = "";
@@ -121,10 +127,12 @@ server_get_key(void *appl, int kvno, des_cblock *key)
     memcpy (key, &afskey, sizeof (afskey));
     return 0;
 }
+#endif
 
 struct rx_securityClass *
 netinit_client_getcred(void)
 {
+#ifdef HAVE_KRB4
     int k_errno;
     CREDENTIALS c;
     struct rx_securityClass *sec;
@@ -155,8 +163,10 @@ netinit_client_getcred(void)
 					c.ticket_st.dat);
 
     return sec;
-}
+#else
+    return NULL;
 #endif
+}
 
 #define N_SECURITY_OBJECTS 3
 
@@ -184,7 +194,7 @@ network_init (int serviceport,
     if (secObjs[0] == NULL ) 
 	errx(1, "cant create security object") ;
 
-#ifdef KERBEROS
+#if defined(KERBEROS) && defined(HAVE_KRB4)
     strlcpy (cell_name, cell_getthiscell(), sizeof (cell_name));
 
     if (krbtkfile[0] == '\0') { 
@@ -236,5 +246,9 @@ network_init (int serviceport,
 
 char *
 netinit_getrealm(void) {
+#if defined(KERBEROS) && defined(HAVE_KRB4)
     return realm_name;
+#else
+    return NULL;
+#endif
 }
