@@ -33,7 +33,7 @@
 
 #include <sfvol_private.h>
 
-RCSID("$Id: common.c,v 1.3 2000/10/03 00:18:41 lha Exp $");
+RCSID("$Id: common.c,v 1.4 2005/05/03 18:20:04 lha Exp $");
 
 /*
  * Translate the `opaque' to the `ino'.
@@ -156,8 +156,8 @@ local_create_name (struct dp_part *dp, int32_t num, char *name, size_t sz)
 		  (unsigned int) (num >> 8) & 0xff,
 		  (unsigned int) num & 0xff);
     
-    if (i >= sz)
-	abort();
+    if (i == -1 || i >= sz)
+	return ENOMEM;
     return 0;
 }
 
@@ -170,15 +170,19 @@ local_create_file (struct dp_part *dp, onode_opaque *o,
     int fd, ret;
     struct stat sb;
     
-    snprintf(nodename, sizeof(nodename), "%s/inodeXXXXXX", DP_NAME(dp));
+    ret = snprintf(nodename, sizeof(nodename), "%s/inodeXXXXXX", DP_NAME(dp));
+    if (ret == -1 || ret >= sizeof(name)) {
+	ret = ENOMEM;
+	goto error;
+    }
     fd = mkstemp(nodename);
     if (fd == -1)
 	return errno;
     ret = fstat(fd, &sb);
     if (ret < 0) {
+	ret = errno;
 	close(fd);
-	unlink(nodename);
-	return errno;
+	goto error;
     }
 
     if (n) {
@@ -190,25 +194,44 @@ local_create_file (struct dp_part *dp, onode_opaque *o,
 	close(fd);
     }
 
-    snprintf(name, sizeof(name), "%s/%02x", DP_NAME(dp), 
-	     (unsigned int) (sb.st_ino >> 24) & 0xff);
+    ret = snprintf(name, sizeof(name), "%s/%02x", DP_NAME(dp), 
+		   (unsigned int) (sb.st_ino >> 24) & 0xff);
+    if (ret == -1 || ret >= sizeof(name)) {
+	ret = ENOMEM;
+	goto error;
+    }
     mkdir(name, 0700);
-    snprintf(name, sizeof(name), "%s/%02x/%02x", DP_NAME(dp), 
-	     (unsigned int) (sb.st_ino >> 24) & 0xff, 
-	     (unsigned int) (sb.st_ino >> 16) & 0xff);
+    ret = snprintf(name, sizeof(name), "%s/%02x/%02x", DP_NAME(dp), 
+		   (unsigned int) (sb.st_ino >> 24) & 0xff, 
+		   (unsigned int) (sb.st_ino >> 16) & 0xff);
+    if (ret == -1 || ret >= sizeof(name)) {
+	ret = ENOMEM;
+	goto error;
+    }
     mkdir(name, 0700);
-    snprintf(name, sizeof(name), "%s/%02x/%02x/%02x", DP_NAME(dp), 
-	     (unsigned int) (sb.st_ino >> 24) & 0xff, 
-	     (unsigned int) (sb.st_ino >> 16) & 0xff,
-	     (unsigned int) (sb.st_ino >> 8) & 0xff);
-    mkdir(name, 0700);
-    local_create_name (dp, sb.st_ino, name, sizeof(name));
+    ret = snprintf(name, sizeof(name), "%s/%02x/%02x/%02x", DP_NAME(dp), 
+		   (unsigned int) (sb.st_ino >> 24) & 0xff, 
+		   (unsigned int) (sb.st_ino >> 16) & 0xff,
+		   (unsigned int) (sb.st_ino >> 8) & 0xff);
+    if (ret == -1 || ret >= sizeof(name)) {
+	ret = ENOMEM;
+	goto error;
+    }
+    ret = mkdir(name, 0700);
+    if (ret) {
+	ret = errno;
+	goto error;
+    }
+    ret = local_create_name (dp, sb.st_ino, name, sizeof(name));
+    if (ret) {
+	unlink (nodename);
+	return ret;
+    }
 
     ret = rename(nodename, name);
     if (ret < 0) {
 	ret = errno;
-	unlink (nodename);
-	return ret;
+	goto error;
     }
 
     ret = local_ino2opaque (&sb.st_ino, o);
@@ -219,5 +242,9 @@ local_create_file (struct dp_part *dp, onode_opaque *o,
     }
 
     return 0;
+
+ error:
+    unlink(nodename);
+    return ret;
 }
 
