@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: vld.c,v 1.63 2003/02/15 14:57:32 map Exp $ */
+/* $Id: vld.c,v 1.66 2006/06/05 08:12:12 lha Exp $ */
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -178,7 +178,7 @@ vld_ref (volume_handle *vol)
 }
 
 /*
- * create a `vol' from the bootstrap infomation in `vs'.
+ * create a `vol' from the bootstrap information in `vs'.
  */
 
 static int
@@ -1403,10 +1403,10 @@ vld_dent_to_fetchstatus (struct volume_handle *vol,
     fs->ParentVnode	= e->ParentVnode;
     fs->ParentUnique	= e->ParentUnique;
     
-    fs->SegSize 	= e->SegSize;
+    fs->ResidencyMask 	= 1;
     fs->ClientModTime 	= e->ServerModTime;
     fs->ServerModTime	= e->ServerModTime;
-    fs->SyncCount 	= 0;
+    fs->SyncCounter 	= 0;
     fs->DataVersionHigh	= vol->info.creationDate;
     fs->LockCount	= 0;
     fs->LengthHigh	= 0;
@@ -1452,10 +1452,10 @@ vld_fent_to_fetchstatus (struct volume_handle *vol,
     fs->ParentVnode	= e->ParentVnode;
     fs->ParentUnique	= e->ParentUnique;
     
-    fs->SegSize 	= e->SegSize;
+    fs->ResidencyMask 	= 1;
     fs->ClientModTime 	= e->ServerModTime;
     fs->ServerModTime	= e->ServerModTime;
-    fs->SyncCount 	= 0;
+    fs->SyncCounter 	= 0;
     fs->DataVersionHigh	= vol->info.creationDate;
     fs->LockCount	= 0;
     fs->LengthHigh	= 0;
@@ -1698,7 +1698,7 @@ vld_check_rights (volume_handle *vol, struct mnode *n,
 }
 
 /*
- * open an mnode `n' in volume `vol' with `flags' in `m' as specifed above.
+ * open an mnode `n' in volume `vol' with `flags' in `m' as specified above.
  */
 
 int
