@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1999-2000, 2002, 2005 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -37,7 +37,7 @@
 
 #include "voldb_locl.h"
 
-RCSID("$Id: vol.c,v 1.7 2002/02/07 17:59:55 lha Exp $");
+RCSID("$Id: vol.c,v 1.10 2005/08/11 09:44:04 tol Exp $");
 
 /*
  * get the partial name of the volume
@@ -48,9 +48,10 @@ RCSID("$Id: vol.c,v 1.7 2002/02/07 17:59:55 lha Exp $");
 int
 vol_getname (uint32_t num, char *str, size_t sz)
 {
-    int i;
-    i = snprintf (str, sz, "vol%08u", num);
-    assert (i > 0);
+    int ret;
+    ret = snprintf (str, sz, "vol%08u", num);
+    if (ret < 0 || ret >= sz)
+	return ENOMEM;
     return 0;
 }
 
@@ -82,8 +83,9 @@ vol_getfullname (uint32_t part, uint32_t num, char *str, size_t sz)
 #endif
     else
 	return EINVAL; /* XXX */
-	
-    assert (ret > 0);
+    
+    if (ret < 0 || ret >= sz)
+	return ENOMEM;
     return 0;
 }
 
@@ -95,7 +97,8 @@ int
 vol_read_header (int fd, volintInfo *info)
 {
     struct stat sb;
-    char *buf, *ptr;
+    char *buf;
+    const char *ptr;
     size_t size;
     int ret;
 
