@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -37,7 +32,7 @@
  */
 
 /*
- * $Id: voldb_locl.h,v 1.1 1999/05/17 14:20:55 lha Exp $
+ * $Id: voldb_locl.h,v 1.4 2000/12/29 20:16:01 tol Exp $
  */
 
 #include <config.h>
@@ -60,7 +55,9 @@
 #include <err.h>
 
 #include <fs_def.h>
+#include <fs.h>
 #include <voldb.h>
+#include <dpart.h>
 
 #include <volumeserver.h>
 
