@@ -52,7 +52,12 @@
 #else
 #include <des.h>
 #endif
+#ifdef HAVE_KRB4
 #include <krb.h>
+#endif
+#ifdef HAVE_KRB5
+#include <krb5.h>
+#endif
 #include <rxkad.h>
 #include "rxkad_locl.h"
 #endif
@@ -75,14 +80,14 @@
 #include <mlog.h>
 #include <mdebug.h>
 
-RCSID("$Id: pr.c,v 1.25 2003/11/14 10:33:31 tol Exp $");
+RCSID("$Id: pr.c,v 1.28 2006/03/23 01:21:01 lha Exp $");
 
 /*
  *
  */
 
 int
-PR_NameToID(struct rx_call *call, const namelist *nlist, idlist *ilist)
+SPR_NameToID(struct rx_call *call, const namelist *nlist, idlist *ilist)
 {
     int i;
     int status;
@@ -121,7 +126,7 @@ PR_NameToID(struct rx_call *call, const namelist *nlist, idlist *ilist)
  */
 
 int
-PR_IDToName(struct rx_call *call, const idlist *ilist, namelist *nlist)
+SPR_IDToName(struct rx_call *call, const idlist *ilist, namelist *nlist)
 {
     int i;
     int status;
@@ -140,7 +145,7 @@ PR_IDToName(struct rx_call *call, const idlist *ilist, namelist *nlist)
 	return 0;
     }
 
-    nlist->val = calloc(1, sizeof(prname) * nlist->len);
+    nlist->val = calloc(nlist->len, sizeof(prname));
     if (nlist->val == NULL)
 	return PRDBBAD;
 
@@ -160,8 +165,8 @@ PR_IDToName(struct rx_call *call, const idlist *ilist, namelist *nlist)
  */
 
 int
-PR_NewEntry(struct rx_call *call, const char *name, 
-	    const int32_t flag, const int32_t oid, int32_t *id)
+SPR_NewEntry(struct rx_call *call, const char *name, 
+	     const int32_t flag, const int32_t oid, int32_t *id)
 {
     int error;
     int32_t owner;
@@ -204,8 +209,8 @@ PR_NewEntry(struct rx_call *call, const char *name,
  */
 
 int
-PR_INewEntry(struct rx_call *call, const char *name, 
-	     const int32_t id, const int32_t oid)
+SPR_INewEntry(struct rx_call *call, const char *name, 
+	      const int32_t id, const int32_t oid)
 {
     int error;
     char *localname;
@@ -241,8 +246,8 @@ PR_INewEntry(struct rx_call *call, const char *name,
  */
 
 int
-PR_ListEntry(struct rx_call *call, const int32_t id,
-	     struct prcheckentry *entry)
+SPR_ListEntry(struct rx_call *call, const int32_t id,
+	      struct prcheckentry *entry)
 {
     prentry pr_entry;
     int status;
@@ -280,8 +285,8 @@ PR_ListEntry(struct rx_call *call, const int32_t id,
  */
 
 int
-PR_DumpEntry(struct rx_call *call, const int32_t pos, 
-	     struct prdebugentry *entry)
+SPR_DumpEntry(struct rx_call *call, const int32_t pos, 
+	      struct prdebugentry *entry)
 {
     mlog_log (MDEBPR, "PR_DumpEntry");
     return -1;
@@ -292,8 +297,8 @@ PR_DumpEntry(struct rx_call *call, const int32_t pos,
  */
 
 int
-PR_ChangeEntry(struct rx_call *call, const int32_t id, const char *name,
-	       const int32_t oid, const int32_t newid)
+SPR_ChangeEntry(struct rx_call *call, const int32_t id, const char *name,
+		const int32_t oid, const int32_t newid)
 {
     mlog_log (MDEBPR, "PR_ChangeEntry");
     return -1;
@@ -305,10 +310,10 @@ PR_ChangeEntry(struct rx_call *call, const int32_t id, const char *name,
  */
 
 int
-PR_SetFieldsEntry(struct rx_call *call, const int32_t id, const int32_t mask,
-		  const int32_t flags, const int32_t ngroups, 
-		  const int32_t nusers,
-		  const int32_t spare1, const int32_t spare2)
+SPR_SetFieldsEntry(struct rx_call *call, const int32_t id, const int32_t mask,
+		   const int32_t flags, const int32_t ngroups, 
+		   const int32_t nusers,
+		   const int32_t spare1, const int32_t spare2)
 {
     mlog_log (MDEBPR, "PR_SetFieldsEntry");
     return -1;
@@ -320,7 +325,7 @@ PR_SetFieldsEntry(struct rx_call *call, const int32_t id, const int32_t mask,
  */
 
 int
-PR_Delete(struct rx_call *call, const int32_t id)
+SPR_Delete(struct rx_call *call, const int32_t id)
 {
     mlog_log (MDEBPR, "PR_Delete");
 
@@ -336,7 +341,7 @@ PR_Delete(struct rx_call *call, const int32_t id)
  */
 
 int
-PR_WhereIsIt(struct rx_call *call, const int32_t id, int32_t *ps)
+SPR_WhereIsIt(struct rx_call *call, const int32_t id, int32_t *ps)
 {
     mlog_log (MDEBPR, "PR_WhereIsIt");
     return -1;
@@ -348,12 +353,12 @@ PR_WhereIsIt(struct rx_call *call, const int32_t id, int32_t *ps)
  */
 
 int
-PR_AddToGroup(struct rx_call *call, const int32_t uid, const int32_t gid)
+SPR_AddToGroup(struct rx_call *call, const int32_t uid, const int32_t gid)
 {
     mlog_log (MDEBPR, "PR_AddToGroup");
 
     if (!sec_is_superuser(call))
-      return PRPERM;
+	return PRPERM;
 
     return addtogroup(uid,gid);
 }
@@ -364,7 +369,7 @@ PR_AddToGroup(struct rx_call *call, const int32_t uid, const int32_t gid)
  */
 
 int
-PR_RemoveFromGroup(struct rx_call *call, const int32_t id, const int32_t gid)
+SPR_RemoveFromGroup(struct rx_call *call, const int32_t id, const int32_t gid)
 {
     mlog_log (MDEBPR, "PR_RemoveFromGroup");
 
@@ -380,7 +385,7 @@ PR_RemoveFromGroup(struct rx_call *call, const int32_t id, const int32_t gid)
  */
 
 int
-PR_ListMax(struct rx_call *call, int32_t *uid, int32_t *gid)
+SPR_ListMax(struct rx_call *call, int32_t *uid, int32_t *gid)
 {
     mlog_log (MDEBPR, "PR_ListMax");
     *uid = pr_header.maxID;
@@ -394,7 +399,7 @@ PR_ListMax(struct rx_call *call, int32_t *uid, int32_t *gid)
  */
 
 int
-PR_SetMax(struct rx_call *call, const int32_t uid, const int32_t gflag)
+SPR_SetMax(struct rx_call *call, const int32_t uid, const int32_t gflag)
 {
     mlog_log (MDEBPR, "PR_SetMax");
 
@@ -413,8 +418,8 @@ PR_SetMax(struct rx_call *call, const int32_t uid, const int32_t gflag)
  */
 
 int
-PR_ListElements(struct rx_call *call, const int32_t id, 
-		prlist *elist, int32_t *over)
+SPR_ListElements(struct rx_call *call, const int32_t id, 
+		 prlist *elist, int32_t *over)
 {
     mlog_log (MDEBPR, "PR_ListElements");
 
@@ -427,22 +432,47 @@ PR_ListElements(struct rx_call *call, const int32_t id,
  */
 
 int
-PR_GetCPS(struct rx_call *call, const int32_t id, 
-	  prlist *elist, int32_t *over)
+SPR_GetCPS(struct rx_call *call, const int32_t id, 
+	   prlist *elist, int32_t *over)
 {
     mlog_log (MDEBPR, "PR_GetCPS");
 
     return listelements(id, elist, TRUE);
 }
 
+/*
+ *
+ */
+
+int
+SPR_GetCPS2(struct rx_call *call, const int32_t id,
+	    const int32_t host, prlist *elist, int32_t *over)
+{
+    mlog_log (MDEBPR, "PR_GetCPS2");
+    
+    return -1;
+}
 
 /*
  *
  */
 
 int
-PR_ListOwned(struct rx_call *call, const int32_t id, 
-	     prlist *elist, int32_t *over)
+SPR_GetHostCPS(struct rx_call *call, const int32_t host,
+	       prlist *elist, int32_t *over)
+{
+    mlog_log (MDEBPR, "PR_GetHostCPS");
+    
+    return -1;
+}
+
+/*
+ *
+ */
+
+int
+SPR_ListOwned(struct rx_call *call, const int32_t id, 
+	      prlist *elist, int32_t *over)
 {
     mlog_log (MDEBPR, "PR_ListOwned");
     return -1;
@@ -454,11 +484,11 @@ PR_ListOwned(struct rx_call *call, const int32_t id,
  */
 
 int
-PR_IsAMemberOf(struct rx_call *call, const int32_t uid, const int32_t gid,
-	       int32_t *flag)
+SPR_IsAMemberOf(struct rx_call *call, const int32_t uid, const int32_t gid,
+		int32_t *flag)
 {
 
-  /* XXX Check authorization */
+    /* XXX Check authorization */
 
     prlist elist;
     int ret=0;
