@@ -50,7 +50,12 @@
 #else
 #include <des.h>
 #endif
+#ifdef HAVE_KRB4
 #include <krb.h>
+#endif
+#ifdef HAVE_KRB5
+#include <krb5.h>
+#endif
 #include <rxkad.h>
 #endif
 
@@ -74,7 +79,7 @@
 #include "ptserver.h"
 #include "pts.ss.h"
 
-RCSID("$Id: ptserver.c,v 1.46 2003/04/09 02:41:28 lha Exp $");
+RCSID("$Id: ptserver.c,v 1.48 2005/01/17 09:12:45 lha Exp $");
 
 static struct rx_service *prservice;
 
@@ -808,7 +813,7 @@ main(int argc, char **argv)
     int ret;
     Log_method *method;
 
-    set_progname (argv[0]);
+    setprogname (argv[0]);
 
     if (agetarg (args, argc, argv, &optind, AARG_GNUSTYLE)) {
 	usage (1);
