@@ -33,21 +33,21 @@
 
 #include "vldb_locl.h"
 
-RCSID("$Id: vldbserver.c,v 1.46 2003/02/15 16:03:35 map Exp $");
+RCSID("$Id: vldbserver.c,v 1.48 2005/03/27 17:03:54 tol Exp $");
 
 /*
  * The rpc - calls
  */
 
 int
-VL_CreateEntry(struct rx_call *call, 
-	       const vldbentry *newentry) 
+SVL_CreateEntry(struct rx_call *call, 
+		const vldbentry *newentry) 
 {
     char *name;
     disk_vlentry diskentry;
     disk_vlentry tempentry;
 
-    mlog_log (MDEBVL, "VL_CreateEntry (name=%s, ids=%d,%d,%d flags=%d)\n",
+    mlog_log (MDEBVL, "SVL_CreateEntry (name=%s, ids=%d,%d,%d flags=%d)\n",
 	      newentry->name,
 	      newentry->volumeId[RWVOL],
 	      newentry->volumeId[ROVOL],
@@ -62,12 +62,12 @@ VL_CreateEntry(struct rx_call *call,
 	(vldb_id_to_name(newentry->volumeId[ROVOL], &name) == 0) ||
 	(vldb_id_to_name(newentry->volumeId[BACKVOL], &name) == 0)) {
 	free(name);
-	mlog_log (MDEBVL, "VL_CreateEntry: id exists\n");
+	mlog_log (MDEBVL, "SVL_CreateEntry: id exists\n");
 	return VL_NAMEEXIST;
     }
 
     if (vldb_read_entry(newentry->name, &tempentry) == 0) {
-	mlog_log (MDEBVL, "VL_CreateEntry: name exists\n");
+	mlog_log (MDEBVL, "SVL_CreateEntry: name exists\n");
 	return VL_NAMEEXIST;
     }
 
@@ -96,15 +96,15 @@ VL_CreateEntry(struct rx_call *call,
 }
 
 int 
-VL_DeleteEntry(struct rx_call *call, 
-	       const int32_t Volid, 
-	       const int32_t voltype)
+SVL_DeleteEntry(struct rx_call *call, 
+		const int32_t Volid, 
+		const int32_t voltype)
 {
     disk_vlentry entry;
     char *name;
     int ret;
 
-    mlog_log (MDEBVL, "VL_DeleteEntry (Volid=%d,Voltype=%d)\n", 
+    mlog_log (MDEBVL, "SVL_DeleteEntry (Volid=%d,Voltype=%d)\n", 
 		  Volid, voltype);
 
     if (!sec_is_superuser(call)) {
@@ -135,25 +135,25 @@ VL_DeleteEntry(struct rx_call *call,
     }
     
     if (vldb_delete_id(name, entry.volumeId[RWVOL])) {
-	mlog_log (MDEBVL, "VL_DeleteEntry failed to remove RW id %d\n",
+	mlog_log (MDEBVL, "SVL_DeleteEntry failed to remove RW id %d\n",
 		  entry.volumeId[RWVOL]);
 	ret =  VL_IO;
 	goto out;
     }
     if (vldb_delete_id(name, entry.volumeId[ROVOL])) {
-	mlog_log (MDEBVL, "VL_DeleteEntry failed to remove RO id %d\n",
+	mlog_log (MDEBVL, "SVL_DeleteEntry failed to remove RO id %d\n",
 		  entry.volumeId[ROVOL]);
 	ret =  VL_IO;
 	goto out;
     }
     if (vldb_delete_id(name, entry.volumeId[BACKVOL])) {
-	mlog_log (MDEBVL, "VL_DeleteEntry failed to remove BK id %d\n",
+	mlog_log (MDEBVL, "SVL_DeleteEntry failed to remove BK id %d\n",
 		  entry.volumeId[BACKVOL]);
 	ret =  VL_IO;
 	goto out;
     }
     if (vldb_delete_entry(name)) {
-	mlog_log (MDEBVL, "VL_DeleteEntry failed to remove data\n");
+	mlog_log (MDEBVL, "SVL_DeleteEntry failed to remove data\n");
 	ret =  VL_IO;
 	goto out;
     }
@@ -165,7 +165,7 @@ VL_DeleteEntry(struct rx_call *call,
     ret = 0;
     
  out:
-    mlog_log (MDEBVL, "VL_DeleteEntry returns %d\n", ret);
+    mlog_log (MDEBVL, "SVL_DeleteEntry returns %d\n", ret);
 
     return ret;
 }
@@ -175,14 +175,14 @@ VL_DeleteEntry(struct rx_call *call,
  */
 
 int
-VL_GetEntryByID(struct rx_call *call,
-		const int32_t Volid, 
-		const int32_t voltype, 
-		vldbentry *entry) 
+SVL_GetEntryByID(struct rx_call *call,
+		 const int32_t Volid, 
+		 const int32_t voltype, 
+		 vldbentry *entry) 
 {
     disk_vlentry diskentry;
     char *name;
-    mlog_log (MDEBVL, "VL_GetEntryByID (Volid=%d,Voltype=%d)\n", 
+    mlog_log (MDEBVL, "SVL_GetEntryByID (Volid=%d,Voltype=%d)\n", 
 		  Volid, voltype);
 
     if (vldb_id_to_name(Volid, &name))
@@ -203,17 +203,17 @@ VL_GetEntryByID(struct rx_call *call,
  */
 
 int
-VL_GetEntryByName(struct rx_call *call, 
-		  const char *volumename, 
-		  vldbentry *entry) 
+SVL_GetEntryByName(struct rx_call *call, 
+		   const char *volumename, 
+		   vldbentry *entry) 
 {
     disk_vlentry diskentry;
 
-    mlog_log (MDEBVL, "VL_GetEntryByName (volumename = %s)\n", 
+    mlog_log (MDEBVL, "SVL_GetEntryByName (volumename = %s)\n", 
 		  volumename);
 
     if (isdigit(volumename[0])) {
-	return VL_GetEntryByID(call, atol(volumename), 0 /* XXX */, entry);
+	return SVL_GetEntryByID(call, atol(volumename), 0 /* XXX */, entry);
     }
 
     if (vldb_read_entry(volumename, &diskentry) != 0)
@@ -229,11 +229,11 @@ VL_GetEntryByName(struct rx_call *call,
  */
 
 int 
-VL_GetNewVolumeId (struct rx_call *call, 
-		   const int32_t bumpcount,
-		   int32_t *newvolumid)
+SVL_GetNewVolumeId (struct rx_call *call, 
+		    const int32_t bumpcount,
+		    int32_t *newvolumid)
 {
-    mlog_log (MDEBVL, "VL_GetNewVolumeId(bumpcount=%d)\n", bumpcount) ;
+    mlog_log (MDEBVL, "SVL_GetNewVolumeId(bumpcount=%d)\n", bumpcount) ;
     
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -253,17 +253,88 @@ VL_GetNewVolumeId (struct rx_call *call,
  */
 
 int
-VL_ReplaceEntry (struct rx_call *call, 
+SVL_ReplaceEntry (struct rx_call *call, 
+		  const int32_t Volid, 
+		  const int32_t voltype,
+		  const vldbentry *newentry,
+		  const int32_t ReleaseType) 
+{
+    mlog_log (MDEBVL, "SVL_ReplaceEntry\n") ;
+
+    if (!sec_is_superuser(call))
+	return VL_PERM;
+
+    return VL_PERM ;
+}
+
+/*
+ *
+ */
+
+int
+SVL_UpdateEntry (struct rx_call *call, 
 		 const int32_t Volid, 
-		 const int32_t voltype,
-		 const vldbentry *newentry,
+		 const int32_t voltype, 
+		 const VldbUpdateEntry *UpdateEntry,
+		 const int32_t ReleaseType)
+{
+    mlog_log (MDEBVL, "SVL_UpdateEntry\n") ;
+
+    if (!sec_is_superuser(call))
+	return VL_PERM;
+
+    return VL_PERM ;
+}
+
+/*
+ *
+ */
+
+int 
+SVL_SetLock (struct rx_call *call, 
+	     const int32_t Volid, 
+	     const int32_t voltype,
+	     const int32_t voloper) 
+{
+    mlog_log (MDEBVL, "SVL_SetLock\n") ;
+
+    if (!sec_is_superuser(call))
+	return VL_PERM;
+
+    return 0;
+}
+
+/*
+ *
+ */
+
+int
+SVL_ReleaseLock (struct rx_call *call, 
+		 const int32_t volid,
+		 const int32_t voltype, 
 		 const int32_t ReleaseType) 
 {
-    mlog_log (MDEBVL, "VL_ReplaceEntry\n") ;
+    mlog_log (MDEBVL, "SVL_ReleaseLock\n") ;
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
 
+
+    return 0;
+}
+
+/*
+ *
+ */
+
+int
+SVL_ListEntry (struct rx_call *call, 
+	       const int32_t previous_index, 
+	       int32_t *count,
+	       int32_t *next_index, 
+	       vldbentry *entry) 
+{
+    mlog_log (MDEBVL, "SVL_ListEntry\n") ;
     return VL_PERM ;
 }
 
@@ -272,17 +343,25 @@ VL_ReplaceEntry (struct rx_call *call,
  */
 
 int
-VL_UpdateEntry (struct rx_call *call, 
-		const int32_t Volid, 
-		const int32_t voltype, 
-		const VldbUpdateEntry *UpdateEntry,
-		const int32_t ReleaseType)
+SVL_ListAttributes (struct rx_call *call, 
+		    const VldbListByAttributes *attributes,
+		    int32_t *nentries,
+		    bulkentries *blkentries) 
 {
-    mlog_log (MDEBVL, "VL_UpdateEntry\n") ;
+    mlog_log (MDEBVL, "SVL_ListAttributes\n") ;
+    return VL_PERM ;
+}
 
-    if (!sec_is_superuser(call))
-	return VL_PERM;
+/*
+ *
+ */
 
+int
+SVL_GetStats (struct rx_call *call, 
+	      vldstats *stats,
+	      vital_vlheader *vital_header) 
+{
+    mlog_log (MDEBVL, "SVL_GetStats") ;
     return VL_PERM ;
 }
 
@@ -291,16 +370,9 @@ VL_UpdateEntry (struct rx_call *call,
  */
 
 int 
-VL_SetLock (struct rx_call *call, 
-	    const int32_t Volid, 
-	    const int32_t voltype,
-	    const int32_t voloper) 
+SVL_Probe(struct rx_call *call)
 {
-    mlog_log (MDEBVL, "VL_SetLock\n") ;
-
-    if (!sec_is_superuser(call))
-	return VL_PERM;
-
+    mlog_log (MDEBVL, "SVL_Probe\n") ;
     return 0;
 }
 
@@ -309,86 +381,14 @@ VL_SetLock (struct rx_call *call,
  */
 
 int
-VL_ReleaseLock (struct rx_call *call, 
-		const int32_t volid,
-		const int32_t voltype, 
-		const int32_t ReleaseType) 
-{
-    mlog_log (MDEBVL, "VL_ReleaseLock\n") ;
-
-    if (!sec_is_superuser(call))
-	return VL_PERM;
-
-
-    return 0;
-}
-
-/*
- *
- */
-
-int
-VL_ListEntry (struct rx_call *call, 
-	      const int32_t previous_index, 
-	      int32_t *count,
-	      int32_t *next_index, 
-	      vldbentry *entry) 
-{
-    mlog_log (MDEBVL, "VL_ListEntry\n") ;
-    return VL_PERM ;
-}
-
-/*
- *
- */
-
-int
-VL_ListAttributes (struct rx_call *call, 
-		   const VldbListByAttributes *attributes,
-		   int32_t *nentries,
-		   bulkentries *blkentries) 
-{
-    mlog_log (MDEBVL, "VL_ListAttributes\n") ;
-    return VL_PERM ;
-}
-
-/*
- *
- */
-
-int
-VL_GetStats (struct rx_call *call, 
-	     vldstats *stats,
-	     vital_vlheader *vital_header) 
-{
-    mlog_log (MDEBVL, "VL_GetStats") ;
-    return VL_PERM ;
-}
-
-/*
- *
- */
-
-int 
-VL_Probe(struct rx_call *call)
-{
-    mlog_log (MDEBVL, "VL_Probe\n") ;
-    return 0;
-}
-
-/*
- *
- */
-
-int
-VL_CreateEntryN(struct rx_call *call,
-		const nvldbentry *entry)
+SVL_CreateEntryN(struct rx_call *call,
+		 const nvldbentry *entry)
 {
     char *name;
     disk_vlentry diskentry;
     disk_vlentry tempentry;
 
-    mlog_log (MDEBVL, "VL_CreateEntryN (name=%s, ids=%d,%d,%d flags=%d)\n",
+    mlog_log (MDEBVL, "SVL_CreateEntryN (name=%s, ids=%d,%d,%d flags=%d)\n",
 	      entry->name,
 	      entry->volumeId[RWVOL],
 	      entry->volumeId[ROVOL],
@@ -403,12 +403,12 @@ VL_CreateEntryN(struct rx_call *call,
 	(vldb_id_to_name(entry->volumeId[ROVOL], &name) == 0) ||
 	(vldb_id_to_name(entry->volumeId[BACKVOL], &name) == 0)) {
 	free(name);
-	mlog_log (MDEBVL, "VL_CreateEntryN: id exists\n");
+	mlog_log (MDEBVL, "SVL_CreateEntryN: id exists\n");
 	return VL_NAMEEXIST;
     }
 
     if (vldb_read_entry(entry->name, &tempentry) == 0) {
-	mlog_log (MDEBVL, "VL_CreateEntryN: name exists\n");
+	mlog_log (MDEBVL, "SVL_CreateEntryN: name exists\n");
 	return VL_NAMEEXIST;
     }
 
@@ -441,14 +441,14 @@ VL_CreateEntryN(struct rx_call *call,
  */
 
 int
-VL_GetEntryByIDN(struct rx_call *call,
-		 const int32_t Volid,
-		 const int32_t voltype,
-		 nvldbentry *entry)
+SVL_GetEntryByIDN(struct rx_call *call,
+		  const int32_t Volid,
+		  const int32_t voltype,
+		  nvldbentry *entry)
 {
     disk_vlentry diskentry;
     char *name;
-    mlog_log (MDEBVL, "VL_GetEntryByIDN (Volid=%d,Voltype=%d)\n", 
+    mlog_log (MDEBVL, "SVL_GetEntryByIDN (Volid=%d,Voltype=%d)\n", 
 	      Volid, voltype);
 
     if (vldb_id_to_name(Volid, &name))
@@ -469,17 +469,17 @@ VL_GetEntryByIDN(struct rx_call *call,
  */
 
 int
-VL_GetEntryByNameN(struct rx_call *call, 
-		   const char *volumename, 
-		   nvldbentry *entry) 
+SVL_GetEntryByNameN(struct rx_call *call, 
+		    const char *volumename, 
+		    nvldbentry *entry) 
 {
     disk_vlentry diskentry;
 
-    mlog_log (MDEBVL, "VL_GetEntryByNameN (volumename = %s)\n", 
+    mlog_log (MDEBVL, "SVL_GetEntryByNameN (volumename = %s)\n", 
 	      volumename);
 
     if (isdigit(volumename[0])) {
-	return VL_GetEntryByIDN(call, atol(volumename), 0 /* XXX */, entry);
+	return SVL_GetEntryByIDN(call, atol(volumename), 0 /* XXX */, entry);
     }
 
     if (vldb_read_entry(volumename, &diskentry) != 0)
@@ -496,11 +496,11 @@ VL_GetEntryByNameN(struct rx_call *call,
  */
 
 int
-VL_GetEntryByNameU(struct rx_call *call, 
-		   const char *volumename, 
-		   uvldbentry *entry) 
+SVL_GetEntryByNameU(struct rx_call *call, 
+		    const char *volumename, 
+		    uvldbentry *entry) 
 {
-    mlog_log (MDEBVL, "VL_GetEntryByNameU %s\n", volumename);
+    mlog_log (MDEBVL, "SVL_GetEntryByNameU %s\n", volumename);
     memset(entry, 0, sizeof(*entry));
     
     return RXGEN_OPCODE;
@@ -512,12 +512,12 @@ VL_GetEntryByNameU(struct rx_call *call,
  */
 
 int
-VL_ListAttributesN (struct rx_call *call, 
-		   const VldbListByAttributes *attributes,
-		   int32_t *nentries,
-		   nbulkentries *blkentries) 
+SVL_ListAttributesN (struct rx_call *call, 
+		     const VldbListByAttributes *attributes,
+		     int32_t *nentries,
+		     nbulkentries *blkentries) 
 {
-    mlog_log (MDEBVL, "VL_ListAttributesN\n");
+    mlog_log (MDEBVL, "SVL_ListAttributesN\n");
     mlog_log (MDEBVL, "  attributes: Mask=(%d=", attributes->Mask);
 
     if (attributes->Mask & VLLIST_SERVER)
@@ -550,12 +550,12 @@ VL_ListAttributesN (struct rx_call *call,
  */
 
 int
-VL_ListAttributesU(struct rx_call *call, 
-		   const VldbListByAttributes *attributes,
-		   int32_t *nentries,
-		   ubulkentries *blkentries) 
+SVL_ListAttributesU(struct rx_call *call, 
+		    const VldbListByAttributes *attributes,
+		    int32_t *nentries,
+		    ubulkentries *blkentries) 
 {
-    mlog_log (MDEBVL, "VL_ListAttributesU\n") ;
+    mlog_log (MDEBVL, "SVL_ListAttributesU\n") ;
     *nentries = 0;
     blkentries->len = 0;
     blkentries->val = NULL;
@@ -568,12 +568,12 @@ VL_ListAttributesU(struct rx_call *call,
  */
 
 int
-VL_UpdateEntryByName(struct rx_call *call,
-		     const char volname[65],
-		     const struct VldbUpdateEntry *UpdateEntry,
-		     const int32_t ReleaseType)
+SVL_UpdateEntryByName(struct rx_call *call,
+		      const char volname[65],
+		      const struct VldbUpdateEntry *UpdateEntry,
+		      const int32_t ReleaseType)
 {
-    mlog_log (MDEBVL, "VL_UpdateEntryByName (not implemented)\n");
+    mlog_log (MDEBVL, "SVL_UpdateEntryByName (not implemented)\n");
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -586,14 +586,14 @@ VL_UpdateEntryByName(struct rx_call *call,
  */
 
 int
-VL_GetAddrsU(struct rx_call *call,
-	     const struct ListAddrByAttributes *inaddr,
-	     struct afsUUID *uuid,
-	     int32_t *uniq,
-	     int32_t *nentries,
-	     bulkaddrs *addrs)
+SVL_GetAddrsU(struct rx_call *call,
+	      const struct ListAddrByAttributes *inaddr,
+	      struct afsUUID *uuid,
+	      int32_t *uniq,
+	      int32_t *nentries,
+	      bulkaddrs *addrs)
 {
-    mlog_log (MDEBVL, "VL_GetAddrsU (not implemented)\n");
+    mlog_log (MDEBVL, "SVL_GetAddrsU (not implemented)\n");
     return RXGEN_OPCODE;
 }
 
@@ -602,12 +602,12 @@ VL_GetAddrsU(struct rx_call *call,
  */
 
 int
-VL_RegisterAddrs(struct rx_call *call,
-		 const struct afsUUID *uid,
-		 const int32_t spare,
-		 const bulkaddrs *addrs)
+SVL_RegisterAddrs(struct rx_call *call,
+		  const struct afsUUID *uid,
+		  const int32_t spare,
+		  const bulkaddrs *addrs)
 {
-    mlog_log (MDEBVL, "VL_RegistersAddrs (not implemented)\n");
+    mlog_log (MDEBVL, "SVL_RegistersAddrs (not implemented)\n");
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -621,10 +621,10 @@ VL_RegisterAddrs(struct rx_call *call,
  */
 
 int
-VL_CreateEntryU(struct rx_call *call,
-		const struct uvldbentry *newentry)
+SVL_CreateEntryU(struct rx_call *call,
+		 const struct uvldbentry *newentry)
 {
-    mlog_log (MDEBVL, "VL_CreateEntryU (not implemented)\n");
+    mlog_log (MDEBVL, "SVL_CreateEntryU (not implemented)\n");
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -639,9 +639,9 @@ VL_CreateEntryU(struct rx_call *call,
  */
 
 int
-VL_ReplaceEntryU(struct rx_call *call)
+SVL_ReplaceEntryU(struct rx_call *call)
 {
-    mlog_log (MDEBVL, "VL_ReplaceEntryU (not implemented)\n");
+    mlog_log (MDEBVL, "SVL_ReplaceEntryU (not implemented)\n");
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -655,13 +655,13 @@ VL_ReplaceEntryU(struct rx_call *call)
  */
 
 int
-VL_ReplaceEntryN(struct rx_call *call,
-		 const int32_t Volid,
-		 const int32_t voltype,
-		 const struct nvldbentry *newentry,
-		 const int32_t ReleaseType)
+SVL_ReplaceEntryN(struct rx_call *call,
+		  const int32_t Volid,
+		  const int32_t voltype,
+		  const struct nvldbentry *newentry,
+		  const int32_t ReleaseType)
 {
-    mlog_log (MDEBVL, "VL_ReplaceEntryN (not implemented)\n");
+    mlog_log (MDEBVL, "SVL_ReplaceEntryN (not implemented)\n");
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -674,11 +674,11 @@ VL_ReplaceEntryN(struct rx_call *call,
  */
 
 int
-VL_ChangeAddrs(struct rx_call *call,
-	       const int32_t old_ip,
-	       const int32_t new_ip)
+SVL_ChangeAddrs(struct rx_call *call,
+		const int32_t old_ip,
+		const int32_t new_ip)
 {
-    mlog_log (MDEBVL, "VL_ChangeAddrs (not implemented)\n");
+    mlog_log (MDEBVL, "SVL_ChangeAddrs (not implemented)\n");
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -692,9 +692,9 @@ VL_ChangeAddrs(struct rx_call *call,
  */
 
 int
-VL_GetEntryByIDU(struct rx_call *call)
+SVL_GetEntryByIDU(struct rx_call *call)
 {
-    mlog_log (MDEBVL, "VL_GetEntryByIDU (not implemented)\n");
+    mlog_log (MDEBVL, "SVL_GetEntryByIDU (not implemented)\n");
     return VL_PERM;
 }
 #endif
@@ -704,13 +704,13 @@ VL_GetEntryByIDU(struct rx_call *call)
  */
 
 int
-VL_ListEntryN(struct rx_call *call,
-	      int32_t previous_index,
-	      int32_t *count,
-	      int32_t *next_index,
-	      nvldbentry *entry)
+SVL_ListEntryN(struct rx_call *call,
+	       int32_t previous_index,
+	       int32_t *count,
+	       int32_t *next_index,
+	       nvldbentry *entry)
 {
-    mlog_log (MDEBVL, "VL_ListEntryN (not implemented)\n");
+    mlog_log (MDEBVL, "SVL_ListEntryN (not implemented)\n");
     return VL_PERM;
 }
 
@@ -720,9 +720,9 @@ VL_ListEntryN(struct rx_call *call,
  */
 
 int
-VL_ListEntryU(struct rx_call *call)
+SVL_ListEntryU(struct rx_call *call)
 {
-    mlog_log (MDEBVL, "VL_ListEntryU (not implemented)\n");
+    mlog_log (MDEBVL, "SVL_ListEntryU (not implemented)\n");
     return VL_PERM;
 }
 #endif
@@ -732,14 +732,14 @@ VL_ListEntryU(struct rx_call *call)
  */
 
 int
-VL_GetAddrs(struct rx_call *call,
-	    const int32_t handle,
-	    const int32_t spare,
-	    struct VL_Callback *spare3,
-	    int32_t *nentries,
-	    bulkaddrs *blkaddr)
+SVL_GetAddrs(struct rx_call *call,
+	     const int32_t handle,
+	     const int32_t spare,
+	     struct VL_Callback *spare3,
+	     int32_t *nentries,
+	     bulkaddrs *blkaddr)
 {
-    mlog_log (MDEBVL, "VL_GetAddrs (not implemented)\n");
+    mlog_log (MDEBVL, "SVL_GetAddrs (not implemented)\n");
     return VL_PERM;
 }
 
@@ -749,9 +749,9 @@ VL_GetAddrs(struct rx_call *call,
  */
 
 int
-VL_LinkedListN(struct rx_call *call)
+SVL_LinkedListN(struct rx_call *call)
 {
-    mlog_log (MDEBVL, "VL_LinkedListN (not implemented)\n");
+    mlog_log (MDEBVL, "SVL_LinkedListN (not implemented)\n");
     return VL_PERM;
 }
 
@@ -760,23 +760,23 @@ VL_LinkedListN(struct rx_call *call)
  */
 
 int
-VL_LinkedListU(struct rx_call *call)
+SVL_LinkedListU(struct rx_call *call)
 {
-    mlog_log (MDEBVL, "VL_LinkedListU (not implemented)\n");
+    mlog_log (MDEBVL, "SVL_LinkedListU (not implemented)\n");
     return VL_PERM;
 }
 #endif
 
 int
-VL_ListAttributesN2(struct rx_call *call,
-		    const struct VldbListByAttributes *attributes,
-		    const char *volumename,
-		    const int32_t startindex,
-		    int32_t *nentries,
-		    nbulkentries *blkentries,
-		    int32_t *nextstartindex)
+SVL_ListAttributesN2(struct rx_call *call,
+		     const struct VldbListByAttributes *attributes,
+		     const char *volumename,
+		     const int32_t startindex,
+		     int32_t *nentries,
+		     nbulkentries *blkentries,
+		     int32_t *nextstartindex)
 {
-    mlog_log (MDEBVL, "VL_ListAttributesN2 (not implemented)\n");
+    mlog_log (MDEBVL, "SVL_ListAttributesN2 (not implemented)\n");
     return VL_PERM;
 }
 /*
@@ -824,7 +824,7 @@ main(int argc, char **argv)
     int optind = 0;
     int ret;
     
-    set_progname (argv[0]);
+    setprogname (argv[0]);
 
     if (agetarg (args, argc, argv, &optind, AARG_GNUSTYLE)) {
 	usage (1);
