@@ -39,7 +39,7 @@
 #include <nnpfs/nnpfs_deb.h>
 #include <nnpfs/nnnpfs.h>
 
-RCSID("$Id: nnpfs_dev.c,v 1.8 2002/09/07 10:44:31 lha Exp $");
+RCSID("$Id: nnpfs_dev.c,v 1.9 2004/06/13 15:02:55 lha Exp $");
 
 /*
  * Queues of nnpfs_links hold outbound messages and processes sleeping
@@ -143,7 +143,7 @@ nnpfs_devopen(dev_t dev,
   else
     chan->status |= CHANNEL_OPENED;
 
-  chan->message_buffer = nnpfs_alloc(MAX_XMSG_SIZE);
+  chan->message_buffer = nnpfs_alloc(NNPFS_MAX_MSG_SIZE);
 
   ASSERT(chan->message_buffer != NULL);
 
@@ -207,7 +207,7 @@ nnpfs_devclose(dev_t dev, chan_t foo_chan)
   }
   
   if (chan->message_buffer) {
-      nnpfs_free(chan->message_buffer, MAX_XMSG_SIZE);
+      nnpfs_free(chan->message_buffer, NNPFS_MAX_MSG_SIZE);
       chan->message_buffer = 0;
   }
       
@@ -271,7 +271,7 @@ nnpfs_devwrite(dev_t dev, struct uio *uiop, chan_t foo_chan, int ext)
   chan = &nnpfs_channel[minor(dev)];
 
   cnt = uiop->uio_resid;
-  error = uiomove((caddr_t) chan->message_buffer, MAX_XMSG_SIZE,
+  error = uiomove((caddr_t) chan->message_buffer, NNPFS_MAX_MSG_SIZE,
 		  UIO_WRITE, uiop);
   if (error != 0)
     return error;
