@@ -43,7 +43,7 @@
 #include <nnpfs/nnpfs_deb.h>
 #include <nnpfs/nnpfs_syscalls.h>
 
-RCSID("$Id: nnpfs_vnodeops.c,v 1.9 2002/09/07 10:44:38 lha Exp $");
+RCSID("$Id: nnpfs_vnodeops.c,v 1.10 2004/06/13 15:03:06 lha Exp $");
 
 static int
 nnpfs_open_valid(struct vnode *vp, struct ucred *cred, u_int tok)
@@ -417,7 +417,7 @@ nnpfs_access(struct vnode *vp,
 	
 	  error = EACCES;
 	
-	  for (i = 0; i < MAXRIGHTS; i++)
+	  for (i = 0; i < NNPFS_MAXRIGHTS; i++)
 	      if (xn->id[i] == pag) {
 		  error = check_rights (xn->rights[i], mode);
 		  break;
