@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2004 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2006 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -39,10 +39,14 @@
 #include <nnpfs/nnpfs_dev.h>
 #include <nnpfs/nnpfs_deb.h>
 
-RCSID("$Id: nnpfs_dev-common.c,v 1.67 2004/06/01 19:00:54 tol Exp $");
+RCSID("$Id: nnpfs_dev-common.c,v 1.76 2006/05/12 08:41:45 lha Exp $");
 
 struct nnpfs nnpfs_dev[NNNPFS];
 
+/*
+ * queue handling
+ */
+
 void
 nnpfs_initq(struct nnpfs_link *q)
 {
@@ -58,14 +62,14 @@ nnpfs_emptyq(const struct nnpfs_link *q)
 }
 
 /* Is this link on any queue? Link *must* be inited! */
-int
+static int
 nnpfs_onq(const struct nnpfs_link *link)
 {
     return link->next != NULL || link->prev != NULL;
 }
 
 /* Append q with p */
-void
+static void
 nnpfs_appendq(struct nnpfs_link *q, struct nnpfs_link *p)
 {
     p->next = q;
@@ -75,7 +79,7 @@ nnpfs_appendq(struct nnpfs_link *q, struct nnpfs_link *p)
 }
 
 /* remove `p' from its queue */
-void
+static void
 nnpfs_outq(struct nnpfs_link *p)
 {
     p->next->prev = p->prev;
@@ -87,7 +91,7 @@ nnpfs_outq(struct nnpfs_link *p)
  * Only allow one open.
  */
 int
-nnpfs_devopen_common(dev_t dev)
+nnpfs_devopen_common(nnpfs_dev_t dev)
 {
     struct nnpfs *chan;
 
@@ -96,32 +100,38 @@ nnpfs_devopen_common(dev_t dev)
 
     chan = &nnpfs_dev[minor(dev)];
 
+    nnpfs_dev_lock(chan);
+
     /* Only allow one reader/writer */
     if (chan->status & CHANNEL_OPENED) {
+	nnpfs_dev_unlock(chan);
 	NNPFSDEB(XDEBDEV, ("nnpfs_devopen: already open\n"));
 	return EBUSY;
-    } else {
-	chan->status |= CHANNEL_OPENED;
     }
+    chan->status |= CHANNEL_OPENED;
 
-    chan->message_buffer = nnpfs_alloc(MAX_XMSG_SIZE, M_NNPFS_MSG);
+    chan->message_buffer = nnpfs_alloc(NNPFS_MAX_MSG_SIZE, M_NNPFS_MSG);
 
     /* initalize the queues if they have not been initialized before */
     nnpfs_initq(&chan->sleepq);
     nnpfs_initq(&chan->messageq);
 
+    chan->proc = NULL;
+
+    nnpfs_dev_unlock(chan);
+
     return 0;
 }
 
-#if defined(HAVE_THREE_ARGUMENT_VFS_BUSY)
+#if defined(__APPLE__)
+#define nnpfs_vfs_busy(mp, flags, lock, proc) 0
+#define nnpfs_vfs_unbusy(mp, proc)
+#elif defined(HAVE_THREE_ARGUMENT_VFS_BUSY)
 #define nnpfs_vfs_busy(mp, flags, lock, proc) vfs_busy((mp), (flags), (lock))
 #define nnpfs_vfs_unbusy(mp, proc) vfs_unbusy((mp))
 #elif defined(HAVE_FOUR_ARGUMENT_VFS_BUSY)
 #define nnpfs_vfs_busy(mp, flags, lock, proc) vfs_busy((mp), (flags), (lock), (proc))
 #define nnpfs_vfs_unbusy(mp, proc) vfs_unbusy((mp), (proc))
-#elif defined(__osf__)
-#define nnpfs_vfs_busy(mp, flags, lock, proc) (0)
-#define nnpfs_vfs_unbusy(mp, proc) (0)
 #else
 #define nnpfs_vfs_busy(mp, flags, lock, proc) vfs_busy((mp))
 #define nnpfs_vfs_unbusy(mp, proc) vfs_unbusy((mp))
@@ -131,11 +141,13 @@ nnpfs_devopen_common(dev_t dev)
  * Wakeup all sleepers and cleanup.
  */
 int
-nnpfs_devclose_common(dev_t dev, d_thread_t *proc)
+nnpfs_devclose_common(nnpfs_dev_t dev, d_thread_t *proc)
 {
     struct nnpfs *chan = &nnpfs_dev[minor(dev)];
     struct nnpfs_link *first;
     
+    nnpfs_dev_lock(chan);
+
     /* Sanity check, paranoia? */
     if (!(chan->status & CHANNEL_OPENED))
 	panic("nnpfs_devclose never opened?");
@@ -168,7 +180,7 @@ nnpfs_devclose_common(dev_t dev, d_thread_t *proc)
 	wakeup((caddr_t) chan);
 
     if (chan->message_buffer) {
-	nnpfs_free(chan->message_buffer, MAX_XMSG_SIZE, M_NNPFS_MSG);
+	nnpfs_free(chan->message_buffer, NNPFS_MAX_MSG_SIZE, M_NNPFS_MSG);
 	chan->message_buffer = NULL;
     }
 
@@ -179,9 +191,13 @@ nnpfs_devclose_common(dev_t dev, d_thread_t *proc)
     if (chan->mp != NULL) {
 	if (nnpfs_vfs_busy(chan->mp, 0, NULL, proc)) {
 	    NNPFSDEB(XDEBNODE, ("nnpfs_dev_close: vfs_busy() --> BUSY\n"));
+	    nnpfs_dev_unlock(chan);
 	    return EBUSY;
 	}
+
+	nnpfs_dev_unlock(chan);
 	nnpfs_free_all_nodes(chan, FORCECLOSE, 0);
+	nnpfs_dev_lock(chan);
 
 	nnpfs_vfs_unbusy(chan->mp, proc);
     }
@@ -192,6 +208,15 @@ nnpfs_devclose_common(dev_t dev, d_thread_t *proc)
 	nnpfs_free_node(chan, xn);
     }
 
+#ifdef __APPLE__
+    if (chan->proc != NULL)
+	proc_rele(chan->proc);
+#endif
+    
+    chan->proc = NULL;
+
+    nnpfs_dev_unlock(chan);
+
     return 0;
 }
 
@@ -203,11 +228,7 @@ nnpfs_devclose_common(dev_t dev, d_thread_t *proc)
 static long
 nnpfs_cursig (d_thread_t *p)
 {
-#if defined(__osf__)
-    thread_t th 	= current_thread();
-    struct np_uthread	*npu = thread_to_np_uthread(th);
-    return CURSIG(p,npu);
-#elif defined(HAVE_FREEBSD_THREAD)
+#if defined(HAVE_FREEBSD_THREAD)
 #ifndef CURSIG
     return 0; /* XXX we would like to use sig_ffs, but that isn't
 	       * exported */
@@ -217,6 +238,8 @@ nnpfs_cursig (d_thread_t *p)
 #else
 #if defined(__NetBSD__) && __NetBSD_Version__ >= 106130000
     return 0; /* XXX CURSIG operates on a struct lwp */
+#elif !defined(CURSIG)
+    return 0;
 #else
     return CURSIG(p);
 #endif
@@ -224,12 +247,22 @@ nnpfs_cursig (d_thread_t *p)
 }
 #endif
 
+
+static d_thread_t *
+nnpfs_get_proc(struct uio *uiop) {
+#ifdef __APPLE__
+    return proc_self();
+#else
+    return nnpfs_uio_to_proc(uiop);
+#endif
+}
+
 /*
  * Move messages from kernel to user space.
  */
 
 int
-nnpfs_devread(dev_t dev, struct uio * uiop, int ioflag)
+nnpfs_devread(nnpfs_dev_t dev, struct uio * uiop, int ioflag)
 {
     struct nnpfs *chan = &nnpfs_dev[minor(dev)];
     struct nnpfs_link *first;
@@ -238,6 +271,8 @@ nnpfs_devread(dev_t dev, struct uio * uiop, int ioflag)
     char devname[64];
 #endif
 
+    nnpfs_dev_lock(chan);
+
     NNPFSDEB(XDEBDEV, ("nnpfs_devread dev = %s\n",
 		     nnpfs_devtoname_r(dev, devname, sizeof(devname))));
 
@@ -246,11 +281,9 @@ nnpfs_devread(dev_t dev, struct uio * uiop, int ioflag)
 		     (unsigned long)chan->messageq.prev,
 		     (unsigned long)chan->messageq.next));
 
-#ifdef HAVE_FREEBSD_THREAD
-    chan->proc = nnpfs_uio_to_thread(uiop);
-#else
-    chan->proc = nnpfs_uio_to_proc(uiop);
-#endif
+
+    if (chan->proc == NULL)
+	chan->proc = nnpfs_get_proc(uiop);
 
  again:
 
@@ -267,7 +300,7 @@ nnpfs_devread(dev_t dev, struct uio * uiop, int ioflag)
 	    NNPFSDEB(XDEBDEV, ("nnpfs_devread: message->size = %u\n",
 			     first->message->size));
 	    
-	    if (first->message->size > uiop->uio_resid)
+	    if (first->message->size > nnpfs_uio_resid(uiop))
 		break;
 
 	    error = uiomove((caddr_t) first->message, first->message->size, 
@@ -281,12 +314,15 @@ nnpfs_devread(dev_t dev, struct uio * uiop, int ioflag)
 		nnpfs_free(first, first->error_or_size, M_NNPFS_LINK);
 	}
     } else {
+	int ret;
 	chan->status |= CHANNEL_WAITING;
-	if (tsleep((caddr_t) chan, (PZERO + 1) | PCATCH, "nnpfsread", 0)) {
-#ifdef HAVE_FREEBSD_THREAD
+
+	ret = nnpfs_xsleep((caddr_t) chan, &chan->dev_lock,
+			   (PZERO + 1) | PCATCH, "nnpfsread");
+	if (ret) {
+#if defined(__APPLE__)
 	    NNPFSDEB(XDEBMSG,
-		   ("caught signal nnpfs_devread: %ld\n",
-		    nnpfs_cursig(nnpfs_uio_to_thread(uiop))));
+		     ("caught signal nnpfs_devread\n"));
 #else
 	    NNPFSDEB(XDEBMSG,
 		   ("caught signal nnpfs_devread: %ld\n",
@@ -299,6 +335,8 @@ nnpfs_devread(dev_t dev, struct uio * uiop, int ioflag)
 	    error = EIO;
     }
     
+    nnpfs_dev_unlock(chan);
+    
     NNPFSDEB(XDEBDEV, ("nnpfs_devread done error = %d\n", error));
 
     return error;
@@ -309,33 +347,36 @@ nnpfs_devread(dev_t dev, struct uio * uiop, int ioflag)
  * wakeup sleepers, insert new data in VFS.
  */
 int
-nnpfs_devwrite(dev_t dev, struct uio *uiop, int ioflag)
+nnpfs_devwrite(nnpfs_dev_t dev, struct uio *uiop, int ioflag)
 {
     struct nnpfs *chan = &nnpfs_dev[minor(dev)];
     char *p;
     int ret, error = 0;
     u_int cnt;
     struct nnpfs_message_header *msg_buf;
+    d_thread_t *pp;
 #ifdef NNPFS_DEBUG
     char devname[64];
 #endif
 
-#ifdef HAVE_FREEBSD_THREAD
-	d_thread_t *pp = nnpfs_uio_to_thread(uiop);
-#else
-	d_thread_t *pp = nnpfs_uio_to_proc(uiop);
-#endif
+    nnpfs_dev_lock(chan);
 
     NNPFSDEB(XDEBDEV, ("nnpfs_devwrite dev = %s\n",
-		     nnpfs_devtoname_r (dev, devname, sizeof(devname))));
+		       nnpfs_devtoname_r (dev, devname, sizeof(devname))));
 
-    chan->proc = pp;
-    cnt = uiop->uio_resid;
-    error = uiomove((caddr_t) chan->message_buffer, MAX_XMSG_SIZE, uiop);
+    if (chan->proc == NULL)
+	chan->proc = nnpfs_get_proc(uiop);
+
+    pp = chan->proc;
+
+    nnpfs_dev_unlock(chan);
+
+    cnt = nnpfs_uio_resid(uiop);
+    error = uiomove((caddr_t) chan->message_buffer, NNPFS_MAX_MSG_SIZE, uiop);
     if (error != 0)
 	return error;
 
-    cnt -= uiop->uio_resid;
+    cnt -= nnpfs_uio_resid(uiop);
 
     /*
      * This thread handles the received message.
@@ -359,6 +400,7 @@ nnpfs_devwrite(dev_t dev, struct uio *uiop, int ioflag)
 	p += msg_buf->size;
 	cnt -= msg_buf->size;
     }
+
     NNPFSDEB(XDEBDEV, ("nnpfs_devwrite error = %d\n", error));
     return error;
 }
@@ -377,8 +419,12 @@ nnpfs_message_send(int fd, struct nnpfs_message_header * message, u_int size)
 
     NNPFSDEB(XDEBMSG, ("nnpfs_message_send opcode = %d\n", message->opcode));
 
-    if (!(chan->status & CHANNEL_OPENED))	/* No receiver? */
+    nnpfs_dev_lock(chan);
+
+    if (!(chan->status & CHANNEL_OPENED)) {	/* No receiver? */
+	nnpfs_dev_unlock(chan);
 	return ENODEV;
+    }
 
     /* Prepare message and copy it later */
     message->size = size;
@@ -396,6 +442,8 @@ nnpfs_message_send(int fd, struct nnpfs_message_header * message, u_int size)
     }
     nnpfs_select_wakeup(chan);
 
+    nnpfs_dev_unlock(chan);
+
     return 0;
 }
 
@@ -403,11 +451,13 @@ nnpfs_message_send(int fd, struct nnpfs_message_header * message, u_int size)
 #define NNPFS_P_EXIT SWEXIT
 #elif defined(P_WEXIT)
 #define NNPFS_P_EXIT P_WEXIT
+#elif defined(__APPLE__)
+/* don't need it */
 #else
 #error what is your exit named ?
 #endif
 
-#if defined(HAVE_STRUCT_PROC_P_SIGMASK) || defined(HAVE_STRUCT_PROC_P_SIGCTX) || defined(HAVE_STRUCT_PROC_P_SIGWAITMASK) || defined(__osf__) || defined(HAVE_FREEBSD_THREAD)
+#if defined(HAVE_STRUCT_PROC_P_SIGMASK) || defined(HAVE_STRUCT_PROC_P_SIGCTX) || defined(HAVE_STRUCT_PROC_P_SIGWAITMASK) || defined(HAVE_FREEBSD_THREAD)
 static void
 nnpfs_block_sigset (sigset_t *sigset)
 {
@@ -437,48 +487,50 @@ nnpfs_block_sigset (sigset_t *sigset)
 
 int
 nnpfs_message_rpc(int fd, struct nnpfs_message_header * message, u_int size,
-		d_thread_t *proc)
+		  d_thread_t *proc)
 {
     int ret;
     struct nnpfs *chan = &nnpfs_dev[fd];
     struct nnpfs_link *this_message;
     struct nnpfs_link *this_process;
     struct nnpfs_message_header *msg;
-#if defined(HAVE_STRUCT_PROC_P_SIGMASK) || defined(HAVE_STRUCT_PROC_P_SIGCTX) || defined(__osf__) || defined(HAVE_FREEBSD_THREAD)
+#if defined(HAVE_STRUCT_PROC_P_SIGMASK) || defined(HAVE_STRUCT_PROC_P_SIGCTX) || defined(HAVE_FREEBSD_THREAD)
     sigset_t oldsigmask;
 #endif
     int catch;
 
+    nnpfs_dev_lock(chan);
+	
     NNPFSDEB(XDEBMSG, ("nnpfs_message_rpc opcode = %d\n", message->opcode));
 
-    if (proc == NULL) {
-#ifdef HAVE_FREEBSD_THREAD
-	proc = nnpfs_curthread();
-#else
+    if (proc == NULL)
 	proc = nnpfs_curproc();
-#endif
-    }
-    if (!(chan->status & CHANNEL_OPENED))	/* No receiver? */
+
+    if (!(chan->status & CHANNEL_OPENED)) {	/* No receiver? */
+	nnpfs_dev_unlock(chan);
 	return ENODEV;
+    }
 
 #ifdef HAVE_FREEBSD_THREAD
     if (chan->proc != NULL && chan->proc->td_proc != NULL &&
-      proc->td_proc->p_pid == chan->proc->td_proc->p_pid) {
+	proc->td_proc != NULL &&
+	proc->td_proc->p_pid == chan->proc->td_proc->p_pid) {
 	printf("nnpfs_message_rpc: deadlock avoided "
 	       "pid = %u == %u\n", proc->td_proc->p_pid, chan->proc->td_proc->p_pid);
 #else
-    if (chan->proc != NULL && proc->p_pid == chan->proc->p_pid) {
-	printf("nnpfs_message_rpc: deadlock avoided "
-	       "pid = %u == %u\n", proc->p_pid, chan->proc->p_pid);
+    if (chan->proc != NULL && proc == chan->proc) {
+      printf("nnpfs_message_rpc: deadlock avoided\n");
 #endif
 #if 0
 	psignal (proc, SIGABRT);
 #endif
+	nnpfs_dev_unlock(chan);
 	return EDEADLK;
     }
 
     if (size < sizeof(struct nnpfs_message_wakeup)) {
 	printf("NNPFS PANIC Error: Message to small to receive wakeup, opcode = %d\n", message->opcode);
+	nnpfs_dev_unlock(chan);
 	return ENOMEM;
     }
     this_message = nnpfs_alloc(sizeof(struct nnpfs_link), M_NNPFS_LINK);
@@ -493,9 +545,18 @@ nnpfs_message_rpc(int fd, struct nnpfs_message_header * message, u_int size,
     this_process->message = msg;
     nnpfs_appendq(&chan->messageq, this_message);
     nnpfs_appendq(&chan->sleepq, this_process);
-    nnpfs_select_wakeup(chan);
     this_process->error_or_size = 0;
 
+    /*
+     * Wakeup daemon that might be blocking select()/poll().
+     */
+
+    nnpfs_select_wakeup(chan);
+
+    /*
+     * Wakeup blocking read in nnpfs_devread
+     */
+
     if (chan->status & CHANNEL_WAITING) {
 	chan->status &= ~CHANNEL_WAITING;
 	wakeup((caddr_t) chan);
@@ -507,7 +568,12 @@ nnpfs_message_rpc(int fd, struct nnpfs_message_header * message, u_int size,
      * bash & co can't handle them.
      */
 
-#ifdef HAVE_FREEBSD_THREAD
+#ifdef __DragonFly__
+    if (proc->td_proc != NULL) {
+        oldsigmask = proc->td_proc->p_sigmask;
+        nnpfs_block_sigset (&proc->td_proc->p_sigmask);
+    }
+#elif defined(HAVE_FREEBSD_THREAD)
     /* FreeBSD 5.1 */
     oldsigmask = proc->td_sigmask;
     nnpfs_block_sigset (&proc->td_sigmask);
@@ -516,17 +582,15 @@ nnpfs_message_rpc(int fd, struct nnpfs_message_header * message, u_int size,
     oldsigmask = proc->p_sigmask;
     nnpfs_block_sigset (&proc->p_sigmask);
 #elif defined(HAVE_STRUCT_PROC_P_SIGCTX)
+#if __NetBSD_Version__ >= 399001400 
+    /* NetBSD 3.99.14 */
+    oldsigmask = proc->l_proc->p_sigctx.ps_sigmask;
+    nnpfs_block_sigset (&proc->l_proc->p_sigctx.ps_sigmask);
+#else
     /* NetBSD 1.6 */
     oldsigmask = proc->p_sigctx.ps_sigmask;
     nnpfs_block_sigset (&proc->p_sigctx.ps_sigmask);
-#elif defined(HAVE_STRUCT_PROC_P_SIGWAITMASK)
-    /* OSF 4.0 */
-    oldsigmask = proc->p_sigwaitmask;
-    nnpfs_block_sigset (&proc->p_sigwaitmask);
-#elif defined(__osf__)
-    /* OSF 5.0 */
-    oldsigmask = u.u_sigmask;
-    nnpfs_block_sigset (&u.u_sigmask);
+#endif
 #endif
 
     /*
@@ -537,8 +601,13 @@ nnpfs_message_rpc(int fd, struct nnpfs_message_header * message, u_int size,
      */
 
     catch = 0;
-#ifdef HAVE_FREEBSD_THREAD
-    if (!(proc->td_proc->p_flag & NNPFS_P_EXIT))
+#ifdef __APPLE__
+    /* XXX */
+    if (0)
+#elif defined(HAVE_FREEBSD_THREAD)
+    if (proc->td_proc && !(proc->td_proc->p_flag & NNPFS_P_EXIT))
+#elif __NetBSD_Version__ >= 399001400 /* NetBSD 3.99.14 */
+    if (!(proc->l_proc->p_flag & NNPFS_P_EXIT))
 #else
     if (!(proc->p_flag & NNPFS_P_EXIT))
 #endif
@@ -552,23 +621,28 @@ nnpfs_message_rpc(int fd, struct nnpfs_message_header * message, u_int size,
     if (!(chan->status & CHANNEL_OPENED)) {
 	NNPFSDEB(XDEBMSG, ("nnpfs_message_rpc: channel went away\n"));
 	this_process->error_or_size = EINTR;
-    } else if ((ret = tsleep((caddr_t) this_process,
-			     (PZERO + 1) | catch, "nnpfs", 0)) != 0) {
-	NNPFSDEB(XDEBMSG, ("caught signal (%d): %ld\n",
-			 ret, nnpfs_cursig(proc)));
-	this_process->error_or_size = EINTR;
+    } else {
+	ret = nnpfs_xsleep((caddr_t)this_process, &chan->dev_lock,
+			   (PZERO + 1) | catch, "nnpfs");
+	if (ret != 0) {
+	    NNPFSDEB(XDEBMSG, ("caught signal (%d): %ld\n", ret, nnpfs_cursig(proc)));
+	    this_process->error_or_size = EINTR;
+	}
     }
 
-#ifdef HAVE_FREEBSD_THREAD
+#ifdef __DragonFly__
+    if (proc->td_proc != NULL)
+        proc->td_proc->p_sigmask = oldsigmask;
+#elif defined(HAVE_FREEBSD_THREAD)
     proc->td_sigmask = oldsigmask;
 #elif HAVE_STRUCT_PROC_P_SIGMASK
     proc->p_sigmask = oldsigmask;
 #elif defined(HAVE_STRUCT_PROC_P_SIGCTX)
+#if defined(__NetBSD__) && __NetBSD_Version__ >= 399001400 /* 3.99.14 */
+    proc->l_proc->p_sigctx.ps_sigmask = oldsigmask;
+#else
     proc->p_sigctx.ps_sigmask = oldsigmask;
-#elif defined(HAVE_STRUCT_PROC_P_SIGWAITMASK)
-    proc->p_sigwaitmask = oldsigmask;
-#elif defined(__osf__)
-    u.u_sigmask = oldsigmask;
+#endif
 #endif
 
     /*
@@ -584,8 +658,9 @@ nnpfs_message_rpc(int fd, struct nnpfs_message_header * message, u_int size,
     ret = this_process->error_or_size;
 
     NNPFSDEB(XDEBMSG, ("nnpfs_message_rpc this_process->error_or_size = %d\n",
-		     this_process->error_or_size));
-    NNPFSDEB(XDEBMSG, ("nnpfs_message_rpc opcode ((nnpfs_message_wakeup*)(this_process->message))->error = %d\n", ((struct nnpfs_message_wakeup *) (this_process->message))->error));
+		       this_process->error_or_size));
+    NNPFSDEB(XDEBMSG, ("nnpfs_message_rpc error = %d\n",
+		       ((struct nnpfs_message_wakeup *) (this_process->message))->error));
 
     bcopy(msg, message, size);
 
@@ -593,6 +668,8 @@ nnpfs_message_rpc(int fd, struct nnpfs_message_header * message, u_int size,
     nnpfs_free(this_process, sizeof(*this_process), M_NNPFS_LINK);
     nnpfs_free(msg, size, M_NNPFS_MSG);
 
+    nnpfs_dev_unlock(chan);
+
     return ret;
 }
 
@@ -681,8 +758,13 @@ nnpfs_message_wakeup(int fd,
 		   d_thread_t *p)
 {
     struct nnpfs *chan = &nnpfs_dev[fd];
-    struct nnpfs_link *sleepq = &chan->sleepq;
-    struct nnpfs_link *t = chan->sleepq.next;	/* Really first in q */
+    struct nnpfs_link *sleepq;
+    struct nnpfs_link *t;
+
+    nnpfs_dev_lock(chan);
+
+    sleepq = &chan->sleepq;
+    t = chan->sleepq.next;	/* Really first in q */
 
     NNPFSDEB(XDEBMSG, ("nnpfs_message_wakeup error: %d\n", message->error));
 
@@ -698,6 +780,8 @@ nnpfs_message_wakeup(int fd,
 	    break;
 	}
 
+    nnpfs_dev_unlock(chan);
+
     return 0;
 }
 
@@ -708,8 +792,13 @@ nnpfs_message_wakeup_data(int fd,
 			d_thread_t *p)
 {
     struct nnpfs *chan = &nnpfs_dev[fd];
-    struct nnpfs_link *sleepq = &chan->sleepq;
-    struct nnpfs_link *t = chan->sleepq.next;	/* Really first in q */
+    struct nnpfs_link *sleepq;
+    struct nnpfs_link *t;
+
+    nnpfs_dev_lock(chan);
+
+    sleepq = &chan->sleepq;
+    t = chan->sleepq.next;	/* Really first in q */
 
     NNPFSDEB(XDEBMSG, ("nnpfs_message_wakeup_data error: %d\n", message->error));
 
@@ -723,6 +812,9 @@ nnpfs_message_wakeup_data(int fd,
 	    wakeup((caddr_t) t);
 	    break;
 	}
+
+    nnpfs_dev_unlock(chan);
+
     return 0;
 }
 
