@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2004 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2006 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -34,10 +34,12 @@
 #include <nnpfs/nnpfs_locl.h>
 #include <nnpfs/nnpfs_common.h>
 #include <nnpfs/nnpfs_fs.h>
+#include <nnpfs/nnpfs_dev.h>
 #include <nnpfs/nnpfs_deb.h>
 #include <nnpfs/nnpfs_vnodeops.h>
+#include <nnpfs/nnpfs_node.h>
 
-RCSID("$Id: nnpfs_node-bsd.c,v 1.75 2004/02/15 21:36:24 lha Exp $");
+RCSID("$Id: nnpfs_node-bsd.c,v 1.92 2006/02/07 14:46:26 lha Exp $");
 
 extern vop_t **nnpfs_vnodeop_p;
 
@@ -51,48 +53,138 @@ extern vop_t **nnpfs_vnodeop_p;
  */
 
 int
-nnpfs_getnewvnode(struct nnpfs *nnpfsp, struct vnode **vpp, 
-		struct nnpfs_handle *handle)
+nnpfs_getnewvnode(struct nnpfs *nnpfsp,
+		  struct vnode **vpp, 
+		  struct nnpfs_handle *handle, 
+		  struct nnpfs_msg_node *node,
+		  d_thread_t *p,
+		  int isrootp)
 {
     struct nnpfs_node *result, *check;
-    struct mount *mp;
     int error;
 
+    result = nnpfs_alloc(sizeof(*result), M_NNPFS_NODE);
+    bzero(result, sizeof(*result));
+
+#ifdef __APPLE__
+    {
+      struct vnode_fsparam p;
+
+      if (!XA_VALID_TYPE(&node->attr)) {
+	nnpfs_free(result, sizeof(*result), M_NNPFS_NODE);
+	return EINVAL;
+      }
+
+      memset(&p, 0, sizeof(p));
+      p.vnfs_mp = NNPFS_TO_VFS(nnpfsp);
+      switch (node->attr.xa_type) {
+      case NNPFS_FILE_NON:
+	p.vnfs_vtype = VNON;
+	break;
+      case NNPFS_FILE_REG:
+	p.vnfs_vtype = VREG;
+	break;
+      case NNPFS_FILE_DIR:
+	p.vnfs_vtype = VDIR;
+	break;
+      case NNPFS_FILE_BLK:
+	p.vnfs_vtype = VBLK;
+	break;
+      case NNPFS_FILE_CHR:
+	p.vnfs_vtype = VCHR;
+	break;
+      case NNPFS_FILE_LNK:
+	p.vnfs_vtype = VLNK;
+	break;
+      case NNPFS_FILE_SOCK:
+	p.vnfs_vtype = VSOCK;
+	break;
+      case NNPFS_FILE_FIFO:
+	p.vnfs_vtype = VFIFO;
+	break;
+      case NNPFS_FILE_BAD:
+	p.vnfs_vtype = VBAD;
+	break;
+      default:
+	printf("nnpfs_getnewnvode: bad value");
+	nnpfs_free(result, sizeof(*result), M_NNPFS_NODE);
+	return EINVAL;
+      }
+      p.vnfs_str = "arla";
+      p.vnfs_dvp = NULL; /* parent vnode */
+      p.vnfs_fsnode = result;
+      p.vnfs_vops = nnpfs_vnodeop_p;
+      p.vnfs_markroot = isrootp;
+      p.vnfs_marksystem = 0;
+      p.vnfs_rdev = 0;
+      if (XA_VALID_SIZE(&node->attr))
+	p.vnfs_filesize = node->attr.xa_size;
+      else
+	p.vnfs_filesize = 0;
+      p.vnfs_cnp = NULL;
+      p.vnfs_flags = 0; /* XXX VNFS_CANTCACHE? */
+
+      error = vnode_create(VNCREATE_FLAVOR, VCREATESIZE, &p, vpp);
+    }
+#else
     error = getnewvnode(VT_AFS, NNPFS_TO_VFS(nnpfsp), nnpfs_vnodeop_p, vpp);
+    if (error == 0)
+      (*vpp)->v_data = result;
+#endif
     if (error)
 	return error;
     
-    result = nnpfs_alloc(sizeof(*result), M_NNPFS_NODE);
-    bzero(result, sizeof(*result));
-    
-    (*vpp)->v_data = result;
     result->vn = *vpp;
     
     result->handle = *handle;
     result->flags = 0;
     result->tokens = 0;
     result->offset = 0;
-#if defined(HAVE_KERNEL_LOCKMGR) || defined(HAVE_KERNEL_DEBUGLOCKMGR)
+#if (defined(HAVE_KERNEL_LOCKMGR) || defined(HAVE_KERNEL_DEBUGLOCKMGR)) && !defined(__APPLE__)
     lockinit (&result->lock, PVFS, "nnpfs_lock", 0, LK_NOPAUSE);
 #else
     result->vnlocks = 0;
 #endif
+
+#ifdef __APPLE__
+    result->writers = 0;
+#endif
+
     result->anonrights = 0;
-    result->rd_cred = NULL;
-    result->wr_cred = NULL;
+    nnpfs_setcred(&result->rd_cred, NOCRED);
+    nnpfs_setcred(&result->wr_cred, NOCRED);
 
 #if defined(__NetBSD_Version__) && __NetBSD_Version__ >= 105280000
     genfs_node_init(*vpp, &nnpfs_genfsops);
 #endif
 
-    check = nnpfs_node_find(&nnpfsp->nodehead, handle);
-    if (check) {
-	vput(*vpp);
-	*vpp = result->vn;
+    nnpfs_dev_lock(nnpfsp);
+
+ retry:
+    error = nnpfs_node_find_locked(nnpfsp, handle, &check);
+    if (error == ENOENT) {
+	nnpfs_insert(&nnpfsp->nodehead, result);
+	nnpfs_dev_unlock(nnpfsp);
+
 	return 0;
     }
 
-    nnpfs_insert(&nnpfsp->nodehead, result);
+    if (nnpfs_do_vget(XNODE_TO_VNODE(check), 0, p))
+	goto retry;
+    
+    nnpfs_vput(*vpp);
+    *vpp = NULL;
+    
+    if (error == EISDIR) {
+	nnpfs_vletgo(XNODE_TO_VNODE(check));
+	nnpfs_dev_unlock(nnpfsp);
+
+	return EEXIST;
+    }
+    
+    *vpp = check->vn;
+
+    nnpfs_dev_unlock(nnpfsp);
 
     return 0;
 }
@@ -106,11 +198,14 @@ nnpfs_getnewvnode(struct nnpfs *nnpfsp, struct vnode **vpp,
 
 int
 nnpfs_new_node(struct nnpfs *nnpfsp,
-	     struct nnpfs_msg_node *node,
-	     struct nnpfs_node **xpp,
-	     d_thread_t *p)
+	       struct nnpfs_msg_node *node,
+	       char *name,
+	       struct nnpfs_node **xpp,
+	       d_thread_t *p,
+	       int isrootp)
 {
     struct nnpfs_node *result;
+    int error;
 
     NNPFSDEB(XDEBNODE, ("nnpfs_new_node (%d,%d,%d,%d)\n",
 		      node->handle.a,
@@ -120,42 +215,63 @@ nnpfs_new_node(struct nnpfs *nnpfsp,
 
 retry:
     /* Does not allow duplicates */
-    result = nnpfs_node_find(&nnpfsp->nodehead, &node->handle);
-    if (result == 0) {
-	int error;
+    error = nnpfs_node_find(nnpfsp, &node->handle, &result);
+    if (error == ENOENT) {
 	struct vnode *v;
 
-	error = nnpfs_getnewvnode(nnpfsp, &v, &node->handle);
+	error = nnpfs_getnewvnode(nnpfsp, &v, &node->handle, node, p, isrootp);
 	if (error)
 	    return error;
 
 	result = VNODE_TO_XNODE(v);
 	result->anonrights = node->anonrights;
-
-	nnpfsp->nnodes++;
+    } else if (error == EISDIR) {
+	/* node is about to be deleted */
+	NNPFSDEB(XDEBNODE, ("nnpfs_new_node: node deleted\n"));
+	return error;
     } else {
 	/* Node is already cached */
-	if(nnpfs_do_vget(XNODE_TO_VNODE(result), 0, p))
+	if (nnpfs_do_vget(XNODE_TO_VNODE(result), 0, p))
 	    goto retry;
     }
 
     /* Init other fields */
     nnpfs_attr2vattr(&node->attr, &result->attr, 1);
+#ifndef __APPLE__
     result->vn->v_type = result->attr.va_type;
+#endif
+
     result->tokens = node->tokens;
+    if ((result->tokens & NNPFS_DATA_MASK) && DATA_FROM_XNODE(result) == NULL) {
+	printf("nnpfs_new_node: tokens and no data (%d,%d,%d,%d) \n",
+	       node->handle.a, node->handle.b, node->handle.c, node->handle.d);
+	result->tokens &= ~NNPFS_DATA_MASK;
+    }
+
     bcopy(node->id, result->id, sizeof(result->id));
     bcopy(node->rights, result->rights, sizeof(result->rights));
 
-#ifdef __APPLE__
-    if (result->vn->v_type == VREG && (!UBCINFOEXISTS(result->vn)))
+#if defined(__APPLE__) && 0
+    if (nnpfs_vnode_isreg(result->vn) && (!UBCINFOEXISTS(result->vn)))
 	ubc_info_init(result->vn);
 #endif
 
+#ifndef __APPLE__
+    if (isrootp) {
+	NNPFS_MAKE_VROOT(result->vn);
+    }
+#endif
+
     *xpp = result;
     NNPFSDEB(XDEBNODE, ("return: nnpfs_new_node\n"));
     return 0;
 }
 
+/*
+ * free node.
+ *
+ * this should always be called with dev lock held
+ */
 void
 nnpfs_free_node(struct nnpfs *nnpfsp, struct nnpfs_node *node)
 {
@@ -166,22 +282,14 @@ nnpfs_free_node(struct nnpfs *nnpfsp, struct nnpfs_node *node)
 		      node->handle.c,
 		      node->handle.d));
 
+
     NNPQUEUE_REMOVE(node, &nnpfsp->freehead, nn_free);
     nnpfs_remove_node(&nnpfsp->nodehead, node);
 
     if (DATA_FROM_XNODE(node)) {
-	vrele(DATA_FROM_XNODE(node));
+	nnpfs_vletgo(DATA_FROM_XNODE(node));
 	DATA_FROM_XNODE(node) = NULL;
     }
-    nnpfsp->nnodes--;
-    if (node->rd_cred) {
-	crfree (node->rd_cred);
-	node->rd_cred = NULL;
-    }
-    if (node->wr_cred) {
-	crfree (node->wr_cred);
-	node->wr_cred = NULL;
-    }
 
     nnpfs_free(node, sizeof(*node), M_NNPFS_NODE);
 
@@ -190,12 +298,15 @@ nnpfs_free_node(struct nnpfs *nnpfsp, struct nnpfs_node *node)
 
 /*
  * FreeBSD 4.4 and newer changed to API to vflush around June 2001
+ * FreeBSD 5.2-CURRENT and newer changed to API to vflush
  */
 
 static int
-nnpfs_vflush(struct mount *mp, int flags)
+nnpfs_vflush(struct mount *mp, int flags, d_thread_t *td)
 {
-#if __FreeBSD__ && __FreeBSD_version > 430000
+#if defined(__FreeBSD__) && __FreeBSD_version > 502123
+    return vflush(mp, 0, flags, td);
+#elif (__FreeBSD__ && __FreeBSD_version > 430000) || __DragonFly__
     return vflush(mp, 0, flags);
 #else
     return vflush(mp, NULL, flags);
@@ -220,7 +331,11 @@ nnpfs_free_all_nodes(struct nnpfs *nnpfsp, int flags, int unmountp)
     if (nnpfsp->root) {
 	NNPFSDEB(XDEBNODE, ("nnpfs_free_all_nodes now removing root\n"));
 
+#if 0
 	vgone(XNODE_TO_VNODE(nnpfsp->root));
+#else
+	nnpfs_vletgo(XNODE_TO_VNODE(nnpfsp->root)); /* XXX ? */
+#endif
 	nnpfsp->root = NULL;
     }
 
@@ -244,7 +359,7 @@ nnpfs_free_all_nodes(struct nnpfs *nnpfsp, int flags, int unmountp)
 	     */
 #if defined(__OpenBSD__)
 	    vgone(mp->mnt_syncer);
-#elif defined(__FreeBSD__)
+#elif defined(__FreeBSD__) || defined(__DragonFly__)
 	    vrele(mp->mnt_syncer);
 #else
 #error what os do you use ?
@@ -254,7 +369,7 @@ nnpfs_free_all_nodes(struct nnpfs *nnpfsp, int flags, int unmountp)
 	}
     }
 #endif
-    error = nnpfs_vflush(mp, flags);
+    error = nnpfs_vflush(mp, flags, nnpfsp->proc);
 #ifdef HAVE_STRUCT_MOUNT_MNT_SYNCER
     if (!unmountp) {
 	NNPFSDEB(XDEBNODE, ("nnpfs_free_all_nodes not flushing syncer vnode\n"));
@@ -281,28 +396,91 @@ nnpfs_free_all_nodes(struct nnpfs *nnpfsp, int flags, int unmountp)
 		(var) = ((var)->field.le_next))
 #endif
 
+#ifdef __APPLE__
 void
-vattr2nnpfs_attr(const struct vattr *va, struct nnpfs_attr *xa)
+vattr2nnpfs_attr(const struct nnpfs_vfs_vattr *va, struct nnpfs_attr *xa)
+{
+    /* XXX macos bitmask handling */
+    bzero(xa, sizeof(*xa));
+    if (VATTR_IS_ACTIVE(va, va_mode))
+	XA_SET_MODE(xa, va->va_mode);
+    if (VATTR_IS_ACTIVE(va, va_nlink))
+	XA_SET_NLINK(xa, va->va_nlink);
+    if (VATTR_IS_ACTIVE(va, va_data_size))
+	XA_SET_SIZE(xa, nnpfs_vattr_get_size(va));
+    if (VATTR_IS_ACTIVE(va, va_uid))
+	XA_SET_UID(xa, va->va_uid);
+    if (VATTR_IS_ACTIVE(va, va_gid))
+	XA_SET_GID(xa, va->va_gid);
+    if (VATTR_IS_ACTIVE(va, va_access_time))
+	XA_SET_ATIME(xa, nnpfs_vattr_get_atime_sec(va));
+    if (VATTR_IS_ACTIVE(va, va_modify_time))
+	XA_SET_MTIME(xa, nnpfs_vattr_get_mtime_sec(va));
+    if (VATTR_IS_ACTIVE(va, va_create_time))
+	XA_SET_CTIME(xa, nnpfs_vattr_get_ctime_sec(va));
+    if (VATTR_IS_ACTIVE(va, va_fileid))
+	XA_SET_FILEID(xa, va->va_fileid);
+
+    if (VATTR_IS_ACTIVE(va, va_type)) {
+	switch (va->va_type) {
+	case VNON:
+	    xa->xa_type = NNPFS_FILE_NON;
+	    break;
+	case VREG:
+	    xa->xa_type = NNPFS_FILE_REG;
+	    break;
+	case VDIR:
+	    xa->xa_type = NNPFS_FILE_DIR;
+	    break;
+	case VBLK:
+	    xa->xa_type = NNPFS_FILE_BLK;
+	    break;
+	case VCHR:
+	    xa->xa_type = NNPFS_FILE_CHR;
+	    break;
+	case VLNK:
+	    xa->xa_type = NNPFS_FILE_LNK;
+	    break;
+	case VSOCK:
+	    xa->xa_type = NNPFS_FILE_SOCK;
+	    break;
+	case VFIFO:
+	    xa->xa_type = NNPFS_FILE_FIFO;
+	    break;
+	case VBAD:
+	    xa->xa_type = NNPFS_FILE_BAD;
+	    break;
+	default:
+	    panic("vattr2nnpfs_attr: bad value");
+	}
+    }
+}
+
+#else
+
+void
+vattr2nnpfs_attr(const struct nnpfs_vfs_vattr *va, struct nnpfs_attr *xa)
 {
     bzero(xa, sizeof(*xa));
     if (va->va_mode != (mode_t)VNOVAL)
 	XA_SET_MODE(xa, va->va_mode);
     if (va->va_nlink != VNOVAL)
 	XA_SET_NLINK(xa, va->va_nlink);
-    if (va->va_size != VNOVAL)
-	XA_SET_SIZE(xa, va->va_size);
+    if (nnpfs_vattr_get_size(va) != VNOVAL)
+	XA_SET_SIZE(xa, nnpfs_vattr_get_size(va));
     if (va->va_uid != VNOVAL)
 	XA_SET_UID(xa, va->va_uid);
     if (va->va_gid != VNOVAL)
 	XA_SET_GID(xa, va->va_gid);
-    if (va->va_atime.tv_sec != VNOVAL)
-	XA_SET_ATIME(xa, va->va_atime.tv_sec);
-    if (va->va_mtime.tv_sec != VNOVAL)
-	XA_SET_MTIME(xa, va->va_mtime.tv_sec);
-    if (va->va_ctime.tv_sec != VNOVAL)
-	XA_SET_CTIME(xa, va->va_ctime.tv_sec);
+    if (nnpfs_vattr_get_atime_sec(va) != VNOVAL)
+	XA_SET_ATIME(xa, nnpfs_vattr_get_atime_sec(va));
+    if (nnpfs_vattr_get_mtime_sec(va) != VNOVAL)
+	XA_SET_MTIME(xa, nnpfs_vattr_get_mtime_sec(va));
+    if (nnpfs_vattr_get_ctime_sec(va) != VNOVAL)
+	XA_SET_CTIME(xa, nnpfs_vattr_get_ctime_sec(va));
     if (va->va_fileid != VNOVAL)
 	XA_SET_FILEID(xa, va->va_fileid);
+
     switch (va->va_type) {
     case VNON:
 	xa->xa_type = NNPFS_FILE_NON;
@@ -332,76 +510,87 @@ vattr2nnpfs_attr(const struct vattr *va, struct nnpfs_attr *xa)
 	xa->xa_type = NNPFS_FILE_BAD;
 	break;
     default:
-	panic("nnpfs_attr2attr: bad value");
+	panic("vattr2nnpfs_attr: bad value");
     }
 }
-
-#define SET_TIMEVAL(X, S, N) do { (X)->tv_sec = (S); (X)->tv_nsec = (N); } while(0)
+#endif
 
 void
-nnpfs_attr2vattr(const struct nnpfs_attr *xa, struct vattr *va, int clear_node)
+nnpfs_attr2vattr(const struct nnpfs_attr *xa, struct nnpfs_vfs_vattr *va, int clear_node)
 {
-    if (clear_node)
+    /* XXX proper macos bitmask handling */
+    if (clear_node) {
+#ifdef __APPLE__
+	VATTR_INIT(va);
+#else
 	VATTR_NULL(va);
+#endif
+    }
     if (XA_VALID_MODE(xa))
-	va->va_mode = xa->xa_mode;
+	nnpfs_vattr_set(va, va_mode, xa->xa_mode);
     if (XA_VALID_NLINK(xa))
-	va->va_nlink = xa->xa_nlink;
+	nnpfs_vattr_set(va, va_nlink, xa->xa_nlink);
     if (XA_VALID_SIZE(xa)) {
-	va->va_size = xa->xa_size;
-	va->va_bytes = va->va_size;
+	nnpfs_vattr_set_size(va, xa->xa_size);
+	nnpfs_vattr_set_bytes(va, xa->xa_size);
     }
     if (XA_VALID_UID(xa))
-	va->va_uid = xa->xa_uid;
+	nnpfs_vattr_set(va, va_uid, xa->xa_uid);
     if (XA_VALID_GID(xa))
-	va->va_gid = xa->xa_gid;
+	nnpfs_vattr_set(va, va_gid, xa->xa_gid);
     if (XA_VALID_ATIME(xa)) {
-	SET_TIMEVAL(&va->va_atime, xa->xa_atime, 0);
+	nnpfs_vattr_set_atime(va, xa->xa_atime, 0);
     }
     if (XA_VALID_MTIME(xa)) {
-	SET_TIMEVAL(&va->va_mtime, xa->xa_mtime, 0);
+	nnpfs_vattr_set_mtime(va, xa->xa_mtime, 0);
     }
     if (XA_VALID_CTIME(xa)) {
-	SET_TIMEVAL(&va->va_ctime, xa->xa_ctime, 0);
+	nnpfs_vattr_set_ctime(va, xa->xa_ctime, 0);
     }
     if (XA_VALID_FILEID(xa)) {
-	va->va_fileid = xa->xa_fileid;
+	nnpfs_vattr_set(va, va_fileid, xa->xa_fileid);
     }
     if (XA_VALID_TYPE(xa)) {
 	switch (xa->xa_type) {
 	case NNPFS_FILE_NON:
-	    va->va_type = VNON;
+	    nnpfs_vattr_set(va, va_type, VNON);
 	    break;
 	case NNPFS_FILE_REG:
-	    va->va_type = VREG;
+	    nnpfs_vattr_set(va, va_type, VREG);
 	    break;
 	case NNPFS_FILE_DIR:
-	    va->va_type = VDIR;
+	    nnpfs_vattr_set(va, va_type, VDIR);
 	    break;
 	case NNPFS_FILE_BLK:
-	    va->va_type = VBLK;
+	    nnpfs_vattr_set(va, va_type, VBLK);
 	    break;
 	case NNPFS_FILE_CHR:
-	    va->va_type = VCHR;
+	    nnpfs_vattr_set(va, va_type, VCHR);
 	    break;
 	case NNPFS_FILE_LNK:
-	    va->va_type = VLNK;
+	    nnpfs_vattr_set(va, va_type, VLNK);
 	    break;
 	case NNPFS_FILE_SOCK:
-	    va->va_type = VSOCK;
+	    nnpfs_vattr_set(va, va_type, VSOCK);
 	    break;
 	case NNPFS_FILE_FIFO:
-	    va->va_type = VFIFO;
+	    nnpfs_vattr_set(va, va_type, VFIFO);
 	    break;
 	case NNPFS_FILE_BAD:
-	    va->va_type = VBAD;
+	    nnpfs_vattr_set(va, va_type, VBAD);
 	    break;
 	default:
 	    panic("nnpfs_attr2vattr: bad value");
 	}
     }
-    va->va_flags = 0;
-    va->va_blocksize = 8192;
+    nnpfs_vattr_set(va, va_flags, 0);
+
+#ifdef __APPLE__
+    VATTR_RETURN(va, va_iosize, 8192);
+    va->va_active = va->va_supported;
+#else
+    nnpfs_vattr_set(va, va_blocksize, 8192);
+#endif
 }
 
 /*
@@ -471,6 +660,12 @@ tbl_lookup (struct componentname *cnp,
 	&& strncmp(tbl.name, cnp->cn_nameptr, tbl.len) == 0) {
 
 	*res = tbl.vp;
+
+#ifdef __APPLE__
+	/* darwin's cache_lookup gives us ref:ed nodes, imitate */
+	nnpfs_do_vget(*res, 0, NULL);
+#endif
+	
 	return -1;
     } else
 	return 0;
@@ -528,7 +723,12 @@ nnpfs_dnlc_enter(struct vnode *dvp,
 	    printf ("NNPFS PANIC WARNING! nnpfs_dnlc_enter: %s already in cache\n",
 		    cnp->cn_nameptr);
 	}
-#else
+#elif defined(__DragonFly__)
+	if (cache_lookup(dvp, NCPNULL, &dummy, NCPPNULL, cnp) != 0) {
+	    printf ("NNPFS PANIC WARNING! nnpfs_dnlc_enter: %s already in cache\n",
+		    cnp->cn_nameptr);
+	}
+#elif !defined(__APPLE__)
 	if (cache_lookup(dvp, &dummy, cnp) != 0) {
 	    printf ("NNPFS PANIC WARNING! nnpfs_dnlc_enter: %s already in cache\n",
 		    cnp->cn_nameptr);
@@ -538,7 +738,11 @@ nnpfs_dnlc_enter(struct vnode *dvp,
 
 	cnp->cn_nameiop = save_nameiop;
 	cnp->cn_flags   = save_flags;
-	cache_enter(dvp, vp, cnp);
+#ifdef __DragonFly__
+	cache_enter(dvp, NCPNULL, vp, cnp);
+#else
+ 	cache_enter(dvp, vp, cnp);
+#endif
     }
 
     if (vp != NULL)
@@ -550,23 +754,16 @@ nnpfs_dnlc_enter(struct vnode *dvp,
 
 static void
 nnpfs_cnp_init (struct componentname *cn,
-	      const char *name,
-	      d_thread_t *proc, struct ucred *cred,
-	      int nameiop)
+		char *name,
+		d_thread_t *proc, struct ucred *cred,
+		int nameiop)
 {
     bzero(cn, sizeof(*cn));
     cn->cn_nameptr = (char *)name;
     cn->cn_namelen = strlen(name);
     cn->cn_flags   = 0;
 #if __APPLE__
-    {
-	const unsigned char *p;
-	int i;
-
-	cn->cn_hash = 0;
-	for (p = cn->cn_nameptr, i = 1; *p; ++p, ++i)
-	    cn->cn_hash += *p * i;
-    }
+    cn->cn_hash = 0; /* Let the vfs compute the hash */
 #elif defined(HAVE_KERNEL_NAMEI_HASH)
     {
 	const char *cp = name + cn->cn_namelen;
@@ -582,12 +779,23 @@ nnpfs_cnp_init (struct componentname *cn,
     }
 #endif
     cn->cn_nameiop = nameiop;
-#ifdef HAVE_FREEBSD_THREAD
+#if defined(__APPLE__)
+    /* apple have not proc */
+#elif defined(__DragonFly__)
+    cn->cn_td = proc;
+#elif defined(HAVE_FREEBSD_THREAD)
     cn->cn_thread = proc;
+#else
+#if defined(__NetBSD__) && __NetBSD_Version__ >= 399001400 /* 3.99.14 */
+    cn->cn_lwp = proc;
 #else
     cn->cn_proc = proc;
 #endif
+#endif
+
+#if !defined(__APPLE__)
     cn->cn_cred = cred;
+#endif
 }
 
 
@@ -597,8 +805,8 @@ nnpfs_cnp_init (struct componentname *cn,
 
 int
 nnpfs_dnlc_enter_name(struct vnode *dvp,
-		    const char *name,
-		    struct vnode *vp)
+		      char *name,
+		      struct vnode *vp)
 {
     struct componentname cn;
 
@@ -640,7 +848,11 @@ nnpfs_dnlc_lookup_int(struct vnode *dvp,
     saved_flags = cnp->cn_flags;
     cnp->cn_flags |= MAKEENTRY | LOCKPARENT | ISLASTCN;
 
+#ifdef __DragonFly__
+    error = cache_lookup(dvp, NCPNULL, res, NCPPNULL, cnp);
+#else
     error = cache_lookup(dvp, res, cnp);
+#endif
 
     cnp->cn_flags = saved_flags;
 
@@ -670,18 +882,14 @@ nnpfs_dnlc_lock(struct vnode *dvp,
      */
 
     if (*res == dvp) {		/* "." */
-	VREF(dvp);
+	nnpfs_vref(dvp);
     } else if (cnp->cn_flags & ISDOTDOT) { /* ".." */
 #ifdef HAVE_STRUCT_VNODE_V_ID
 	u_long vpid = dvp->v_id;
 #endif
 
-#ifdef HAVE_FREEBSD_THREAD
-	nnpfs_vfs_unlock(dvp, nnpfs_cnp_to_thread(cnp));
-	error = nnpfs_do_vget(*res, LK_EXCLUSIVE, nnpfs_cnp_to_thread(cnp));
-	nnpfs_vfs_writelock(dvp, nnpfs_cnp_to_thread(cnp));
-#else
 	nnpfs_vfs_unlock(dvp, nnpfs_cnp_to_proc(cnp));
+#ifndef __APPLE__
 	error = nnpfs_do_vget(*res, LK_EXCLUSIVE, nnpfs_cnp_to_proc(cnp));
 	nnpfs_vfs_writelock(dvp, nnpfs_cnp_to_proc(cnp));
 #endif
@@ -693,9 +901,7 @@ nnpfs_dnlc_lock(struct vnode *dvp,
 	}
 #endif
     } else {
-#ifdef HAVE_FREEBSD_THREAD
-	error = nnpfs_do_vget(*res, LK_EXCLUSIVE, nnpfs_cnp_to_thread(cnp));
-#else
+#ifndef __APPLE__
 	error = nnpfs_do_vget(*res, LK_EXCLUSIVE, nnpfs_cnp_to_proc(cnp));
 #endif
     }
@@ -784,7 +990,9 @@ nnpfs_dnlc_purge_mp(struct mount *mp)
     NNPFSDEB(XDEBDNLC, ("nnpfs_dnlc_purge_mp()\n"));
 
     tbl_clear ();
+#ifndef __APPLE__
     cache_purgevfs(mp);
+#endif
 }
 
 /*
@@ -796,7 +1004,7 @@ nnpfs_has_pag(const struct nnpfs_node *xn, nnpfs_pag_t pag)
 {
     int i;
 
-    for (i = 0; i < MAXRIGHTS; i++)
+    for (i = 0; i < NNPFS_MAXRIGHTS; i++)
 	if (xn->id[i] == pag)
 	    return 1;
 
@@ -804,19 +1012,13 @@ nnpfs_has_pag(const struct nnpfs_node *xn, nnpfs_pag_t pag)
 }
 
 void
-nnpfs_update_write_cred(struct nnpfs_node *xn, struct ucred *cred)
+nnpfs_setcred(nnpfs_cred *ncred, struct ucred *ucred)
 {
-    if (xn->wr_cred)
-	crfree (xn->wr_cred);
-    crhold (cred);
-    xn->wr_cred = cred;
-}
-
-void
-nnpfs_update_read_cred(struct nnpfs_node *xn, struct ucred *cred)
-{
-    if (xn->rd_cred)
-	crfree (xn->rd_cred);
-    crhold (cred);
-    xn->rd_cred = cred;
+    if (ucred == NOCRED) {
+	ncred->uid = 0;
+	ncred->pag = NNPFS_ANONYMOUSID;
+    } else {
+	ncred->uid = ucred->cr_uid;
+	ncred->pag = nnpfs_get_pag(ucred);
+    }
 }
