@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2002 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2006 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -35,10 +35,6 @@
  * NNPFS operations.
  */
 
-#ifdef __APPLE__
-#define MACH_KERNEL 1
-#endif
-
 #include <nnpfs/nnpfs_locl.h>
 #include <nnpfs/nnpfs_message.h>
 #include <nnpfs/nnpfs_common.h>
@@ -51,7 +47,17 @@
 #include <vm/vnode_pager.h>
 #endif
 
-RCSID("$Id: nnpfs_vnodeops-bsd.c,v 1.130.2.1 2004/06/13 11:32:23 lha Exp $");
+RCSID("$Id: nnpfs_vnodeops-bsd.c,v 1.155 2006/04/10 12:13:46 tol Exp $");
+
+
+#ifdef HAVE_FREEBSD_THREAD
+#define NNPFS_AP_PROC(ap) ((ap)->a_td)
+#elif defined(__NetBSD__) && __NetBSD_Version__ >= 399001400 /* 3.99.14 */
+#define NNPFS_AP_PROC(ap) ((ap)->a_l)
+#else
+#define NNPFS_AP_PROC(ap) ((ap)->a_p)
+#endif
+
 
 /*
  * vnode functions
@@ -71,11 +77,10 @@ nnpfs_open(struct vop_open_args * ap)
 #endif
   }; */
 {
-#ifdef HAVE_FREEBSD_THREAD
-    return nnpfs_open_common (ap->a_vp, ap->a_mode, ap->a_cred, ap->a_td);
-#else
-    return nnpfs_open_common (ap->a_vp, ap->a_mode, ap->a_cred, ap->a_p);
-#endif
+    nnpfs_vfs_context ctx;
+    nnpfs_vfs_context_init(ctx, NNPFS_AP_PROC(ap), ap->a_cred);
+    
+    return nnpfs_open_common (ap->a_vp, ap->a_mode, ctx);
 }
 #endif /* HAVE_VOP_OPEN */
 
@@ -91,12 +96,18 @@ nnpfs_fsync(struct vop_fsync_args * ap)
 };  */
 {
 #ifdef HAVE_STRUCT_VOP_FSYNC_ARGS_A_FLAGS
-    return nnpfs_fsync_common(ap->a_vp, ap->a_cred, ap->a_flags, ap->a_p);
+#if defined(__NetBSD__) && __NetBSD_Version__ >= 399001400 /* 3.99.14 */
+    return nnpfs_fsync_common(ap->a_vp, ap->a_cred, NULL, ap->a_flags, ap->a_l);
 #else
-#ifdef HAVE_FREEBSD_THREAD
-    return nnpfs_fsync_common(ap->a_vp, ap->a_cred, ap->a_waitfor, ap->a_td);
+    return nnpfs_fsync_common(ap->a_vp, ap->a_cred, NULL, ap->a_flags, ap->a_p);
+#endif
 #else
-    return nnpfs_fsync_common(ap->a_vp, ap->a_cred, ap->a_waitfor, ap->a_p);
+#ifdef __DragonFly__
+    return nnpfs_fsync_common(ap->a_vp, ap->a_td->td_proc->p_ucred, NULL,
+                              ap->a_waitfor, ap->a_td);
+#else
+    return nnpfs_fsync_common(ap->a_vp, ap->a_cred, NULL,
+			      ap->a_waitfor, NNPFS_AP_PROC(ap));
 #endif
 #endif
 }
@@ -112,10 +123,11 @@ nnpfs_close(struct vop_close_args * ap)
 	IN struct proc *p;
   }; */
 {
-#ifdef HAVE_FREEBSD_THREAD
-    return nnpfs_close_common(ap->a_vp, ap->a_fflag, ap->a_td, ap->a_cred);
+#ifdef __DragonFly__
+    return nnpfs_close_common(ap->a_vp, ap->a_fflag, ap->a_td,
+                              ap->a_td->td_proc->p_ucred);
 #else
-    return nnpfs_close_common(ap->a_vp, ap->a_fflag, ap->a_p, ap->a_cred);
+    return nnpfs_close_common(ap->a_vp, ap->a_fflag, NNPFS_AP_PROC(ap), ap->a_cred);
 #endif
 }
 #endif /* HAVE_VOP_CLOSE */
@@ -144,7 +156,10 @@ nnpfs_write(struct vop_write_args * ap)
 	IN struct ucred *cred;
    }; */
 {
-    return nnpfs_write_common(ap->a_vp, ap->a_uio, ap->a_ioflag, ap->a_cred);
+    nnpfs_vfs_context ctx;
+    nnpfs_vfs_context_init(ctx, nnpfs_uio_to_proc(ap->a_uio), ap->a_cred);
+
+    return nnpfs_write_common(ap->a_vp, ap->a_uio, ap->a_ioflag, ctx);
 }
 #endif /* HAVE_VOP_WRITE */
 
@@ -216,10 +231,11 @@ nnpfs_getattr(struct vop_getattr_args * ap)
 	    struct ucred *cred,
 	    struct proc *p) */
 {
-#ifdef HAVE_FREEBSD_THREAD
-    return nnpfs_getattr_common(ap->a_vp, ap->a_vap, ap->a_cred, ap->a_td);
+#ifdef __DragonFly__
+    return nnpfs_getattr_common(ap->a_vp, ap->a_vap,
+                                ap->a_td->td_proc->p_ucred, ap->a_td);
 #else
-    return nnpfs_getattr_common(ap->a_vp, ap->a_vap, ap->a_cred, ap->a_p);
+    return nnpfs_getattr_common(ap->a_vp, ap->a_vap, ap->a_cred, NNPFS_AP_PROC(ap));
 #endif
 }
 #endif /* HAVE_VOP_GETATTR */
@@ -233,11 +249,7 @@ nnpfs_setattr(struct vop_setattr_args * ap)
 	    struct proc *p)
 	    */
 {
-#ifdef HAVE_FREEBSD_THREAD
-    return nnpfs_setattr_common(ap->a_vp, ap->a_vap, ap->a_cred, ap->a_td);
-#else
-    return nnpfs_setattr_common(ap->a_vp, ap->a_vap, ap->a_cred, ap->a_p);
-#endif
+    return nnpfs_setattr_common(ap->a_vp, ap->a_vap, ap->a_cred, NNPFS_AP_PROC(ap));
 }
 #endif /* HAVE_VOP_SETATTR */
 
@@ -251,11 +263,7 @@ struct vnode *vp,
 	   struct proc *p)
 	   */
 {
-#ifdef HAVE_FREEBSD_THREAD
-    return nnpfs_access_common(ap->a_vp, ap->a_mode, ap->a_cred, ap->a_td);
-#else
-    return nnpfs_access_common(ap->a_vp, ap->a_mode, ap->a_cred, ap->a_p);
-#endif
+    return nnpfs_access_common(ap->a_vp, ap->a_mode, ap->a_cred, NNPFS_AP_PROC(ap));
 }
 #endif /* HAVE_VOP_ACCESS */
 
@@ -270,7 +278,11 @@ nnpfs_lookup(struct vop_lookup_args * ap)
 }; */
 {
     struct componentname *cnp = ap->a_cnp;
+    d_thread_t *p  = nnpfs_cnp_to_proc(cnp);
+    struct ucred *cred = nnpfs_proc_to_cred(p);
+    nnpfs_vfs_context ctx;
     int error;
+
     int lockparent = (cnp->cn_flags & (LOCKPARENT | ISLASTCN))
 	== (LOCKPARENT | ISLASTCN);
 
@@ -284,7 +296,8 @@ nnpfs_lookup(struct vop_lookup_args * ap)
     cnp->cn_flags &= ~PDIRUNLOCK;
 #endif
 
-    error = nnpfs_lookup_common(ap->a_dvp, cnp, ap->a_vpp);
+    nnpfs_vfs_context_init(ctx, p, cred);
+    error = nnpfs_lookup_common(ap->a_dvp, cnp, ap->a_vpp, ctx);
 
     if (error == ENOENT
 	&& (cnp->cn_nameiop == CREATE || cnp->cn_nameiop == RENAME)
@@ -301,11 +314,8 @@ nnpfs_lookup(struct vop_lookup_args * ap)
 	} else if (!(cnp->cn_flags & ISLASTCN) || !lockparent) {
 	    /* if we isn't last component and is isn't requested,
 	     * return parent unlocked */
-#ifdef HAVE_FREEBSD_THREAD
-	    nnpfs_vfs_unlock (ap->a_dvp, nnpfs_cnp_to_thread(cnp));
-#else
-	    nnpfs_vfs_unlock (ap->a_dvp, nnpfs_cnp_to_proc(cnp));
-#endif
+
+	    nnpfs_vfs_unlock (ap->a_dvp, p);
 #ifdef PDIRUNLOCK
 	    cnp->cn_flags |= PDIRUNLOCK;
 #endif
@@ -369,22 +379,20 @@ nnpfs_create(struct vop_create_args *ap)
     struct componentname *cnp = ap->a_cnp;
     const char *name   = cnp->cn_nameptr;
     struct ucred *cred = cnp->cn_cred;
-#ifdef HAVE_FREEBSD_THREAD
-    d_thread_t *p     = nnpfs_cnp_to_thread(cnp);
-#else
     d_thread_t *p     = nnpfs_cnp_to_proc(cnp);
-#endif
+    nnpfs_vfs_context ctx;
     int error;
 
+    nnpfs_vfs_context_init(ctx, p, cred);
     error = nnpfs_create_common(dvp, name, ap->a_vap, cred, p);
 
     if (error == 0) {
-	error = nnpfs_lookup_common(dvp, cnp, ap->a_vpp);
+	error = nnpfs_lookup_common(dvp, cnp, ap->a_vpp, ctx);
     }
 
     cleanup_cnp (cnp, error);
 
-#if defined(__NetBSD__) || defined(__OpenBSD__) || defined(__APPLE__)
+#if defined(__NetBSD__) || defined(__OpenBSD__)
     vput (dvp);
 #endif
 
@@ -405,17 +413,12 @@ nnpfs_remove(struct vop_remove_args * ap)
     struct vnode *dvp = ap->a_dvp;
     struct vnode *vp  = ap->a_vp;
 
-#ifdef HAVE_FREEBSD_THREAD
     int error = nnpfs_remove_common(dvp, vp, cnp->cn_nameptr, 
-				  cnp->cn_cred, nnpfs_cnp_to_thread(cnp));
-#else
-    int error = nnpfs_remove_common(dvp, vp, cnp->cn_nameptr, 
-				  cnp->cn_cred, nnpfs_cnp_to_proc(cnp));
-#endif
+				    cnp->cn_cred, nnpfs_cnp_to_proc(cnp));
 
     cleanup_cnp (cnp, error);
 
-#ifndef __FreeBSD__
+#if !(defined(__FreeBSD__) || defined(__DragonFly__))
     if (dvp == vp)
 	vrele(vp);
     else
@@ -423,11 +426,6 @@ nnpfs_remove(struct vop_remove_args * ap)
     vput(dvp);
 #endif
     
-#ifdef __APPLE__
-    if (error == 0 && UBCINFOEXISTS(vp))
-	ubc_uncache(vp);
-#endif
-
     return error;
 }
 #endif /* HAVE_VOP_REMOVE */
@@ -456,11 +454,7 @@ nnpfs_rename(struct vop_rename_args * ap)
 				  tvp,
 				  ap->a_tcnp->cn_nameptr,
 				  ap->a_tcnp->cn_cred,
-#ifdef HAVE_FREEBSD_THREAD
-				  nnpfs_cnp_to_thread (ap->a_fcnp));
-#else
 				  nnpfs_cnp_to_proc (ap->a_fcnp));
-#endif
     if(tdvp == tvp)
 	vrele(tdvp);
     else
@@ -486,21 +480,19 @@ nnpfs_mkdir(struct vop_mkdir_args * ap)
     struct componentname *cnp = ap->a_cnp;
     const char *name   = cnp->cn_nameptr;
     struct ucred *cred = cnp->cn_cred;
-#ifdef HAVE_FREEBSD_THREAD
-    d_thread_t *p     = nnpfs_cnp_to_thread(cnp);
-#else
     d_thread_t *p     = nnpfs_cnp_to_proc(cnp);
-#endif
+    nnpfs_vfs_context ctx;
     int error;
 
+    nnpfs_vfs_context_init(ctx, p, cred);
     error = nnpfs_mkdir_common(dvp, name, ap->a_vap, cred, p);
 
     if (error == 0)
-	error = nnpfs_lookup_common(dvp, cnp, ap->a_vpp);
+	error = nnpfs_lookup_common(dvp, cnp, ap->a_vpp, ctx);
 
     cleanup_cnp (cnp, error);
 
-#ifndef __FreeBSD__
+#if !(defined(__FreeBSD__) || defined(__DragonFly__))
     vput(dvp);
 #endif
 
@@ -518,19 +510,18 @@ nnpfs_rmdir(struct vop_rmdir_args * ap)
    struct componentname *cnp */
 {
     struct componentname *cnp = ap->a_cnp;
+#if !(defined(__FreeBSD__) || defined(__DragonFly__))
     struct vnode *dvp = ap->a_dvp;
     struct vnode *vp  = ap->a_vp;
-    int error = nnpfs_rmdir_common(ap->a_dvp, ap->a_vp, 
-				 cnp->cn_nameptr,
-				 cnp->cn_cred,
-#ifdef HAVE_FREEBSD_THREAD
-				 nnpfs_cnp_to_thread(cnp));
-#else
-				 nnpfs_cnp_to_proc(cnp));
 #endif
-
+    
+    int error = nnpfs_rmdir_common(ap->a_dvp, ap->a_vp, 
+				   cnp->cn_nameptr,
+				   cnp->cn_cred,
+				   nnpfs_cnp_to_proc(cnp));
+    
     cleanup_cnp (cnp, error);
-#ifndef __FreeBSD__
+#if !(defined(__FreeBSD__) || defined(__DragonFly__))
     if (dvp == vp)
 	vrele(vp);
     else
@@ -544,7 +535,7 @@ nnpfs_rmdir(struct vop_rmdir_args * ap)
 
 #ifdef HAVE_VOP_READDIR
 
-#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__APPLE__)
+#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
 typedef u_long nnpfs_cookie_t;
 #elif defined(__NetBSD__)
 typedef off_t nnpfs_cookie_t;
@@ -558,18 +549,14 @@ nnpfs_readdir(struct vop_readdir_args * ap)
 	    struct uio *uiop,
 	    struct ucred *cred) */
 {
+    nnpfs_vfs_context ctx;
     int error;
     off_t off;
 
-    off = ap->a_uio->uio_offset;
+    nnpfs_vfs_context_init(ctx, nnpfs_uio_to_proc(ap->a_uio), ap->a_cred);
+    off = nnpfs_uio_offset(ap->a_uio);
 
-    error = nnpfs_readdir_common(ap->a_vp, ap->a_uio, ap->a_cred,
-#ifdef HAVE_FREEBSD_THREAD
-			       nnpfs_uio_to_thread (ap->a_uio),
-#else
-			       nnpfs_uio_to_proc (ap->a_uio),
-#endif
-			       ap->a_eofflag);
+    error = nnpfs_readdir_common(ap->a_vp, ap->a_uio, ap->a_eofflag, ctx);
 
     if (!error && ap->a_ncookies != NULL) {
 	struct uio *uio = ap->a_uio;
@@ -580,7 +567,7 @@ nnpfs_readdir(struct vop_readdir_args * ap)
 	if (uio->uio_segflg != UIO_SYSSPACE || uio->uio_iovcnt != 1)
 	    panic("nnpfs_readdir: mail arla-drinkers and tell them to bake burned cookies");
 	dp = (const struct dirent *)
-	    ((const char *)uio->uio_iov->iov_base - (uio->uio_offset - off));
+	    ((const char *)uio->uio_iov->iov_base - (nnpfs_uio_offset(uio) - off));
 
 	dp_end = (const struct dirent *) uio->uio_iov->iov_base;
 	for (dp_start = dp, ncookies = 0;
@@ -606,7 +593,7 @@ nnpfs_readdir(struct vop_readdir_args * ap)
     }
     return error;
 }
-#endif /* HAVE_VOP_READDIR */
+#endif
 
 #ifdef HAVE_VOP_LINK
 int
@@ -620,16 +607,11 @@ nnpfs_link(struct vop_link_args * ap)
     struct componentname *cnp = ap->a_cnp;
     struct vnode *vp = ap->a_vp;
     struct vnode *dvp;
-#ifdef HAVE_FREEBSD_THREAD
-    d_thread_t *p = cnp->cn_thread;
-#else
-    d_thread_t *p = cnp->cn_proc;
-#endif
     int error;
 
 #if defined (__OpenBSD__) || defined(__NetBSD__)
     dvp = ap->a_dvp;
-#elif defined(__FreeBSD__) || defined(__APPLE__)
+#elif defined(__FreeBSD__) || defined(__DragonFly__)
     dvp = ap->a_tdvp;
 #else
 #error what kind of BSD is this?
@@ -651,33 +633,31 @@ nnpfs_link(struct vop_link_args * ap)
     }
     /* FreeBSD 5.0 doesn't need to lock the vnode in VOP_LINK */
 #if !defined(__FreeBSD_version) || __FreeBSD_version < 500043
-
-    if (dvp != vp && (error = nnpfs_vfs_writelock(vp, p))) {
+    
+    if (dvp != vp && (error = nnpfs_vfs_writelock(vp,
+						  nnpfs_cnp_to_proc(cnp)))) {
 #ifdef HAVE_VOP_ABORTOP
 	    VOP_ABORTOP(dvp, cnp);
 #endif
 	    goto out;
     }
 #endif /* defined(__FreeBSD_version) || __FreeBSD_version < 500043 */
-
-    error = nnpfs_link_common(
-			   dvp,
-			   vp,
-			   cnp->cn_nameptr,
-			   cnp->cn_cred,
-#ifdef HAVE_FREEBSD_THREAD
-			   nnpfs_cnp_to_thread (cnp));
-#else
-			   nnpfs_cnp_to_proc (cnp));
-#endif
+    
+    error = nnpfs_link_common(dvp,
+			      vp,
+			      cnp->cn_nameptr,
+			      cnp->cn_cred,
+			      nnpfs_cnp_to_proc (cnp));
 
     cleanup_cnp (cnp, error);
 
+#if !defined(__FreeBSD_version) || __FreeBSD_version < 500043
     if (dvp != vp)
-	nnpfs_vfs_unlock(vp, p);
+	nnpfs_vfs_unlock(vp, nnpfs_cnp_to_proc(cnp));
+#endif
 
 out:
-#ifndef __FreeBSD__
+#if !(defined(__FreeBSD__) || defined(__DragonFly__))
     vput(dvp);
 #endif
 
@@ -697,24 +677,29 @@ nnpfs_symlink(struct vop_symlink_args * ap)
   */
 {
     struct componentname *cnp = ap->a_cnp;
+    d_thread_t *proc  = nnpfs_cnp_to_proc(cnp);
+    struct ucred *cred = nnpfs_proc_to_cred(proc);
+    nnpfs_vfs_context ctx;
     struct vnode *dvp  = ap->a_dvp;
     struct vnode **vpp = ap->a_vpp;
+    int error;
 
-    int error = nnpfs_symlink_common(dvp,
-				   vpp,
-				   cnp,
-				   ap->a_vap,
-				   ap->a_target);
-
+    nnpfs_vfs_context_init(ctx, proc, cred);
+    error = nnpfs_symlink_common(dvp,
+				 vpp,
+				 cnp,
+				 ap->a_vap,
+				 ap->a_target,
+				 ctx);
     if (error == 0) {
-	error = nnpfs_lookup_common(dvp, cnp, vpp);
-#if (!defined(__FreeBSD__) || __FreeBSD_version < 400012) && (!defined(__NetBSD__) || __NetBSD_Version__ < 105240000) && (!defined(__OpenBSD__) || OpenBSD <= 200211)
+	error = nnpfs_lookup_common(dvp, cnp, vpp, ctx);
+#if (!defined(__FreeBSD__) || __FreeBSD_version < 400012) && (!defined(__NetBSD__) || __NetBSD_Version__ < 105240000) && !defined(__DragonFly__)
 	if (error == 0)
 	    vput (*vpp);
 #endif
     }
     cleanup_cnp (cnp, error);
-#ifndef __FreeBSD__
+#if !(defined(__FreeBSD__) || defined(__DragonFly__))
     if (error || dvp != *vpp)
         vput(dvp);
 #endif
@@ -730,18 +715,19 @@ nnpfs_readlink(struct vop_readlink_args * ap)
 	     struct uio *uiop,
 	     struct ucred *cred) */
 {
-    return nnpfs_readlink_common(ap->a_vp, ap->a_uio, ap->a_cred);
+    nnpfs_vfs_context ctx;
+    nnpfs_vfs_context_init(ctx, nnpfs_uio_to_proc(ap->a_uio), ap->a_cred);
+
+    return nnpfs_readlink_common(ap->a_vp, ap->a_uio, ctx);
 }
 #endif /* HAVE_VOP_READLINK */
 
 #ifdef HAVE_VOP_INACTIVE
 int
 nnpfs_inactive(struct vop_inactive_args * ap)
-     /*struct vnode *vp,
-	     struct ucred *cred)*/
 {
 #ifdef HAVE_FREEBSD_THREAD
-    return nnpfs_inactive_common(ap->a_vp, nnpfs_curthread());
+    return nnpfs_inactive_common(ap->a_vp, ap->a_td);
 #else
     return nnpfs_inactive_common(ap->a_vp, nnpfs_curproc());
 #endif
@@ -782,10 +768,10 @@ nnpfs_lock(struct vop_lock_args * ap)
     int ret;
 
     NNPFSDEB(XDEBVNOPS, ("nnpfs_lock: %lx, flags 0x%x\n",
-		       (unsigned long)vp, flags));
+			 (unsigned long)vp, flags));
 
     if (l == NULL)
-      panic("nnpfs_lock: lock NULL");
+	panic("nnpfs_lock: lock NULL");
 
     NNPFSDEB(XDEBVNOPS, ("nnpfs_lock before: lk flags: %d share: %d "
 			 "wait: %d excl: %d holder: 0x%llx\n",
@@ -796,22 +782,17 @@ nnpfs_lock(struct vop_lock_args * ap)
 
 #ifndef	DEBUG_LOCKS
 #ifdef HAVE_FOUR_ARGUMENT_LOCKMGR
-#ifdef HAVE_FREEBSD_THREAD
-    ret = lockmgr(l, flags, &vp->v_interlock, ap->a_td);
+#ifdef __DragonFly__
+    ret = lockmgr(l, flags, ap->a_vlock, ap->a_td);
 #else
-    ret = lockmgr(l, flags, &vp->v_interlock, ap->a_p);
+    ret = lockmgr(l, flags, &vp->v_interlock, NNPFS_AP_PROC(ap));
 #endif
 #else
     ret = lockmgr(l, flags, &vp->v_interlock);
 #endif
 #else
-#ifdef HAVE_FREEBSD_THREAD
-    ret = debuglockmgr(l, flags, &vp->v_interlock, ap->a_td,
-			"nnpfs_lock", ap->a_vp->filename, ap->a_vp->line);
-#else
-    ret = debuglockmgr(l, flags, &vp->v_interlock, ap->a_p,
-			"nnpfs_lock", ap->a_vp->filename, ap->a_vp->line);
-#endif
+    ret = debuglockmgr(l, flags, &vp->v_interlock, NNPFS_AP_PROC(ap),
+		       "nnpfs_lock", ap->a_vp->filename, ap->a_vp->line);
 #endif
     NNPFSDEB(XDEBVNOPS, ("nnpfs_lock: lk flags: %d share: %d "
 			 "wait: %d excl: %d holder: 0x%llx\n",
@@ -834,38 +815,33 @@ nnpfs_unlock(struct vop_unlock_args * ap)
     int ret;
 
     if (l == NULL)
-      panic("nnpfs_unlock: lock NULL");
+	panic("nnpfs_unlock: lock NULL");
 
     NNPFSDEB(XDEBVNOPS,
-	   ("nnpfs_unlock: %lx, flags 0x%x, l %lx, ap %lx\n",
-	    (unsigned long)vp, flags,
-	    (unsigned long)l,
-	    (unsigned long)ap));
-
+	     ("nnpfs_unlock: %lx, flags 0x%x, l %lx, ap %lx\n",
+	      (unsigned long)vp, flags,
+	      (unsigned long)l,
+	      (unsigned long)ap));
+    
     NNPFSDEB(XDEBVNOPS, ("nnpfs_unlock: lk flags: %d share: %d "
-			 "wait: %d excl: %d holder: 0x%lld\n",
+			 "wait: %d excl: %d holder: 0x%llx\n",
 			 l->lk_flags, l->lk_sharecount, l->lk_waitcount,
 			 l->lk_exclusivecount, 
 			 (unsigned long long)
 			 (nnpfs_uintptr_t)l->lk_lockholder));
 #ifndef	DEBUG_LOCKS
 #ifdef HAVE_FOUR_ARGUMENT_LOCKMGR
-#ifdef HAVE_FREEBSD_THREAD
-    ret = lockmgr (l, flags | LK_RELEASE, &vp->v_interlock, ap->a_td);
+#ifdef __DragonFly__
+    ret = lockmgr (l, flags | LK_RELEASE, ap->a_vlock, ap->a_td);
 #else
-    ret = lockmgr (l, flags | LK_RELEASE, &vp->v_interlock, ap->a_p);
+    ret = lockmgr (l, flags | LK_RELEASE, &vp->v_interlock, NNPFS_AP_PROC(ap));
 #endif
 #else
     ret = lockmgr (l, flags | LK_RELEASE, &vp->v_interlock);
 #endif
 #else
-#ifdef HAVE_FREEBSD_THREAD
-    ret = debuglockmgr (l, flags | LK_RELEASE, &vp->v_interlock, ap->a_td,
+    ret = debuglockmgr (l, flags | LK_RELEASE, &vp->v_interlock, NNPFS_AP_PROC(ap),
 			"nnpfs_lock", ap->a_vp->filename, ap->a_vp->line);
-#else
-    ret = debuglockmgr (l, flags | LK_RELEASE, &vp->v_interlock, ap->a_p,
-			"nnpfs_lock", ap->a_vp->filename, ap->a_vp->line);
-#endif
 #endif
     NNPFSDEB(XDEBVNOPS, ("nnpfs_unlock: return %d\n", ret));
     return ret;
@@ -878,17 +854,22 @@ nnpfs_islocked (struct vop_islocked_args *ap)
 {
     struct vnode *vp    = ap->a_vp;
     struct nnpfs_node *xn = VNODE_TO_XNODE(vp);
-    nnpfs_vnode_lock *l   = &xn->lock;
+    nnpfs_vnode_lock *l;
 
     NNPFSDEB(XDEBVNOPS, ("nnpfs_islocked: %lx\n",
-		       (unsigned long)vp));
+			 (unsigned long)vp));
+    
+    /*
+     * If the node doesn't exist, we haven't locked it.  This keeps
+     * FreeBSD's DEBUG_VFS_LOCKS asserts happy after reclaim.
+     */
+    if (!xn)
+        return 0;
+
+    l = &xn->lock;
 
 #if defined(HAVE_TWO_ARGUMENT_LOCKSTATUS)
-#ifdef HAVE_FREEBSD_THREAD
-    return lockstatus (l, ap->a_td);
-#else
-    return lockstatus (l, ap->a_p);
-#endif
+    return lockstatus (l, NNPFS_AP_PROC(ap));
 #elif defined(HAVE_ONE_ARGUMENT_LOCKSTATUS)
     return lockstatus (l);
 #else
@@ -911,7 +892,7 @@ nnpfs_lock(struct vop_lock_args * ap)
 
     while (vp->v_flag & VXLOCK) {
 	vp->v_flag |= VXWANT;
-	(void) tsleep((caddr_t)vp, PINOD, "nnpfs_vnlock", 0);
+	(void) nnpfs_tsleep((caddr_t)vp, PINOD, "nnpfs_vnlock");
     }
     if (vp->v_tag == VT_NON)
 	return (ENOENT);
@@ -1021,6 +1002,90 @@ nnpfs_bmap(struct vop_bmap_args *ap)
 
 #ifdef HAVE_VOP_GETPAGES
 
+#ifdef __DragonFly__
+#include <sys/sfbuf.h>
+#include <vm/vm.h>
+#include <vm/vm_object.h>
+#include <vm/vm_page.h>
+#include <vm/vm_pager.h>
+#include <vm/vm_map.h>
+#include <vm/vnode_pager.h>
+#include <vm/vm_extern.h>
+
+int
+nnpfs_getpages (struct vop_getpages_args *ap)
+{
+    int i, count, error, size;
+    struct ucred *creds;
+    vm_object_t object;
+    struct iovec aiov;
+    struct uio auio;
+    vm_offset_t kva;
+    struct sf_buf *sf;
+    vm_page_t m;
+
+    count = ap->a_count / PAGE_SIZE;
+    m = ap->a_m[ap->a_reqpage];
+    object = ap->a_vp->v_object;
+
+    for (i = 0; i < count; i++)
+	if (i != ap->a_reqpage)
+	    vnode_pager_freepage(ap->a_m[i]);
+
+    mycpu->gd_cnt.v_vnodein++;
+    mycpu->gd_cnt.v_vnodepgsin++;
+
+    creds = VNODE_TO_XNODE(ap->a_vp)->rd_cred;
+
+    /* is the start or end of the page EOF ? */
+    if (IDX_TO_OFF(m->pindex) >= object->un_pager.vnp.vnp_size)
+	return VM_PAGER_BAD;
+
+    size = PAGE_SIZE;
+    if (IDX_TO_OFF(m->pindex) + size > object->un_pager.vnp.vnp_size)
+	size = object->un_pager.vnp.vnp_size - IDX_TO_OFF(m->pindex);
+    
+    sf = sf_buf_alloc(m, 0);
+    kva = sf_buf_kva(sf);
+    
+    aiov.iov_base = (caddr_t) kva;
+    aiov.iov_len = size;
+    auio.uio_iov = &aiov;
+    auio.uio_iovcnt = 1;
+    nnpfs_uio_setoffset(auio, IDX_TO_OFF(m->pindex));
+    auio.uio_segflg = UIO_SYSSPACE;
+    auio.uio_rw = UIO_READ;
+    auio.uio_resid = size;
+    auio.uio_td = nnpfs_curthread();
+    
+    error = VOP_READ(object->handle, &auio, 0, creds);
+    if (error == 0) {
+	int count = size - auio.uio_resid;
+	
+	if (count == 0)
+	    error = EINVAL;
+	else if (count != PAGE_SIZE)
+	    bzero((caddr_t) kva + count, PAGE_SIZE - count);
+    }
+    sf_buf_free(sf);
+
+    pmap_clear_modify(m);
+    vm_page_undirty(m);
+    vm_page_flag_clear(m, PG_ZERO);
+    if (error == 0)
+	m->valid = VM_PAGE_BITS_ALL;
+
+    NNPFSDEB(XDEBVNOPS, ("nnpfs_getpages = %d\n", error));
+
+    if (error)
+	return VM_PAGER_ERROR;
+    return VM_PAGER_OK;
+}
+
+#else  /* !__DragonFly__ */
+
+#if !HAVE_KERNEL_VNODE_PAGER_GENERIC_GETPAGES
+
 static size_t
 get_pages_endlength (struct vop_getpages_args *ap)
 {
@@ -1032,6 +1097,8 @@ get_pages_endlength (struct vop_getpages_args *ap)
 #endif
 }
 
+#endif /* !HAVE_KERNEL_VNODE_PAGER_GENERIC_GETPAGES */
+
 int
 nnpfs_getpages (struct vop_getpages_args *ap)
      /* Old BSD
@@ -1060,9 +1127,8 @@ nnpfs_getpages (struct vop_getpages_args *ap)
     error = vnode_pager_generic_getpages (ap->a_vp, ap->a_m, 
 					  ap->a_count, ap->a_reqpage);
 #else
-    error = nnpfs_data_valid (ap->a_vp, VNODE_TO_XNODE(ap->a_vp)->rd_cred,
-			    nnpfs_curproc(), NNPFS_DATA_R,
-			    get_pages_endlength(ap));
+    error = nnpfs_data_valid(ap->a_vp, &VNODE_TO_XNODE(ap->a_vp)->rd_cred,
+			     NNPFS_DATA_R, get_pages_endlength(ap));
     if (error == 0)
 	error = VOP_GETPAGES(DATA_FROM_VNODE(ap->a_vp), 
 			     ap->a_offset, ap->a_m,
@@ -1072,9 +1138,18 @@ nnpfs_getpages (struct vop_getpages_args *ap)
     NNPFSDEB(XDEBVNOPS, ("nnpfs_getpages = %d\n", error));
     return error;
 }
+
+#endif /* __DragonFly__ */
+
 #endif /* HAVE_VOP_GETPAGES */
 
 #ifdef HAVE_VOP_PUTPAGES
+
+#ifdef __FreeBSD__
+#include <vm/vm_page.h>
+#include <vm/vm_pager.h>
+#endif
+
 int
 nnpfs_putpages (struct vop_putpages_args *ap)
      /* Old BSD
@@ -1095,7 +1170,6 @@ nnpfs_putpages (struct vop_putpages_args *ap)
     struct vnode *vp    = ap->a_vp;
     struct nnpfs_node *xn = VNODE_TO_XNODE(vp);
     struct vnode *t     = DATA_FROM_XNODE(xn);
-    int error;
 
     NNPFSDEB(XDEBVNOPS, ("nnpfs_putpages\n"));
 
@@ -1105,8 +1179,25 @@ nnpfs_putpages (struct vop_putpages_args *ap)
 #ifdef HAVE_STRUCT_VOP_PUTPAGES_ARGS_A_SYNC /* FreeBSD-style */
     xn->flags |= NNPFS_DATA_DIRTY;
 
-    return VOP_PUTPAGES(t, ap->a_m, ap->a_count, ap->a_sync, ap->a_rtvals,
-			ap->a_offset);
+    {
+	int count = ap->a_count / PAGE_SIZE;
+	int *retvals = ap->a_rtvals;
+	int ret;
+	int i;
+	
+	nnpfs_vfs_writelock(t, nnpfs_curproc());
+	ret = VOP_PUTPAGES(t, ap->a_m, ap->a_count, ap->a_sync, ap->a_rtvals,
+			   ap->a_offset);
+	nnpfs_vfs_unlock(t, nnpfs_curproc());
+
+	for (i = 0; i < count; i++) {
+	    if (retvals[i] == VM_PAGER_OK)
+		vm_page_undirty(ap->a_m[i]);
+	} 
+
+	return ret;
+    }
+
 #else /* NetBSD-style */
 #if defined(__NetBSD__) && __NetBSD_Version__  >= 105250000 
     /* XXX should only walk over those pages that is requested */
@@ -1210,10 +1301,15 @@ nnpfs_advlock(struct vop_advlock_args *v)
     ret = nnpfs_advlock_common (xn, );
 
     return ret;
-#elif defined(HAVE_KERNEL_LF_ADVLOCK) && !defined(__OpenBSD__) && !defined(__APPLE__)
+#elif defined(HAVE_KERNEL_LF_ADVLOCK)
     struct nnpfs_node *xn = VNODE_TO_XNODE(ap->a_vp);
  
-    return lf_advlock(ap, &xn->lockf, xn->attr.va_size);
+#ifdef __OpenBSD__
+    return lf_advlock(&xn->lockf, nnpfs_vattr_get_size(&xn->attr), ap->a_id,
+		      ap->a_op, ap->a_fl, ap->a_flags);
+#else
+    return lf_advlock(ap, &xn->lockf, nnpfs_vattr_get_size(&xn->attr));
+#endif /* __OpenBSD__ */
 #else
      return EOPNOTSUPP;
 #endif
@@ -1234,97 +1330,6 @@ nnpfs_revoke(struct vop_revoke_args *v)
 }
 #endif /* HAVE_VOP_REVOKE */
 
-#ifdef HAVE_VOP_PAGEIN
-int
-nnpfs_pagein(struct vop_pagein_args *ap)
-{
-#ifdef __APPLE__
-    struct uio uio;
-    struct iovec iov;
-    int ret;
-
-    kernel_upl_map(kernel_map, ap->a_pl, &iov.iov_base);
-    iov.iov_base+=ap->a_pl_offset;
-    iov.iov_len=ap->a_size;
-
-    uio.uio_iov=&iov;
-    uio.uio_iovcnt=1;
-    uio.uio_offset=ap->a_f_offset;
-    uio.uio_resid=ap->a_size;
-    uio.uio_segflg=UIO_SYSSPACE; /* XXX what is it? */
-    uio.uio_rw=UIO_READ;
-    uio.uio_procp=nnpfs_curproc();
-
-    ret = VOP_READ(ap->a_vp, &uio, 0, ap->a_cred);
-
-    /* Zero out rest of last page if there wasn't enough data in the file */
-    if (ret == 0 && uio.uio_resid > 0)
-	bzero(iov.iov_base, uio.uio_resid);
-    
-    kernel_upl_unmap(kernel_map, ap->a_pl);
-
-    if (ret) {
-	kernel_upl_abort_range(ap->a_pl, ap->a_pl_offset, ap->a_size,
-			       UPL_ABORT_ERROR | UPL_ABORT_FREE_ON_EMPTY);
-    } else {
-	kernel_upl_commit_range(ap->a_pl, ap->a_pl_offset, ap->a_size,
-				UPL_COMMIT_CLEAR_DIRTY |
-				UPL_COMMIT_FREE_ON_EMPTY,
-				UPL_GET_INTERNAL_PAGE_LIST(ap->a_pl),
-				MAX_UPL_TRANSFER);
-    }
-
-    return ret;
-#else
-#error pagein on non apple ?
-#endif
-}
-  
-#endif
-
-#ifdef HAVE_VOP_PAGEOUT
-int
-nnpfs_pageout(struct vop_pageout_args *ap)
-{
-#ifdef __APPLE__
-    struct uio uio;
-    struct iovec iov;
-    int ret;
-
-    kernel_upl_map(kernel_map, ap->a_pl, &iov.iov_base);
-    iov.iov_base+=ap->a_pl_offset;
-    iov.iov_len=ap->a_size;
-
-    uio.uio_iov=&iov;
-    uio.uio_iovcnt=1;
-    uio.uio_offset=ap->a_f_offset;
-    uio.uio_resid=ap->a_size;
-    uio.uio_segflg=UIO_SYSSPACE; /* XXX what is it? */
-    uio.uio_rw=UIO_WRITE;
-    uio.uio_procp=nnpfs_curproc();
-
-    ret = VOP_WRITE(ap->a_vp, &uio, 0, ap->a_cred);
-
-    kernel_upl_unmap(kernel_map, ap->a_pl);
-
-    if (ret) {
-	kernel_upl_abort_range(ap->a_pl, ap->a_pl_offset, ap->a_size,
-			       UPL_ABORT_FREE_ON_EMPTY);
-    } else {
-	kernel_upl_commit_range(ap->a_pl, ap->a_pl_offset, ap->a_size,
-				UPL_COMMIT_CLEAR_DIRTY |
-				UPL_COMMIT_FREE_ON_EMPTY,
-				UPL_GET_INTERNAL_PAGE_LIST(ap->a_pl),
-				MAX_UPL_TRANSFER);
-    }
-
-    return ret;
-#else
-#error pageout on non apple ?
-#endif
-}
-#endif
-
 #ifdef HAVE_VOP_CREATEVOBJECT
 int
 nnpfs_createvobject(struct vop_createvobject_args *ap)
@@ -1413,15 +1418,102 @@ nnpfs_returnzero (struct vop_generic_args *ap)
 }
 
 void
-nnpfs_pushdirty(struct vnode *vp, struct ucred *cred, d_thread_t *p)
+nnpfs_pushdirty(struct vnode *vp)
 {
-#if defined(__NetBSD_Version__) && __NetBSD_Version__ >= 105280000
+#ifdef __NetBSD__
+    if ((vp->v_flag & VONWORKLST) == 0)
+	return;
+    simple_lock(&vp->v_interlock);
     VOP_PUTPAGES(vp, 0, 0, PGO_ALLPAGES|PGO_SYNCIO|PGO_CLEANIT);
-#elif defined(__APPLE__)
-    ubc_pushdirty(vp);
 #endif
 }
 
+#ifdef __FreeBSD__
+
+static void
+filt_nnpfsdetach(struct knote *kn)
+{
+    struct vnode *vp = (struct vnode *)kn->kn_hook;
+    knlist_remove(&vp->v_pollinfo->vpi_selinfo.si_note, kn, 0);
+}
+
+static int
+filt_nnpfsread(struct knote *kn, long hint)
+{
+    struct vnode *vp = (struct vnode *)kn->kn_hook;
+    struct nnpfs_node *xn = VNODE_TO_XNODE(vp);
+
+    if (hint == NOTE_REVOKE)
+	kn->kn_flags |= (EV_EOF | EV_ONESHOT);
+
+    kn->kn_data = nnpfs_vattr_get_size(&xn->attr) - kn->kn_fp->f_offset;
+    return (kn->kn_data != 0);
+
+    kn->kn_data = 0;
+    return (1);
+}
+
+static int
+filt_nnpfswrite(struct knote *kn, long hint)
+{
+    if (hint == NOTE_REVOKE)
+	kn->kn_flags |= (EV_EOF | EV_ONESHOT);
+
+    kn->kn_data = 0;
+    return (1);
+}
+
+static int
+filt_nnpfsvnode(struct knote *kn, long hint)
+{
+    if (kn->kn_sfflags & hint)
+	kn->kn_fflags |= hint;
+    if (hint == NOTE_REVOKE) {
+	kn->kn_flags |= EV_EOF;
+	return (1);
+    }
+    return (kn->kn_fflags != 0);
+}
+
+static struct filterops nnpfsread_filtops = 
+	{ 1, NULL, filt_nnpfsdetach, filt_nnpfsread };
+static struct filterops nnpfswrite_filtops = 
+	{ 1, NULL, filt_nnpfsdetach, filt_nnpfswrite };
+static struct filterops nnpfsvnode_filtops = 
+	{ 1, NULL, filt_nnpfsdetach, filt_nnpfsvnode };
+
+int
+nnpfs_kqfilter(struct vop_kqfilter_args *ap)
+{
+    struct vnode *vp = ap->a_vp;
+    struct knote *kn = ap->a_kn;
+
+    switch (kn->kn_filter) {
+    case EVFILT_READ:
+	kn->kn_fop = &nnpfsread_filtops;
+	break;
+    case EVFILT_WRITE:
+	kn->kn_fop = &nnpfswrite_filtops;
+	break;
+    case EVFILT_VNODE:
+	kn->kn_fop = &nnpfsvnode_filtops;
+	break;
+    default:
+	return (1);
+    }
+
+    kn->kn_hook = (caddr_t)vp;
+
+    if (vp->v_pollinfo == NULL)
+	v_addpollinfo(vp);
+    if (vp->v_pollinfo == NULL)
+	return ENOMEM;
+    knlist_add(&vp->v_pollinfo->vpi_selinfo.si_note, kn, 0);
+
+    return (0);
+}
+
+#endif /* __FreeBSD__ */
 
 
 static struct vnodeopv_entry_desc nnpfs_vnodeop_entries[] = {
@@ -1555,6 +1647,9 @@ static struct vnodeopv_entry_desc nnpfs_vnodeop_entries[] = {
 #endif
 #ifdef HAVE_VOP_PATHCONF
     {&vop_pathconf_desc, (vop_t *) nnpfs_pathconf },
+#endif
+#ifdef __FreeBSD__
+    {&vop_kqfilter_desc, (vop_t *) nnpfs_kqfilter },
 #endif
     {(struct vnodeop_desc *) NULL, (int (*) (void *)) NULL}
 };
