@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2002, 2004 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2002, 2004 - 2005 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -44,17 +44,19 @@
 #include <nnpfs/nnpfs_syscalls.h>
 #include <nnpfs/nnpfs_vnodeops.h>
 
-RCSID("$Id: nnpfs_vnodeops-common.c,v 1.99 2004/05/04 20:42:47 lha Exp $");
+RCSID("$Id: nnpfs_vnodeops-common.c,v 1.113 2005/11/21 11:25:40 tol Exp $");
 
 static void
 nnpfs_handle_stale(struct nnpfs_node *xn)
 {
+#if __APPLE__
     struct vnode *vp = XNODE_TO_VNODE(xn);
+#endif
 
     if ((xn->flags & NNPFS_STALE) == 0)
 	return;
 
-#if __APPLE__
+#if 0
     if (UBCISVALID(vp) && !ubc_isinuse(vp, 1)) {
 	xn->flags &= ~NNPFS_STALE;
 	ubc_setsize(vp, 0);
@@ -66,11 +68,11 @@ nnpfs_handle_stale(struct nnpfs_node *xn)
 }
 
 int
-nnpfs_open_valid(struct vnode *vp, struct ucred *cred, d_thread_t *p,
-	       u_int tok)
+nnpfs_open_valid(struct vnode *vp, nnpfs_vfs_context ctx, u_int tok)
 {
     struct nnpfs *nnpfsp = NNPFS_FROM_VNODE(vp);
     struct nnpfs_node *xn = VNODE_TO_XNODE(vp);
+    struct ucred *cred = nnpfs_vfs_context_ucred(ctx);
     int error = 0;
 
     NNPFSDEB(XDEBVFOPS, ("nnpfs_open_valid\n"));
@@ -87,10 +89,11 @@ nnpfs_open_valid(struct vnode *vp, struct ucred *cred, d_thread_t *p,
 	    msg.handle = xn->handle;
 	    msg.tokens = tok;
 
-	    error = nnpfs_message_rpc(nnpfsp->fd, &msg.header, sizeof(msg), p);
-
+	    error = nnpfs_message_rpc(nnpfsp->fd, &msg.header,
+				      sizeof(msg), nnpfs_vfs_context_proc(ctx));
 	    if (error == 0)
-		error = ((struct nnpfs_message_wakeup *) & msg)->error;
+		error = NNPFS_MSG_WAKEUP_ERROR(&msg);
+
 	} else {
 	    goto done;
 	}
@@ -104,7 +107,7 @@ done:
 
 int
 nnpfs_attr_valid(struct vnode *vp, struct ucred *cred, d_thread_t *p,
-	       u_int tok)
+		 u_int tok)
 {
     struct nnpfs *nnpfsp = NNPFS_FROM_VNODE(vp);
     struct nnpfs_node *xn = VNODE_TO_XNODE(vp);
@@ -121,7 +124,7 @@ nnpfs_attr_valid(struct vnode *vp, struct ucred *cred, d_thread_t *p,
 	    msg.handle = xn->handle;
 	    error = nnpfs_message_rpc(nnpfsp->fd, &msg.header, sizeof(msg), p);
 	    if (error == 0)
-		error = ((struct nnpfs_message_wakeup *) & msg)->error;
+		error = NNPFS_MSG_WAKEUP_ERROR(&msg);
 	} else {
 	    goto done;
 	}
@@ -132,26 +135,26 @@ done:
 }
 
 int
-nnpfs_data_valid(struct vnode *vp, struct ucred *cred, d_thread_t *p,
-	       u_int tok, uint32_t want_offset)
+nnpfs_data_valid(struct vnode *vp, nnpfs_cred *cred,
+		 u_int tok, off_t want_offset)
 {
     struct nnpfs *nnpfsp = NNPFS_FROM_VNODE(vp);
     struct nnpfs_node *xn = VNODE_TO_XNODE(vp);
     int error = 0;
-    uint32_t offset;
+    off_t offset;
     struct nnpfs_message_getdata msg;
 
     do {
 	offset = want_offset;
-	if (NNPFS_TOKEN_GOT(xn, tok|NNPFS_ATTR_R) && offset > xn->attr.va_size) {
-	    offset = xn->attr.va_size;
+	if (NNPFS_TOKEN_GOT(xn, tok|NNPFS_ATTR_R) && offset > nnpfs_vattr_get_size(&xn->attr)) {
+	    offset = nnpfs_vattr_get_size(&xn->attr);
 	}
     
 	NNPFSDEB(XDEBVNOPS, ("nnpfs_data_valid: offset: want %ld has %ld, "
 			   "tokens: want %lx has %lx length: %ld\n",
 			   (long) offset, (long) xn->offset,
 			   (long) tok, (long) xn->tokens,
-			   (long) xn->attr.va_size));
+			   (long) nnpfs_vattr_get_size(&xn->attr)));
 
 	if (NNPFS_TOKEN_GOT(xn, tok)) {
 	    if (offset <= xn->offset || xn->attr.va_type == VDIR) {
@@ -160,16 +163,15 @@ nnpfs_data_valid(struct vnode *vp, struct ucred *cred, d_thread_t *p,
 	}
 
 	msg.header.opcode = NNPFS_MSG_GETDATA;
-	msg.cred.uid = cred->cr_uid;
-	msg.cred.pag = nnpfs_get_pag(cred);
+	msg.cred = *cred;
 	msg.handle = xn->handle;
 	msg.tokens = tok;
 	msg.offset = offset;
 	
-	error = nnpfs_message_rpc(nnpfsp->fd, &msg.header, sizeof(msg), p);
-	
+	error = nnpfs_message_rpc(nnpfsp->fd, &msg.header, sizeof(msg),
+				  nnpfs_curproc());
 	if (error == 0)
-	    error = ((struct nnpfs_message_wakeup *) & msg)->error;
+	    error = NNPFS_MSG_WAKEUP_ERROR(&msg);
 	
     } while (error == 0);
 
@@ -178,56 +180,57 @@ nnpfs_data_valid(struct vnode *vp, struct ucred *cred, d_thread_t *p,
 
 int
 nnpfs_open_common(struct vnode *vp,
-		int mode,
-		struct ucred *cred,
-		d_thread_t *p)
+		  int mode,
+		  nnpfs_vfs_context ctx)
 {
     struct nnpfs_node *xn = VNODE_TO_XNODE(vp);
+    struct ucred *cred = nnpfs_vfs_context_ucred(ctx);
     int ret;
 
     NNPFSDEB(XDEBVNOPS, ("nnpfs_open\n"));
 
     if (mode & FWRITE) {
-	ret = nnpfs_open_valid(vp, cred, p, NNPFS_OPEN_NW);
+	ret = nnpfs_open_valid(vp, ctx, NNPFS_OPEN_NW);
+#ifdef __APPLE__
+	xn->writers++;
+#endif	
     } else {
-	ret = nnpfs_open_valid(vp, cred, p, NNPFS_OPEN_NR);
+	ret = nnpfs_open_valid(vp, ctx, NNPFS_OPEN_NR);
     }
 
     /* always update the read cred */
 
     if (mode & FWRITE)
-	nnpfs_update_write_cred(xn, cred);
-    nnpfs_update_read_cred(xn, cred);
+	nnpfs_setcred(&xn->wr_cred, cred);
+    nnpfs_setcred(&xn->rd_cred, cred);
 
     return ret;
 }
 
 static int
 do_fsync(struct nnpfs *nnpfsp,
-	 struct nnpfs_node *xn,
-	 struct ucred *cred,
-	 d_thread_t *p,
-	 u_int flag)
+         struct nnpfs_node *xn,
+         struct ucred *cred,
+         nnpfs_cred *ncred,
+         d_thread_t *p,
+         u_int flag)
 {
-    int error;
     struct nnpfs_message_putdata msg;
+    int error;
 
     msg.header.opcode = NNPFS_MSG_PUTDATA;
-    if (cred != NOCRED) {
-	msg.cred.uid = cred->cr_uid;
-	msg.cred.pag = nnpfs_get_pag(cred);
-    } else {
-	msg.cred.uid = 0;
-	msg.cred.pag = NNPFS_ANONYMOUSID;
-    }
     msg.handle = xn->handle;
     vattr2nnpfs_attr(&xn->attr, &msg.attr);
     msg.flag   = flag;
 
-    error = nnpfs_message_rpc(nnpfsp->fd, &msg.header, sizeof(msg), p);
+    if (ncred != NULL)
+	msg.cred = *ncred;
+    else
+	nnpfs_setcred(&msg.cred, cred);
 
+    error = nnpfs_message_rpc(nnpfsp->fd, &msg.header, sizeof(msg), p);
     if (error == 0)
-	error = ((struct nnpfs_message_wakeup *) & msg)->error;
+	error = NNPFS_MSG_WAKEUP_ERROR(&msg);
 
     if (error == 0)
 	xn->flags &= ~NNPFS_DATA_DIRTY;
@@ -236,8 +239,8 @@ do_fsync(struct nnpfs *nnpfsp,
 }
 
 int
-nnpfs_fsync_common(struct vnode *vp, struct ucred *cred,
-		 int waitfor, d_thread_t *proc)
+nnpfs_fsync_common(struct vnode *vp, struct ucred *cred, nnpfs_cred *ncred,
+		   int waitfor, d_thread_t *proc)
 {
     struct nnpfs *nnpfsp = NNPFS_FROM_VNODE(vp);
     struct nnpfs_node *xn = VNODE_TO_XNODE(vp);
@@ -255,7 +258,7 @@ nnpfs_fsync_common(struct vnode *vp, struct ucred *cred,
 	return 0;
     }
     
-    nnpfs_pushdirty(vp, cred, proc);
+    nnpfs_pushdirty(vp);
 
     if (xn->flags & NNPFS_DATA_DIRTY) {
 #ifdef FSYNC_RECLAIM
@@ -266,7 +269,7 @@ nnpfs_fsync_common(struct vnode *vp, struct ucred *cred,
 	    return 0;
 	}
 #endif    
-	error = do_fsync(nnpfsp, xn, cred, proc, NNPFS_WRITE | NNPFS_FSYNC);
+	error = do_fsync(nnpfsp, xn, cred, ncred, proc, NNPFS_WRITE | NNPFS_FSYNC);
     }
 
     return error;
@@ -274,29 +277,35 @@ nnpfs_fsync_common(struct vnode *vp, struct ucred *cred,
 
 int
 nnpfs_close_common(struct vnode *vp, int fflag,
-		 d_thread_t *proc, struct ucred *cred)
+		   d_thread_t *proc, struct ucred *cred)
 {
     struct nnpfs *nnpfsp = NNPFS_FROM_VNODE(vp);
     struct nnpfs_node *xn = VNODE_TO_XNODE(vp);
     int error = 0;
     
     NNPFSDEB(XDEBVNOPS,
-	   ("nnpfs_close cred = %lx, fflag = %x, xn->flags = %x\n",
-	    (unsigned long)cred, fflag, xn->flags));
-
-    if (vp->v_type == VREG)
-	nnpfs_pushdirty(vp, cred, proc);
-
-    if (fflag & FWRITE && xn->flags & NNPFS_DATA_DIRTY)
-	error = do_fsync(nnpfsp, xn, cred, proc, NNPFS_WRITE);
-
+	     ("nnpfs_close cred = %lx, fflag = %x, xn->flags = %x\n",
+	      (unsigned long)cred, fflag, xn->flags));
+    
+    if (nnpfs_vnode_isreg(vp))
+        nnpfs_pushdirty(vp);
+    
+    if (fflag & FWRITE && xn->flags & NNPFS_DATA_DIRTY) {
+	error = do_fsync(nnpfsp, xn, cred, NULL, proc, NNPFS_WRITE);
+	
+#ifdef __APPLE__
+	if (--xn->writers < 0)
+	    panic("xn -ve writers");
+#endif	
+    }
+    
     return error;
 }
 
-size_t
+off_t
 nnpfs_uio_end_length (struct uio *uio)
 {
-#if DIAGNOSTIC
+#if defined(DIAGNOSTIC) && !defined(__APPLE__)
     size_t sz = 0;
     int i;
 
@@ -305,42 +314,27 @@ nnpfs_uio_end_length (struct uio *uio)
     if (sz != uio->uio_resid)
 	panic("nnpfs_uio_end_length");
 #endif
-    return uio->uio_offset + uio->uio_resid;
+    return nnpfs_uio_offset(uio) + nnpfs_uio_resid(uio);
 }
 
 
 int
-nnpfs_read_common(struct vnode *vp, struct uio *uio, int ioflag,
-		struct ucred *cred)
+nnpfs_read_common(struct vnode *vp, struct uio *uio, int ioflag, struct ucred *cred)
 {
     int error = 0;
-    int i;
 
     NNPFSDEB(XDEBVNOPS, ("nnpfs_read\n"));
 
-    nnpfs_update_read_cred(VNODE_TO_XNODE(vp), cred);
+    nnpfs_setcred(&(VNODE_TO_XNODE(vp))->rd_cred, cred); 
 
-#ifdef HAVE_FREEBSD_THREAD
-    error = nnpfs_data_valid(vp, cred, nnpfs_uio_to_thread(uio), NNPFS_DATA_R,
-			   nnpfs_uio_end_length(uio));
-#else
-    error = nnpfs_data_valid(vp, cred, nnpfs_uio_to_proc(uio), NNPFS_DATA_R,
-			   nnpfs_uio_end_length(uio));
-#endif
-
-    NNPFSDEB(XDEBVNOPS, ("nnpfs_read: iovcnt: %d\n", uio->uio_iovcnt));
-    for (i = 0; i < uio->uio_iovcnt; i++)
-	NNPFSDEB(XDEBVNOPS, ("  base: %lx len: %lu\n",
-			   (unsigned long)uio->uio_iov[i].iov_base,
-			   (unsigned long)uio->uio_iov[i].iov_len));
+    error = nnpfs_data_valid(vp, &(VNODE_TO_XNODE(vp))->rd_cred,
+			     NNPFS_DATA_R, nnpfs_uio_end_length(uio));
 
     if (error == 0) {
 	struct vnode *t = DATA_FROM_VNODE(vp);
 
-#ifdef HAVE_FREEBSD_THREAD
-	nnpfs_vfs_readlock(t, nnpfs_uio_to_thread(uio));
-	nnpfs_vop_read(t, uio, ioflag, cred, error);
-	nnpfs_vfs_unlock(t, nnpfs_uio_to_thread(uio));
+#if defined(__APPLE__)
+	nnpfs_vop_read(t, uio, ioflag, NULL, error);
 #else
 	nnpfs_vfs_readlock(t, nnpfs_uio_to_proc(uio));
 	nnpfs_vop_read(t, uio, ioflag, cred, error);
@@ -349,8 +343,8 @@ nnpfs_read_common(struct vnode *vp, struct uio *uio, int ioflag,
     }
 
     NNPFSDEB(XDEBVNOPS, ("nnpfs_read offset: %lu resid: %lu\n",
-		       (unsigned long)uio->uio_offset,
-		       (unsigned long)uio->uio_resid));
+			 (unsigned long)nnpfs_uio_offset(uio),
+			 (unsigned long)nnpfs_uio_resid(uio)));
     NNPFSDEB(XDEBVNOPS, ("nnpfs_read error: %d\n", error));
 
     return error;
@@ -358,60 +352,57 @@ nnpfs_read_common(struct vnode *vp, struct uio *uio, int ioflag,
 
 int
 nnpfs_write_common(struct vnode *vp, struct uio *uiop, int ioflag,
-		 struct ucred *cred)
+		   nnpfs_vfs_context ctx)
 {
+    struct ucred *cred = nnpfs_vfs_context_ucred(ctx);
     struct nnpfs_node *xn = VNODE_TO_XNODE(vp);
     int error = 0;
 
     NNPFSDEB(XDEBVNOPS, ("nnpfs_write\n"));
 
-    nnpfs_update_write_cred(xn, cred);
+    nnpfs_setcred(&xn->wr_cred, cred); 
 
-#ifdef HAVE_FREEBSD_THREAD
-    error = nnpfs_data_valid(vp, cred, nnpfs_uio_to_thread(uiop), NNPFS_DATA_W,
-			   VNODE_TO_XNODE(vp)->attr.va_size);
-#else
-    error = nnpfs_data_valid(vp, cred, nnpfs_uio_to_proc(uiop), NNPFS_DATA_W,
-			   VNODE_TO_XNODE(vp)->attr.va_size);
-#endif
+    error = nnpfs_data_valid(vp, &xn->wr_cred, NNPFS_DATA_W,
+			     nnpfs_vattr_get_size(&xn->attr));
 
     if (error == 0) {
 	struct vnode *t = DATA_FROM_XNODE(xn);
-	struct vattr sub_attr;
+	struct nnpfs_vfs_vattr sub_attr;
 	int error2 = 0;
  
- #ifdef HAVE_FREEBSD_THREAD
-	nnpfs_vfs_writelock(t, nnpfs_uio_to_thread(uiop));
-	nnpfs_vop_write(t, uiop, ioflag, cred, error);
-	VNODE_TO_XNODE(vp)->flags |= NNPFS_DATA_DIRTY;
-	nnpfs_vop_getattr(t, &sub_attr, cred, nnpfs_uio_to_thread(uiop), error2);
- #else
-	nnpfs_vfs_writelock(t, nnpfs_uio_to_proc(uiop));
-	nnpfs_vop_write(t, uiop, ioflag, cred, error);
-	VNODE_TO_XNODE(vp)->flags |= NNPFS_DATA_DIRTY;
-	nnpfs_vop_getattr(t, &sub_attr, cred, nnpfs_uio_to_proc(uiop), error2);
- #endif
+#if defined(__APPLE__)
+	nnpfs_vop_write(t, uiop, ioflag, ctx, error);
 
-	if (error2 == 0) {
-	    xn->attr.va_size  = sub_attr.va_size;
-	    xn->attr.va_bytes = sub_attr.va_size;
-	    xn->attr.va_mtime = sub_attr.va_mtime;
-	    nnpfs_set_vp_size(vp, sub_attr.va_size);
-	    xn->offset = sub_attr.va_size;
-	}
-#ifdef HAVE_FREEBSD_THREAD
-	nnpfs_vfs_unlock(t, nnpfs_uio_to_thread(uiop));
+	VATTR_INIT(&sub_attr);
+	VATTR_WANTED(&sub_attr, va_data_size);
+	VATTR_WANTED(&sub_attr, va_total_alloc);
+	VATTR_WANTED(&sub_attr, va_modify_time);
 #else
-	nnpfs_vfs_unlock(t, nnpfs_uio_to_proc(uiop));
+	nnpfs_vfs_writelock(t, nnpfs_vfs_context_proc(ctx));
+	nnpfs_vop_write(t, uiop, ioflag, cred, error);
 #endif
+
+	VNODE_TO_XNODE(vp)->flags |= NNPFS_DATA_DIRTY;
+	nnpfs_vop_getattr(t, &sub_attr, ctx, error2);
+	if (error2 == 0) {
+	    nnpfs_vattr_set_size(&xn->attr, nnpfs_vattr_get_size(&sub_attr));
+	    nnpfs_vattr_set_bytes(&xn->attr, nnpfs_vattr_get_size(&sub_attr));
+	    nnpfs_vattr_set_mtime(&xn->attr,
+				  nnpfs_vattr_get_mtime_sec(&sub_attr),
+				  nnpfs_vattr_get_mtime_nsec(&sub_attr));
+
+            nnpfs_set_vp_size(vp, nnpfs_vattr_get_size(&sub_attr));
+	    xn->offset = nnpfs_vattr_get_size(&sub_attr);
+	}
+	nnpfs_vfs_unlock(t, nnpfs_vfs_context_proc(ctx));
     }
 
     return error;
 }
 
 int
-nnpfs_getattr_common(struct vnode *vp, struct vattr *vap,
-		   struct ucred *cred, d_thread_t *p)
+nnpfs_getattr_common(struct vnode *vp, struct nnpfs_vfs_vattr *vap,
+		     struct ucred *cred, d_thread_t *p)
 {
     int error = 0;
 
@@ -420,18 +411,51 @@ nnpfs_getattr_common(struct vnode *vp, struct vattr *vap,
     NNPFSDEB(XDEBVNOPS, ("nnpfs_getattr\n"));
 
     error = nnpfs_attr_valid(vp, cred, p, NNPFS_ATTR_R);
-    if (error == 0) {
+    if (error == 0)
 	*vap = xn->attr;
-#ifdef __APPLE__
-        vap->va_fsid = vp->v_mount->mnt_stat.f_fsid.val[0]; 
-#endif
-    }
+
     return error;
 }
 
+static int
+setattr_is_noop(struct nnpfs_node *xn, struct nnpfs_vfs_vattr *vap)
+{
+#ifdef __APPLE__
+    
+#define CHECK_NNPFSATTR(A) (!VATTR_IS_ACTIVE(vap, A) || vap->A == xn->attr.A)
+    if (CHECK_NNPFSATTR(va_mode) &&
+	CHECK_NNPFSATTR(va_nlink) &&
+	CHECK_NNPFSATTR(va_data_size) &&
+	CHECK_NNPFSATTR(va_uid) &&
+	CHECK_NNPFSATTR(va_gid) &&
+	CHECK_NNPFSATTR(va_fileid) && /* we ignore va_type */
+	(!VATTR_IS_ACTIVE(vap, va_modify_time)
+	 || vap->va_modify_time.tv_sec == xn->attr.va_modify_time.tv_sec))
+	return 1;
+#undef CHECK_NNPFSATTR
+
+#else
+
+#define CHECK_NNPFSATTR(A, cast) (vap->A == cast VNOVAL || vap->A == xn->attr.A)
+    if (CHECK_NNPFSATTR(va_mode,(mode_t)) &&
+	CHECK_NNPFSATTR(va_nlink,(short)) &&
+	CHECK_NNPFSATTR(va_size,(va_size_t)) &&
+	CHECK_NNPFSATTR(va_uid,(uid_t)) &&
+	CHECK_NNPFSATTR(va_gid,(gid_t)) &&
+	CHECK_NNPFSATTR(va_mtime.tv_sec,(time_t)) &&
+	CHECK_NNPFSATTR(va_fileid,(long)) &&
+	CHECK_NNPFSATTR(va_type,(enum vtype)))
+	return 1;
+#undef CHECK_NNPFSATTR
+
+#endif /* ! __APPLE__ */
+
+    return 0;
+}
+
 int
-nnpfs_setattr_common(struct vnode *vp, struct vattr *vap,
-		   struct ucred *cred, d_thread_t *p)
+nnpfs_setattr_common(struct vnode *vp, struct nnpfs_vfs_vattr *vap,
+		     struct ucred *cred, d_thread_t *p)
 {
     struct nnpfs *nnpfsp = NNPFS_FROM_VNODE(vp);
     struct nnpfs_node *xn = VNODE_TO_XNODE(vp);
@@ -439,17 +463,8 @@ nnpfs_setattr_common(struct vnode *vp, struct vattr *vap,
 
     NNPFSDEB(XDEBVNOPS, ("nnpfs_setattr\n"));
 
-#define CHECK_NNPFSATTR(A, cast) (vap->A == cast VNOVAL || vap->A == xn->attr.A)
-	if (CHECK_NNPFSATTR(va_mode,(mode_t)) &&
-	    CHECK_NNPFSATTR(va_nlink,(short)) &&
-	    CHECK_NNPFSATTR(va_size,(va_size_t)) &&
-	    CHECK_NNPFSATTR(va_uid,(uid_t)) &&
-	    CHECK_NNPFSATTR(va_gid,(gid_t)) &&
-	    CHECK_NNPFSATTR(va_mtime.tv_sec,(unsigned int)) &&
-	    CHECK_NNPFSATTR(va_fileid,(long)) &&
-	    CHECK_NNPFSATTR(va_type,(enum vtype)))
-		return 0;		/* Nothing to do */
-#undef CHECK_NNPFSATTR
+    if (setattr_is_noop(xn, vap))
+	return 0; /* nothing to do */
 
     if (NNPFS_TOKEN_GOT(xn, NNPFS_ATTR_W)) {
 	/* Update attributes and mark them dirty. */
@@ -460,48 +475,85 @@ nnpfs_setattr_common(struct vnode *vp, struct vattr *vap,
 	struct nnpfs_message_putattr msg;
 
 	msg.header.opcode = NNPFS_MSG_PUTATTR;
-	if (cred != NOCRED) {
-	    msg.cred.uid = cred->cr_uid;
-	    msg.cred.pag = nnpfs_get_pag(cred);
-	} else {
-	    msg.cred.uid = 0;
-	    msg.cred.pag = NNPFS_ANONYMOUSID;
-	}
+	nnpfs_setcred(&msg.cred, cred);
 	msg.handle = xn->handle;
 	vattr2nnpfs_attr(vap, &msg.attr);
-	if (NNPFS_TOKEN_GOT(xn, NNPFS_DATA_R)) {
-	    if (vp->v_type == VREG) {
-		if (vap->va_size != (va_size_t)VNOVAL)
-		    XA_SET_SIZE(&msg.attr, vap->va_size);
-		else
-		    XA_SET_SIZE(&msg.attr, xn->attr.va_size);
-#ifdef __APPLE__
-		/* XXX needed ? */
-		if (UBCINFOEXISTS(vp))
-		    ubc_setsize(vp, msg.attr.xa_size);
-#endif
-	    }
-	    if (vap->va_mtime.tv_sec != (unsigned int)VNOVAL)
-		XA_SET_MTIME(&msg.attr, vap->va_mtime.tv_sec);
-	    else
-		XA_SET_MTIME(&msg.attr, xn->attr.va_mtime.tv_sec);
-	}
 
-	NNPFS_TOKEN_CLEAR(xn, NNPFS_ATTR_VALID, NNPFS_ATTR_MASK);
+	if (NNPFS_TOKEN_GOT(xn, NNPFS_DATA_R)) {
+	    if (nnpfs_vnode_isreg(vp)) {
+		if (nnpfs_vattr_size_isactive(vap))
+		    XA_SET_SIZE(&msg.attr, nnpfs_vattr_get_size(vap));
+		else
+		    XA_SET_SIZE(&msg.attr, nnpfs_vattr_get_size(&xn->attr));
+	    }
+
+	    if (nnpfs_vattr_mtime_isactive(vap))
+		XA_SET_MTIME(&msg.attr, nnpfs_vattr_get_mtime_sec(vap));
+	    else
+		XA_SET_MTIME(&msg.attr, nnpfs_vattr_get_mtime_sec(&xn->attr));
+	}
+	
 	error = nnpfs_message_rpc(nnpfsp->fd, &msg.header, sizeof(msg), p);
 	if (error == 0)
-	    error = ((struct nnpfs_message_wakeup *) & msg)->error;
+	    error = NNPFS_MSG_WAKEUP_ERROR(&msg);
     }
 
 done:
+    NNPFS_VN_KNOTE(vp, NOTE_ATTRIB);
+
     return error;
 }
 
 static int
-check_rights (u_char rights, int mode)
+check_rights (nnpfs_rights rights, int mode, int dirp)
 {
     int error = 0;
 
+#ifdef __APPLE__
+    /* VNOP_ACCESS passes a kauth action instead of ordinary unix mode */
+
+    if (dirp) {
+	if (mode & (KAUTH_VNODE_LIST_DIRECTORY | KAUTH_VNODE_SEARCH))
+	    if ((rights & NNPFS_RIGHT_X) == 0)
+		error = EACCES;
+	if (mode & KAUTH_VNODE_DELETE_CHILD)
+	    if ((rights & NNPFS_RIGHT_AD) == 0)
+		error = EACCES;
+	if (mode & (KAUTH_VNODE_ADD_FILE | KAUTH_VNODE_ADD_SUBDIRECTORY))
+	    if ((rights & NNPFS_RIGHT_AI) == 0)
+		error = EACCES;
+
+	/* XXX can't check KAUTH_VNODE_DELETE, have no info */
+
+    } else {
+	if (mode & (KAUTH_VNODE_READ_DATA | KAUTH_VNODE_READ_ATTRIBUTES))
+	    if ((rights & NNPFS_RIGHT_AR) == 0)
+		error = EACCES;
+	if (mode & (KAUTH_VNODE_WRITE_DATA | KAUTH_VNODE_APPEND_DATA | KAUTH_VNODE_WRITE_ATTRIBUTES | KAUTH_VNODE_TAKE_OWNERSHIP))
+	    if ((rights & NNPFS_RIGHT_W) == 0)
+		error = EACCES;
+	if (mode & KAUTH_VNODE_EXECUTE)
+	    if ((rights & NNPFS_RIGHT_X) == 0)
+		error = EACCES;
+	if (mode & KAUTH_VNODE_DELETE)
+	    if ((rights & NNPFS_RIGHT_AD) == 0)
+		error = EACCES;
+    }
+    
+#if 0
+KAUTH_VNODE_READ_EXTATTRIBUTES
+KAUTH_VNODE_WRITE_EXTATTRIBUTES
+KAUTH_VNODE_READ_SECURITY
+KAUTH_VNODE_WRITE_SECURITY
+KAUTH_VNODE_SYNCHRONIZE notused
+KAUTH_VNODE_LINKTARGET like insert, but for target???
+KAUTH_VNODE_CHECKIMMUTABLE always ok
+KAUTH_VNODE_ACCESS (advisory)
+KAUTH_VNODE_NOIMMUTABLE ?
+#endif
+
+#else /* !__APPLE__ */
+
     if (mode & VREAD)
 	if ((rights & NNPFS_RIGHT_R) == 0)
 	    error = EACCES;
@@ -511,13 +563,18 @@ check_rights (u_char rights, int mode)
     if (mode & VEXEC)
 	if ((rights & NNPFS_RIGHT_X) == 0)
 	    error = EACCES;
+
+#endif /* !__APPLE__ */
+
     return error;
 }
 
 int
 nnpfs_access_common(struct vnode *vp, int mode, struct ucred *cred,
-		  d_thread_t *p)
+		    d_thread_t *p)
 {
+    int dirp = nnpfs_vnode_isdir(vp);
+
     int error = 0;
     nnpfs_pag_t pag = nnpfs_get_pag(cred);
 
@@ -528,7 +585,7 @@ nnpfs_access_common(struct vnode *vp, int mode, struct ucred *cred,
 	struct nnpfs_node *xn = VNODE_TO_XNODE(vp);
 	int i;
 
-	error = check_rights (xn->anonrights, mode);
+	error = check_rights(xn->anonrights, mode, dirp);
 
 	if (error == 0)
 	    goto done;
@@ -537,9 +594,9 @@ nnpfs_access_common(struct vnode *vp, int mode, struct ucred *cred,
 
 	error = EACCES;		/* default to EACCES if pag isn't in xn->id */
 
-	for (i = 0; i < MAXRIGHTS; i++)
+	for (i = 0; i < NNPFS_MAXRIGHTS; i++)
 	    if (xn->id[i] == pag) {
-		error = check_rights (xn->rights[i], mode);
+		error = check_rights(xn->rights[i], mode, dirp);
 		break;
 	    }
     }
@@ -552,20 +609,16 @@ done:
 
 int
 nnpfs_lookup_common(struct vnode *dvp, 
-		  nnpfs_componentname *cnp, 
-		  struct vnode **vpp)
+		    nnpfs_componentname *cnp, 
+		    struct vnode **vpp,
+		    nnpfs_vfs_context ctx)
 {
     struct nnpfs_message_getnode msg;
+    struct ucred *cred = nnpfs_vfs_context_ucred(ctx);
+    d_thread_t *p  = nnpfs_vfs_context_proc(ctx);
     struct nnpfs *nnpfsp = NNPFS_FROM_VNODE(dvp);
     struct nnpfs_node *d = VNODE_TO_XNODE(dvp);
     int error = 0;
-#ifdef HAVE_FREEBSD_THREAD
-    d_thread_t *proc  = nnpfs_cnp_to_thread(cnp);
-    struct ucred *cred = nnpfs_thread_to_cred(proc);
-#else
-    d_thread_t *proc  = nnpfs_cnp_to_proc(cnp);
-    struct ucred *cred = nnpfs_proc_to_cred(proc);
-#endif
 
     NNPFSDEB(XDEBVNOPS, ("nnpfs_lookup_common: enter\n"));
 
@@ -574,17 +627,21 @@ nnpfs_lookup_common(struct vnode *dvp,
     if (cnp->cn_namelen >= NNPFS_MAX_NAME)
 	return ENAMETOOLONG;
 	
-    if (dvp->v_type != VDIR)
+    if (!nnpfs_vnode_isdir(dvp))
 	return ENOTDIR;
 
     if (cnp->cn_namelen == 1 && cnp->cn_nameptr[0] == '.') {
 	*vpp = dvp;
-	VREF(*vpp);
+#ifdef __APPLE__
+	nnpfs_do_vget(*vpp, 0 /* XXX flag */, NULL /*proc */);
+#else
+	nnpfs_vref(*vpp);
+#endif
 	return 0;
     }
     
     do {
-	nnpfs_vop_access(dvp, VEXEC, cred, proc, error);
+	nnpfs_lookup_access(dvp, ctx, p, error);
 	if (error != 0)
 	    goto done;
 
@@ -611,20 +668,13 @@ nnpfs_lookup_common(struct vnode *dvp,
 #endif
 
 	    msg.header.opcode = NNPFS_MSG_GETNODE;
-	    if (cnp->cn_cred != NOCRED) {
-		msg.cred.uid = cnp->cn_cred->cr_uid;
-		msg.cred.pag = nnpfs_get_pag(cnp->cn_cred);
-	    } else {
-		msg.cred.uid = 0;
-		msg.cred.pag = NNPFS_ANONYMOUSID;
-	    }
+	    nnpfs_setcred(&msg.cred, cred);
 	    msg.parent_handle = d->handle;
 	    memcpy(msg.name, cnp->cn_nameptr, cnp->cn_namelen);
 	    msg.name[cnp->cn_namelen] = '\0';
-	    error = nnpfs_message_rpc(nnpfsp->fd, &msg.header,
-				    sizeof(msg), proc);
+	    error = nnpfs_message_rpc(nnpfsp->fd, &msg.header, sizeof(msg), p);
 	    if (error == 0)
-		error = ((struct nnpfs_message_wakeup *) & msg)->error;
+		error = NNPFS_MSG_WAKEUP_ERROR(&msg);
 	    if(error == ENOENT && cnp->cn_nameiop != CREATE) {
 		NNPFSDEB(XDEBVNOPS, ("nnpfs_lookup: neg cache %lx (%s, %ld)\n",
 				   (unsigned long)dvp,
@@ -637,17 +687,20 @@ nnpfs_lookup_common(struct vnode *dvp,
 	}
     } while (error == 0);
 
-done:
-    NNPFSDEB(XDEBVNOPS, ("nnpfs_lookup_common: return error = %d\n", error));
+ done:
+    NNPFS_VN_KNOTE(dvp, NOTE_WRITE);
+    
+    NNPFSDEB(XDEBVNOPS, ("nnpfs_lookup_common: return %d, vn %p\n",
+			 error, *vpp));
     return error;
 }
 
 int
 nnpfs_create_common(struct vnode *dvp,
-		  const char *name,
-		  struct vattr *vap, 
-		  struct ucred *cred,
-		  d_thread_t *p)
+		    const char *name,
+		    struct nnpfs_vfs_vattr *vap, 
+		    struct ucred *cred,
+		    d_thread_t *p)
 {
     struct nnpfs *nnpfsp = NNPFS_FROM_VNODE(dvp);
     struct nnpfs_node *xn = VNODE_TO_XNODE(dvp);
@@ -665,19 +718,16 @@ nnpfs_create_common(struct vnode *dvp,
 	vattr2nnpfs_attr(vap, &msg.attr);
 
 	msg.mode = 0;		       /* XXX - mode */
-	if (cred != NOCRED) {
-	    msg.cred.uid = cred->cr_uid;
-	    msg.cred.pag = nnpfs_get_pag(cred);
-	} else {
-	    msg.cred.uid = 0;
-	    msg.cred.pag = NNPFS_ANONYMOUSID;
-	}
+	nnpfs_setcred(&msg.cred, cred);
 
+#ifdef __APPLE__
+	/* needed for subsequent writes to succeed with n */
+	msg.attr.valid &= ~XA_V_UID; 
+#endif
 
 	error = nnpfs_message_rpc(nnpfsp->fd, &msg.header, sizeof(msg), p);
-
 	if (error == 0)
-	    error = ((struct nnpfs_message_wakeup *) & msg)->error;
+	    error = NNPFS_MSG_WAKEUP_ERROR(&msg);
     }
 
 #if 0
@@ -690,10 +740,10 @@ nnpfs_create_common(struct vnode *dvp,
 
 int
 nnpfs_remove_common(struct vnode *dvp,
-		  struct vnode *vp,
-		  const char *name,
-		  struct ucred *cred,
-		  d_thread_t *p)
+		    struct vnode *vp,
+		    const char *name,
+		    struct ucred *cred,
+		    d_thread_t *p)
 {
     struct nnpfs *nnpfsp  = NNPFS_FROM_VNODE(dvp);
     struct nnpfs_node *xn = VNODE_TO_XNODE(dvp);
@@ -712,33 +762,38 @@ nnpfs_remove_common(struct vnode *dvp,
     else
 	error = nnpfs_message_rpc(nnpfsp->fd, &msg.header, sizeof(msg), p);
     if (error == 0)
-	error = ((struct nnpfs_message_wakeup *) &msg)->error;
+	error = NNPFS_MSG_WAKEUP_ERROR(&msg);
 
-    if (error == 0)
+    if (error == 0) {
 	nnpfs_dnlc_purge (vp);
+	NNPFS_VN_KNOTE(vp, NOTE_DELETE);
+	NNPFS_VN_KNOTE(dvp, NOTE_WRITE);
+    }
 
     return error;
 }
 
 int
 nnpfs_rename_common(struct vnode *fdvp, 
-		  struct vnode *fvp,
-		  const char *fname,
-		  struct vnode *tdvp,
-		  struct vnode *tvp,
-		  const char *tname,
-		  struct ucred *cred,
-		  d_thread_t *p)
+		    struct vnode *fvp,
+		    const char *fname,
+		    struct vnode *tdvp,
+		    struct vnode *tvp,
+		    const char *tname,
+		    struct ucred *cred,
+		    d_thread_t *p)
 {
     struct nnpfs *nnpfsp = NNPFS_FROM_VNODE(fdvp);
     int error;
 
     NNPFSDEB(XDEBVNOPS, ("nnpfs_rename: %s %s\n", fname, tname));
 
+#if 0
     if ((fvp->v_mount != tdvp->v_mount)
 	|| (tvp && (fvp->v_mount != tvp->v_mount))) {
 	return  EXDEV;
     }
+#endif
 
     {
 	struct nnpfs_message_rename msg;
@@ -758,6 +813,9 @@ nnpfs_rename_common(struct vnode *fdvp,
 
     }
 
+    NNPFS_VN_KNOTE(fdvp, NOTE_WRITE);
+    NNPFS_VN_KNOTE(fvp, NOTE_WRITE);
+
     NNPFSDEB(XDEBVNOPS, ("nnpfs_rename: error = %d\n", error));
 
     return error;
@@ -765,10 +823,10 @@ nnpfs_rename_common(struct vnode *fdvp,
 
 int
 nnpfs_mkdir_common(struct vnode *dvp, 
-		 const char *name,
-		 struct vattr *vap, 
-		 struct ucred *cred,
-		 d_thread_t *p)
+		   const char *name,
+		   struct nnpfs_vfs_vattr *vap, 
+		   struct ucred *cred,
+		   d_thread_t *p)
 {
     struct nnpfs *nnpfsp = NNPFS_FROM_VNODE(dvp);
     struct nnpfs_node *xn = VNODE_TO_XNODE(dvp);
@@ -783,27 +841,24 @@ nnpfs_mkdir_common(struct vnode *dvp,
 	if (strlcpy(msg.name, name, sizeof(msg.name)) >= NNPFS_MAX_NAME)
 	    return ENAMETOOLONG;
 	vattr2nnpfs_attr(vap, &msg.attr);
-	if (cred != NOCRED) {
-	    msg.cred.uid = cred->cr_uid;
-	    msg.cred.pag = nnpfs_get_pag(cred);
-	} else {
-	    msg.cred.uid = 0;
-	    msg.cred.pag = NNPFS_ANONYMOUSID;
-	}
+	nnpfs_setcred(&msg.cred, cred);
 	error = nnpfs_message_rpc(nnpfsp->fd, &msg.header, sizeof(msg), p);
 	if (error == 0)
-	    error = ((struct nnpfs_message_wakeup *) & msg)->error;
+	    error = NNPFS_MSG_WAKEUP_ERROR(&msg);
     }
 
+    if (error == 0)
+	NNPFS_VN_KNOTE(dvp, NOTE_WRITE | NOTE_LINK);
+
     return error;
 }
 
 int
 nnpfs_rmdir_common(struct vnode *dvp,
-		 struct vnode *vp,
-		 const char *name,
-		 struct ucred *cred,
-		 d_thread_t *p)
+		   struct vnode *vp,
+		   const char *name,
+		   struct ucred *cred,
+		   d_thread_t *p)
 {
     struct nnpfs *nnpfsp  = NNPFS_FROM_VNODE(dvp);
     struct nnpfs_node *xn = VNODE_TO_XNODE(dvp);
@@ -826,6 +881,9 @@ nnpfs_rmdir_common(struct vnode *dvp,
     if (error == 0)
 	nnpfs_dnlc_purge (vp);
 
+    NNPFS_VN_KNOTE(dvp, NOTE_WRITE | NOTE_LINK);
+    NNPFS_VN_KNOTE(vp, NOTE_DELETE);
+
     NNPFSDEB(XDEBVNOPS, ("nnpfs_rmdir error: %d\n", error));
 
     return error;
@@ -833,60 +891,49 @@ nnpfs_rmdir_common(struct vnode *dvp,
 
 int
 nnpfs_readdir_common(struct vnode *vp, 
-		   struct uio *uiop, 
-		   struct ucred *cred,
-		   d_thread_t *p,
-		   int *eofflag)
+		     struct uio *uiop, 
+		     int *eofflag,
+		     nnpfs_vfs_context ctx)
 {
     int error = 0;
+    nnpfs_cred cred;
 
     NNPFSDEB(XDEBVNOPS, ("nnpfs_readdir\n"));
 
     if(eofflag)
 	*eofflag = 0;
-#ifdef HAVE_FREEBSD_THREAD
-    error = nnpfs_data_valid(vp, cred, nnpfs_uio_to_thread(uiop), NNPFS_DATA_R,
-			   nnpfs_uio_end_length(uiop));
-#else
-    error = nnpfs_data_valid(vp, cred, nnpfs_uio_to_proc(uiop), NNPFS_DATA_R,
-			   nnpfs_uio_end_length(uiop));
-#endif
+
+    nnpfs_setcred(&cred, nnpfs_vfs_context_ucred(ctx));
+    error = nnpfs_data_valid(vp, &cred, NNPFS_DATA_R,
+			     nnpfs_uio_end_length(uiop));
     if (error == 0) {
 	struct vnode *t = DATA_FROM_VNODE(vp);
 
-#ifdef HAVE_FREEBSD_THREAD
-	nnpfs_vfs_readlock(t, nnpfs_uio_to_thread(uiop));
-#else
 	nnpfs_vfs_readlock(t, nnpfs_uio_to_proc(uiop));
-#endif
-	nnpfs_vop_read(t, uiop, 0, cred, error);
+	nnpfs_vop_read(t, uiop, 0, NULL, error);
 	if (eofflag) {
-	    struct vattr t_attr;
+	    struct nnpfs_vfs_vattr t_attr;
 	    int error2;
 
-#ifdef HAVE_FREEBSD_THREAD
-	    nnpfs_vop_getattr(t, &t_attr, cred, nnpfs_uio_to_thread(uiop), error2);
-#else
-	    nnpfs_vop_getattr(t, &t_attr, cred, nnpfs_uio_to_proc(uiop), error2);
+#ifdef __APPLE__
+	    VATTR_INIT(&t_attr);
+	    VATTR_WANTED(&t_attr, va_data_size);
 #endif
+	    nnpfs_vop_getattr(t, &t_attr, ctx, error2);  /* XXX check bitmask */
 	    if (error2 == 0)
-		*eofflag = t_attr.va_size <= uiop->uio_offset;
+	      *eofflag = nnpfs_vattr_get_size(&t_attr) <= nnpfs_uio_offset(uiop);
 	}
-#ifdef HAVE_FREEBSD_THREAD
-	nnpfs_vfs_unlock(t, nnpfs_uio_to_thread(uiop));
-#else
 	nnpfs_vfs_unlock(t, nnpfs_uio_to_proc(uiop));
-#endif
     }
     return error;
 }
 
 int
 nnpfs_link_common(struct vnode *dvp, 
-		struct vnode *vp, 
-		const char *name,
-		struct ucred *cred,
-		d_thread_t *p)
+		  struct vnode *vp, 
+		  const char *name,
+		  struct ucred *cred,
+		  d_thread_t *p)
 {
     struct nnpfs *nnpfsp = NNPFS_FROM_VNODE(dvp);
     struct nnpfs_node *xn = VNODE_TO_XNODE(dvp);
@@ -906,27 +953,24 @@ nnpfs_link_common(struct vnode *dvp,
 
     error = nnpfs_message_rpc(nnpfsp->fd, &msg.header, sizeof(msg), p);
     if (error == 0)
-	error = ((struct nnpfs_message_wakeup *) & msg)->error;
+	error = NNPFS_MSG_WAKEUP_ERROR(&msg);
     
+    NNPFS_VN_KNOTE(vp, NOTE_LINK);
+
     return error;
 }
 
 int
 nnpfs_symlink_common(struct vnode *dvp,
-		   struct vnode **vpp,
-		   nnpfs_componentname *cnp,
-		   struct vattr *vap,
-		   char *target)
+		     struct vnode **vpp,
+		     nnpfs_componentname *cnp,
+		     struct nnpfs_vfs_vattr *vap,
+		     char *target,
+		     nnpfs_vfs_context ctx)
 {
     struct nnpfs *nnpfsp = NNPFS_FROM_VNODE(dvp);
     struct nnpfs_node *xn = VNODE_TO_XNODE(dvp);
-#ifdef HAVE_FREEBSD_THREAD
-    d_thread_t *proc  = nnpfs_cnp_to_thread(cnp);
-    struct ucred *cred = nnpfs_thread_to_cred(proc);
-#else
-    d_thread_t *proc  = nnpfs_cnp_to_proc(cnp);
-    struct ucred *cred = nnpfs_proc_to_cred(proc);
-#endif
+    struct ucred *cred = nnpfs_vfs_context_ucred(ctx);
     struct nnpfs_message_symlink msg;
     const char *name = cnp->cn_nameptr;
     int error = 0;
@@ -946,52 +990,54 @@ nnpfs_symlink_common(struct vnode *dvp,
 	error = ENAMETOOLONG;
 	goto done;
     }
-    error = nnpfs_message_rpc(nnpfsp->fd, &msg.header, sizeof(msg), proc);
+    error = nnpfs_message_rpc(nnpfsp->fd, &msg.header, sizeof(msg),
+			      nnpfs_vfs_context_proc(ctx));
     if (error == 0)
-	error = ((struct nnpfs_message_wakeup *) & msg)->error;
+	error = NNPFS_MSG_WAKEUP_ERROR(&msg);
 
  done:
+    NNPFS_VN_KNOTE(dvp, NOTE_WRITE);
+
     return error;
 }
 
 int
-nnpfs_readlink_common(struct vnode *vp, struct uio *uiop, struct ucred *cred)
+nnpfs_readlink_common(struct vnode *vp, struct uio *uiop, nnpfs_vfs_context ctx)
 {
     int error = 0;
+    nnpfs_cred cred;
+    d_thread_t *proc = nnpfs_vfs_context_proc(ctx);
 
     NNPFSDEB(XDEBVNOPS, ("nnpfs_readlink\n"));
 
-#ifdef HAVE_FREEBSD_THREAD
-    error = nnpfs_data_valid(vp, cred, nnpfs_uio_to_thread(uiop), NNPFS_DATA_R,
-			   nnpfs_uio_end_length(uiop));
-#else
-    error = nnpfs_data_valid(vp, cred, nnpfs_uio_to_proc(uiop), NNPFS_DATA_R,
-			   nnpfs_uio_end_length(uiop));
-#endif
+    if (!nnpfs_vnode_islnk(vp)) {
+	return EINVAL;
+    }
+
+    nnpfs_setcred(&cred, nnpfs_vfs_context_ucred(ctx));
+    error = nnpfs_data_valid(vp, &cred, NNPFS_DATA_R, nnpfs_uio_end_length(uiop));
     if (error == 0) {
 	struct vnode *t = DATA_FROM_VNODE(vp);
 
-#ifdef HAVE_FREEBSD_THREAD
-	nnpfs_vfs_readlock(t, nnpfs_uio_to_thread(uiop));
-	nnpfs_vop_read(t, uiop, 0, cred, error);
-	nnpfs_vfs_unlock(t, nnpfs_uio_to_thread(uiop));
+#if defined(__APPLE__)
+	nnpfs_vop_read(t, uiop, 0, ctx, error);
 #else
-	nnpfs_vfs_readlock(t, nnpfs_uio_to_proc(uiop));
-	nnpfs_vop_read(t, uiop, 0, cred, error);
-	nnpfs_vfs_unlock(t, nnpfs_uio_to_proc(uiop));
+	nnpfs_vfs_readlock(t, proc);
+	nnpfs_vop_read(t, uiop, 0, nnpfs_vfs_context_ucred(ctx), error);
+	nnpfs_vfs_unlock(t, proc);
 #endif
     }
     return error;
 }
 
 int
-nnpfs_inactive_common(struct vnode *vp, d_thread_t *p)
+nnpfs_inactive_common(struct vnode *vp, d_thread_t *proc)
 {
     int error;
     struct nnpfs_node *xn = VNODE_TO_XNODE(vp);
 
     NNPFSDEB(XDEBVNOPS, ("nnpfs_inactive, %lx\n",
-		       (unsigned long)vp));
+			 (unsigned long)vp));
 
     /*
      * This seems rather bogus, but sometimes we get an already
@@ -1005,10 +1051,10 @@ nnpfs_inactive_common(struct vnode *vp, d_thread_t *p)
 
     /* xn->wr_cred not set -> NOCRED */
 
-    if (vp->v_type == VREG)
-	nnpfs_pushdirty(vp, xn->wr_cred, p);
+    if (nnpfs_vnode_isreg(vp))
+	nnpfs_pushdirty(vp);
 
-    error = nnpfs_fsync_common(vp, xn->wr_cred, /* XXX */ 0, p);
+    error = nnpfs_fsync_common(vp, NULL, &xn->wr_cred, 0, proc);
     if (error) {
 	printf ("nnpfs_inactive: failed writing back data: %d\n", error);
 	xn->flags &= ~NNPFS_DATA_DIRTY;
@@ -1018,18 +1064,11 @@ nnpfs_inactive_common(struct vnode *vp, d_thread_t *p)
     if (!NNPFS_TOKEN_GOT(xn, NNPFS_ATTR_R | NNPFS_ATTR_W)
 	|| (xn->flags & NNPFS_STALE) == NNPFS_STALE)
     {
-#ifndef __osf__
-	nnpfs_vfs_unlock(vp, p);
+	nnpfs_vfs_unlock(vp, proc);
         NNPFSDEB(XDEBVNOPS, ("nnpfs_inactive: vrecycle\n"));
-        vrecycle(vp, 0, p);
-#else /* __osf__ */
-	NNPFSDEB(XDEBVNOPS, ("nnpfs_inactive: vp = %lx vp->v_usecount= %d\n",
-			     (unsigned long)vp, vp?vp->v_usecount:0));
-#endif /* __osf__ */
+        nnpfs_vrecycle(vp, 0, proc);
     } else {
-#ifndef __osf__
-	nnpfs_vfs_unlock(vp, p);
-#endif
+	nnpfs_vfs_unlock(vp, proc);
 	xn->flags &= ~NNPFS_STALE;
     }
 
@@ -1047,6 +1086,8 @@ nnpfs_reclaim_common(struct vnode *vp)
     NNPFSDEB(XDEBVNOPS, ("nnpfs_reclaim: %lx\n",
 			 (unsigned long)vp));
 
+    nnpfs_dev_lock(nnpfsp);
+
     xn->flags |= NNPFS_LIMBO;
 
     NNPFS_TOKEN_CLEAR(xn,
@@ -1055,7 +1096,7 @@ nnpfs_reclaim_common(struct vnode *vp)
 		      NNPFS_DATA_MASK | NNPFS_LOCK_MASK);
     /* Release, data if we still have it. */
     if (DATA_FROM_XNODE(xn) != NULL) {
-        vrele(DATA_FROM_XNODE(xn));
+        nnpfs_vletgo(DATA_FROM_XNODE(xn));
 	DATA_FROM_XNODE(xn) = NULL;
     }
 
@@ -1066,12 +1107,22 @@ nnpfs_reclaim_common(struct vnode *vp)
     if (nnpfsp->status & CHANNEL_OPENED) {
 	struct nnpfs_message_inactivenode msg;
 
+	nnpfs_dev_unlock(nnpfsp);
+
 	msg.header.opcode = NNPFS_MSG_INACTIVENODE;
 	msg.handle = xn->handle;
 	msg.flag   = NNPFS_NOREFS | NNPFS_DELETE;
 	nnpfs_message_send(nnpfsp->fd, &msg.header, sizeof(msg));
-    } else
+    } else {
 	nnpfs_free_node(nnpfsp, xn);
+	nnpfs_dev_unlock(nnpfsp);
+    }
+
+#if 0
+	vnode_removefsref(vp);
+	cache_purge(vp);
+	vnode_clearfsnode(vp);
+#endif
 
     return 0;
 }
@@ -1101,16 +1152,11 @@ nnpfs_advlock_common(struct vnode *dvp,
 	msg.locktype = locktype;
 	msg.lockid = lockid;
 
-	if (cred != NOCRED) {
-	    msg.cred.uid = cred->cr_uid;
-	    msg.cred.pag = nnpfs_get_pag(cred);
-	} else {
-	    msg.cred.uid = 0;
-	    msg.cred.pag = NNPFS_ANONYMOUSID;
-	}
-	error = nnpfs_message_rpc(nnpfsp->fd, &msg.header, sizeof(msg), p);
+	nnpfs_setcred(&msg.cred, cred);
+	error = nnpfs_message_rpc(nnpfsp->fd, &msg.header, sizeof(msg), 
+				  nnpfs_vfs_context_proc(ctx));
 	if (error == 0)
-	    error = ((struct nnpfs_message_wakeup *) & msg)->error;
+	    error = NNPFS_MSG_WAKEUP_ERROR(&msg);
     }
 
     if (error == 0) {
@@ -1143,5 +1189,5 @@ nnpfs_printnode_common (struct vnode *vp)
     printf ("\tdata: %svalid\n", 
 	    NNPFS_TOKEN_GOT(xn, NNPFS_DATA_VALID) ? "": "in");
     printf ("\tflags: 0x%x\n", xn->flags);
-    printf ("\toffset: %d\n", xn->offset);
+    printf ("\toffset: %lu\n", (unsigned long)xn->offset);
 }
