@@ -34,14 +34,14 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: nnpfs_locl.h,v 1.67.2.1 2004/06/16 15:49:38 lha Exp $ */
+/* $Id: nnpfs_locl.h,v 1.73 2006/01/10 09:17:00 tol Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include <asm/current.h>
 #include <linux/config.h>
+#include <asm/current.h>
 #include <linux/version.h>
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
@@ -68,13 +68,8 @@
 
 #include <linux/locks.h>
 
-#ifdef HAVE_LINUX_DEVFS_FS_KERNEL_H
-# include <linux/devfs_fs_kernel.h>
-#else
-# ifdef HAVE_LINUX_DEVFS_FS_H
-#  include <linux/devfs_fs.h>
-# endif
-#endif
+#include <linux/devfs_fs_kernel.h>
+
 #ifdef CONFIG_DEVFS_FS
 # define DEVFS_SUPPORT
 #endif
@@ -217,7 +212,7 @@ nnpfs_dcount (const struct dentry *dentry)
 
 #define NNPFS_RESET_I_MAPPING(node) (node)->i_mapping = &((node)->i_data)
 
-extern struct address_space_operations nnpfs_aops;
+extern struct address_space_operations nnpfs_null_aops;
 
 #ifndef list_for_each
 #define list_for_each(pos, head) for (pos = (head)->next; pos != (head); pos = pos->next)
@@ -234,7 +229,7 @@ extern struct address_space_operations nnpfs_aops;
 #define init_MUTEX(m)	*(m) = MUTEX
 #endif
 
-typedef int (*nnpfs_sys_call_function)(void);
+typedef asmlinkage long (*nnpfs_sys_call_function)(void);
 
 int
 nnpfs_fixup_syscall_lossage(void);
@@ -269,3 +264,6 @@ nnpfs_fixup_syscall_lossage(void);
 
 #endif /* !LINUX2_5 */
 
+#ifndef O_LARGEFILE
+#define O_LARGEFILE 0
+#endif
