@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2004 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2006 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -40,7 +40,7 @@
 #include <linux/poll.h>
 
 #ifdef RCSID
-RCSID("$Id: nnpfs_dev.c,v 1.110 2004/06/01 19:24:02 tol Exp $");
+RCSID("$Id: nnpfs_dev.c,v 1.116 2006/04/04 07:04:46 tol Exp $");
 #endif
 
 #include <nnpfs/nnpfs_message.h>
@@ -197,7 +197,7 @@ nnpfs_process_inactive_queue (struct nnpfs *nnpfsp)
  * throw away all inactive nodes on `nnpfsp'
  */
 
-static void
+void
 nnpfs_empty_inactive_queue (struct nnpfs *nnpfsp)
 {
     int i = 0;
@@ -254,7 +254,7 @@ nnpfs_devopen(struct inode *inode, struct file *file)
 	goto out;
     }
 
-    nnpfsp->message_buffer = nnpfs_alloc(MAX_XMSG_SIZE, NNPFS_MEM_MSGBUF);
+    nnpfsp->message_buffer = nnpfs_alloc(NNPFS_MAX_MSG_SIZE, NNPFS_MEM_MSGBUF);
     if (nnpfsp->message_buffer == NULL) {
 	NNPFS_MOD_DEREF;
 	ret = -ENOMEM;
@@ -311,8 +311,6 @@ nnpfs_devclose(struct inode * inode, struct file * file)
     }
     up(&nnpfsp->channel_sem);
 
-    nnpfs_empty_inactive_queue (nnpfsp);
-    
     if (nnpfsp->message_buffer) {
 	nnpfs_free(nnpfsp->message_buffer, NNPFS_MEM_MSGBUF);
 	nnpfsp->message_buffer = 0;
@@ -320,9 +318,11 @@ nnpfs_devclose(struct inode * inode, struct file * file)
 
     /* Free all nnpfs_nodes. */
     free_all_nnpfs_nodes(&nnpfs[MINOR(inode->i_rdev)]);
-    
+    nnpfs_empty_inactive_queue(nnpfsp);
+
     NNPFS_MOD_DEREF;
     unlock_kernel ();
+
     return 0;
 }
 
@@ -336,7 +336,7 @@ nnpfs_devread(struct file *file, char *buf, size_t count, loff_t *ppos)
     nnpfs_dev_t dev = file->f_dentry->d_inode->i_rdev;
     struct nnpfs *nnpfsp = &nnpfs[MINOR(dev)];
     struct nnpfs_link *first;
-    int ret = 0;
+    int ret = 0, error = 0;
     
     lock_kernel ();
 
@@ -359,7 +359,11 @@ nnpfs_devread(struct file *file, char *buf, size_t count, loff_t *ppos)
 	NNPFSDEB(XDEBDEV, ("nnpfs_devread: message->size = %u\n",
 			   first->message->size));
 	
-	copy_to_user (buf, first->message, first->message->size);
+	if (copy_to_user (buf, first->message, first->message->size)) {
+	    error = -EFAULT;
+	    break;
+	}
+
 	buf += first->message->size;
 	count -= first->message->size;
 	ret += first->message->size;
@@ -371,9 +375,12 @@ nnpfs_devread(struct file *file, char *buf, size_t count, loff_t *ppos)
     }
     up(&nnpfsp->channel_sem);
     
-    *ppos += ret;
-    
     unlock_kernel ();
+
+    if (error)
+	return error;
+
+    *ppos += ret;
     return ret;
 }
 
@@ -399,9 +406,12 @@ nnpfs_devwrite(struct file *file, const char *buf, size_t count, loff_t *ppos)
     dev = file->f_dentry->d_inode->i_rdev;
     nnpfsp = &nnpfs[MINOR(dev)];
 
-    if (count > MAX_XMSG_SIZE)
-	count = MAX_XMSG_SIZE;
-    copy_from_user(nnpfsp->message_buffer, buf, count);
+    if (count > NNPFS_MAX_MSG_SIZE)
+	count = NNPFS_MAX_MSG_SIZE;
+    if (copy_from_user(nnpfsp->message_buffer, buf, count)) {
+	error = -EFAULT;
+	goto out;
+    }
     
     /*
      * This thread handles the received message.
@@ -426,10 +436,11 @@ nnpfs_devwrite(struct file *file, const char *buf, size_t count, loff_t *ppos)
 	p += msg_buf->size;
 	left -= msg_buf->size;
     }
+ out:
     unlock_kernel ();
 
     NNPFSDEB(XDEBDEV, ("nnpfs_devwrite error = %d\n", error));
-
+    
     if (error)
 	return error;
 
@@ -818,6 +829,7 @@ nnpfs_message_wakeup_data(struct nnpfs *nnpfsp,
 }
 
 struct file_operations nnpfs_fops = {
+    owner:		THIS_MODULE,
     read:		nnpfs_devread,
     write:		nnpfs_devwrite,
     poll:		nnpfs_devpoll,
@@ -838,7 +850,10 @@ nnpfs_init_device(void)
 	nnpfs_initq(&nnpfs[i].sleepq);
 	init_MUTEX(&nnpfs[i].inactive_sem);
 	init_MUTEX(&nnpfs[i].channel_sem);
+	init_MUTEX(&nnpfs[i].node_sem);
 	INIT_LIST_HEAD(&nnpfs[i].inactive_list);
+	xn_init_head(&nnpfs[i].node_head);
+	nnpfs[i].nnodes = 0;
     }
     return 0;
 }
@@ -846,6 +861,7 @@ nnpfs_init_device(void)
 void
 nnpfs_print_sleep_queue(void)
 {
+#if 0
     int i;
 
     for (i = 0; i < NNNPFS; ++i) {
@@ -866,4 +882,5 @@ nnpfs_print_sleep_queue(void)
 	    }
 	}
     }
+#endif
 }
