@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995-2004 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995-2006 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -49,7 +49,7 @@
 #endif
 
 #ifdef RCSID
-RCSID("$Id: nnpfs_inodeops.c,v 1.201 2004/05/08 08:04:45 tol Exp $");
+RCSID("$Id: nnpfs_inodeops.c,v 1.216 2006/04/04 14:02:57 tol Exp $");
 #endif
 
 static int
@@ -142,6 +142,12 @@ nnpfs_write_file(struct file *file, const char *buf, size_t count, loff_t *ppos)
 static ssize_t
 nnpfs_read_file(struct file *file, char *buf, size_t count, loff_t *ppos);
 
+#ifdef LINUX2_5
+static ssize_t
+nnpfs_sendfile(struct file *file, loff_t *ppos, size_t count,
+	       read_actor_t actor, void *target);
+#endif
+
 static int
 #ifdef HAVE_GETATTR_THREE_ARGS
 nnpfs_getattr(struct vfsmount *mnt, struct dentry *, struct kstat *stat);
@@ -149,7 +155,11 @@ nnpfs_getattr(struct vfsmount *mnt, struct dentry *, struct kstat *stat);
 nnpfs_getattr (struct dentry *dentry, struct iattr *attr);
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,13)
+static void *
+#else
 static int
+#endif
 nnpfs_follow_link (struct dentry *dentry, struct nameidata *nd);
 
 /*
@@ -213,6 +223,9 @@ struct file_operations nnpfs_file_operations = {
     flush:	nnpfs_flush,
     release:	nnpfs_release_file,
     fsync:	nnpfs_fsync,
+#ifdef LINUX2_5
+    sendfile:	nnpfs_sendfile,
+#endif
 };
 
 struct file_operations nnpfs_dead_operations = {
@@ -263,7 +276,9 @@ struct inode_operations nnpfs_dead_inode_operations = {
 struct inode_operations nnpfs_link_inode_operations = {
     readlink: 		nnpfs_readlink,
     follow_link: 	nnpfs_follow_link,
-#ifndef LINUX2_5
+#ifdef LINUX2_5
+    put_link: 		page_put_link,
+#else
     revalidate:		nnpfs_revalidate_inode,
 #endif
 };
@@ -481,20 +496,37 @@ static int
 nnpfs_open(struct inode *i, struct file *f)
 {
   int ret;
+  int done = 0;
+  int tries = 0;
 
   NNPFSDEB(XDEBVNOPS, ("nnpfs_open inode: %p f->f_mode: %d aliases:",
-		     i, f->f_mode));
+		       i, f->f_mode));
   nnpfs_print_aliases(i);
 
-  if (f->f_mode & FMODE_WRITE)
-      ret = nnpfs_open_valid(i, NNPFS_OPEN_NW);
-  else
-      ret = nnpfs_open_valid(i, NNPFS_OPEN_NR);
+  while (!done) {
+      if (f->f_mode & FMODE_WRITE)
+	  ret = nnpfs_open_valid(i, NNPFS_OPEN_NW);
+      else
+	  ret = nnpfs_open_valid(i, NNPFS_OPEN_NR);
+      
+      done = 1; /* common case */
 
 #ifdef LINUX2_5
-  if (ret == 0)
-      f->f_mapping = i->i_mapping;
+      if (ret == 0) {
+	  f->f_mapping = i->i_mapping;
+	  
+	  if (f->f_mapping->a_ops == &nnpfs_null_aops) {
+	      if (++tries > 4) {
+		  printk("nnpfs_open: failed, aops was null\n");
+		  return -ENODEV;
+	      }
+	      
+	      done = 0; /* bad aops, try again */
+	      printk("nnpfs_open: avoided using null aops\n");
+	  }
+      }
 #endif
+  }
 
   return ret;
 }
@@ -648,7 +680,7 @@ done:
  */
 
 static int
-check_rights (u_char rights, int mode)
+check_rights (nnpfs_rights rights, int mode)
 {
     int error = 0;
 
@@ -697,7 +729,7 @@ nnpfs_permission(struct inode *inode, int mode)
 	
 	error = -EACCES;
 	
-	for (i = 0; i < MAXRIGHTS; i++)
+	for (i = 0; i < NNPFS_MAXRIGHTS; i++)
 	    if (xn->id[i] == pag) {
 		error = check_rights (xn->rights[i], mode);
 		break;
@@ -714,12 +746,12 @@ done:
  */
 
 static int
-nnpfs_data_valid(struct inode *vp, u_int tok, uint32_t want_offset)
+nnpfs_data_valid(struct inode *vp, u_int tok, loff_t want_offset)
 {
     struct nnpfs *nnpfsp;
     struct nnpfs_node *xn;
     int error = 0;
-    uint32_t offset;
+    loff_t offset;
     struct nnpfs_message_getdata msg;
 
     error = nnpfs_inode_valid(vp);
@@ -734,11 +766,11 @@ nnpfs_data_valid(struct inode *vp, u_int tok, uint32_t want_offset)
 	if (NNPFS_TOKEN_GOT(xn, tok|NNPFS_ATTR_R) && offset > xn->attr.xa_size)
 	    offset = xn->attr.xa_size;
 
-	NNPFSDEB(XDEBVNOPS, ("nnpfs_data_valid: offset: want %ld has %ld, "
-			   "tokens: want %lx has %lx length: %ld\n",
-			   (long) offset, (long) xn->offset,
-			   (long) tok, (long) xn->tokens,
-			   (long) xn->attr.xa_size));
+	NNPFSDEB(XDEBVNOPS, ("nnpfs_data_valid: offset: want %lld has %lld, "
+			   "tokens: want %x has %x length: %lld\n",
+			   (long long) offset, (long long) xn->offset,
+			   tok, xn->tokens,
+			   (long long) xn->attr.xa_size));
 
 	if (NNPFS_TOKEN_GOT(xn, tok))
 	    if(offset <= xn->offset || xn->attr.xa_type == NNPFS_FILE_DIR)
@@ -790,8 +822,11 @@ update_from_cache_node (struct inode *inode)
 
     inode->i_size   = t_inode->i_size;
     inode->i_blocks = inode->i_size >> I_BLOCKS_BITS; /* / I_BLOCKS_UNIT */
-    inode->i_mtime  = t_inode->i_mtime;
     VNODE_TO_XNODE(inode)->flags |= NNPFS_DATA_DIRTY;
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
+    inode->i_mtime  = t_inode->i_mtime;
+#endif
 }
 
 /*
@@ -855,6 +890,13 @@ nnpfs_read_file(struct file *file, char *buf,size_t count, loff_t *ppos)
 	NNPFSDEB(XDEBVNOPS, ("nnpfs_read_file: data not valid %d\n", error));
 	return error;
     }
+    
+#ifdef LINUX2_5
+    if (file->f_mapping->a_ops->readpage == NULL) {
+        printk("nnpfs_read_file: bad aops\n");
+        return -EINVAL;
+    }
+#endif
 
     error = generic_file_read (file, buf, count, ppos);
 
@@ -1281,15 +1323,6 @@ nnpfs_readdir(struct file * file, void * dirent, filldir_t filldir)
 	error = NNPFS_DIRENT_BLOCKSIZE;
 	file->f_pos += error;
 
-	if (error != NNPFS_DIRENT_BLOCKSIZE) {
-	    printk(KERN_EMERG
-		   "NNPFS Panic: expected %d bytes, got %d in nnpfs_readdir\n",
-		   NNPFS_DIRENT_BLOCKSIZE, error);
-	    kill_proc (current->pid, SIGSEGV, 0);
-	    kunmap (page);
-	    page_cache_release (page);
-	    return -EINVAL;
-	}
 	NNPFSDEB(XDEBREADDIR, ("nnpfs_readdir error: %d\n",error));
 	while (offset < NNPFS_DIRENT_BLOCKSIZE) {
 	    struct nnpfs_dirent *xdirent = (struct nnpfs_dirent *) (buf + offset);
@@ -1313,6 +1346,22 @@ nnpfs_readdir(struct file * file, void * dirent, filldir_t filldir)
 		return 0;
 	    }
 	    offset += xdirent->d_reclen;
+
+	    if (xdirent->d_reclen == 0) {
+		struct inode *cache_inode = DENTRY_TO_INODE(t);
+		printk(KERN_EMERG "NNPFS Panic: "
+		       "empty dirent at %lld in nnpfs_readdir, pos %d size %d\n",
+		       (long long)offset, (int)file->f_pos,
+		       (int)cache_inode->i_size);
+		NNPFSDEB(XDEBVNOPS, ("inode: %p aliases:", inode));
+		nnpfs_print_aliases(inode);
+
+		NNPFSDEB(XDEBVNOPS, ("cache_inode: %p aliases:", cache_inode));
+		nnpfs_print_aliases(cache_inode);
+
+		error = -EIO;
+		break;
+	    }
 	}
 	kunmap (page);
 	page_cache_release (page);
@@ -1334,8 +1383,13 @@ nnpfs_readlink (struct dentry *dentry, char *buffer, int buflen)
     NNPFSDEB(XDEBVNOPS, ("nnpfs_readlink\n"));
     
     error = nnpfs_data_valid(inode, NNPFS_DATA_R, inode->i_size);
-    if (error == 0)
+    if (error == 0) {
+#ifdef LINUX2_5
+	error = generic_readlink(dentry, buffer, buflen);
+#else
 	error = page_readlink(dentry, buffer, buflen);
+#endif
+    }
 
     return error;
 }
@@ -1344,7 +1398,11 @@ nnpfs_readlink (struct dentry *dentry, char *buffer, int buflen)
  *
  */
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,13)
+static void *
+#else
 static int
+#endif
 nnpfs_follow_link (struct dentry *dentry,
 		 struct nameidata *nd)
 {
@@ -1355,9 +1413,17 @@ nnpfs_follow_link (struct dentry *dentry,
     
     error = nnpfs_data_valid(inode, NNPFS_DATA_R, inode->i_size);
     if (error)
-	return error;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,13)
+       return ERR_PTR(error);
+#else
+       return error;
+#endif
 
+#ifdef LINUX2_5
+    return page_follow_link_light(dentry, nd);
+#else
     return page_follow_link(dentry, nd);
+#endif
 }
 
 /*
@@ -1448,13 +1514,21 @@ nnpfs_setattr (struct dentry *dentry, struct iattr *attr)
 	msg.cred.pag = nnpfs_get_pag();
         msg.handle = xn->handle;
         vattr2nnpfs_attr(attr, &msg.attr);
-	inode_setattr(inode, attr);
+
 	if (NNPFS_TOKEN_GOT(xn, NNPFS_DATA_R)) {
-	    if (S_ISREG(inode->i_mode))
-		XA_SET_SIZE(&msg.attr,  inode->i_size);
-	    XA_SET_MTIME(&msg.attr, NNPFS_GET_TIME_SEC(inode->i_mtime));
+	    if (S_ISREG(inode->i_mode)) {
+		if (attr->ia_valid & ATTR_SIZE)
+		    XA_SET_SIZE(&msg.attr,  attr->ia_size);
+		else
+		    XA_SET_SIZE(&msg.attr,  inode->i_size);
+	    }
+
+	    if (attr->ia_valid & ATTR_MTIME_SET)
+		XA_SET_MTIME(&msg.attr, NNPFS_GET_TIME_SEC(attr->ia_mtime));
+            else
+		XA_SET_MTIME(&msg.attr, NNPFS_GET_TIME_SEC(inode->i_mtime));
 	}
-        NNPFS_TOKEN_CLEAR(xn, NNPFS_ATTR_VALID, NNPFS_ATTR_MASK);
+
         error = nnpfs_message_rpc(nnpfsp, &msg.header, sizeof(msg));
         if (error == 0) {
             error = ((struct nnpfs_message_wakeup *) & msg)->error;
@@ -1603,3 +1677,28 @@ nnpfs_d_delete(struct dentry *dentry)
     }
     return 0;
 }
+
+#ifdef LINUX2_5
+static ssize_t
+nnpfs_sendfile(struct file *file, loff_t *ppos, size_t count,
+	       read_actor_t actor, void *target)
+{
+    int error = 0;
+    struct inode *inode = file->f_dentry->d_inode;
+    struct nnpfs_node *xn = VNODE_TO_XNODE(inode);
+    
+    if (xn != NULL)
+	NNPFSDEB(XDEBVNOPS, ("nnpfs_sendfile: tokens: 0x%x\n", xn->tokens));
+
+    error = nnpfs_data_valid(inode, NNPFS_DATA_R, *ppos + count);
+    if (error) {
+	NNPFSDEB(XDEBVNOPS, ("nnpfs_sendfile: data not valid %d\n", error));
+	return error;
+    }
+
+    error = generic_file_sendfile(file, ppos, count, actor, target);
+
+    NNPFSDEB(XDEBVNOPS, ("nnpfs_sendfile: error = %d\n", error));
+    return error;
+}
+#endif
