@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2004 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995-2004, 2006 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -44,7 +44,7 @@
 #include <nnpfs/nnpfs_dev.h>
 
 #ifdef RCSID
-RCSID("$Id: nnpfs_message.c,v 1.136 2004/06/01 22:10:21 tol Exp $");
+RCSID("$Id: nnpfs_message.c,v 1.141 2006/05/04 07:37:01 tol Exp $");
 #endif
 
 static void
@@ -260,8 +260,7 @@ nnpfs_message_installattr(struct nnpfs *nnpfsp,
     }
     
     if (!NNPFS_TOKEN_GOT(t, NNPFS_DATA_R) && DATA_FROM_XNODE(t)) {
-	printk(KERN_EMERG 
-	       "NNPFS SoftAssert: nnpfs_message_installattr: "
+	printk("NNPFS SoftAssert: nnpfs_message_installattr: "
 	       "got data w/o token!\n");
     }
     
@@ -273,7 +272,7 @@ nnpfs_message_installattr(struct nnpfs *nnpfsp,
     
     memmove(t->id, message->node.id, sizeof(t->id));
     memmove(t->rights, message->node.rights, sizeof(t->rights));
-    for (i = 0; i < MAXRIGHTS; i++) {
+    for (i = 0; i < NNPFS_MAXRIGHTS; i++) {
 	NNPFSDEB(XDEBMSG, ("rights %d:", t->id[i]));
 	NNPFSDEB(XDEBMSG, (t->rights[i]&NNPFS_RIGHT_R?"r":"-"));
 	NNPFSDEB(XDEBMSG, (t->rights[i]&NNPFS_RIGHT_W?"w":"-"));
@@ -291,7 +290,7 @@ nnpfs_message_installdata(struct nnpfs *nnpfsp,
 			  u_int size)
 {
     struct nnpfs_node *t;
-    struct dentry *dentry;
+    struct dentry *dentry = ERR_PTR(-EINVAL);
     struct inode *inode;
     int error = 0;
     
@@ -309,6 +308,8 @@ nnpfs_message_installdata(struct nnpfs *nnpfsp,
     
     message->cache_name[sizeof(message->cache_name)-1] = '\0';
     NNPFSDEB(XDEBMSG, ("cache_name '%s'\n", message->cache_name));
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,15)
     if (message->flag & NNPFS_ID_HANDLE_VALID) {
 	dentry = nnpfs_fh_to_dentry(&message->cache_handle);
 	
@@ -317,9 +318,9 @@ nnpfs_message_installdata(struct nnpfs *nnpfsp,
 		   "nnpfs_message_installdata failed "
 		   "nnpfs_fh_to_dentry = %s, errno: %ld\n",
 		   message->cache_name, PTR_ERR(dentry));
-    } else
-	dentry = ERR_PTR(-EINVAL);
-    
+    }
+#endif
+
     NNPFSDEB(XDEBMSG, ("nnpfs_message_installdata dentry: %p\n",
 		       dentry));
     
@@ -360,8 +361,7 @@ nnpfs_message_installdata(struct nnpfs *nnpfsp,
 	
 	t->tokens = message->node.tokens;
 	if (!NNPFS_TOKEN_GOT(t, NNPFS_DATA_R)) {
-	    printk(KERN_EMERG 
-		   "NNPFS SoftAssert: nnpfs_message_installdata: "
+	    printk("NNPFS SoftAssert: nnpfs_message_installdata: "
 		   "got data w/o token!\n");
 	}
 	
@@ -369,12 +369,12 @@ nnpfs_message_installdata(struct nnpfs *nnpfsp,
 			   t->tokens));
 	
 	t->attr = message->node.attr;
-	NNPFSDEB(XDEBMSG, ("nnpfs_message_installdata size before: %ld\n",
-			   (long) inode->i_size));
+	NNPFSDEB(XDEBMSG, ("nnpfs_message_installdata size before: %lld\n",
+			   (long long) inode->i_size));
 	nnpfs_attr2inode (&t->attr, inode, 1);
 	t->offset = message->offset;
-	NNPFSDEB(XDEBMSG, ("nnpfs_message_installdata size after: %ld\n",
-			   (long) inode->i_size));
+	NNPFSDEB(XDEBMSG, ("nnpfs_message_installdata size after: %lld\n",
+			   (long long) inode->i_size));
 	memmove(t->id, message->node.id, sizeof(t->id));
 	memmove(t->rights, message->node.rights, sizeof(t->rights));
 	t->anonrights = message->node.anonrights;
@@ -414,7 +414,11 @@ clear_all_children (struct inode *inode, int parent)
 	subdirs = dentry->d_subdirs.next;
 	while (subdirs != &dentry->d_subdirs) {
 	    struct list_head *tmp = subdirs;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
+	    struct dentry *child = list_entry(tmp, struct dentry, d_u.d_child);
+#else
 	    struct dentry *child = list_entry(tmp, struct dentry, d_child);
+#endif
 	    subdirs = tmp->next;
 	    NNPFSDEB(XDEBMSG, ("clear_all_children child: %.*s inode: %p/%p "
 			       "dcount: %d aliases:\n",
