@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2004 Kungliga Tekniska Hgskolan
+ * Copyright (c) 2003-2004, 2006 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -47,11 +47,9 @@
 #include <linux/unistd.h>
 
 #ifdef RCSID
-RCSID("$Id: nnpfs_syscalls-lossage.c,v 1.5.2.3 2004/06/27 04:54:54 lha Exp $");
+RCSID("$Id: nnpfs_syscalls-lossage.c,v 1.15 2006/04/04 10:48:05 tol Exp $");
 #endif
 
-#ifndef HAVE_KERNEL_SYS_CALL_TABLE
-
 #ifdef LINUX2_5
 #include <linux/kallsyms.h>
 static void *lower_bound = &kernel_thread;
@@ -60,9 +58,31 @@ static void *lower_bound = &kernel_thread;
 static void *lower_bound = &empty_zero_page;
 #endif
 
-nnpfs_sys_call_function *sys_call_table;
+void * __attribute__((weak)) sys_call_table(void);
+nnpfs_sys_call_function *nnpfs_sys_call_table =
+(nnpfs_sys_call_function *)sys_call_table;
+
+const char * __attribute__((weak))
+    kallsyms_lookup(unsigned long addr,
+		    unsigned long *symbolsize,
+		    unsigned long *offset,
+		    char **modname, char *namebuf);
+
+static void **
+get_start_addr(void) {
+#ifdef __x86_64__
+    return (void **)&tasklist_lock - 0x1000;
+#else
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
+    return (void **)&mutex_lock;
+#else
+    return (void **)&init_mm;
+#endif
+
+#endif
+}
 
-#ifdef HAVE_KALLSYMS_LOOKUP
 static inline int 
 kallsym_is_equal(unsigned long addr, const char *name)
 {
@@ -77,7 +97,6 @@ kallsym_is_equal(unsigned long addr, const char *name)
 
     return 0;
 }
-#endif
 
 static inline int
 verify(void **p) {
@@ -124,17 +143,15 @@ verify(void **p) {
 	    return 0;
     
 #ifndef LINUX2_5
-    if ((p[__NR_close] != (void*)&sys_close)
-	|| (p[__NR_chdir] != (void*)&sys_chdir))
+    if (p[__NR_close] != (void*)&sys_close)
 	return 0;
 #endif
     
-#ifdef HAVE_KALLSYMS_LOOKUP
-    if (!kallsym_is_equal((unsigned long)p[__NR_close], "sys_close")
-	|| !kallsym_is_equal((unsigned long)p[__NR_chdir], "sys_chdir"))
+    if (kallsyms_lookup
+	&& (!kallsym_is_equal((unsigned long)p[__NR_close], "sys_close")
+	    || !kallsym_is_equal((unsigned long)p[__NR_chdir], "sys_chdir")))
 	return 0;
-#endif
-
+    
     return 1;
 }
 
@@ -148,14 +165,18 @@ static inline int looks_good(void **p)
 int
 nnpfs_fixup_syscall_lossage(void)
 {
-    void **ptr = (void **)&init_mm;
+    void **ptr = get_start_addr();
     void **limit;
 
+    if (nnpfs_sys_call_table != NULL) {
+	printk("nnpfs_sys_call_table: %p\n", nnpfs_sys_call_table);
+	return 0;
+    }
+
     lower_bound = (void*)((unsigned long)lower_bound & ~0xfffff);
-    sys_call_table = NULL;
 
     for (limit = ptr + 16 * 1024;
-	 ptr < limit && sys_call_table == NULL; ptr++)
+	 ptr < limit && nnpfs_sys_call_table == NULL; ptr++)
     {
 	int ok = 1;
 	int i;
@@ -169,19 +190,17 @@ nnpfs_fixup_syscall_lossage(void)
 	}
 
 	if (ok && verify(ptr)) {
-	    sys_call_table = (nnpfs_sys_call_function*)ptr;
+	    nnpfs_sys_call_table = (nnpfs_sys_call_function*)ptr;
 	    break;
 	}
     }
 
-    if (sys_call_table == NULL) {
+    if (nnpfs_sys_call_table == NULL) {
 	printk("Failed to find address of sys_call_table\n");
  	return -EIO;
     }
 
-    printk("Found sys_call_table at %p\n", sys_call_table);
+    printk("Found sys_call_table at %p\n", nnpfs_sys_call_table);
 
     return 0;
 }
-
-#endif /* !HAVE_KERNEL_SYS_CALL_TABLE */
