@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2004 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2006 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -38,25 +38,35 @@
 
 #include <nnpfs/nnpfs_locl.h>
 #include <nnpfs/nnpfs_dev.h>
-#include <kafs.h>
+#include <arla-pioctl.h>
 #include <nnpfs/nnpfs_syscalls.h>
 #include <nnpfs/nnpfs_common.h>
 #include <linux/file.h>
 #include <linux/smp_lock.h>
 #include <linux/smp.h>
 #include <linux/proc_fs.h>
+#include <linux/namespace.h>
 
 #ifdef CONFIG_SECURITY
 #include <linux/security.h>
 #endif
 
+#ifdef CONFIG_COMPAT
+#include <linux/ioctl32.h>
+#endif
+
+#ifdef CONFIG_DEBUG_RODATA
+#undef SYSCALLHACK
+#endif
+
 #ifdef RCSID
-RCSID("$Id: nnpfs_syscalls.c,v 1.92.2.6 2004/06/24 09:06:43 lha Exp $");
+RCSID("$Id: nnpfs_syscalls.c,v 1.120 2006/06/08 14:09:56 tol Exp $");
 #endif
 
 static struct proc_dir_entry *nnpfs_procfs_dir;
 
-#define VIOC_SYSCALL _IOW('C',1,void *)
+#define ARLA_VIOC_SYSCALL _IOW('C',1,void *)
+#define ARLA_VIOC_SYSCALL32 _IOW('C',1,u32)
 #define NNPFS_PROC_DIR "nnpfs"
 #define NNPFS_PROC_NODE "afs_ioctl"
  
@@ -77,10 +87,11 @@ static struct file_operations nnpfs_procfs_fops = {
 };
 
 
+#ifdef SYSCALLHACK
 typedef asmlinkage long (*sys_afs_function)(int operation,
 					    char *a_pathP,
 					    int a_opcode,
-					    struct ViceIoctl *a_paramsP,
+					    struct arlaViceIoctl *a_paramsP,
 					    int a_followSymlinks);
 
 typedef asmlinkage long (*sys_setgroups_function)(int, gid_t *);
@@ -89,37 +100,40 @@ typedef asmlinkage long (*sys_setgroups_function)(int, gid_t *);
 typedef int (*sys_setgroups16_function)(int, old_gid_t *);
 #endif
 
-#ifdef HAVE_KERNEL_SYS_CALL_TABLE
-extern nnpfs_sys_call_function sys_call_table[];
-#ifdef NEED_VICEIOCTL32
-extern uint32_t sys_call_table32[];
-#endif
-#else
-extern nnpfs_sys_call_function *sys_call_table;
+extern nnpfs_sys_call_function *nnpfs_sys_call_table;
+#if 0
 #ifdef NEED_VICEIOCTL32
 extern uint32_t *sys_call_table32;
 #endif
 #endif
 
 static nnpfs_sys_call_function old_afs_syscall = NULL;
+#if 0
 #ifdef NEED_VICEIOCTL32
 static uint32_t old_afs_syscall32=0;
 #endif
+#endif
 
 static nnpfs_sys_call_function old_setgroups = NULL;
 
 #ifdef ARLA_NR_setgroups16
 static nnpfs_sys_call_function old_setgroups16 = NULL;
 #endif
+#endif /* SYSCALLHACK */
 
+#if defined(SYSCALLHACK) || defined(CONFIG_SECURITY)
 static int nnpfs_sec_registered = 0;
+#endif
 
 #ifdef LINUX2_5    
 
-/* valid PAGs are [NNPFS_PAG_LLIM, NNPFS_PAG_ULIM) */
-
-#define NNPFS_PAG_LLIM 0x10000000
-#define NNPFS_PAG_ULIM 0x7fffffff
+/*
+ * Valid PAGs are [NNPFS_PAG_LLIM, NNPFS_PAG_ULIM).
+ * Use the same PAG range as when using 16-bit gids
+ * and stay out of the PTS ID range [0, 0x80000000).
+ */
+#define NNPFS_PAG_LLIM 0x83007F00
+#define NNPFS_PAG_ULIM 0x8700BF00
 #define NNPFS_PAG_NOTFOUND NNPFS_PAG_ULIM
 
 #ifdef CONFIG_SECURITY
@@ -132,19 +146,22 @@ static int nnpfs_sec_registered = 0;
  * fallback on group list version
  */
 
+#define SEC2PAG(s) (nnpfs_pag_t)(unsigned long)(s)
+#define PAG2SEC(p) (void *)(unsigned long)(p)
+
 static int
 nnpfs_sec_task_alloc(struct task_struct *p)
 {
-    nnpfs_pag_t pag = (nnpfs_pag_t)current->security;
+    nnpfs_pag_t pag = SEC2PAG(current->security);
     if (pag)
-	p->security = (void *)pag;
+	p->security = PAG2SEC(pag);
     return 0;
 }
 
 static void
 nnpfs_sec_task_free(struct task_struct *p)
 {
-    nnpfs_pag_t pag = (nnpfs_pag_t)p->security;
+    nnpfs_pag_t pag = SEC2PAG(p->security);
     if (pag)
 	p->security = NULL;
 }
@@ -152,7 +169,7 @@ nnpfs_sec_task_free(struct task_struct *p)
 static nnpfs_pag_t
 nnpfs_get_pag_sec(void)
 {
-    nnpfs_pag_t pag = (nnpfs_pag_t)current->security;
+    nnpfs_pag_t pag = SEC2PAG(current->security);
     if (pag)
 	return pag;
 
@@ -168,7 +185,7 @@ nnpfs_set_pag_sec(void)
 	return -ENOMEM;
     /* pagnum = NNPFS_PAG_LLIM; */
     
-    current->security = (void *)pagnum;
+    current->security = PAG2SEC(pagnum);
     pagnum++;
     return 0;
 }
@@ -181,6 +198,7 @@ static struct security_operations nnpfs_sec_ops = {
 
 #endif /* CONFIG_SECURITY */
 
+#ifdef GROUPPAGS
 /* find pag index in group list, or return NNPFS_PAG_NOTFOUND */
 static int
 find_pag(struct group_info *gi)
@@ -191,7 +209,7 @@ find_pag(struct group_info *gi)
 
     for (i = gi->ngroups - 1; i >= 0; i--) {
 	gid_t group = GROUP_AT(gi, i);
-	if (group >= NNPFS_PAG_LLIM && group < NNPFS_PAG_ULIM) {
+	if ((nnpfs_pag_t)group >= NNPFS_PAG_LLIM && (nnpfs_pag_t)group < NNPFS_PAG_ULIM) {
 	    NNPFSDEB(XDEBSYS,
 		     ("find_pag: Existing pag %u at pos %u\n", group, i));
 	    return i;
@@ -287,6 +305,7 @@ nnpfs_set_pag_group(void)
 
     return ret;
 }
+#endif /* GROUPPAGS */
 
 nnpfs_pag_t
 nnpfs_get_pag()
@@ -296,7 +315,11 @@ nnpfs_get_pag()
 	return nnpfs_get_pag_sec();
 #endif /* !CONFIG_SECURITY */
 
+#ifdef GROUPPAGS
     return nnpfs_get_pag_group();
+#endif /* GROUPPAGS */
+
+    return current->uid;
 }
 
 static inline int
@@ -307,9 +330,15 @@ nnpfs_setpag_call(void)
 	return nnpfs_set_pag_sec();
 #endif /* !CONFIG_SECURITY */
 
+#ifdef GROUPPAGS
     return nnpfs_set_pag_group();
+#endif /* GROUPPAGS */
+
+    return -EINVAL;
 }
 
+#ifdef GROUPPAGS
+#ifdef SYSCALLHACK
 /*
  * A wrapper around sys_setgroups that tries to preserve the pag.
  */
@@ -338,8 +367,8 @@ nnpfs_setgroups (int gidsetsize, gid_t __user *usergrouplist)
 	    if (grouplist[i] == current_pag)
 		pag_in_new = 1;
 	    else 
-		if (grouplist[i] >= NNPFS_PAG_LLIM
-		    && grouplist[i] <= NNPFS_PAG_ULIM)
+		if ((nnpfs_pag_t)(grouplist[i]) >= NNPFS_PAG_LLIM
+		    && (nnpfs_pag_t)(grouplist[i]) <= NNPFS_PAG_ULIM)
 		    return -EINVAL; /* User fakes pag attempt */
 	}
     }
@@ -392,6 +421,8 @@ nnpfs_setgroups16 (int gidsetsize, old_gid_t *grouplist)
 }
 #endif /* 0 */
 #endif /* ARLA_NR_setgroups16 */
+#endif /* SYSCALLHACK */
+#endif /* GROUPPAGS */
 
 #else /* !LINUX2_5 */
 
@@ -406,9 +437,12 @@ nnpfs_setgroups16 (int gidsetsize, old_gid_t *grouplist)
 #define NNPFS_PAG2_LLIM 32512
 #define NNPFS_PAG2_ULIM 48896
 
+#ifdef GROUPPAGS
 static gid_t pag_part_one = NNPFS_PAG1_LLIM ;
 static gid_t pag_part_two = NNPFS_PAG2_LLIM ;
+#endif /* GROUPPAGS */
 
+#ifdef GROUPPAGS
 static int
 nnpfs_is_pag(void)
 {
@@ -422,19 +456,24 @@ nnpfs_is_pag(void)
     else
         return 0;
 }
+#endif /* GROUPPAGS */
 
 nnpfs_pag_t
 nnpfs_get_pag(void)
 {
+#ifdef GROUPPAGS
     if (nnpfs_is_pag()) {
 	
 	return (((current->groups[0] << 16) & 0xFFFF0000) |
 		((current->groups[1] & 0x0000FFFF))) ;
 
-    } else
-	return current->uid;
+    }
+#endif /* GROUPPAGS */
+
+    return current->uid;
 }
 
+#ifdef GROUPPAGS
 static int
 store_pag (gid_t part1, gid_t part2)
 {
@@ -453,6 +492,7 @@ store_pag (gid_t part1, gid_t part2)
     return 0;
 }
 
+#ifdef SYSCALLHACK
 /*
  * Nuke any existing pag.
  */
@@ -471,10 +511,13 @@ unpag (void)
     }
     return 0;
 }
+#endif /* SYSCALLHACK */
+#endif /* GROUPPAGS */
 
 static int
 nnpfs_setpag_call(void)
 {
+#ifdef GROUPPAGS
     int i;
     int ret;
 
@@ -499,8 +542,13 @@ nnpfs_setpag_call(void)
 	pag_part_one = NNPFS_PAG1_LLIM ;
 	
     return 0;
+#endif /* GROUPPAGS */
+
+    return -EINVAL;
 }
 
+#ifdef GROUPPAGS
+#ifdef SYSCALLHACK
 /*
  * A wrapper around sys_setgroups that tries to preserve the pag. If
  * the pag wouldn't fit after setting the group list, we return E2BIG.
@@ -592,6 +640,8 @@ nnpfs_setgroups16 (int gidsetsize, old_gid_t *grouplist)
     }
 }
 #endif /* ARLA_NR_setgroups16 */
+#endif /* SYSCALLHACK */
+#endif /* GROUPPAGS */
 
 #endif /* !LINUX2_5 */
 
@@ -601,13 +651,44 @@ struct file_handle {
     __u32 gen;
 };
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,15)
+
+/*
+ * Find the first vfsmount of inode,
+ * (hope it's good enough)
+ */
+
+static struct vfsmount *
+nnpfs_find_first_vfsmount(struct inode *inode) {
+    struct dentry *root = dget(inode->i_sb->s_root);
+    struct namespace *namespace = current->namespace;
+    struct vfsmount *mnt = NULL;
+    struct vfsmount *tmp = NULL;
+    struct list_head *head;
+    
+    down_read(&namespace->sem);
+    list_for_each(head, &namespace->list) {
+	tmp = list_entry(head, struct vfsmount, mnt_list);
+	if (tmp->mnt_root == root) {
+	    mnt = tmp;
+	    mntget(mnt);
+	    break;
+	}
+    }
+    up_read(&namespace->sem);
+
+    dput(root);
+    return mnt;
+}
+#endif
+
 /*
  * copy a `file_handle' of `dentry' to the buffer pointed to in
  * `vice_ioctl'.  return 0 or -errno.
  */
 
 static int
-fhget_call(struct ViceIoctl *vice_ioctl, const struct dentry *dentry)
+fhget_call(struct arlaViceIoctl *vice_ioctl, const struct dentry *dentry)
 {
     struct file_handle fh;
     struct nnpfs_cache_handle temp_fh;
@@ -643,6 +724,8 @@ fhget_call(struct ViceIoctl *vice_ioctl, const struct dentry *dentry)
     return 0;
 }
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,15)
+
 /*
  * lookup file identified by `handle' and return a dentry or an error.
  */
@@ -698,6 +781,7 @@ nnpfs_fh_to_dentry(struct nnpfs_cache_handle *handle)
 
     return dentry;
 }
+#endif
 
 /*
  * open the file identified by the file handle in `vice_ioctl' with
@@ -706,9 +790,13 @@ nnpfs_fh_to_dentry(struct nnpfs_cache_handle *handle)
  */
 
 static int
-fhopen_call(struct ViceIoctl *vice_ioctl, int mode)
+fhopen_call(struct arlaViceIoctl *vice_ioctl, int mode)
 {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,15)
+    return -EINVAL;
+#else
     struct dentry *dentry;
+    struct vfsmount *mnt;
     struct nnpfs_cache_handle handle;
     int fd;
     struct inode *inode;
@@ -728,24 +816,36 @@ fhopen_call(struct ViceIoctl *vice_ioctl, int mode)
 	return PTR_ERR(dentry);
     inode = dentry->d_inode;
 
-    f = dentry_open(dentry, NULL, O_RDWR);
-    if (IS_ERR(f))
-	return PTR_ERR(f);
     fd = get_unused_fd();
     if (fd < 0) {
-	put_filp(f);
 	dput(dentry);
 	return fd;
     }
+
+    mnt = nnpfs_find_first_vfsmount(inode);
+    if (mnt == NULL) {
+	put_unused_fd(fd);
+	dput(dentry);
+	return -ENOENT;
+    }
+
+    f = dentry_open(dentry, mnt, O_RDWR | O_LARGEFILE);
+    if (IS_ERR(f)) {
+	/* dentry_open puts mnt and dentry for us */
+	put_unused_fd(fd);
+	return PTR_ERR(f);
+    }
+
     fd_install(fd, f);
 
     NNPFSDEB(XDEBSYS, ("fhopen_call: returns fd: %d\n", fd));
 
     return fd;
+#endif
 }
 
 static int
-nnpfs_debug (struct ViceIoctl *vice_ioctl)
+nnpfs_debug (struct arlaViceIoctl *vice_ioctl)
 {
     if (!capable(CAP_SYS_ADMIN))
         return -EPERM;
@@ -780,7 +880,7 @@ nnpfs_debug (struct ViceIoctl *vice_ioctl)
 }
 
 static int
-nnpfs_debug_print (struct ViceIoctl *vice_ioctl, struct dentry *node)
+nnpfs_debug_print (struct arlaViceIoctl *vice_ioctl, struct dentry *node)
 {
     if (!capable(CAP_SYS_ADMIN))
         return -EPERM;
@@ -809,7 +909,7 @@ nnpfs_debug_print (struct ViceIoctl *vice_ioctl, struct dentry *node)
 		nnpfs_print_aliases(node->d_inode);
 		nnpfs_print_children(node);
 	    } else {
-		print_nodes(&nnpfs[0]);
+		nnpfs_print_nodestats(&nnpfs[0]);
 	    }
 	    return 0;
 	default:
@@ -855,21 +955,21 @@ asmlinkage long
 sys_afs_int (int operation,
 	     char *a_pathP,
 	     int a_opcode,
-	     struct ViceIoctl *a_paramsP,
+	     struct arlaViceIoctl *a_paramsP,
 	     int a_followSymlinks)
 {
     long error = 0;
-    struct ViceIoctl vice_ioctl; 
+    struct arlaViceIoctl vice_ioctl; 
     struct nnpfs_message_pioctl msg;
     struct nnpfs_message_wakeup_data *msg2;
     struct dentry *dentry = NULL;
     struct nameidata nd;
     
-    NNPFSDEB(XDEBSYS, ("sys_afs locking kernel; cpu: %d\n", smp_processor_id()));
+    NNPFSDEB(XDEBSYS, ("sys_afs locking kernel\n"));
 
     lock_kernel();
 
-    NNPFSDEB(XDEBSYS, ("sys_afs kernel locked; cpu: %d\n", smp_processor_id()));
+    NNPFSDEB(XDEBSYS, ("sys_afs kernel locked\n"));
 
     NNPFSDEB(XDEBSYS, ("sys_afs operation: %d a_pathP: %p "
 		     "a_opcode: %d a_paramsP: %p "
@@ -878,11 +978,11 @@ sys_afs_int (int operation,
 		     a_paramsP, a_followSymlinks));
     
     switch (operation) {
-    case AFSCALL_PIOCTL:
+    case arla_AFSCALL_PIOCTL:
 	NNPFSDEB(XDEBSYS, ("nnpfs_pioctl\n"));
 	memcpy(&vice_ioctl,a_paramsP,sizeof(*a_paramsP));
 
-	if (vice_ioctl.in_size < 0) {
+	if (((int)vice_ioctl.in_size) < 0) {
 	    printk(KERN_EMERG 
 		   "nnpfs: remote pioctl: got a negative data size: opcode: %d",
 		   a_opcode);
@@ -923,27 +1023,27 @@ sys_afs_int (int operation,
 	}
 
 	switch (a_opcode) {
-	case VIOC_FHGET:
-#ifdef VIOC_FHGET_32
-	case VIOC_FHGET_32:
+	case ARLA_VIOC_FHGET:
+#ifdef ARLA_VIOC_FHGET_32
+	case ARLA_VIOC_FHGET_32:
 #endif
 	    error = fhget_call(&vice_ioctl, dentry);
 	    goto unlock;
-	case VIOC_FHOPEN:
-#ifdef VIOC_FHOPEN_32
-	case VIOC_FHOPEN_32:
+	case ARLA_VIOC_FHOPEN:
+#ifdef ARLA_VIOC_FHOPEN_32
+	case ARLA_VIOC_FHOPEN_32:
 #endif
 	    error = fhopen_call(&vice_ioctl, a_followSymlinks);
 	    goto unlock;
-	case VIOC_NNPFSDEBUG:
-#ifdef VIOC_NNPFSDEBUG_32
-	case VIOC_NNPFSDEBUG_32:
+	case ARLA_VIOC_NNPFSDEBUG:
+#ifdef ARLA_VIOC_NNPFSDEBUG_32
+	case ARLA_VIOC_NNPFSDEBUG_32:
 #endif
 	    error = nnpfs_debug (&vice_ioctl);
 	    goto unlock;
-	case VIOC_NNPFSDEBUG_PRINT:
-#ifdef VIOC_NNPFSDEBUG_PRINT_32
-	case VIOC_NNPFSDEBUG_PRINT_32:
+	case ARLA_VIOC_NNPFSDEBUG_PRINT:
+#ifdef ARLA_VIOC_NNPFSDEBUG_PRINT_32
+	case ARLA_VIOC_NNPFSDEBUG_PRINT_32:
 #endif
 	    error = nnpfs_debug_print (&vice_ioctl, dentry);
 	    goto unlock;
@@ -983,7 +1083,7 @@ sys_afs_int (int operation,
 	    error = -EINVAL;
 
 	if (error == 0 && msg2->header.opcode == NNPFS_MSG_WAKEUP_DATA) {
-	    if (vice_ioctl.out_size < 0)
+	    if (((int)vice_ioctl.out_size) < 0)
 		msg2->len = 0;
 	    else if (msg2->len > vice_ioctl.out_size)
 		msg2->len = vice_ioctl.out_size;
@@ -999,7 +1099,7 @@ sys_afs_int (int operation,
 	}
 
 	break;
-    case AFSCALL_SETPAG:
+    case arla_AFSCALL_SETPAG:
 	error = nnpfs_setpag_call();
 	break;
     default:
@@ -1014,13 +1114,14 @@ sys_afs_int (int operation,
 
     NNPFSDEB(XDEBSYS, ("nnpfs_syscall returns error: %ld\n", error));
 
-    NNPFSDEB(XDEBSYS, ("sys_afs kernel unlock; cpu: %d\n", smp_processor_id()));
+    NNPFSDEB(XDEBSYS, ("sys_afs kernel unlock\n"));
     unlock_kernel();
 
     return error;
 }
 
-#ifdef NEED_VICEIOCTL32
+#if 0
+#if defined CONFIG_COMPAT || (defined SYSCALLHACK && defined NEED_VICEIOCTL32)
 asmlinkage long
 sys32_afs (int operation,
 	   char *a_pathP,
@@ -1028,10 +1129,10 @@ sys32_afs (int operation,
 	   struct ViceIoctl32 *a_paramsP,
 	   int a_followSymlinks)
 {
-    struct ViceIoctl vice_ioctl;
+    struct arlaViceIoctl vice_ioctl;
     struct ViceIoctl32 vice_ioctl32;
 
-    if (operation == AFSCALL_PIOCTL) {
+    if (operation == arla_AFSCALL_PIOCTL) {
 	if(copy_from_user(&vice_ioctl32, a_paramsP, sizeof(*a_paramsP)) != 0)
 	    return -EFAULT;
 
@@ -1045,17 +1146,18 @@ sys32_afs (int operation,
 		       a_followSymlinks);
 }
 #endif
+#endif /* 0 */
 
 asmlinkage long
 sys_afs (int operation,
 	 char *a_pathP,
 	 int a_opcode,
-	 struct ViceIoctl *a_paramsP,
+	 struct arlaViceIoctl *a_paramsP,
 	 int a_followSymlinks)
 {
-    struct ViceIoctl vice_ioctl;
+    struct arlaViceIoctl vice_ioctl;
 
-    if (operation == AFSCALL_PIOCTL) {
+    if (operation == arla_AFSCALL_PIOCTL) {
 	if(copy_from_user(&vice_ioctl, a_paramsP, sizeof(*a_paramsP)) != 0)
 	    return -EFAULT;
     }
@@ -1071,7 +1173,7 @@ nnpfs_procfs_ioctl(struct inode *inode, struct file *file,
     afsprocdata args;
     long ret;
 
-    if (cmd != VIOC_SYSCALL)
+    if (cmd != ARLA_VIOC_SYSCALL)
 	return -EINVAL;
 
     if (copy_from_user(&args, (void *)arg, sizeof(args)))
@@ -1079,11 +1181,41 @@ nnpfs_procfs_ioctl(struct inode *inode, struct file *file,
     
     ret = sys_afs((int)args.syscall,
 		  (char *)args.param1, (int)args.param2,
-		  (struct ViceIoctl *)args.param3, (int)args.param4);
+		  (struct arlaViceIoctl *)args.param3, (int)args.param4);
     return ret;
 }
 
-static int __init nnpfs_init_procfs(void)
+#if 0
+#ifdef CONFIG_COMPAT
+typedef struct afsprocdata32 {
+    u32 param4;
+    u32 param3;
+    u32 param2;
+    u32 param1;
+    u32 syscall;
+} afsprocdata32;
+
+static int
+nnpfs_procfs_ioctl32(unsigned int fd, unsigned int cmd, unsigned long arg,
+		     struct file *file) {
+    afsprocdata32 args;
+    long ret;
+
+    if (cmd != ARLA_VIOC_SYSCALL32)
+	return -EINVAL;
+
+    if (copy_from_user(&args, (void *)arg, sizeof(args)))
+	return -EFAULT;
+    
+    ret = sys32_afs((int)args.syscall,
+		    (char *)(long)args.param1, (int)args.param2,
+		    (struct ViceIoctl32 *)(long)args.param3, (int)args.param4);
+    return ret;
+}
+#endif /* CONFIG_COMPAT */
+#endif
+
+static int nnpfs_init_procfs(void)
 {
     struct proc_dir_entry *entry;
     
@@ -1102,30 +1234,49 @@ static int __init nnpfs_init_procfs(void)
     
     entry->owner = THIS_MODULE;
     entry->proc_fops = &nnpfs_procfs_fops;
+
+#if 0
+#ifdef CONFIG_COMPAT
+    if (register_ioctl32_conversion(ARLA_VIOC_SYSCALL32, nnpfs_procfs_ioctl32)) {
+	printk(KERN_EMERG "nnpfs_init_procfs: unable to register ioctl32\n");
+    }
+#endif /* CONFIG_COMPAT */
+#endif
     
     NNPFSDEB(XDEBSYS, ("nnpfs_init_procfs: success\n"));
 
     return 0;
 }
 
-static void __exit nnpfs_exit_procfs(void)
+static void nnpfs_exit_procfs(void)
 {
+#if 0
+#ifdef CONFIG_COMPAT
+    if (unregister_ioctl32_conversion(ARLA_VIOC_SYSCALL32)) {
+	printk(KERN_EMERG "nnpfs_exit_procfs: error unregistering ioctl32\n");
+    }
+#endif /* CONFIG_COMPAT */
+#endif
     remove_proc_entry(NNPFS_PROC_NODE, nnpfs_procfs_dir);
     remove_proc_entry(NNPFS_PROC_DIR, proc_root_fs);
 }
 
+#ifdef GROUPPAGS
+#ifdef SYSCALLHACK
 static void
 install_setgroups(void)
 {
-    old_setgroups = sys_call_table[ARLA_NR_setgroups];
-    sys_call_table[ARLA_NR_setgroups] = 
+    old_setgroups = nnpfs_sys_call_table[ARLA_NR_setgroups];
+    nnpfs_sys_call_table[ARLA_NR_setgroups] = 
 	(nnpfs_sys_call_function)&nnpfs_setgroups;
 #ifdef ARLA_NR_setgroups16
-    old_setgroups16 = sys_call_table[ARLA_NR_setgroups16];
-    sys_call_table[ARLA_NR_setgroups16] = 
+    old_setgroups16 = nnpfs_sys_call_table[ARLA_NR_setgroups16];
+    nnpfs_sys_call_table[ARLA_NR_setgroups16] = 
 	(nnpfs_sys_call_function)&nnpfs_setgroups16;
 #endif
 }
+#endif /* SYSCALLHACK */
+#endif /* GROUPPAGS */
 
 
 void
@@ -1141,24 +1292,26 @@ install_afs_syscall(void)
 
     nnpfs_init_procfs();
 
-#ifndef HAVE_KERNEL_SYS_CALL_TABLE
+#ifdef SYSCALLHACK
     if (nnpfs_fixup_syscall_lossage()) {
 	NNPFSDEB(XDEBSYS,
 		 ("install_afs_syscall: no syscalltable found\n"));
 	return;
     }
-#endif
 
     if (!nnpfs_sec_registered)
 	install_setgroups();	
 
-    old_afs_syscall = sys_call_table[__NR_afs_syscall];
-    sys_call_table[__NR_afs_syscall] = 
+    old_afs_syscall = nnpfs_sys_call_table[__NR_afs_syscall];
+    nnpfs_sys_call_table[__NR_afs_syscall] = 
 	(nnpfs_sys_call_function)&sys_afs;
+#if 0
 #ifdef NEED_VICEIOCTL32
     old_afs_syscall32 = sys_call_table32[__NR_afs_syscall];
     sys_call_table32[__NR_afs_syscall] = (uint32_t)&sys32_afs;
 #endif
+#endif
+#endif /* SYSCALLHACK */
 }
 
 void
@@ -1166,16 +1319,20 @@ restore_afs_syscall (void)
 {
     nnpfs_exit_procfs();
 
+#ifdef SYSCALLHACK
     if (old_afs_syscall) {
-	sys_call_table[__NR_afs_syscall] = old_afs_syscall;
+	nnpfs_sys_call_table[__NR_afs_syscall] = old_afs_syscall;
 	old_afs_syscall = NULL;
     }
+#if 0
 #ifdef NEED_VICEIOCTL32
     if (old_afs_syscall32) {
 	sys_call_table32[__NR_afs_syscall] = old_afs_syscall32;
 	old_afs_syscall32 = 0;
     }
 #endif
+#endif
+#endif /* SYSCALLHACK */
 
 #ifdef CONFIG_SECURITY
     if (nnpfs_sec_registered)
@@ -1183,14 +1340,18 @@ restore_afs_syscall (void)
 	    printk(KERN_EMERG "nnpfs_exit_sec: couldn't unregister\n");
 #endif /* !CONFIG_SECURITY */
 
+#ifdef GROUPPAGS
+#ifdef SYSCALLHACK
     if (old_setgroups) {
-	sys_call_table[ARLA_NR_setgroups] = old_setgroups;
+	nnpfs_sys_call_table[ARLA_NR_setgroups] = old_setgroups;
 	old_setgroups = NULL;
     }
 #ifdef ARLA_NR_setgroups16
     if (old_setgroups16) {
-	sys_call_table[ARLA_NR_setgroups16] = old_setgroups16;
+	nnpfs_sys_call_table[ARLA_NR_setgroups16] = old_setgroups16;
 	old_setgroups16 = NULL;
     }
 #endif
+#endif /* SYSCALLHACK */
+#endif /* GROUPPAGS */
 }
