@@ -39,7 +39,7 @@
 #include <nnpfs/nnpfs_deb.h>
 #include <nnpfs/nnnpfs.h>
 
-RCSID("$Id: nnpfs_dev.c,v 1.29 2002/09/07 10:47:37 lha Exp $");
+RCSID("$Id: nnpfs_dev.c,v 1.30 2004/06/13 15:06:14 lha Exp $");
 
 /*
  * Queues of nnpfs_links hold outbound messages and processes sleeping
@@ -265,7 +265,7 @@ nnpfs_devwrite(dev_t dev, struct uio *uiop, cred_t *credp)
     ASSERT(chan->message_buffer != NULL);
 
     cnt = uiop->uio_resid;
-    error = uiomove((caddr_t) chan->message_buffer, MAX_XMSG_SIZE,
+    error = uiomove((caddr_t) chan->message_buffer, NNPFS_MAX_MSG_SIZE,
 		    UIO_WRITE, uiop);
     if (error != 0) {
 	mutex_exit(&chan->msg_buf_mutex);
@@ -742,7 +742,7 @@ nnpfs_dev_attach(dev_info_t *dip,
 	mutex_init (&state->sleep_mutex, "nnpfs:sleepq", MUTEX_DRIVER, NULL);
 	mutex_init (&state->msg_buf_mutex, "nnpfs:msg buf", MUTEX_DRIVER, NULL);
 	state->nsequence = 0;
-	state->message_buffer = nnpfs_alloc(MAX_XMSG_SIZE);
+	state->message_buffer = nnpfs_alloc(NNPFS_MAX_MSG_SIZE);
 	ASSERT(state->message_buffer != NULL);
 	state->status = 0;
 	/* how is the pollhead supposed to be initialized? */
@@ -782,7 +782,7 @@ nnpfs_dev_detach(dev_info_t *dip, ddi_detach_cmd_t cmd)
 	mutex_destroy(&state->sleep_mutex);
 	mutex_destroy(&state->msg_buf_mutex);
 	ASSERT(state->message_buffer != NULL);
-	nnpfs_free(state->message_buffer, MAX_XMSG_SIZE);
+	nnpfs_free(state->message_buffer, NNPFS_MAX_MSG_SIZE);
 	ddi_remove_minor_node(dip, NULL);
 	ddi_soft_state_free(nnpfs_dev_state, instance);
 	ret = DDI_SUCCESS;
