@@ -46,7 +46,7 @@
 #include <sys/flock.h>
 #include <sys/fs_subr.h>
 
-RCSID("$Id: nnpfs_vnodeops.c,v 1.41 2002/09/07 10:47:44 lha Exp $");
+RCSID("$Id: nnpfs_vnodeops.c,v 1.42 2004/06/13 15:06:27 lha Exp $");
 
 static int
 nnpfs_open_valid(struct vnode *vp, struct cred *cred, u_int tok)
@@ -428,7 +428,7 @@ nnpfs_access(struct vnode *vp,
 	
 	error = EACCES;
 	
-	for (i = 0; i < MAXRIGHTS; i++)
+	for (i = 0; i < NNPFS_MAXRIGHTS; i++)
 	    if (xn->id[i] == pag) {
 		error = check_rights (xn->rights[i], mode);
 		break;
