@@ -424,7 +424,7 @@ nnpfs_access(vp, mode, cred)
 	{
 	  int i;
 	  error = EACCES;	/* Until otherwise proven */
-	  for (i = 0; i < MAXRIGHTS; i++)
+	  for (i = 0; i < NNPFS_MAXRIGHTS; i++)
 	    if ((xn->id[i] == cred->cr_uid)
 		&& (mode & ~xn->rights[i]))
 	      {
