@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: nnpfs_dev.c,v 1.14 2004/05/21 22:23:56 tol Exp $ */
+/* $Id: nnpfs_dev.c,v 1.15 2004/06/13 15:07:11 lha Exp $ */
 
 #include <nnpfs_locl.h>
 #include <nnpfs_errno.h>
@@ -85,7 +85,7 @@ nnpfs_opcode_valid(char *buf, unsigned len)
 			msg->opcode, msg->size, len);
 	if (msg->opcode >= NNPFS_MSG_COUNT
 	    || msg->opcode < 0
-	    || msg->size > MAX_XMSG_SIZE) {
+	    || msg->size > NNPFS_MAX_MSG_SIZE) {
 	    nnpfs_debug(XDEBDEV, "nnpfs_opcode_valid: FALSE\n");
 	    return FALSE;
 	}
