@@ -1,4 +1,4 @@
-/* $Id: rx.h,v 1.28 2003/01/19 08:49:53 lha Exp $ */
+/* $Id: rx.h,v 1.32 2005/01/09 01:31:57 lha Exp $ */
 
 /*
 ****************************************************************************
@@ -70,6 +70,7 @@ struct rx_service *rx_NewService(u_short, u_short, char *,
 				 struct rx_securityClass **, int, int32_t (*)());
 struct rx_connection *rx_NewConnection(uint32_t, u_short, u_short,
 				       struct rx_securityClass *, int);
+void		rxi_KillConnection(uint32_t host, u_short);
 struct rx_call *
 		rx_NewCall(struct rx_connection *);
 struct rx_call *
@@ -77,15 +78,10 @@ struct rx_call *
 int32_t		rx_EndCall(struct rx_call *, int32_t);
 int		rx_AllocPackets(void);
 void		rx_FreePackets(void);
-int 		rx_Write(struct rx_call *, const void *, int);
-int 		rx_Read(struct rx_call *, void *, int);
+ssize_t		rx_Write(struct rx_call *, const void *, size_t);
+ssize_t		rx_Read(struct rx_call *, void *, size_t);
 void		rx_FlushWrite(struct rx_call *);
 
-#ifdef RXDEBUG
-void rx_PrintStats(FILE *);
-#else
-#define rx_PrintStats(a)
-#endif
 void rx_PrintPeerStats(FILE *file, struct rx_peer *peer);
 void rx_SetArrivalProc(struct rx_call * call, void (*proc) (),
 		       void *handle, void *arg);
@@ -996,8 +992,6 @@ struct rx_stats {		       /* General rx statistics */
     uint32_t spares[8];
 };
 
-void rx_PrintTheseStats(FILE *file, struct rx_stats *s, int size);
-
 /* structures for debug input and output packets */
 
 /* debug input types */
@@ -1081,4 +1075,7 @@ struct rx_debugConn {
 #define	RX_OTHER_IN	1	       /* packets avail in in queue */
 #define	RX_OTHER_OUT	2	       /* packets avail in out queue */
 
+void rx_PrintStats(FILE *);
+void rx_PrintTheseStats(FILE *file, struct rx_stats *s, int size);
+
 #endif				       /* _RX_	 End of rx.h */
