@@ -1,4 +1,4 @@
-/* $Id: rx_globs.h,v 1.10 2003/01/19 08:49:58 lha Exp $ */
+/* $Id: rx_globs.h,v 1.11 2004/11/18 05:10:32 lha Exp $ */
 
 /*
 ****************************************************************************
@@ -158,9 +158,14 @@ EXT kmutex_t rx_freeCallQueue_lock;
 #endif
 EXT long rxi_nCalls INIT(0);
 
-/* Basic socket for client requests; other sockets (for receiving server requests) are in the service structures */
+/* Basic socket for client requests; other sockets (for receiving
+ * server requests) are in the service structures */
+
 EXT osi_socket rx_socket;
 
+/* socket for icmp messages */
+EXT osi_socket rx_socket_icmp;
+
 /* Port requested at rx_Init.  If this is zero, the actual port used will be different--but it will only be used for client operations.  If non-zero, server provided services may use the same port. */
 EXT uint16_t rx_port;
 
@@ -219,6 +224,7 @@ struct rx_peer *	rxi_FindPeer(uint32_t, uint16_t);
 struct rx_call *	rxi_NewCall(struct rx_connection *, int);
 void			rxi_FreeCall(struct rx_call *);
 void			rxi_Listener(void);
+void			rxi_ReadIcmp(osi_socket);
 int			rxi_ReadPacket(int, struct rx_packet *, 
 				       uint32_t *, uint16_t *);
 struct rx_packet *	rxi_ReceivePacket(struct rx_packet *, osi_socket,
