@@ -1,4 +1,4 @@
-/* $Id: rx_kern.h,v 1.4 2003/01/19 08:47:23 lha Exp $ */
+/* $Id: rx_kern.h,v 1.5 2004/11/18 05:10:43 lha Exp $ */
 
 /*
 ****************************************************************************
@@ -59,8 +59,8 @@ typedef struct socket *osi_socket;
 
 #define	OSI_NULLSOCKET	((osi_socket) 0)
 
-extern rx_ReScheduleEvents();
-extern osi_socket rxi_GetUDPSocket();
+osi_socket rxi_GetUDPSocket(uint16_t, uint16_t *);
+osi_socket rxi_GetICMPSocket(void);
 
 #define	osi_Msg printf)(
 #if !defined(AFS_SGI_ENV) && !defined(AFS_SUN5_ENV)
