@@ -1,6 +1,6 @@
 #include "rx_locl.h"
 
-RCSID("$Id: rx_misc.c,v 1.6 2003/06/10 16:55:01 lha Exp $");
+RCSID("$Id: rx_misc.c,v 1.7 2005/03/27 06:21:32 lha Exp $");
 
 /*
  * We currently only include below the errors that
@@ -15,20 +15,20 @@ RCSID("$Id: rx_misc.c,v 1.6 2003/06/10 16:55:01 lha Exp $");
 uint32_t
 hton_syserr_conv(uint32_t code)
 {
-    uint32_t err;
+    uint32_t error;
 
 #if 0
     if (code == ENOSPC)
-	err = VDISKFULL;
+	error = VDISKFULL;
 #ifndef	AFS_SUN5_ENV
     /* EDQUOT doesn't exist on solaris */
     else if (code == EDQUOT)
-	err = VOVERQUOTA;
+	error = VOVERQUOTA;
 #endif
     else
 #endif
-	err = code;
-    return err;
+	error = code;
+    return error;
 }
 
 
@@ -39,19 +39,19 @@ hton_syserr_conv(uint32_t code)
 uint32_t
 ntoh_syserr_conv(uint32_t code)
 {
-    uint32_t err;
+    uint32_t error;
 
 #if 0
     if (code == VDISKFULL)
-	err = ENOSPC;
+	error = ENOSPC;
     else if (code == VOVERQUOTA)
 #ifdef	AFS_SUN5_ENV
-	err = ENOSPC;
+	error = ENOSPC;
 #else
-	err = EDQUOT;
+	error = EDQUOT;
 #endif
     else
 #endif
-	err = code;
-    return err;
+	error = code;
+    return error;
 }
