@@ -21,10 +21,13 @@
 
 #include "rx_locl.h"
 
-RCSID("$Id: rx_multi.c,v 1.6 2000/03/03 08:56:12 assar Exp $");
+RCSID("$Id: rx_multi.c,v 1.8 2005/03/27 06:22:49 lha Exp $");
+
+static void multi_Ready(struct rx_call *, struct multi_handle *, void *);
 
 /* multi.c and multi.h, together with some rxgen hooks, provide a way of making multiple, but similar, rx calls to multiple hosts simultaneously */
 
+
 struct multi_handle *
 multi_Init(struct rx_connection **conns, int nConns)
 {
@@ -51,7 +54,7 @@ multi_Init(struct rx_connection **conns, int nConns)
 	struct rx_call *call;
 
 	call = mh->calls[i] = rx_NewCall(conns[i]);
-	rx_SetArrivalProc(call, multi_Ready, (void *) mh, (void *) i);
+	rx_SetArrivalProc(call, multi_Ready, mh, NULL);
     }
     return mh;
 }
@@ -80,11 +83,16 @@ multi_Select(struct multi_handle *mh)
  * Called by Rx when the first reply packet of a call is received, or the
  * call is aborted. 
  */
-void 
-multi_Ready(struct rx_call *call, struct multi_handle *mh,
-	    int index)
+static void 
+multi_Ready(struct rx_call *call, struct multi_handle *mh, void *unused)
 {
-    *mh->firstNotReady++ = index;
+    int callindex;
+    for (callindex = 0; callindex < RX_MAXCALLS; callindex++)
+	if (mh->calls[callindex] == call)
+	    break;
+    if (callindex >= RX_MAXCALLS)
+	abort();
+    *mh->firstNotReady++ = callindex;
     mh->nReady++;
     osi_rxWakeup(mh);
 }
