@@ -1,6 +1,11 @@
 #include "rx_locl.h"
 
-RCSID("$Id: rx_pkt.c,v 1.22 2003/04/08 22:15:08 lha Exp $");
+#include <netinet/in_systm.h>
+#include <netinet/ip.h>
+#include <netinet/ip_icmp.h>
+#include <netinet/udp.h>
+
+RCSID("$Id: rx_pkt.c,v 1.25 2005/03/27 06:01:28 lha Exp $");
 
 struct rx_packet *rx_mallocedP = 0;
 struct rx_cbuf *rx_mallocedC = 0;
@@ -11,6 +16,67 @@ static char cml_version_number[CML_VERSION_NUMBER_SIZE]= PACKAGE "-" VERSION ;
 
 extern int (*rx_almostSent) ();
 
+
+static void
+put32 (unsigned char **p, uint32_t u)
+{
+    (*p)[0] = (u >> 24) & 0xFF;
+    (*p)[1] = (u >> 16) & 0xFF;
+    (*p)[2] = (u >>  8) & 0xFF;
+    (*p)[3] = (u >>  0) & 0xFF;
+    (*p) += 4;
+}
+
+static uint32_t
+get32 (unsigned char **p)
+{
+    uint32_t u;
+
+    u = ((*p)[0] << 24) | ((*p)[1] << 16) | ((*p)[2] << 8) | (*p)[3];
+    (*p) += 4;
+    return u;
+}
+
+void
+rxi_ReadIcmp(osi_socket asocket)
+{
+    unsigned char buffer[2000];
+    ssize_t length;
+    uint32_t h;
+    unsigned char *p;
+    u_short port;
+
+    length = recv(asocket, buffer, sizeof(buffer), 0);
+    if (length < 0 || length > sizeof(buffer))
+	return;
+
+    p = buffer;
+
+    if (length < (5 * 4) + 8 + (5 * 4) + 8)
+	return;
+    if (*p != 0x45) /* header comes with options, punt */
+	return;
+
+#ifndef ICMP_UNREACH
+#define ICMP_UNREACH 3
+#endif
+#ifndef ICMP_UNREACH_NEEDFRAG
+#define ICMP_UNREACH_NEEDFRAG 4
+#endif
+    if (p[5 * 4] != ICMP_UNREACH) 
+	return;
+    if (p[6 * 4] == ICMP_UNREACH_NEEDFRAG) /* ignore need frag for now */
+	return;
+    if (p[(5 * 4) + 8] != 0x45) /* header comes with options, punt */
+	return;
+
+    p = &p[(5 * 4) + 8 + (4 * 4)];
+    h = get32(&p);
+    port = (p[2] << 8) | p[3];
+
+    rxi_KillConnection(h, port);
+}
+
 /*
  * some rules about packets:
  * 1.  When a packet is allocated, the final iov_buf contains room for
@@ -574,7 +640,7 @@ CountFDs(int amax)
  * The header is decoded.
  */
 int 
-rxi_ReadPacket(int socket, struct rx_packet *p,
+rxi_ReadPacket(int asocket, struct rx_packet *p,
 	       uint32_t *host, uint16_t *port)
 {
     struct sockaddr_in from;
@@ -624,7 +690,7 @@ rxi_ReadPacket(int socket, struct rx_packet *p,
     msg.msg_accrights = NULL;
     msg.msg_accrightslen = 0;
 #endif
-    nbytes = recvmsg(socket, &msg, 0);
+    nbytes = recvmsg(asocket, &msg, 0);
 
     /* restore the vec to its correct state */
     p->wirevec[--p->niovecs].iov_base = NULL;
@@ -665,7 +731,7 @@ rxi_ReadPacket(int socket, struct rx_packet *p,
 
 /* Send a udp datagram */
 int 
-osi_NetSend(osi_socket socket, char *addr, struct iovec *dvec,
+osi_NetSend(osi_socket asocket, char *addr, struct iovec *dvec,
 	    int nvecs, int length)
 {
     struct msghdr msg;
@@ -681,8 +747,8 @@ osi_NetSend(osi_socket socket, char *addr, struct iovec *dvec,
     msg.msg_accrightslen = 0;
 #endif
 
-    while (sendmsg(socket, &msg, 0) == -1) {
-	int err;
+    while (sendmsg(asocket, &msg, 0) == -1) {
+	int error;
 	fd_set sfds;
 
 	rx_stats.sendSelects++;
@@ -696,11 +762,11 @@ osi_NetSend(osi_socket socket, char *addr, struct iovec *dvec,
 	dpf(("rx_send failed with %d\n", errno));
 
 	FD_ZERO(&sfds);
-	if (socket >= FD_SETSIZE)
+	if (asocket >= FD_SETSIZE)
 	    osi_Panic("osi_NetSend: fd too large");
-	FD_SET(socket, &sfds);
-	while ((err = select(socket + 1, 0, &sfds, 0, 0)) != 1) {
-	    if (err >= 0 || errno != EINTR)
+	FD_SET(asocket, &sfds);
+	while ((error = select(asocket + 1, 0, &sfds, 0, 0)) != 1) {
+	    if (error >= 0 || errno != EINTR)
 		osi_Panic("osi_NetSend: select error %d.%d", err, errno);
 	}
     }
@@ -1087,7 +1153,7 @@ rxi_SendPacket(struct rx_connection *conn,
 {
     struct sockaddr_in addr;
     struct rx_peer *peer = conn->peer;
-    osi_socket socket;
+    osi_socket asocket;
 
 #ifdef RXDEBUG
     char deliveryType = 'S';
@@ -1150,8 +1216,8 @@ rxi_SendPacket(struct rx_connection *conn,
      * Send the packet out on the same socket that related packets are being
      * received on
      */
-    socket = (conn->type == RX_CLIENT_CONNECTION
-	      ? rx_socket : conn->service->socket);
+    asocket = (conn->type == RX_CLIENT_CONNECTION
+	       ? rx_socket : conn->service->socket);
 
 #ifdef RXDEBUG
     /* Possibly drop this packet,  for testing purposes */
@@ -1169,7 +1235,7 @@ rxi_SendPacket(struct rx_connection *conn,
 	 * the socket block in send mode, and not block in receive mode
 	 */
 	GLOBAL_UNLOCK();
-	if (osi_NetSend(socket, (char *)&addr, p->wirevec, 
+	if (osi_NetSend(asocket, (char *)&addr, p->wirevec, 
 			p->niovecs, p->length + RX_HEADER_SIZE)) {
 	    /* send failed, so let's hurry up the resend, eh? */
 	    rx_stats.netSendFailures++;
@@ -1275,27 +1341,6 @@ rxi_SendSpecial(struct rx_call *call,
     return optionalPacket;
 }
 
-
-static void
-put32 (unsigned char **p, uint32_t u)
-{
-    (*p)[0] = (u >> 24) & 0xFF;
-    (*p)[1] = (u >> 16) & 0xFF;
-    (*p)[2] = (u >>  8) & 0xFF;
-    (*p)[3] = (u >>  0) & 0xFF;
-    (*p) += 4;
-}
-
-static uint32_t
-get32 (unsigned char **p)
-{
-    uint32_t u;
-
-    u = ((*p)[0] << 24) | ((*p)[1] << 16) | ((*p)[2] << 8) | (*p)[3];
-    (*p) += 4;
-    return u;
-}
-
 /* Encode the packet's header (from the struct header in the packet to
  * the net byte order representation in the wire representation of the
  * packet, which is what is actually sent out on the wire) */
