@@ -26,7 +26,7 @@
 
 #include "rx_locl.h"
 
-RCSID("$Id: rx_user.c,v 1.22 2004/06/01 18:54:46 tol Exp $");
+RCSID("$Id: rx_user.c,v 1.25 2005/11/23 11:33:16 lha Exp $");
 
 #ifndef	IPPORT_USERRESERVED
 /*
@@ -124,6 +124,42 @@ rxi_StartServerProcs(int nExistingProcs)
     }
 }
 
+/*
+ * Add the socket to the listing queue and add it to the select mask.
+ */
+
+static int
+addSocket(const char *name, osi_socket socketFd)
+{
+    osi_socket *sockets;
+
+    if (socketFd >= FD_SETSIZE) {
+	osi_Msg(("%s: socket fd too large\n", name));
+	return OSI_NULLSOCKET;
+    }
+
+    sockets = realloc(rx_sockets, (num_rx_sockets + 1) * sizeof(*rx_sockets));
+    if (sockets == NULL) {
+	perror("socket");
+	osi_Msg(("%sunable to allocated memory for socket\n", name));
+	return 1;
+    }
+    rx_sockets = sockets;
+
+    rx_sockets[num_rx_sockets] = socketFd;
+    num_rx_sockets++;
+
+    if (rx_maxSocketNumber < 0)
+	FD_ZERO(&rx_selectMask);
+
+    FD_SET(socketFd, &rx_selectMask);
+    if (socketFd > rx_maxSocketNumber)
+	rx_maxSocketNumber = socketFd;
+
+    return 0;
+}
+
+
 /*
  * Make a socket for receiving/sending IP packets.  Set it into non-blocking
  * and large buffering modes.  If port isn't specified, the kernel will pick
@@ -138,18 +174,9 @@ rxi_GetUDPSocket(uint16_t port, uint16_t *retport)
 {
     int code;
     osi_socket socketFd = OSI_NULLSOCKET;
-    osi_socket *sockets;
     struct sockaddr_in taddr;
     char *name = "rxi_GetUDPSocket: ";
-    int sa_size;
-
-    sockets = realloc(rx_sockets, (num_rx_sockets + 1) * sizeof(*rx_sockets));
-    if (sockets == NULL) {
-	perror("socket");
-	osi_Msg(("%sunable to allocated memory for UDP socket\n", name));
-	return OSI_NULLSOCKET;
-    }
-    rx_sockets = sockets;
+    socklen_t sa_size;
 
     socketFd = socket(AF_INET, SOCK_DGRAM, 0);
     if (socketFd < 0) {
@@ -164,16 +191,6 @@ rxi_GetUDPSocket(uint16_t port, uint16_t *retport)
 	return OSI_NULLSOCKET;
     }
 
-    rx_sockets[num_rx_sockets] = socketFd;
-    num_rx_sockets++;
-
-    if (rx_maxSocketNumber < 0)
-	FD_ZERO(&rx_selectMask);
-
-    FD_SET(socketFd, &rx_selectMask);
-    if (socketFd > rx_maxSocketNumber)
-	rx_maxSocketNumber = socketFd;
-
     memset (&taddr, 0, sizeof(taddr));
     taddr.sin_family = AF_INET;
     taddr.sin_port   = port;
@@ -223,17 +240,57 @@ rxi_GetUDPSocket(uint16_t port, uint16_t *retport)
 	osi_Msg(("%sunable to set non-blocking mode on socket\n", name));
 	goto error;
     }
+
+    if (addSocket(name, socketFd))
+	goto error;
+
     return socketFd;
 
 error:
-    num_rx_sockets--;
-    rx_sockets[num_rx_sockets] = OSI_NULLSOCKET;
-
     close(socketFd);
 
     return OSI_NULLSOCKET;
 }
 
+/*
+ * Open icmp socket
+ */
+
+osi_socket
+rxi_GetICMPSocket(void)
+{
+    int socketFd = -1;
+    char *name = "rxi_GetICMPSocket: ";
+
+    socketFd = socket(AF_INET, SOCK_RAW, IPPROTO_ICMP);
+    if (socketFd < 0) {
+	if (getuid() == 0) {
+	    perror("socket");
+	    osi_Msg(("%sunable to create ICMP socket\n", name));
+	}
+	goto error;
+    }
+
+    /*
+     * Put it into non-blocking mode so that rx_Listener can do a polling
+     * read before entering select
+     */
+    if (fcntl(socketFd, F_SETFL, FNDELAY) == -1) {
+	perror("fcntl");
+	osi_Msg(("%sunable to set non-blocking mode on socket\n", name));
+	goto error;
+    }
+    if (addSocket(name, socketFd))
+	goto error;
+
+    return socketFd;
+
+error:
+    if (socketFd >= 0)
+	close(socketFd);
+    return OSI_NULLSOCKET;
+}
+
 /*
  * The main loop which listens to the net for datagrams, and handles timeouts
  * and retransmissions, etc.  It also is responsible for scheduling the
@@ -268,8 +325,7 @@ rxi_Listener(void)
     uint16_t port;
     struct rx_packet *p = NULL;
     fd_set rfds;
-    int socket;
-    int fds;
+    int i, fds;
     struct clock cv;
     long nextPollTime;		       /* time to next poll FD before
 				        * sleeping */
@@ -330,13 +386,15 @@ rxi_Listener(void)
 	    if (doingPoll)
 		lastPollWorked = 1;
 
-	    for (socket = 0; socket < num_rx_sockets; socket++) {
+	    for (i = 0; i < num_rx_sockets; i++) {
 		if (p == NULL)
 		    break;
-		if (FD_ISSET(rx_sockets[socket], &rfds) &&
-		    rxi_ReadPacket(rx_sockets[socket], p, &host, &port))
-		{
-		    p = rxi_ReceivePacket(p, rx_sockets[socket], host, port);
+		else if (FD_ISSET(rx_sockets[i], &rfds)) {
+		    if (rx_sockets[i] == rx_socket_icmp) {
+			rxi_ReadIcmp(rx_sockets[i]);
+		    } else if (rxi_ReadPacket(rx_sockets[i],p, &host, &port)) {
+			p = rxi_ReceivePacket(p, rx_sockets[i], host, port);
+		    }
 		}
 	    }
 	}
