@@ -1,4 +1,4 @@
-/* $Id: rx_user.h,v 1.9 2003/04/08 22:13:44 lha Exp $ */
+/* $Id: rx_user.h,v 1.11 2004/12/13 15:33:47 tol Exp $ */
 
 /*
 ****************************************************************************
@@ -46,8 +46,6 @@ extern FILE *rx_debugFile;
 #define ISAFS_GLOCK()
 #endif
 
-extern PROCESS rx_listenerPid;	       /* LWP process id of socket listener
-				        * process */
 void rxi_StartListener(void);
 void rxi_StartServerProcs(int);
 void rxi_ReScheduleEvents(void);
@@ -60,6 +58,7 @@ void rxi_PacketsUnWait(void);
 typedef short osi_socket;
 
 osi_socket rxi_GetUDPSocket(uint16_t, uint16_t *);
+osi_socket rxi_GetICMPSocket(void);
 
 #define	OSI_NULLSOCKET	((osi_socket) -1)
 
