@@ -44,7 +44,7 @@
 #include "config.h"
 #include "roken.h"
 
-RCSID ("$Id: rxdebug.c,v 1.15 2003/01/19 08:50:12 lha Exp $");
+RCSID ("$Id: rxdebug.c,v 1.17 2005/11/23 11:36:04 lha Exp $");
 
 #define	TIMEOUT	    20
 
@@ -75,7 +75,6 @@ MakeCall (int asocket, uint32_t ahost, uint16_t aport, char *adata,
     int code;
     struct timeval tv;
     struct sockaddr_in taddr, faddr;
-    int faddrLen;
     fd_set imask;
     char *tp;
 
@@ -123,7 +122,8 @@ MakeCall (int asocket, uint32_t ahost, uint16_t aport, char *adata,
 	}
 	if (code > 0) {
 	    /* now receive a packet */
-	    faddrLen = sizeof(struct sockaddr_in);
+	    socklen_t faddrLen = sizeof(struct sockaddr_in);
+
 	    code = recvfrom(asocket, tbuffer, sizeof(tbuffer), 0,
 			    (struct sockaddr *)&faddr, &faddrLen);
 	    if (code == -1) {
@@ -155,7 +155,6 @@ GetVersion(int asocket, uint32_t ahost, uint16_t aport,
     int code;
     struct timeval tv;
     struct sockaddr_in taddr, faddr;
-    int faddrLen;
     fd_set imask;
     char *tp;
 
@@ -208,7 +207,7 @@ GetVersion(int asocket, uint32_t ahost, uint16_t aport,
 	}
 	if (code > 0) {
 	    /* now receive a packet */
-	    faddrLen = sizeof(struct sockaddr_in);
+	    socklen_t faddrLen = sizeof(struct sockaddr_in);
 
 	    code = recvfrom(asocket, tbuffer, sizeof(tbuffer), 0,
 			    (struct sockaddr *)&faddr, &faddrLen);
@@ -262,19 +261,19 @@ MapOldConn (char version, struct rx_debugConn *tconn)
     return 0;
 }
 
+static int nodally_flag;
+static int allconns_flag;
+static int rxstats_flag;
+static int onlyServer_flag;
+static int onlyClient_flag;
+static int version_flag;
+static int noConns_flag;
+static int help_flag;
 static char *hostName;
 static char *portName;
-static int nodally;
-static int allconns;
-static int rxstats;
+static char *onlyAuthName;
 static char *onlyPortName;
 static char *onlyHostName;
-static int onlyServer;
-static int onlyClient;
-static char *onlyAuthName;
-static int version_flag;
-static int noConns;
-static int helpflag;
 
 static int
 MainCommand (void)
@@ -286,7 +285,6 @@ MainCommand (void)
     uint32_t host;
     struct in_addr hostAddr;
     uint16_t port;
-    struct hostent *th;
     struct rx_debugIn tin;
     int code;
     uint32_t onlyHost;
@@ -319,6 +317,7 @@ MainCommand (void)
     if (onlyHostName) {
 	char *name = onlyHostName;
 	struct hostent *th;
+
 	th = gethostbyname(name);
 	if (!th) {
 	    printf("rxdebug: host %s not found in host table\n", name);
@@ -345,6 +344,8 @@ MainCommand (void)
 
     /* lookup host */
     if (hostName) {
+	struct hostent *th;
+
 	th = gethostbyname(hostName);
 	if (!th) {
 	    printf("rxdebug: host %s not found in host table\n", hostName);
@@ -427,13 +428,12 @@ MainCommand (void)
 	printf("%ld calls waiting for a thread\n",
 	       (long)ntohl(tstats.nWaiting));
     
-    if (rxstats && withRxStats) {
+    if (rxstats_flag && withRxStats) {
 	if(!withRxStats) {
 	    fprintf (stderr, "WARNING: Server doens't support "
 		     "retrieval of Rx statistics\n");
 	} else {
 	    struct rx_stats rxstats;
-	    int i;
 	    uint32_t *lp;
 	
 	    memset (&rxstats, 0, sizeof(rxstats));
@@ -448,7 +448,7 @@ MainCommand (void)
 	    }
 
 	    if ((code == sizeof(tin)) &&
-		(ntohl(((struct rx_debugIn *)(&rxstats))->type) ==
+		(ntohl(((struct rx_debugIn *)(void*)&rxstats)->type) ==
 		 RX_DEBUGI_BADTYPE)) {
 		fprintf (stderr, "WARNING: Server doens't support "
 			 "retrieval of Rx statistics\n");
@@ -469,15 +469,15 @@ MainCommand (void)
 	}
     }
     
-    if (noConns)
+    if (noConns_flag)
 	return 0;
     
     tin.type = htonl(RX_DEBUGI_GETCONN);
-    if (allconns)
+    if (allconns_flag)
 	tin.type = htonl(RX_DEBUGI_GETALLCONN);
     
-    if (onlyServer) printf ("Showing only server connections\n");
-    if (onlyClient) printf ("Showing only client connections\n");
+    if (onlyServer_flag) printf ("Showing only server connections\n");
+    if (onlyClient_flag) printf ("Showing only client connections\n");
     if (onlyAuth != 999) {
 	static char *name[] =
 	    {"unauthenticated", "rxkad_clear", "rxkad_auth", "rxkad_crypt"};
@@ -507,7 +507,7 @@ MainCommand (void)
 	}
 	
 	/* see if we're in nodally mode and all calls are dallying */
-	if (nodally) {
+	if (nodally_flag) {
 	    flag = 0;
 	    for(j=0;j<RX_MAXCALLS;j++) {
 		if (tconn.callState[j] != RX_STATE_NOTINIT &&
@@ -527,8 +527,8 @@ MainCommand (void)
 	}
 	if ((onlyHost != -1) && (onlyHost != tconn.host)) continue;
 	if ((onlyPort != 0) && (onlyPort != tconn.port)) continue;
-	if (onlyServer && (tconn.type != RX_SERVER_CONNECTION)) continue;
-	if (onlyClient && (tconn.type != RX_CLIENT_CONNECTION)) continue;
+	if (onlyServer_flag && (tconn.type != RX_SERVER_CONNECTION)) continue;
+	if (onlyClient_flag && (tconn.type != RX_CLIENT_CONNECTION)) continue;
 	if (onlyAuth != 999) {
 	    if (onlyAuth == -1) {
 		if (tconn.securityIndex != 0) continue;
@@ -663,7 +663,8 @@ MainCommand (void)
 	    printf("\n");
 	}
     }
-    if (nodally) printf("Skipped %d dallying connections.\n", dallyCounter);
+    if (nodally_flag) 
+	printf("Skipped %d dallying connections.\n", dallyCounter);
     return 0;
 }
 
@@ -672,15 +673,15 @@ static struct agetargs args[] = {
      "server machine", NULL, aarg_mandatory},
     {"port", 0, aarg_string, &portName,
      "IP port", NULL, aarg_optional_swless },
-    {"nodally", 0, aarg_flag, &nodally,
+    {"nodally", 0, aarg_flag, &nodally_flag,
      "don't show dallying conns", NULL },
-    {"allconnections", 0, aarg_flag, &allconns,
+    {"allconnections", 0, aarg_flag, &allconns_flag,
      "don't filter out uninteresting connections on server"},
-    {"rxstats", 0, aarg_flag, &rxstats,
+    {"rxstats", 0, aarg_flag, &rxstats_flag,
      "show Rx statistics", NULL },
-    {"onlyserver", 0, aarg_flag, &onlyServer,
+    {"onlyserver", 0, aarg_flag, &onlyServer_flag,
      "only show server conns", NULL },
-    {"onlyclient", 0, aarg_flag, &onlyClient,
+    {"onlyclient", 0, aarg_flag, &onlyClient_flag,
      "only show client conns", NULL},
     {"onlyport", 0, aarg_integer, &onlyPortName,
      "show only <port>", NULL },
@@ -690,8 +691,10 @@ static struct agetargs args[] = {
      "show only <auth level>", NULL },
     {"version", 0, aarg_flag, &version_flag,
      "show AFS version id", NULL },
-    {"noconns", 0, aarg_flag, &noConns,
+    {"noconns", 0, aarg_flag, &noConns_flag,
      "show no connections", NULL },
+    {"help", 0, aarg_flag, &help_flag,
+     "show help", NULL },
     {NULL}
 };
 
@@ -708,20 +711,20 @@ int
 main(int argc, char **argv)
 {
 
-    int optind = 0;
+    int o = 0;
     
-    if (agetarg (args, argc, argv, &optind, AARG_AFSSTYLE)) {
+    if (agetarg (args, argc, argv, &o, AARG_AFSSTYLE)) {
         usage();
         return 0;
     }
 
-    if(helpflag) {
+    if(help_flag) {
         usage();
         return 0;
     }
 
-    argc -= optind;
-    argv += optind;
+    argc -= o;
+    argv += o;
 
     if (argc > 0) {
         fprintf (stderr, "create volume: unparsed arguments\n");
