@@ -43,7 +43,7 @@
 #define HAVE_GETRUSAGE 1
 
 #ifdef RCSID
-RCSID("$Id: rxperf.c,v 1.23 2003/04/08 00:19:47 lha Exp $");
+RCSID("$Id: rxperf.c,v 1.26 2005/11/23 11:39:21 lha Exp $");
 #endif
 
 #include <stdarg.h>
@@ -79,8 +79,6 @@ RCSID("$Id: rxperf.c,v 1.23 2003/04/08 00:19:47 lha Exp $");
 #define uint32_t afs_uint32
 #endif
 
-static const char *__progname;
-
 #ifndef HAVE_WARNX
 static void
 warnx(const char *fmt, ...)
@@ -88,7 +86,7 @@ warnx(const char *fmt, ...)
     va_list args;
 
     va_start(args, fmt);
-    fprintf(stderr, "%s: ", __progname);
+    fprintf(stderr, "%s: ", "rxperf");
     vfprintf (stderr, fmt, args);
     fprintf(stderr, "\n");
     va_end(args);
@@ -102,7 +100,7 @@ errx(int eval, const char *fmt, ...)
     va_list args;
 
     va_start(args, fmt);
-    fprintf(stderr, "%s: ", __progname);
+    fprintf(stderr, "%s: ", "rxperf");
     vfprintf (stderr, fmt, args);
     fprintf(stderr, "\n");
     va_end(args);
@@ -119,7 +117,7 @@ warn(const char *fmt, ...)
     char *errstr;
 
     va_start(args, fmt);
-    fprintf(stderr, "%s: ", __progname);
+    fprintf(stderr, "%s: ", "rxperf");
     vfprintf (stderr, fmt, args);
 
     errstr = strerror(errno);
@@ -137,7 +135,7 @@ err(int eval, const char *fmt, ...)
     char *errstr;
 
     va_start(args, fmt);
-    fprintf(stderr, "%s: ", __progname);
+    fprintf(stderr, "%s: ", "rxperf");
     vfprintf (stderr, fmt, args);
 
     errstr = strerror(errno);
@@ -565,7 +563,7 @@ do_client (const char *server, int port, char *filename,
     struct rx_securityClass *secureobj;
     int secureindex;
     int32_t data;
-    int32_t num;
+    uint32_t num;
     int ret;
     int i;
     int readp = FALSE;
@@ -734,17 +732,17 @@ usage()
 #define COMMON ""
 
     fprintf(stderr, "usage: %s client -c send -b <bytes>\n",
-	    __progname);
+	    getprogname());
     fprintf(stderr, "usage: %s client -c recv -b <bytes>\n",
-	    __progname);
+	    getprogname());
     fprintf(stderr, "usage: %s client -c rpc  -S <rpc_sendbytes> -R <rpc_recvbytes>\n",
-	    __progname);
+	    getprogname());
     fprintf(stderr, "usage: %s client -c file -f filename\n", 
-	    __progname);
+	    getprogname());
     fprintf (stderr, "%s: usage:	common option to the client "
 	     "-w <rx_write size> -r <rx_read size> -T times -p port -s server\n",
-	     __progname);
-    fprintf(stderr, "usage: %s server -p port\n", __progname);
+	     getprogname());
+    fprintf(stderr, "usage: %s server -p port\n", getprogname());
 #undef COMMMON
     exit(1);
 }
@@ -776,8 +774,8 @@ rxperf_server (int argc, char **argv)
 	    if (ptr != 0 && ptr[0] != '\0')
 		errx (1, "can't resolve readsize");
 	    if (rxread_size > sizeof(somebuf))
-	      errx(1, "%d > sizeof(somebuf) (%d)",
-		   rxread_size, sizeof(somebuf));
+	      errx(1, "%d > sizeof(somebuf) (%lu)",
+		   rxread_size, (unsigned long)sizeof(somebuf));
 	    break;
 	case 'p':
 	    port = strtol(optarg, &ptr, 0);
@@ -789,8 +787,8 @@ rxperf_server (int argc, char **argv)
 	    if (ptr != 0 && ptr[0] != '\0')
 		errx (1, "can't resolve writesize");
 	    if (rxwrite_size > sizeof(somebuf))
-		errx(1, "%d > sizeof(somebuf) (%d)",
-		     rxwrite_size, sizeof(somebuf));
+		errx(1, "%d > sizeof(somebuf) (%lu)",
+		     rxwrite_size, (unsigned long)sizeof(somebuf));
 	    break;
 	default:
 	    usage();
@@ -870,8 +868,8 @@ rxperf_client (int argc, char **argv)
 	    if (ptr != 0 && ptr[0] != '\0')
 		errx (1, "can't resolve readsize");
 	    if (rxread_size > sizeof(somebuf))
-		errx(1, "%d > sizeof(somebuf) (%d)",
-		     rxread_size, sizeof(somebuf));
+		errx(1, "%d > sizeof(somebuf) (%lu)",
+		     rxread_size, (unsigned long)sizeof(somebuf));
 	    break;
 	case 's':
 	    host = strdup(optarg);
@@ -883,8 +881,8 @@ rxperf_client (int argc, char **argv)
 	    if (ptr != 0 && ptr[0] != '\0')
 		errx (1, "can't resolve writesize");
 	    if (rxwrite_size > sizeof(somebuf))
-		errx(1, "%d > sizeof(somebuf) (%d)",
-		     rxwrite_size, sizeof(somebuf));
+		errx(1, "%d > sizeof(somebuf) (%lu)",
+		     rxwrite_size, (unsigned long)sizeof(somebuf));
 	    break;
 	case 'T':
 	    times = strtol (optarg, &ptr, 0);
@@ -925,8 +923,8 @@ rxperf_client (int argc, char **argv)
 	    printf("no stats\n");
 	else
 	    printf ("Status:\n"
-		    "- utime = (%ld, %ld)\n"
-		    "- stime = (%ld, %ld)\n"
+		    "- utime = (%ld, %lu)\n"
+		    "- stime = (%ld, %lu)\n"
 		    "- maxrss = %ld\n"
 		    "- ixrss = %ld\n"
 		    "- idrss = %ld\n"
@@ -939,12 +937,12 @@ rxperf_client (int argc, char **argv)
 		    "- msgsnd = %ld\n"
 		    "- msgrcv = %ld\n"
 		    "- nsignals = %ld\n"
-		    "- nvcsw = %ld\n"
-		    "- nivcws = %ld\n",
+		    "- nvcsw = %lu\n"
+		    "- nivcws = %lu\n",
 		    rusage.ru_utime.tv_sec,
-		    rusage.ru_utime.tv_usec,
+		    (unsigned long)rusage.ru_utime.tv_usec,
 		    rusage.ru_stime.tv_sec,
-		    rusage.ru_stime.tv_usec,
+		    (unsigned long)rusage.ru_stime.tv_usec,
 		    rusage.ru_maxrss,
 		    rusage.ru_ixrss,
 		    rusage.ru_idrss,
@@ -957,8 +955,8 @@ rxperf_client (int argc, char **argv)
 		    rusage.ru_msgsnd,
 		    rusage.ru_msgrcv,
 		    rusage.ru_nsignals,
-		    rusage.ru_nvcsw,
-		    rusage.ru_nivcsw);
+		    (unsigned long)rusage.ru_nvcsw,
+		    (unsigned long)rusage.ru_nivcsw);
     }
 #endif /* HAVE_GETRUSAGE */
     return 0;
@@ -973,11 +971,7 @@ main(int argc, char **argv)
 {
     PROCESS pid;
 
-    __progname = strrchr(argv[0], '/');
-    if (__progname)
-	__progname++;
-    else
-	__progname = argv[0];
+    setprogname(argv[0]);
 
     signal (SIGUSR1, sigusr1);
     signal (SIGINT, sigint);
