@@ -37,14 +37,19 @@
 
 #include "rx_clock.h"
 
-RCSID("$Id: test_rx_clock.c,v 1.3 2000/10/03 00:41:44 lha Exp $");
+RCSID("$Id: test_rx_clock.c,v 1.6 2005/08/09 18:35:23 lha Exp $");
 
 int
 main(int argc, char **argv)
 {
     struct clock clock_old = {0, 0};
+    struct clock first;
 
     clock_Init ();
+
+    clock_NewTime ();
+    clock_GetTime (&first);
+
     for (;;) {
 	struct clock now;
 
@@ -53,7 +58,9 @@ main(int argc, char **argv)
 	if (now.sec < clock_old.sec ||
 	    (now.sec == clock_old.sec && now.usec < clock_old.usec))
 	    abort ();
-	printf ("%6ld.%6ld\r", now.sec, now.usec);
+	/* printf ("%6ld.%6ld\r", now.sec, now.usec); */
+	if (now.sec > first.sec + 10)
+	    break;
     }
     return 0;
 }
