@@ -1,8 +1,41 @@
 /*
- * Common defintions used by several *.xg files
+ * Copyright (c) 1995 - 2004 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
+ * All rights reserved.
+ * 
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 
+ * 3. Neither the name of the Institute nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
+ * 
+ * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  */
 
-/* $Id: common.h,v 1.19 2003/03/06 20:39:47 lha Exp $ */
+/*
+ * Common definition used by several *.xg files
+ */
+
+/* $Id: common.h,v 1.22 2005/10/20 19:41:16 lha Exp $ */
 
 %#ifndef _COMMON_
 %#define _COMMON_
@@ -65,11 +98,11 @@ struct AFSFetchStatus {
      uint32_t UnixModeBits;
      uint32_t ParentVnode;
      uint32_t ParentUnique;
-     uint32_t SegSize;
+     uint32_t ResidencyMask;
      uint32_t ClientModTime;
      uint32_t ServerModTime;
      uint32_t Group;
-     uint32_t SyncCount;
+     uint32_t SyncCounter;
      uint32_t DataVersionHigh;	/* For AFS/DFS translator */
      uint32_t LockCount;
      uint32_t LengthHigh;
