@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 2003 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -33,16 +33,19 @@
 
 #include "rxkad_locl.h"
 
-RCSID("$Id: rxk_info.c,v 1.4 2000/10/03 00:38:33 lha Exp $");
+RCSID("$Id: rxk_info.c,v 1.6 2003/11/13 05:48:34 lha Exp $");
 
-int32
-rxkad_GetServerInfo(struct rx_connection *con,
-		    rxkad_level *level,
-		    u_int32 *expiration,
-		    char *name,
-		    char *instance,
-		    char *cell,
-		    int32 *kvno)
+/*
+ * Return value of `name' depends on `con', its free()ed when con is
+ * free()ed so don't do that.
+ */
+
+int32_t
+rxkad_GetServerInfoNew(struct rx_connection *con,
+		       rxkad_level *level,
+		       uint32_t *expiration,
+		       char **name,
+		       int32_t *kvno)
 {
   serv_con_data *cdat = (serv_con_data *) con->securityData;
 
@@ -55,11 +58,7 @@ rxkad_GetServerInfo(struct rx_connection *con,
       if (expiration)
 	*expiration = cdat->expires;
       if (name)
-	strcpy(name, cdat->user->name);
-      if (instance)
-	strcpy(instance, cdat->user->instance);
-      if (cell)
-	strcpy(cell, cdat->user->realm);
+	*name = cdat->user;
       if (kvno)
 	*kvno = -1;		/* Where do we find this and who needs it? */
       return 0;
