@@ -33,17 +33,18 @@
  * SUCH DAMAGE.
  */
 
-/* @(#)$Id: rxkad.h,v 1.14 2003/11/13 16:14:52 lha Exp $ */
+/* @(#)$Id: rxkad.h,v 1.15 2005/01/17 09:28:10 lha Exp $ */
 
 #ifndef __RXKAD_H
 #define __RXKAD_H
 
-/* Krb4 tickets can't have a key version number of 256. This is used
+/* Krb4 tickets can't have a key version number of 213 or 256. This is used
  * as a magic kvno to indicate that this is really a krb5 ticket. The
  * real kvno can be retrieved from the cleartext portion of the
  * ticket. For more info see the Transarc header file rxkad.h.
  */
 #define RXKAD_TKT_TYPE_KERBEROS_V5 256
+#define RXKAD_TKT_TYPE_KERBEROS_V5_ENCPART_ONLY 213
 
 /* Is this really large enough for a krb5 ticket? */
 #define MAXKRB5TICKETLEN	1024
