@@ -48,7 +48,7 @@
 #include <roken.h>
 
 #ifdef RCSID
-RCSID("$Id: create-files.c,v 1.6.2.1 2002/07/11 14:00:54 lha Exp $");
+RCSID("$Id: create-files.c,v 1.8 2002/09/17 18:27:03 lha Exp $");
 #endif
 
 static int
@@ -90,7 +90,7 @@ creat_files (int count, long startsize)
 static void
 usage (int ret)
 {
-    fprintf (stderr, "%s number-of-files size-of-files\n", __progname);
+    fprintf (stderr, "%s number-of-files size-of-files\n", getprogname());
     exit (ret);
 }
 
