@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995-1999, 2002, 2006 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -48,7 +48,7 @@
 #include <roken.h>
 
 #ifdef RCSID
-RCSID("$Id: large-dir2.c,v 1.5 2002/09/17 18:27:11 lha Exp $");
+RCSID("$Id: large-dir2.c,v 1.7 2006/04/03 08:59:17 tol Exp $");
 #endif
 
 static int
@@ -68,10 +68,7 @@ creat_files (const char *dirname, int count)
 	err (1, "chdir %s", dirname);
     if (stat (".", &sb) < 0)
 	err (1, "stat .");
-#if 0
-    if (sb.st_size != 2048)
-	errx (1, "size != 2048");
-#endif
+
     for (i = 0; i < count; ++i) {
 	char fname[256];
 	int len;
@@ -85,17 +82,27 @@ creat_files (const char *dirname, int count)
 	fname[j] = '\0';
 
 	fd = open (fname, O_CREAT | O_EXCL, 0777);
-	if (fd < 0)
+	if (fd < 0) {
+	    /* We sometimes get collisions; if so, try again. */
+	    if (errno == EEXIST) {
+		i--;
+		continue;
+	    }
 	    err (1, "open %s", fname);
+	}
+
 	if (close (fd) < 0)
 	    err (1, "close %s", fname);
     }
+
     if (stat (".", &sb) < 0)
 	err (1, "stat .");
 
     d = opendir (".");
     if (d == NULL)
 	err (1, "opendir .");
+
+    i = 0;
     while ((dp = readdir (d)) != NULL) {
 	if (strcmp (dp->d_name, ".") == 0
 	    || strcmp (dp->d_name, "..") == 0)
@@ -103,12 +110,18 @@ creat_files (const char *dirname, int count)
 
 	if (unlink (dp->d_name) < 0)
 	    err (1, "unlink %s", dp->d_name);
+	i++;
     }
     closedir (d);
+
+    if (i != count)
+	errx(1, "removed %d entries, %d expected", i, count);
+
     if (chdir ("..") < 0)
 	err (1, "chdir ..");
     if (rmdir (dirname) < 0)
 	err (1, "rmdir");
+
     return 0;
 }
 
