@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999 - 2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1999 - 2000, 2005 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -48,7 +48,7 @@
 #include <roken.h>
 
 #ifdef RCSID
-RCSID("$Id: symlink.c,v 1.4 2002/07/15 12:45:36 lha Exp $");
+RCSID("$Id: symlink.c,v 1.5 2005/10/28 14:33:40 tol Exp $");
 #endif
 
 int
@@ -56,6 +56,7 @@ main(int argc, char **argv)
 {
     int ret;
     struct stat sb;
+    char buf[MAXPATHLEN];
 
     setprogname (argv[0]);
 
@@ -67,6 +68,14 @@ main(int argc, char **argv)
 	err (1, "lstat bar");
     if ((sb.st_mode & S_IFLNK) != S_IFLNK)
 	errx (1, "bar is not symlink");
+
+    ret = readlink("bar", buf, sizeof(buf) - 1);
+    if (ret < 0)
+	err (1, "readlink bar");
+
+    buf[ret] = '\0';
+    if (strcmp("foo", buf))
+	err (1, "strcmp");
     ret = unlink ("bar");
     if (ret < 0)
 	err (1, "unlink bar");
