@@ -37,7 +37,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: log.c,v 1.38 2003/03/13 14:50:58 lha Exp $");
+RCSID("$Id: log.c,v 1.39 2005/05/03 18:30:42 lha Exp $");
 #endif
 
 #include <sys/types.h>
@@ -727,6 +727,10 @@ _print_unit (Log_unit *unit, char *buf, size_t sz)
 {
     size_t ret, orig_sz = sz;
     ret = snprintf (buf, sz, "%s:", unit->name);
+    if (ret == -1)
+	ret = 0;
+    else if (ret >= sz)
+	ret = sz - 1;
     UPDATESZ(buf,sz,ret);
     ret = unparse_flags (log_get_mask (unit), unit->unit, buf, sz);
     UPDATESZ(buf,sz,ret);
@@ -748,6 +752,10 @@ log_mask2str (Log_method *method, Log_unit *unit, char *buf, size_t sz)
 	if (log_get_mask (method->units[i])) {
 	    if (printed) {
 		ret = snprintf (buf, sz, ";");
+		if (ret == -1)
+		    ret = 0;
+		else if (ret >= sz)
+		    ret = sz - 1;
 		UPDATESZ(buf,sz,ret);
 	    }
 	    ret = _print_unit (method->units[i], buf, sz);
