@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2002 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2004 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -33,7 +33,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: output.c,v 1.91 2003/04/16 09:07:03 lha Exp $");
+RCSID("$Id: output.c,v 1.100 2005/10/20 19:41:59 lha Exp $");
 #endif
 
 #include <stdio.h>
@@ -435,7 +435,7 @@ encode_function (Type *type, EncodeType encodetype)
 	 switch (type->type) {
 	 case YDR_TLONGLONG:
 	 case YDR_TULONGLONG:
-	     return "htole64";
+	     return "htobe64";
 	 default:
 	     return "htonl";
 	     break;
@@ -444,7 +444,7 @@ encode_function (Type *type, EncodeType encodetype)
 	 switch (type->type) {
 	 case YDR_TLONGLONG:
 	 case YDR_TULONGLONG:
-	     return "letoh64";
+	     return "be64toh";
 	 default:
 	     return "ntohl";
 	     break;
@@ -1627,7 +1627,7 @@ generate_printfunction_definition (Symbol *s, FILE *f)
 }
 
 /*
- * Generate a defintion for a function to free `s', writing it to `f'
+ * Generate a definition for a function to free `s', writing it to `f'
  */
 
 static void
@@ -2173,14 +2173,48 @@ generate_standard_c_prologue (FILE *f,
 {
      fprintf (f, "/* Generated from %s.xg */\n", basename);
      fprintf (f, "#include \"%s.h\"\n\n", basename);
+     fprintf (f, "#ifndef _GNU_SOURCE\n");
+     fprintf (f, "#define _GNU_SOURCE 1\n");
+     fprintf (f, "#endif /* _GNU_SOURCE */\n");
+     fprintf (f, "#include <sys/types.h>\n");
      fprintf (f, "#include <stdio.h>\n");
      fprintf (f, "#include <stdlib.h>\n");
      fprintf (f, "#include <string.h>\n");
      fprintf (f, "#include <netinet/in.h>\n");
      fprintf (f, "#include <errno.h>\n");
      fprintf (f, "#ifdef RCSID\n"
-	      "RCSID(\"%s generated from %s.xg with $Id: output.c,v 1.91 2003/04/16 09:07:03 lha Exp $\");\n"
+	      "RCSID(\"%s generated from %s.xg with $Id: output.c,v 1.100 2005/10/20 19:41:59 lha Exp $\");\n"
 	      "#endif\n\n", filename, basename);
+     fprintf(f, 
+	     "/* crap for operationssystem that doesn't "
+	     "provide us 64 bit ops */\n"
+	     "#ifndef be64toh\n"
+	     "#if BYTE_ORDER == LITTLE_ENDIAN\n"
+	     "static inline uint64_t\n"
+	     "ydr_swap64(uint64_t x)\n"
+	     "{\n"
+	     "#define LT(n) n##ULL\n"
+	     "   return \n"
+	     "      ((LT(0x00000000000000ff) & x) << 56) | \n"
+	     "      ((LT(0x000000000000ff00) & x) << 40) | \n"
+	     "      ((LT(0x0000000000ff0000) & x) << 24) | \n"
+	     "      ((LT(0x00000000ff000000) & x) << 8) | \n"
+	     "      ((LT(0x000000ff00000000) & x) >> 8) | \n"
+	     "      ((LT(0x0000ff0000000000) & x) >> 24) | \n"
+	     "      ((LT(0x00ff000000000000) & x) >> 40) | \n"
+	     "      ((LT(0xff00000000000000) & x) >> 56) ; \n"
+	     "#undef LT\n"
+	     "}\n"
+	     "\n"
+	     "#define be64toh(x) ydr_swap64((x))\n"
+	     "#define htobe64(x) ydr_swap64((x))\n"
+	     "#endif /* BYTE_ORDER */\n"
+	     "#if BYTE_ORDER == BIG_ENDIAN\n"
+	     "#define be64toh(x) (x)\n"
+	     "#define htobe64(x) (x)\n"
+	     "#endif /* BYTE_ORDER */\n"
+	     "#endif /* be64toh */\n"
+	 );
 }
 
 /*
